package org.gridschool.unicore.plugins.boltzmann;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.UserTask;
import org.unicore.resources.Resource;
import org.unicore.resources.SoftwareResource;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * A basic example container executing a ApplicationResource and adding
 * an OutcomePanel to the Job Monitor area
 *
 *@author     Ralf Ratering
 *@version    $Id: BoltzmannContainer.java,v 1.5 2003/07/12 16:02:03 rrate Exp $
 */
public class BoltzmannContainer
	extends UserContainer {

	// name of our ApplicationResource
	private static final String APPLICATION_NAME = "Boltzmann";

	// id for compatibility between versions
	private static final long serialVersionUID = -4933274194596975934L;


	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent job group
	 */
	public BoltzmannContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Build the AJO ActionGroup that will be plugged into parent job group.
	 */
	protected void buildExecuteGroup() {
		logger.info("Building execution ActionGroup");

		// get a clone the task's AJO ResourceSet
		ResourceSet taskResourceSet = getResourceSet().getResourceSetClone();

		// add ApplicationResource that was set in checkContents
		// to ResourceSet
		taskResourceSet.add(getPreinstalledSoftware());

		// build task to execute a SoftwareResource
		UserTask executeTask = new UserTask(
			// task name
			getName(), null,
			// use ResourceSet that contains ApplicationResource
			taskResourceSet,
			getEnv(),
			getCommandLine(),
			null,
			getRedirectStdout(),
			getRedirectStderr(),
			isVerboseOn(),
			isVersionOn(),
			null,
			getMeasureTime(),
			getDebug(),
			getProfile());
		
	
		// build execute group and add UserTask to it
		executeGroup = new ActionGroup(getName() + "_EXECUTION");
		executeGroup.add(executeTask);


	}

	/**
	 *  Add container specific errors to error set of super classes and select
	 *  appropriate icon for display in Job preparation tree.
	 *
	 *@return    ErrorSet set containing all errors including those from
	 * super classes
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();

		if (id == null || getName().equals("")) {
			err.add(new UError(getIdentifier(), "No task name specified"));
		}

		// look for software resource if a vsite was selected
		if(getVsite()!=null) {
			checkSoftwareResource(err);
		}
		
		setErrors(err);
		return err;
	}

	/**
	 * Get ResourceSet of currently selected Vsite from ResourceManager
	 * and look for our application resource. If we found a matching
	 * resource, store it in the field of super class UserContainer.
	 * If no matching resource was found add an error to the ErrorSet
	 * parameter
	 *
	 *@param  errorSet ErrorSet of container
	 */
	private void checkSoftwareResource(ErrorSet errorSet) {
		boolean softwareAvailable = false;
		// get Vsite ResourceSet
		ResourceSet siteResources =
			ResourceManager.getResourceSet(getVsite()).getResourceSet();
		// Iterate over set elements and check resource names
		ResourceEnumeration siteEnum = siteResources.elements();
		while (siteEnum.hasMoreElements()) {
			Resource siteResource = (Resource) siteEnum.nextElement();
			if (siteResource instanceof SoftwareResource) {
				SoftwareResource siteSoftware = (SoftwareResource) siteResource;
				// is this one our application?
				if (siteSoftware
					.getName()
					.toUpperCase()
					.startsWith(APPLICATION_NAME.toUpperCase())) {
					softwareAvailable = true;
					// store it in field of super class
					setPreinstalledSoftware(siteSoftware);
				}
			}
		}

		if (!softwareAvailable) {
			errorSet.add(
				new UError(
					getIdentifier(),
					"Application "
						+ APPLICATION_NAME
						+ " is not supported at site "
						+  getVsite()));
		}

	}


}
