package com.pallas.ogsa.server;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Vector;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;

import org.apache.axis.server.AxisServer;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.helpers.XMLReaderFactory;

import com.pallas.security.KeyStoreManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.util.queues.Dispatcher;

/**
 *@author     Ralf Ratering
 *@author     Thomas Kentemich
 *@created    December 20, 2002
 *@version
 *@version    $Id: Server.java,v 1.11 2003/07/11 08:39:49 thomas Exp $
 */
public class Server implements IObserver, Runnable {

	private final int NUMTHREADS = 10;
	private int numThreads = NUMTHREADS;
	private static Logger logger = Logger.getLogger("com.pallas.ogsa.server");
	private static FileHandler filePlainHandler = null;
	private String logFile = "server.log";
	private String logPrefix = "";

	private static WatchDog watchdog;

	private AxisServer server = new AxisServer();
	private Dispatcher dispatcher = new Dispatcher();

	private boolean stopped = false;
	private PrintStream logStream = System.err;

	private URL url;
	private String name = "UNKNOWN";
	private SSLContext sslContext;
	private ServerSocket serverSocket;

	private KeyStoreManager manager;

	private int acceptedConnections = 0;
	private static final int MAX_CONNECTIONS = 50;

	private String CONFIG_FILE_NAME =
		".." + File.separator + "etc" + File.separator + "serverconfig.xml";
	private String keystoreFilename = ".." + File.separator + "server.keystore";
	private String password = "changeit";
	private String keystoreProvider = "SunJCE";
	private String keystoreType = "JCEKS";
	private String loggingDir = ".";
	private boolean useSSL = false;
	private int port = 8080;
	private long watchdogDelay = 10000;
	private long watchdogPeriod = 30000;
	private boolean watchdogMail = false;
	private String mailTo = "";

	private Deployment deployment;

	/**
	 *  Constructor for the Server object
	 *
	 *@param  args  Description of the Parameter
	 */
	public Server(String[] args, String name) {
		this.name = name;
		logPrefix = "[" + name + "] ";
		deployment = new Deployment(server);

		// look for config dir
		if (args.length > 0) {
			// overwrite config
			CONFIG_FILE_NAME = args[0];
			logger.info("Configfile is: " + CONFIG_FILE_NAME);
		}
		try {
			readServerConfiguration();
			File logDir = new File(loggingDir);
			logger.info("Logging directory is now " + loggingDir);
			if (!logDir.exists()) {
				logDir.mkdir();
			}

			if (!logDir.isDirectory()) {
				//hmm seems to be a file; reset dir  to .
				loggingDir = ".";
				logger.warning("Logging directory reset to [.]");
			}
			filePlainHandler =
				new FileHandler(
					loggingDir + File.separator + logFile,
					1048576,
					3,
					true);
			filePlainHandler.setFormatter(new SimpleFormatter());
			logger.addHandler(filePlainHandler);
		} catch (Exception e) {
			System.err.println("cannot register logging handler");
			e.printStackTrace();
		}

		try {
			Security.addProvider(
				new org.bouncycastle.jce.provider.BouncyCastleProvider());
			logger.finest(logPrefix + "Adding new security provider: ");
		} catch (Exception ex) {
			logger.log(
				Level.SEVERE,
				logPrefix + "Security provider BC not found",
				ex);
			System.exit(1);
		}
	}

	public int getPort() {
		return port;
	}

	public Deployment getDeployment() {
		return deployment;
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	private void readServerConfiguration() throws Exception {
		XMLReader parser = null;
		try {
			parser =
				XMLReaderFactory.createXMLReader(
					"org.apache.xerces.parsers.SAXParser");
		} catch (Exception e) {
			Parser sax1Parser =
				ParserFactory.makeParser("org.apache.xerces.parsers.SAXParser");
			parser = new ParserAdapter(sax1Parser);
			logger.warning(
				logPrefix
					+ "Features and Properties not supported by SAX1 parsers");
		}
		ServerSaxHandler handler = new ServerSaxHandler();
		parser.setContentHandler(handler);
		parser.setErrorHandler(handler);
		File configFile = new File(CONFIG_FILE_NAME);
		if (!configFile.exists()) {
			throw new FileNotFoundException(
				logPrefix + "Config file " + CONFIG_FILE_NAME + " not found.");
		}
		FileInputStream fis = new FileInputStream(configFile);

		InputSource source = new InputSource(fis);
		parser.parse(source);

	}

	/**
	 *  Sets the logStream attribute of the Server object
	 *
	 *@param  logStream  The new logStream value
	 */
	public void setLogStream(PrintStream logStream) {
		this.logStream = logStream;
	}

	/**
	 *  Gets the uRL attribute of the Server object
	 *
	 *@return    The uRL value
	 */
	public URL getURL() {
		return url;
	}

	/**
	 *  Gets the axisServer attribute of the Server object
	 *
	 *@return    The axisServer value
	 */
	public AxisServer getAxisServer() {
		return server;
	}

	/**
	 *  Gets the serverSocket attribute of the Server object
	 *
	 *@return    The serverSocket value
	 */
	public ServerSocket getServerSocket() {
		return serverSocket;
	}

	public void saveServerStatus() {
		server.saveConfiguration();
	}

	/**
	 *  Stop this server. This will interrupt any pending accept().
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public void stop() throws Exception {
		/*
		 *  Close the server socket cleanly, but avoid fresh accepts while
		 *  the socket is closing.
		 */

		stopped = true;
		watchdog.stop();
		try {
			serverSocket.close();
		} catch (Exception e) {
			logger.severe(logPrefix + "Exception on open");
			e.printStackTrace();
		}

		saveServerStatus();
		logger.info(logPrefix + "Shutdown Server");
	}

	/**
	  *  Implementation of the IObserver interface - check for the returned
	  *  object - setup a innner class "Runnable" - let the event-dispatchng
	  *  thread perform the update see http://java.sun.com/products/jfc/tsc/articles/threads/threads1.html
	  *  This method will be called from the running threads that have submitted
	  *  jobs to the njs
	  *
	  *@param  theObserved  Description of Parameter
	  *@param  changeCode   Description of Parameter
	  */
	public void observableUpdate(Object theObserved, Object changeCode) {
		logger.info(
			logPrefix
				+ "Connection dropped: "
				+ ((Socket) changeCode).getInetAddress()
				+ " port: "
				+ ((Socket) changeCode).getPort());
		acceptedConnections--;
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public void run() {
		watchdog =
			new WatchDog(watchdogDelay, watchdogPeriod, watchdogMail, this);
		watchdog.start();
		configureDispatcher();
		try {
			if (useSSL) {
				setupSSL();
			}
			initSocket(useSSL);
			listenOnSocket();
		} catch (Exception e) {
			logger.log(Level.SEVERE, logPrefix + "Server died.", e);
			try {
				stop();
			} catch (Exception ex) {
				logger.log(
					Level.SEVERE,
					logPrefix + "Could not stop server ",
					ex);
			}
			return;
		}
	}

	private void setupSSL() throws Exception {
		manager =
			KeyStoreManager.getInstance(
				keystoreProvider,
				keystoreType,
				keystoreFilename);
		manager.loadKeyStore(password.toCharArray(), password.toCharArray());
		manager.createSSLKeystore(
			true,
			true,
			password.toCharArray(),
			password.toCharArray());
		X509Certificate x509[] =
			manager.getCertificateByAlias(manager.getDefaultAlias());
		sslContext = manager.initializeSSLContext(new SecureRandom(), x509[0]);

	}

	/**
	 *  Description of the Method
	 */
	private void configureDispatcher() {
		dispatcher.createThreads(numThreads);
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	private void initSocket(boolean ssl) throws Exception {
		//
		if (ssl) {
			SSLServerSocketFactory sslSrvFact =
				sslContext.getServerSocketFactory();
			serverSocket = (ServerSocket) sslSrvFact.createServerSocket(port);
			((SSLServerSocket) serverSocket).setNeedClientAuth(true);
		} else {
			serverSocket = new ServerSocket(port);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	private void listenOnSocket() throws Exception {
		boolean stopped = false;
		Socket socket = null;
		java.security.cert.X509Certificate cert = null;

		String message = "Waiting for connection on: " + serverSocket;
		logger.info(logPrefix + message);
		if (logStream != null) {
			logStream.println(message);
		}

		while (!stopped) {
			try {

				socket = serverSocket.accept();
				if (acceptedConnections >= MAX_CONNECTIONS) {
					// drop cinnection
					socket.close();
					continue;
				}
				if (socket instanceof SSLSocket) {
					SSLSession session = ((SSLSocket) socket).getSession();

					try {
						// extract peer cerificate
						javax.security.cert.X509Certificate tmpchain =
							session.getPeerCertificateChain()[0];

						// Do a class change to the one required by the AJO and exposing
						// the full V3 options - note that the documentation says that
						// the javax class is a wrapper around the java class so
						// hopefully the getEncoded encode the full info.
						byte[] blah;
						try {
							blah = tmpchain.getEncoded();
						} catch (
							javax
								.security
								.cert
								.CertificateEncodingException ceex) {
							logger.info(
								"Connection from <"
									+ socket.getInetAddress()
									+ "> has unreadable certificate, rejecting.");
							socket.close();
							continue;
						}

						java.security.cert.CertificateFactory cf;
						try {
							cf =
								java
									.security
									.cert
									.CertificateFactory
									.getInstance(
									"X.509");
							cert =
								(
									java
										.security
										.cert
										.X509Certificate) cf
										.generateCertificate(
									new ByteArrayInputStream(blah));
						} catch (Exception ex) {
							logger.log(
								Level.SEVERE,
								"Problems converting certificate from <"
									+ socket.getInetAddress()
									+ ">.",
								ex);
							socket.close();
							continue;
						}

						// So far certificate is from trusted CA, check not revoked
						/*if(crl_checker.isRevoked(cert)) {
							socket.close();
							continue;
							} */

						// ... and still valid
						try {
							cert.checkValidity();
						} catch (Exception ex) {
							logger.info(
								"Invalid certificate presented from <"
									+ cert.getSubjectDN()
									+ "> on <"
									+ socket.getInetAddress().toString()
									+ ">");
							socket.close();
							continue;
						}

						logger.info(
							logPrefix
								+ "Peer connected with: "
								+ cert.getSubjectDN());
					} catch (Exception pue) {
						logger.warning("Peer unverified");
					}
				}

			} catch (IOException ioe) {
				logger.log(
					Level.SEVERE,
					logPrefix + "Exception on socket ",
					ioe);
				continue;
			}

			if (logStream != null) {
				logStream.println(message);
			}

			if (socket != null) {
				ServiceRequest newRequest =
					new ServiceRequest(socket, this, cert);
				newRequest.setLogStream(logStream);
				newRequest.addObserver(this);
				acceptedConnections++;
				message =
					"New connection accepted from: "
						+ socket.getInetAddress()
						+ " port "
						+ socket.getPort()
						+ " Currently "
						+ acceptedConnections
						+ " active.";
				logger.info(logPrefix + message);
				dispatcher.pushRequest(newRequest);

			}
		}
	}

	/**
	 *  Description of the Class
	 *
	 *@author     thomas
	 *@created    September 27, 2001
	 */
	private class ServerSaxHandler extends DefaultHandler {
		Vector foundSites = new Vector();

		//===========================================================
		// SAX DocumentHandler methods
		//===========================================================

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void startDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@exception  SAXException  Description of Exception
		 */
		public void endDocument() throws SAXException {
		}

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  lName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@param  attrs             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void startElement(
			String namespaceURI,
			String lName,
			String qName,
			Attributes attrs)
			throws SAXException {
			String elementName = lName;
			if ("".equals(elementName)) {
				elementName = qName;

				// namespaceAware = false
			}

			// read keystore information

			if (elementName.equals("keystore")) {
				if (attrs != null) {
					String keystore = null;
					String provider = null;
					String type = null;

					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("file")) {
							keystore = attrs.getValue(i);
						}

						if (aName.equals("provider")) {
							provider = attrs.getValue(i);
						}

						if (aName.equals("type")) {
							type = attrs.getValue(i);
						}
					}

					if (keystore != null && provider != null && type != null) {
						try {
							;
							keystoreFilename =
								keystore.replace('/', File.separatorChar);
							keystoreProvider = provider;
							keystoreType = type;
						} catch (Exception e) {
							logger.log(Level.SEVERE, "Error parsing XML.", e);
						}
					} else {
						logger.log(
							Level.SEVERE,
							logPrefix
								+ "Error parsing configfile "
								+ "keystore");
					}

				}
			}

			// read log dir
			if (elementName.equals("logdir")) {
				if (attrs != null) {
					String logdir = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("file")) {
							logdir = attrs.getValue(i);
						}
					}

					if (logdir != null) {
						try {
							loggingDir =
								logdir.replace('/', File.separatorChar);
						} catch (Exception e) {
							logger.log(
								Level.SEVERE,
								logPrefix + "Error parsing XML.",
								e);
						}
					} else {
						logger.log(
							Level.SEVERE,
							logPrefix + "Error parsing configfile " + "logdir");
					}

				}
			}

			// read ssl
			if (elementName.equals("ssl")) {
				if (attrs != null) {
					String ssl = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("usessl")) {
							ssl = attrs.getValue(i);
						}
					}

					if (ssl != null) {
						try {
							if (ssl.equalsIgnoreCase("true")) {
								useSSL = true;
							} else if (ssl.equalsIgnoreCase("false")) {
								useSSL = false;
							}
						} catch (Exception e) {
							logger.log(
								Level.SEVERE,
								logPrefix + "Error parsing XML.",
								e);
						}
					} else {
						logger.log(
							Level.SEVERE,
							logPrefix + "Error parsing configfile " + "ssl");
					}

				}

			}

			// read port
			if (elementName.equals("port")) {
				if (attrs != null) {
					String portString = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("listen")) {
							portString = attrs.getValue(i);
						}
					}

					if (portString != null) {
						try {
							port = Integer.parseInt(portString);

						} catch (Exception e) {
							logger.log(
								Level.SEVERE,
								logPrefix + "Error parsing XML.",
								e);
						}
					} else {
						logger.log(
							Level.SEVERE,
							logPrefix + "Error parsing configfile " + "port");
					}

				}

			}

			// read watchdog delay
			if (elementName.equals("watchdog")) {
				if (attrs != null) {
					String delayString = null;
					String periodString = null;
					String mailString = null;
					String mailtoString = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("delay")) {
							delayString = attrs.getValue(i);
						}
						if (aName.equals("period")) {
							periodString = attrs.getValue(i);
						}
						if (aName.equals("mail")) {
							mailString = attrs.getValue(i);
						}
						if (aName.equals("mailto")) {
							mailtoString = attrs.getValue(i);
						}
					}

					if (delayString != null
						&& periodString != null
						&& mailString != null
						&& mailtoString != null) {
						try {
							watchdogDelay = Long.parseLong(delayString);
							watchdogPeriod = Long.parseLong(periodString);
							if (mailString.equalsIgnoreCase("true")) {
								watchdogMail = true;
							} else if (mailString.equalsIgnoreCase("false")) {
								watchdogMail = false;
							}
							mailTo = mailtoString;
						} catch (Exception e) {
							logger.log(
								Level.SEVERE,
								logPrefix + "Error parsing XML.",
								e);
						}
					} else {
						logger.log(
							Level.SEVERE,
							logPrefix
								+ "Error parsing configfile "
								+ "watchdog");
					}

				}

			}

			// read threads
			if (elementName.equals("threads")) {
				if (attrs != null) {
					String numberString = null;
					for (int i = 0; i < attrs.getLength(); i++) {
						String aName = attrs.getLocalName(i);

						// Attr name
						if ("".equals(aName)) {
							aName = attrs.getQName(i);
						}
						if (aName.equals("number")) {
							numberString = attrs.getValue(i);
						}
						if (numberString != null) {
							numThreads = Integer.parseInt(numberString);
						}
					}
				}
			}

		} //startElement

		/**
		 *  Description of the Method
		 *
		 *@param  namespaceURI      Description of Parameter
		 *@param  sName             Description of Parameter
		 *@param  qName             Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void endElement(String namespaceURI, String sName, String qName)
			throws SAXException {
			// qualified name
		}

		/**
		 *  Description of the Method
		 *
		 *@param  buf               Description of Parameter
		 *@param  offset            Description of Parameter
		 *@param  len               Description of Parameter
		 *@exception  SAXException  Description of Exception
		 */
		public void characters(char[] buf, int offset, int len)
			throws SAXException {
		}
	}

}
