package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.plaf.metal.DefaultMetalTheme;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.SystemDefaults;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.controls.SimpleToolBar;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.panels.ThemeEditorPanel;
import com.pallas.unicore.client.themes.LookAndFeelManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.LoggingLevels;
import com.pallas.unicore.utility.UserMessages;

/**
 *  User interface to change user default settings.
 *
 *@author     Ralf Ratering
 *@created    21. August 2001
 *@version    $Id: UserDefaultsDialog.java,v 1.59 2002/02/27 09:16:50 thomas Exp
 *      $
 */
public class UserDefaultsDialog extends GenericDialog {

    /**
     *  Description of the Field
     */
    public final static String XML_FORMAT = "XML";
    /**
     *  Description of the Field
     */
    public final static String PLAIN_FORMAT = "PLAIN";
    /**
     *  Description of the Field
     */
    public final static String XML_AND_PLAIN_FORMAT = "BOTH";
    static ResourceBundle res = ResourceBundle.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
    private LoggingLevels logLevels = null;

    // text fields
    private JTextField
            emailTextField,
            xloginTextField,
            projectTextField,
            homeTextField,
            jobTextField,
            resourceTextField,
            importTextField,
            exportTextField,
            pluginTextField,
            keystoreTextField,
            logTextField,
            tmpTextField,
            crlURLTextField,
            proxyHostTextField;

    private IntTextField
            proxyPortTextField,
            socketTimeoutTextField;

    // text areas
    private JTextArea
            usiteURLTextArea,
            noProxyTextArea;

    // combo box for plafs and themes
    private JComboBox
            plafComboBox,
            themeComboBox,
            logLevelComboBox,
            logFormatComboBox;

    // check boxes
    private JCheckBox
            redirectCheckBox,
            autoFetchCheckBox,
            detailedJobStatusBox,
            proxyCheckBox,
            storageFormatCheckBox,
            preemptiveModeCheckBox;

    // buttons
    private JButton
            usiteURLChooserButton,
            usiteURLDefaultButton,
            crlURLChooserButton,
            crlURLDefaultButton,
            jobDirChooserButton,
            resourceDirChooserButton,
            importDirChooserButton,
            exportDirChooserButton,
            keystoreChooserButton,
            logDirChooserButton,
            tmpDirChooserButton,
            pluginDirChooserButton;

    // some panels that will be put into a tabbed pane
    private JPanel userPanel;
    private JPanel pathPanel;
    private JPanel systemPanel;
    private JPanel interfacePanel;
    private JPanel logPanel;
    private ThemeEditorPanel editorPanel;

    // the user defaults to be modified
    private UserDefaults userDefaults;

    private SystemDefaults systemDefaults;

    // remember the proxy status until the dialog stops
    private boolean proxyWasEnabled = false;


    /**
     *  Constructor takes parent frame as input and builds modal dialog
     *
     *@param  parentFrame  reference to parent JFrame
     */
    public UserDefaultsDialog(JFrame parentFrame) {
        super(parentFrame, res.getString("USERDEFAULT_TITLE"), true, OK_CANCEL);

        // get reference to user defaults from static resource manager
        userDefaults = ResourceManager.getUserDefaults();

        systemDefaults = ResourceManager.getSystemDefaults();

        buildComponents();

        pack();
    }


    /**
     *  Gets the validURL attribute of the UserDefaultsDialog object
     *
     *@param  urlStrings   Description of the Parameter
     *@param  description  Description of the Parameter
     *@return              The validURL value
     */
    private boolean isValidURL(String urlStrings, String description) {
        boolean returnCode = true;

        // urlString is actually a list of URLs separated by "\t"
        StringTokenizer st = new StringTokenizer(urlStrings, "\t");
        while(st.hasMoreElements()) {
            String urlString = st.nextToken();

            try {
                URL url = new URL(urlString);
            }
            catch(MalformedURLException e) {
                UserMessages.error("Entry for " + description + ": \n<" + urlString + ">\n" +
                        "has not correct URL syntax.\nPlease fix this or reset to system default.");
                returnCode = false;
            }
        }

        return returnCode;
    }



    /**
     *  Gets the formats attribute of the UserDefaultsDialog object
     *
     *@return    The formats value
     */
    private Vector getFormats() {
        Vector formats = new Vector();

        formats.addElement(XML_FORMAT);
        formats.addElement(PLAIN_FORMAT);
        formats.addElement(XML_AND_PLAIN_FORMAT);

        return formats;
    }


    /**
     *  Description of the Method
     *
     *@param  args  Description of Parameter
     */
    public static void main(String args[]) {
        UserDefaultsDialog d = new UserDefaultsDialog(new JFrame());
        d.show();
    }


    /**
     *  Overwrite show method to let email text field always grab focus.
     */
    public void show() {
        emailTextField.requestFocus();
        getRootPane().setDefaultButton(okButton);
        super.show();
    }


    /**
     *  Apply values will be called from parent class
     *
     *@return    true if applying was successful
     */
    protected boolean applyValues() {
        String usiteStrings = usiteURLTextArea.getText().trim();
        // change newlines and commas (inserted by the user into the list of URLs) to tabs
        usiteStrings = usiteStrings.replaceAll("\n", "\t").replaceAll(",", "\t");
        if(!isValidURL(usiteStrings, "UNICOREpro Site Server")) {
            return false;
        }
        userDefaults.setUsiteURL(usiteStrings);

        String crlString = crlURLTextField.getText().trim();
        if(!isValidURL(crlString, "Certificate Revocation List")) {
            return false;
        }
        userDefaults.setCrlURL(crlString);

        // copy text field entries to user defaults
        userDefaults.setUserEmail(emailTextField.getText().trim());
        userDefaults.setXlogin(xloginTextField.getText().trim());
        userDefaults.setProject(projectTextField.getText().trim());
        userDefaults.setJobDirectory(jobTextField.getText().trim());
        userDefaults.setResourceDirectory(resourceTextField.getText().trim());
        userDefaults.setImportDirectory(importTextField.getText().trim());
        userDefaults.setExportDirectory(exportTextField.getText().trim());
        userDefaults.setPluginDirectory(pluginTextField.getText().trim());
        userDefaults.setKeyStoreName(keystoreTextField.getText().trim());

        String logDir = this.logTextField.getText().trim();
        if(!logDir.equals(userDefaults.getLogDirectory())) {
            ResourceManager.resetFileHandlers();
            userDefaults.setLogDirectory(logTextField.getText().trim());
        }

        userDefaults.setTmpDirectory(tmpTextField.getText().trim());
        userDefaults.setProxyEnabled(proxyCheckBox.isSelected());
        userDefaults.setProxyHost(proxyHostTextField.getText());
        userDefaults.setProxyPort(proxyPortTextField.getValue());
        userDefaults.setNoProxy(noProxyTextArea.getText());
        //logging settings
        userDefaults.setLogLevel(((Level)logLevelComboBox.getSelectedItem()).toString());
        userDefaults.setLogFormat(logFormatComboBox.getSelectedItem().toString());
        userDefaults.setRedirect(redirectCheckBox.isSelected());

        userDefaults.setPreemptiveFileSelectorMode(this.preemptiveModeCheckBox.isSelected());
        userDefaults.setSocketTimeout(socketTimeoutTextField.getValue());
        //system settings
        userDefaults.setJobStorageFormat(storageFormatCheckBox.isSelected() ? "xml" : "serialized");
        userDefaults.setAutofetchOutcome(autoFetchCheckBox.isSelected());
        userDefaults.setDetailedJobStatus(detailedJobStatusBox.isSelected());

        // plaf
        String plaf = (String)plafComboBox.getSelectedItem();
        userDefaults.setLookAndFeel(LookAndFeelManager.getPlafByName(plaf));

        // theme
        String themeName = (String)themeComboBox.getSelectedItem();
        userDefaults.setTheme(LookAndFeelManager.getThemeByName(themeName));

        ResourceManager.redirectLog(redirectCheckBox.isSelected(), (String)logFormatComboBox.getSelectedItem(), (Level)logLevelComboBox.getSelectedItem());

        // write defaults to file
        if(!userDefaults.writeToFile()) {
            UserMessages.error(res.getString("COULD_NOT_WRITE"));
            setStatusMessage(res.getString("COULD_NOT_WRITE"));
        }
        else {
            setStatusMessage(res.getString("SUCESSFULLY_STORED"));
        }

        Client client = (Client)getParent();
        client.updateLookAndFeel();
      //  userDefaults.update();

	if (proxyCheckBox.isSelected() && !proxyWasEnabled) {
	    firePropertyChange("PROXY_ENABLED", null, null);
	}
	
	// remove all connections proxy settings may have changed
	ResourceManager.getConnectionManager().removeAllConnections();

        // returning true closes dialog if ok was pressed
        return true;
    }


    /**
     *  When cancel was pressed reload user defaults from file
     *
     *@return    if canceling was successful
     */
    protected boolean cancelValues() {
        userDefaults.loadFromFile();
        resetValues();
        return true;
    }


    /**
     *  Fill text fields with current class member values
     */
    protected void resetValues() {
        super.resetValues();

        emailTextField.setText(userDefaults.getUserEmail());
        xloginTextField.setText(userDefaults.getXlogin());
        projectTextField.setText(userDefaults.getProject());
        jobTextField.setText(userDefaults.getJobDirectory());
        resourceTextField.setText(userDefaults.getResourceDirectory());
        importTextField.setText(userDefaults.getImportDirectory());
        exportTextField.setText(userDefaults.getExportDirectory());
        pluginTextField.setText(userDefaults.getPluginDirectory());
        keystoreTextField.setText(userDefaults.getKeyStoreName());
        logTextField.setText(userDefaults.getLogDirectory());
        tmpTextField.setText(userDefaults.getTmpDirectory());
        // replace "\t" (and "," in old specification) of usite URLs by "\n"
        usiteURLTextArea.setText(userDefaults.getUsiteURL().replaceAll("\t", "\n").replaceAll(",", "\n"));
        //logging settings
        logLevelComboBox.setSelectedItem((logLevels.get(userDefaults.getLogLevel())));
        logFormatComboBox.setSelectedItem(userDefaults.getLogFormat());

        redirectCheckBox.setSelected(userDefaults.getRedirect());
        proxyCheckBox.setSelected(userDefaults.isProxyEnabled());
        proxyHostTextField.setText(userDefaults.getProxyHost());
        proxyPortTextField.setValue(userDefaults.getProxyPort());
        noProxyTextArea.setText(userDefaults.getNoProxy());
        crlURLTextField.setText(userDefaults.getCrlURL());

        storageFormatCheckBox.setSelected((userDefaults.getJobStorageFormat().equals("xml")));
        autoFetchCheckBox.setSelected(userDefaults.getAutofetchOutcome());
        detailedJobStatusBox.setSelected(userDefaults.getDetailedJobStatus());
        this.preemptiveModeCheckBox.setSelected(userDefaults.getPreemptiveFileSelectorMode());
        socketTimeoutTextField.setValue(userDefaults.getSocketTimeout());

        // find plaf
        String selectedPlaf =
                LookAndFeelManager.getPlafByFullName(userDefaults.getLookAndFeel());
        plafComboBox.setSelectedItem(selectedPlaf);

        // disable controls if metal was not selected
        boolean metalSelected = selectedPlaf.equals(LookAndFeelManager.getCrossPlatformPlafName());
        themeComboBox.setEnabled(metalSelected);
        editorPanel.setEnabled(metalSelected);

        // find theme
        if(userDefaults.getTheme() != null) {
            themeComboBox.setSelectedItem(userDefaults.getTheme().getName());
            editorPanel.setTheme(userDefaults.getTheme());
        }

	proxyWasEnabled = proxyCheckBox.isSelected();

    }


    /**
     *  Description of the Method
     */
    private void buildComponents() {
        addTooltipsToButtons();
        buildUserPanel();
        buildPathPanel();
        buildInterfacePanel();
        buildSystemPanel();
        buildLogPanel();
        this.logFormatComboBox.setEnabled(this.redirectCheckBox.isSelected());
        resetValues();

        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.add(userPanel, res.getString("GENERAL"));
        tabbedPane.add(pathPanel, res.getString("PATHS"));
        tabbedPane.add(systemPanel, res.getString("SYSTEM_SETTINGS"));
        tabbedPane.add(interfacePanel, res.getString("USER_INTERFACE"));
        tabbedPane.add(logPanel, res.getString("LOGGING_SETTINGS"));

        getContentPane().add(tabbedPane, BorderLayout.CENTER);

    }


    /**
     *  Description of the Method
     */
    private void buildLogPanel() {

        GridBagLayout g = new GridBagLayout();
        this.logPanel = new JPanel();
        logPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();

        JLabel levelLabel = new JLabel(res.getString("LOGGING_LEVEL"));
        c = LayoutTools.makegbc(0, 0, 1, 0, false);
        //specifies the external padding of the component, the minimum amount
        //of space between the component and the edges of its display area.
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(levelLabel, c);

        JLabel formatLabel = new JLabel(res.getString("LOGGING_FORMAT"));
        c = LayoutTools.makegbc(0, 1, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(formatLabel, c);

        JLabel logOutputLabel = new JLabel(res.getString("REDIRECT"));
        c = LayoutTools.makegbc(0, 2, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(logOutputLabel, c);

        logLevels = new LoggingLevels();
        Vector levels = logLevels.getLevels();
        logLevelComboBox = new JComboBox(levels);
        c = LayoutTools.makegbc(1, 0, 1, 100, true);
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(logLevelComboBox, c);

        Vector formats = this.getFormats();
        this.logFormatComboBox = new JComboBox(formats);
        c = LayoutTools.makegbc(1, 1, 1, 100, true);
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(logFormatComboBox, c);

        this.redirectCheckBox = new JCheckBox();
        this.redirectCheckBox.addItemListener(new CheckBoxListener());
        c = LayoutTools.makegbc(1, 2, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        this.logPanel.add(this.redirectCheckBox, c);

        Border matteBorder = BorderFactory.createMatteBorder(0, 170, 0, 170, this.getBackground());
        Border lineBorder = BorderFactory.createLineBorder(Color.GRAY);
        this.logPanel.setBorder(BorderFactory.createCompoundBorder(lineBorder, matteBorder));
    }


    /**
     *  Build sub panel
     */
    private void buildUserPanel() {
        this.userPanel = new JPanel();
        GridBagLayout g = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        userPanel.setLayout(g);

        c = LayoutTools.makegbc(0, 0, 1, 0, false);
        this.userPanel.add(new JLabel("User Email: "), c);

        c = LayoutTools.makegbc(0, 1, 1, 0, false);
        c.insets = new Insets(10, 1, 10, 1);
        //       this.userPanel.add(new JLabel("Login: "), c);

        c = LayoutTools.makegbc(0, 2, 1, 0, false);
        c.insets = new Insets(1, 1, 0, 1);
        //       this.userPanel.add(new JLabel("Project: ") ,c);

        c = LayoutTools.makegbc(0, 3, 1, 0, false);
        c.insets = new Insets(6, 1, 0, 1);
        this.userPanel.add(new JLabel(res.getString("USITE_URL")), c);

        c = LayoutTools.makegbc(0, 4, 1, 1, false);
        c.insets = new Insets(0, 1, 5, 1);
        this.userPanel.add(new JLabel(res.getString("CRL")), c);

        c = LayoutTools.makegbc(0, 5, 1, 1, false);
        this.userPanel.add(new JLabel(res.getString("PROXY_ADDRESS")), c);
        
		c = LayoutTools.makegbc(0, 6, 1, 1, false);
		this.userPanel.add(new JLabel("No proxy for: "), c);

        // fill text fields with previous defaults
        emailTextField = new JTextField(20);
        c = LayoutTools.makegbc(1, 0, 5, 100, true);
        this.userPanel.add(emailTextField, c);

        xloginTextField = new JTextField(20);
        c = LayoutTools.makegbc(1, 1, 5, 100, true);
        c.insets = new Insets(10, 1, 10, 1);
        //       this.userPanel.add(xloginTextField, c);

        projectTextField = new JTextField(20);
        c = LayoutTools.makegbc(1, 2, 5, 100, true);
        c.insets = new Insets(1, 1, 0, 1);
        //      this.userPanel.add(projectTextField, c);

        ButtonListener buttonListener = new ButtonListener();
        usiteURLTextArea = new JTextArea(1, 20);

        usiteURLDefaultButton = new JButton(ResourceManager.getIcon(ResourceManager.UNDO));
        // several system default Usite URLs may be seperated by "\t"
        usiteURLDefaultButton.setToolTipText("Reset value to system default: " + systemDefaults.getUsiteURL().replaceAll("\t", "\n"));
        usiteURLDefaultButton.setSize(usiteURLDefaultButton.getMinimumSize());
        usiteURLDefaultButton.addActionListener(buttonListener);

        usiteURLChooserButton = new JButton(ResourceManager.getIcon(ResourceManager.OPEN));
        usiteURLChooserButton.setToolTipText("Add local file");
        usiteURLChooserButton.setSize(usiteURLChooserButton.getMinimumSize());
        usiteURLChooserButton.addActionListener(buttonListener);

        c = LayoutTools.makegbc(1, 3, 4, 100, true);
//        c.insets = new Insets(6, 1, 0, 3);
        c.insets = new Insets(6, 1, 0, 1);
        this.userPanel.add(usiteURLTextArea, c);

        SimpleToolBar usitePanel =
                new SimpleToolBar(usiteURLChooserButton, usiteURLDefaultButton);
        c = LayoutTools.makegbc(5, 3, 1, 0, false);
        c.insets = new Insets(6, 0, 0, 0);
        c.anchor = GridBagConstraints.EAST;
        this.userPanel.add(usitePanel, c);

        crlURLTextField = new JTextField(20);
        c = LayoutTools.makegbc(1, 4, 4, 100, true);
//        c.insets = new Insets(0, 1, 5, 18);
        c.insets = new Insets(0, 1, 5, 1);
        this.userPanel.add(crlURLTextField, c);

        crlURLDefaultButton = new JButton(ResourceManager.getIcon(ResourceManager.UNDO));
        crlURLDefaultButton.setToolTipText("Reset value to system default: " + systemDefaults.getCrlURL());
        crlURLDefaultButton.addActionListener(buttonListener);

        SimpleToolBar crlPanel = new SimpleToolBar(crlURLDefaultButton);
        c = LayoutTools.makegbc(5, 4, 1, 0, false);
        c.insets = new Insets(0, 0, 5, 0);
        c.anchor = GridBagConstraints.EAST;
        this.userPanel.add(crlPanel, c);

        proxyHostTextField = new JTextField(20);
        c = LayoutTools.makegbc(1, 5, 1, 100, true);
        c.insets = new Insets(1, 1, 1, 3);
        this.userPanel.add(proxyHostTextField, c);

        c = LayoutTools.makegbc(2, 5, 1, 0, false);
        this.userPanel.add(new JLabel("Port: "), c);

        proxyPortTextField = new IntTextField(8080, 0, 99999);
        c = LayoutTools.makegbc(3, 5, 1, 100, true);
        c.insets = new Insets(1, 1, 1, 3);
        this.userPanel.add(proxyPortTextField, c);

        c = LayoutTools.makegbc(4, 5, 1, 0, false);
        c.anchor = GridBagConstraints.EAST;
        this.userPanel.add(new JLabel(res.getString("ON_OFF")), c);

        proxyCheckBox = new JCheckBox();
        c = LayoutTools.makegbc(5, 5, 1, 0, false);
        c.anchor = GridBagConstraints.EAST;
        this.userPanel.add(proxyCheckBox, c);
        
		noProxyTextArea = new JTextArea(1, 20); 
		noProxyTextArea.setToolTipText("Enter a list of non-proxy host seperated by `|`.");
		c = LayoutTools.makegbc(1, 6, 4, 100, true);
		c.insets = new Insets(6, 1, 0, 1);
		this.userPanel.add(noProxyTextArea, c);

        Border matteBorder = BorderFactory.createMatteBorder(10, 25, 10, 25, this.getBackground());
        Border lineBorder = BorderFactory.createLineBorder(Color.GRAY);
        this.userPanel.setBorder(BorderFactory.createCompoundBorder(lineBorder, matteBorder));
    }


    /**
     *  Build sub panel
     */
    private void buildPathPanel() {
        pathPanel = new JPanel();
        GridBagLayout g = new GridBagLayout();
        pathPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();

        c = LayoutTools.makegbc(0, 0, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("PLUGIN_LABEL")), c);

        c = LayoutTools.makegbc(0, 1, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("JOB_LABEL")), c);

        c = LayoutTools.makegbc(0, 2, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("RESOURCE_LABEL")), c);

        c = LayoutTools.makegbc(0, 3, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("IMPORT_LABEL")), c);

        c = LayoutTools.makegbc(0, 4, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("EXPORT_LABEL")), c);

        c = LayoutTools.makegbc(0, 5, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("LOG_LABEL")), c);

        c = LayoutTools.makegbc(0, 6, 1, 0, false);
        pathPanel.add(new JLabel(res.getString("TMP_LABEL")), c);

        jobTextField = new JTextField(25);
        resourceTextField = new JTextField(25);
        importTextField = new JTextField(25);
        exportTextField = new JTextField(25);
        pluginTextField = new JTextField(25);
        keystoreTextField = new JTextField(25);
        logTextField = new JTextField(25);
        tmpTextField = new JTextField(25);

        jobTextField.setToolTipText(res.getString("JOB_DIR"));
        resourceTextField.setToolTipText(res.getString("RESOURCE_DIR"));
        importTextField.setToolTipText(res.getString("IMPORT_DIR"));
        exportTextField.setToolTipText(res.getString("EXPORT_DIR"));
        pluginTextField.setToolTipText(res.getString("PLUGIN_DIR"));
        keystoreTextField.setToolTipText(res.getString("KEYSTORE_FILE"));
        logTextField.setToolTipText(res.getString("LOG_DIR"));
        tmpTextField.setToolTipText(res.getString("TMP_DIR"));

        // file chooser buttons
        Icon openIcon = ResourceManager.getIcon(ResourceManager.OPEN);
        ButtonListener buttonListener = new ButtonListener();

        jobDirChooserButton = new JButton(openIcon);
        jobDirChooserButton.addActionListener(buttonListener);
        jobDirChooserButton.setToolTipText(res.getString("JOB_DIR_TOOLTIP"));

        resourceDirChooserButton = new JButton(openIcon);
        resourceDirChooserButton.addActionListener(buttonListener);
        resourceDirChooserButton.setToolTipText(res.getString("RESOURCE_DIR_TOOLTIP"));

        importDirChooserButton = new JButton(openIcon);
        importDirChooserButton.addActionListener(buttonListener);
        importDirChooserButton.setToolTipText(res.getString("IMPORT_DIR_TOOLTIP"));

        exportDirChooserButton = new JButton(openIcon);
        exportDirChooserButton.addActionListener(buttonListener);
        exportDirChooserButton.setToolTipText(res.getString("EXPORT_DIR_TOOLTIP"));

        pluginDirChooserButton = new JButton(openIcon);
        pluginDirChooserButton.addActionListener(buttonListener);
        pluginDirChooserButton.setToolTipText(res.getString("PLUGIN_DIR_TOOLTIP"));

        keystoreChooserButton = new JButton(openIcon);
        keystoreChooserButton.addActionListener(buttonListener);
        keystoreChooserButton.setToolTipText(res.getString("KEYSTORE_TOOLTIP"));

        logDirChooserButton = new JButton(openIcon);
        logDirChooserButton.addActionListener(buttonListener);
        logDirChooserButton.setToolTipText(res.getString("LOG_DIR_TOOLTIP"));

        tmpDirChooserButton = new JButton(openIcon);
        tmpDirChooserButton.addActionListener(buttonListener);
        tmpDirChooserButton.setToolTipText(res.getString("TMP_DIR_TOOLTIP"));

        SimpleToolBar pluginPanel = new SimpleToolBar(pluginTextField, pluginDirChooserButton);
        c = LayoutTools.makegbc(1, 0, 2, 100, true);
        pathPanel.add(pluginPanel, c);

        SimpleToolBar jobPanel = new SimpleToolBar(jobTextField, jobDirChooserButton);
        c = LayoutTools.makegbc(1, 1, 2, 100, true);
        pathPanel.add(jobPanel, c);

        SimpleToolBar resourcePanel = new SimpleToolBar(resourceTextField, resourceDirChooserButton);
        c = LayoutTools.makegbc(1, 2, 2, 100, true);
        pathPanel.add(resourcePanel, c);

        SimpleToolBar importPanel = new SimpleToolBar(importTextField, importDirChooserButton);
        c = LayoutTools.makegbc(1, 3, 2, 100, true);
        pathPanel.add(importPanel, c);

        SimpleToolBar exportPanel = new SimpleToolBar(exportTextField, exportDirChooserButton);
        c = LayoutTools.makegbc(1, 4, 2, 100, true);
        pathPanel.add(exportPanel, c);

        SimpleToolBar logPanel = new SimpleToolBar(logTextField, logDirChooserButton);
        c = LayoutTools.makegbc(1, 5, 2, 100, true);
        pathPanel.add(logPanel, c);

        SimpleToolBar tmpPanel = new SimpleToolBar(tmpTextField, tmpDirChooserButton);
        c = LayoutTools.makegbc(1, 6, 2, 100, true);
        pathPanel.add(tmpPanel, c);

        Border matteBorder = BorderFactory.createMatteBorder(10, 25, 10, 25, this.getBackground());
        Border lineBorder = BorderFactory.createLineBorder(Color.GRAY);
        pathPanel.setBorder(BorderFactory.createCompoundBorder(lineBorder, matteBorder));
    }


    /**
     *  Build sub panel
     */
    private void buildInterfacePanel() {
        JPanel guiPanel = new JPanel();
        GridBagLayout g = new GridBagLayout();
        guiPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();

        ComboListener listener = new ComboListener();

        c = LayoutTools.makegbc(0, 0, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        guiPanel.add(new JLabel(res.getString("PLAF_LABEL")), c);

        plafComboBox = new JComboBox(LookAndFeelManager.getPlafNames());
        plafComboBox.addItemListener(listener);
        c = LayoutTools.makegbc(1, 0, 1, 100, true);
        c.insets = new Insets(5, 5, 5, 5);
        guiPanel.add(plafComboBox, c);

        c = LayoutTools.makegbc(0, 1, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        guiPanel.add(new JLabel(res.getString("THEME_LABEL")), c);

        themeComboBox = new JComboBox(LookAndFeelManager.getThemeNames());
        themeComboBox.setToolTipText(res.getString("PLAF_TIP") +
                LookAndFeelManager.getCrossPlatformPlaf());
        themeComboBox.addItemListener(listener);
        c = LayoutTools.makegbc(1, 1, 1, 100, true);
        c.insets = new Insets(5, 5, 5, 5);
        guiPanel.add(themeComboBox, c);

        guiPanel.setBorder(BorderFactory.createMatteBorder(0, 170, 0, 170, this.getBackground()));
        editorPanel = new ThemeEditorPanel();

        this.interfacePanel = new JPanel(new GridLayout(2, 1, 0, 10));
        this.interfacePanel.add(guiPanel);
        this.interfacePanel.add(editorPanel);

        Border matteBorder = BorderFactory.createMatteBorder(10, 10, 10, 10, this.getBackground());
        Border lineBorder = BorderFactory.createLineBorder(Color.GRAY);
        this.interfacePanel.setBorder(BorderFactory.createCompoundBorder(lineBorder, matteBorder));
    }


    /**
     *  Description of the Method
     */
    private void buildSystemPanel() {
        JPanel subPanel = new JPanel();
        GridBagLayout g = new GridBagLayout();
        subPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();

        c = LayoutTools.makegbc(0, 0, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(new JLabel(res.getString("LOADXML_LABEL")), c);

        storageFormatCheckBox = new JCheckBox();
        storageFormatCheckBox.setEnabled(true);
        c = LayoutTools.makegbc(1, 0, 2, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(storageFormatCheckBox, c);

        c = LayoutTools.makegbc(0, 1, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(new JLabel(res.getString("AUTOFETCH_LABEL")), c);

        autoFetchCheckBox = new JCheckBox();
        c = LayoutTools.makegbc(1, 1, 2, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(autoFetchCheckBox, c);

        c = LayoutTools.makegbc(0, 2, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(new JLabel(res.getString("DETAILEDJOBSTATUS_LABEL")), c);

        detailedJobStatusBox = new JCheckBox();
        c = LayoutTools.makegbc(1, 2, 2, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(detailedJobStatusBox, c);

        c = LayoutTools.makegbc(0, 3, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(new JLabel(res.getString("PREEMPTIVE_SELECTORDIALOG_MODE")), c);

        preemptiveModeCheckBox = new JCheckBox();
        c = LayoutTools.makegbc(1, 3, 2, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(preemptiveModeCheckBox, c);

        c = LayoutTools.makegbc(0, 4, 1, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(new JLabel("Socket timeout (ms)"), c);

        socketTimeoutTextField = new IntTextField(userDefaults.getSocketTimeout(), 0, 1000000);
        c = LayoutTools.makegbc(1, 4, 2, 0, false);
        c.insets = new Insets(5, 5, 5, 5);
        subPanel.add(socketTimeoutTextField, c);

        //Border matteBorder = BorderFactory.createMatteBorder(0, 170, 0, 100, this.getBackground());
        //Border lineBorder = BorderFactory.createLineBorder(Color.GRAY);
        this.systemPanel = new JPanel(new BorderLayout());
        this.systemPanel.add(subPanel, BorderLayout.CENTER);
        this.systemPanel.setBorder(BorderFactory.createLineBorder(Color.GRAY));
    }


    /**
     *  Stupid little helper method for GridBagLayout
     *
     *@param  panel  The feature to be added to the ToPanel attribute
     *@param  l      The feature to be added to the ToPanel attribute
     *@param  r      The feature to be added to the ToPanel attribute
     *@param  g      The feature to be added to the ToPanel attribute
     *@param  c      The feature to be added to the ToPanel attribute
     */
    private void addToPanel(JPanel panel,
            JComponent l,
            JComponent r,
            GridBagLayout g,
            GridBagConstraints c) {
        c.gridwidth = GridBagConstraints.RELATIVE;
        g.setConstraints(l, c);
        panel.add(l);

        c.gridwidth = GridBagConstraints.REMAINDER;
        g.setConstraints(r, c);
        panel.add(r);
    }


    /**
     *  Add tooltips to buttons from super class
     */
    private void addTooltipsToButtons() {
        okButton.setToolTipText(res.getString("OK_TIP"));
        cancelButton.setToolTipText(res.getString("CLOSE_DEFAULTS_TIP"));
    }


    /**
     *  Display a filechooser to select a directory
     *
     *@param  title           file chooser title
     *@param  startDirectory  Description of Parameter
     *@param  mode            Description of Parameter
     *@return                 selected directory
     */
    private String chooseDirectory(String title, String startDirectory, int mode) {
        SelectorDialog chooser = ResourceManager.getSelectorDialog(new File(startDirectory), null);
        chooser.setTitle(title);
        chooser.setFileSelectionMode(mode);
        int result = chooser.showDialog();

        if(result == JFileChooser.APPROVE_OPTION) {
            try {
                return chooser.getSelectedFile().getCanonicalPath();
            }
            catch(IOException e) {
                return null;
            }
        }
        return null;
    }


    /**
     *  Listen to button clicks
     *
     *@author     Ralf Ratering
     *@created    21. August 2001
     */
    private class ButtonListener implements ActionListener {
        /**
         *  Description of the Method
         *
         *@param  event  Description of the Parameter
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();
            if(source == usiteURLChooserButton) {
                String value =
                        chooseDirectory("UNICOREpro: Choose local site server file",
                        userDefaults.getUnicoreDir(), JFileChooser.FILES_ONLY);
                if(value != null) {
                    try {
                        StringBuffer usitesString = new StringBuffer(usiteURLTextArea.getText().trim());
                        if(usitesString.length() > 0) {
                            usitesString.append("\n");
                        }
                        usitesString.append(new File(value).toURL().toString());

                        usiteURLTextArea.setText(usitesString.toString());
                    }
                    catch(MalformedURLException e) {
                        // should not occur...
                        logger.log(Level.SEVERE, "", e);
                    }
                }
            }
            else if(source == usiteURLDefaultButton) {
                // several system default Usite URLs may be seperated by "\t"
                usiteURLTextArea.setText(systemDefaults.getUsiteURL().replaceAll("\t", "\n"));
            }
            else if(source == crlURLDefaultButton) {
                crlURLTextField.setText(systemDefaults.getCrlURL());
            }
            else if(source == jobDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_JOB_DIR"),
                        jobTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    jobTextField.setText(value);
                }
            }
            else if(source == resourceDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_RESOURCE_DIR"),
                        resourceTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    resourceTextField.setText(value);
                }
            }
            else if(source == importDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_IMPORT_DIR"),
                        importTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    importTextField.setText(value);
                }
            }
            else if(source == exportDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_EXPORT_DIR"),
                        exportTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    exportTextField.setText(value);
                }
            }
            else if(source == pluginDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_PLUGIN_DIR"),
                        pluginTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    pluginTextField.setText(value);
                }
            }
            else if(source == keystoreChooserButton) {
                String value = chooseDirectory(res.getString("SELECT_KEYSTORE"),
                        keystoreTextField.getText(), JFileChooser.FILES_AND_DIRECTORIES);
                if(value != null) {
                    keystoreTextField.setText(value);
                }
            }
            else if(source == logDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_LOG_DIR"),
                        logTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    logTextField.setText(value);
                }
            }
            else if(source == tmpDirChooserButton) {
                String value =
                        chooseDirectory(res.getString("SELECT_TMP_DIR"),
                        tmpTextField.getText(), JFileChooser.DIRECTORIES_ONLY);
                if(value != null) {
                    tmpTextField.setText(value);
                }
            }

        }
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    21. August 2001
     */
    private class ComboListener implements ItemListener {
        /**
         *  Description of the Method
         *
         *@param  e  Description of Parameter
         */
        public void itemStateChanged(ItemEvent e) {
            if(e.getStateChange() == ItemEvent.DESELECTED) {
                return;
            }
            Object source = e.getSource();

            if(source.equals(plafComboBox)) {
                String selectedPlaf = (String)plafComboBox.getSelectedItem();
                boolean metalSelected = selectedPlaf.equals(LookAndFeelManager.getCrossPlatformPlafName());
                themeComboBox.setEnabled(metalSelected);
                editorPanel.setEnabled(metalSelected);
            }
            else if(source.equals(themeComboBox)) {
                String themeName = (String)themeComboBox.getSelectedItem();
                DefaultMetalTheme theme = LookAndFeelManager.getThemeByName(themeName);
                editorPanel.setTheme(theme);
            }
        }

    }


    /**
     *  Description of the Class
     *
     *@author     kfoerst
     *@created    13. August 2002
     */
    private class CheckBoxListener implements ItemListener {
        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void itemStateChanged(ItemEvent e) {
            Object source = e.getSource();

            if(source.equals(redirectCheckBox)) {
                logFormatComboBox.setEnabled(redirectCheckBox.isSelected());
            }
            if(!redirectCheckBox.isSelected()) {
                logFormatComboBox.setSelectedItem(PLAIN_FORMAT);
            }
        }
    }
}

