package com.pallas.unicore.client.graphs;

import java.util.Vector;

import com.pallas.unicore.utility.VectorSorter;

/**
 *  Directed acyclic graph
 *
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: DAG.java,v 1.2 2003/06/30 06:54:10 rrate Exp $
 */
public class DAG extends Graph {
	private int maxDepth;

	/**
	 *  Adds a feature to the Edge attribute of the DAG object
	 *
	 *@param  edge           The feature to be added to the Edge attribute
	 *@exception  Exception  Description of the Exception
	 */
	public void addEdge(Edge edge) throws Exception {
		if (causesCycle(edge)) {
			throw new Exception(
				"Cannot add "
					+ edge
					+ ", because it would cause a cycle in the directed acyclic graph.");
		}
		if (!edges.contains(edge)) {
			edges.add(edge);
			edge.getParentVertex().addEdge(edge);
			edge.getChildVertex().addEdge(edge);
			edge.setColor(color);
			edge.setSelectionColor(selectionColor);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  edge  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	public boolean causesCycle(Edge edge) {
		return pathExists(
			(DAGVertex) edge.getChildVertex(),
			(DAGVertex) edge.getParentVertex());
	}

	/**
	 *  Compute depth of a single vertex
	 *
	 *@param  vertex  vertex to be computed
	 */
	private void computeDepth(DAGVertex vertex) {
		Vector children = vertex.getChildren();
		for (int i = 0; i < children.size(); i++) {
			DAGVertex child = (DAGVertex) children.get(i);
			int newDepth = vertex.getDepth() + 1;
			// only increase depths
			if (newDepth > child.getDepth()) {
				child.setDepth(vertex.getDepth() + 1);
				computeDepth(child);
			}
		}

	}

	/**
	 *  Compute depth for vertices
	 */
	public void computeDepths() {
		// reset depths
		for (int i = 0; i < getNumberOfVertices(); i++) {
			((DAGVertex) getVertex(i)).setDepth(0);
		}

		// loop over vertices and start recursion for root vertices (depth==0)
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.isRoot()) {
				computeDepth(vertex);
			}
		}
		computeMaxDepth();
	}

	/**
	 *  Find out maximum depth in graph
	 */
	private void computeMaxDepth() {
		maxDepth = 0;
		// loop over vertices and start recursion for root vertices (depth==0)
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.getDepth() > maxDepth) {
				maxDepth = vertex.getDepth();
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  edge  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	public boolean contains(Edge edge) {
		for (int i = 0; i < edges.size(); i++) {
			if (edge.equals(edges.elementAt(i))) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Gets the Depth attribute of the DAG object
	 *
	 *@return    The Depth value
	 */
	public int getDepth() {
		return maxDepth;
	}

	/**
	 *  Get all nodes at a certain depth
	 *
	 *@param  depth  depth level for nodes
	 *@return        vector of vertices
	 */
	public Vector getNodesByDepth(int depth) {
		Vector result = new Vector();
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.getDepth() == depth) {
				result.add(vertex);
			}
		}
		return result;
	}

	/**
	 *  Get a Vector containing all possile successors of the DAG vertex
	 *
	 *@param  vertex  vertex to get successors from
	 *@return         a Vector containing all possible successors
	 */
	public Vector getPossibleSuccs(Vertex vertex) {
		return vertices;
	}

	/**
	 *  Gets the sortedVertices attribute of the DAG object
	 *
	 *@param  ascending  Description of the Parameter
	 *@return            The sortedVertices value
	 */
	public Vector getSortedVertices(boolean ascending) {
		// depths will be used for sorting
		computeDepths();
		// Vector preSorted = VectorSorter.sort(vertices, ascending, true);
		return VectorSorter.sort(vertices, ascending);
	}

	/**
	 *  Check if a path exists between two vertices
	 *
	 *@param  start  start vertex
	 *@param  end    end vertex
	 *@return        true, if path exists
	 */
	public boolean pathExists(DAGVertex start, DAGVertex end) {
		if (start == end) {
			return true;
		}
		Vector children = start.getChildren();
		for (int i = 0; i < children.size(); i++) {
			if (pathExists((DAGVertex) children.elementAt(i), end)) {
				return true;
			}
		}
		return false;
	}
}
