package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.DependencyContainer;
import com.pallas.unicore.container.FileTransferContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Editor for UNICOREpro container dependency graphcs
 *
 *@author     Ralf Ratering
 *@created    22. August 2001
 *@version    $Id: DependencyEditor.java,v 1.1.1.1 2002/04/22 08:30:25 thomas
 *      Exp $
 */

public class DependencyEditor extends DAGEditor {
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.graphs");

	/**
	 *  The main program for the DependencyEditor class
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		ResourceManager.init(new com.pallas.unicore.client.Client(args));
		JobContainer jobContainer = new JobContainer();

		FileTransferContainer task1 = new FileTransferContainer(jobContainer);
		task1.setName("Eins");
		FileTransferContainer task2 = new FileTransferContainer(jobContainer);
		task2.setName("Zwei");
		FileTransferContainer task3 = new FileTransferContainer(jobContainer);
		task3.setName("Drei");
		FileTransferContainer task4 = new FileTransferContainer(jobContainer);
		task4.setName("Vier");

		DependencyContainer[] dependencies =
			{
				new DependencyContainer(task1, task2),
				new DependencyContainer(task1, task3),
				new DependencyContainer(task1, task4),
				new DependencyContainer(task2, task3),
				new DependencyContainer(task2, task4),
				new DependencyContainer(task3, task4)};
		jobContainer.setDependencies(dependencies);

		DependencyEditor dependencyEditor = new DependencyEditor(jobContainer);
		dependencyEditor.setBackground(new Color(0, 100, 200));
		dependencyEditor.setColor(Color.white);
		dependencyEditor.setSelectionColor(Color.green);

		JScrollPane scrollPane = new JScrollPane(dependencyEditor);

		JFrame frame = new JFrame("Dependency Editor");
		frame.getContentPane().add(scrollPane);
		frame.setSize(400, 400);
		frame.show();
	}
	private GroupContainer container;
	private Hashtable taskTable = new Hashtable();

	/**
	 *  Constructor
	 *
	 *@param  container  Description of the Parameter
	 */
	public DependencyEditor(GroupContainer container) {
		// our graph will be a dag
		super(new DAG());
		this.container = container;
		resetValues();
	}

	/**
	 *  Apply dependencies and positions to JobContainer
	 */
	public void applyValues() {
		updateModifiedTime();

		// apply icon positions to containers
		for (int i = 0; i < graph.getNumberOfVertices(); i++) {
			ContainerVertex vertex = (ContainerVertex) graph.getVertex(i);
			vertex.getContainer().setIconPosition(vertex.getCenter());
		}

		// set deps
		DependencyContainer[] deps =
			new DependencyContainer[graph.getNumberOfEdges()];
		for (int i = 0; i < graph.getNumberOfEdges(); i++) {
			Edge edge = graph.getEdge(i);
			ContainerVertex pred = (ContainerVertex) edge.getParentVertex();
			ContainerVertex succ = (ContainerVertex) edge.getChildVertex();
			deps[i] =
				new DependencyContainer(
					pred.getContainer(),
					succ.getContainer());
		}
		container.setDependencies(deps);
	}

	/**
	 *  find a nice position for a vertex
	 *
	 *@param  vertex  vertex to be placed
	 */
	private void findPosition(ContainerVertex vertex) {

		int width = getWidth();
		int height = getHeight();

		Point point = new Point(width / 2, height / 2);
		vertex.setCenter(point);
		int counter = 0;
		while (!goodPosition(vertex) && counter < 100) {
			point.setLocation(Math.random() * width, Math.random() * height);
			vertex.setCenter(point);
			counter++;
		}
	}

	/**
	 *  Check, if a vertex has a nice position
	 *
	 *@param  vertex  Vertex to be tested
	 *@return         true, if it's a nice position
	 */
	private boolean goodPosition(Vertex vertex) {
		double x = vertex.getCenter().getX();
		double y = vertex.getCenter().getY();
		if (x < 40 || x > getWidth() - 40 || y < 40 || y > getHeight() - 40) {
			return false;
		}

		Rectangle bounds = vertex.getBounds();
		for (int i = 0; i < graph.getNumberOfVertices(); i++) {
			if (bounds.intersects(graph.getVertex(i).getBounds())) {
				return false;
			}
		}

		for (int i = 0; i < graph.getNumberOfEdges(); i++) {
			if (bounds.intersectsLine(graph.getEdge(i).getLine())) {
				return false;
			}
		}

		return true;
	}

	/**
	 *  Overwrite method from sub class to directly remove dependency from
	 *  container as well
	 *
	 *@param  edge  Description of the Parameter
	 */
	protected void removeEdge(Edge edge) {
		super.removeEdge(edge);
		ContainerVertex pred = (ContainerVertex) edge.getParentVertex();
		ContainerVertex succ = (ContainerVertex) edge.getChildVertex();
		ActionContainer predCont = pred.getContainer();
		ActionContainer succCont = succ.getContainer();
		DependencyContainer[] deps = container.getDependencies();
		for (int i = 0; i < deps.length; i++) {
			if (deps[i].getPred().equals(predCont)
				&& deps[i].getSucc().equals(succCont)) {
				container.removeDependency(deps[i]);
				return;
			}
		}
	}

	/**
	 *  Apply values from container to graph
	 */
	public void resetValues() {
		updateValues();

		// apply last container icon positions
		for (int i = 0; i < graph.getNumberOfVertices(); i++) {
			ContainerVertex vertex = (ContainerVertex) graph.getVertex(i);
			Point lastPos = vertex.getContainer().getIconPosition();
			if (lastPos != null) {
				vertex.setCenter(lastPos);
			}
		}

		// remove old edges
		graph.removeAllEdges();
		DependencyContainer[] dependencies = container.getDependencies();
		for (int i = 0; i < dependencies.length; i++) {
			DependencyContainer dep = dependencies[i];
			Vertex pred = (Vertex) taskTable.get(dep.getPred());
			Vertex succ = (Vertex) taskTable.get(dep.getSucc());
			Edge edge = new Edge(pred, succ);
			try {
				graph.addEdge(edge);
			} catch (Exception e) {
				logger.log(Level.WARNING, "Cycle in dependencies.", e);
			}
		}
		repaint();
	}

	/**
	 *  Sets the Foreground attribute of the DependencyEditor object
	 *
	 *@param  color  The new Color value
	 */
	public void setColor(Color color) {
		graph.setColor(color);
	}

	/**
	 *  Sets the SelectionColor attribute of the DependencyEditor object
	 *
	 *@param  color  The new SelectionColor value
	 */
	public void setSelectionColor(Color color) {
		graph.setSelectionColor(color);
	}

	/**
	 *  Check if something has changed in the graph
	 */
	private void updateModifiedTime() {
		// compare icon positions to containers
		for (int i = 0; i < graph.getNumberOfVertices(); i++) {
			ContainerVertex vertex = (ContainerVertex) graph.getVertex(i);
			if (vertex.getContainer().getIconPosition() != null
				&& !vertex.getContainer().getIconPosition().equals(
					vertex.getCenter())) {
				container.setModifiedTime(new Date());
				return;
			}
		}

		// compare edges to container deps
		DependencyContainer[] deps = container.getDependencies();
		// compare numbers
		if (deps != null && deps.length != graph.getNumberOfEdges()) {
			container.setModifiedTime(new Date());
			return;
		}

		// find matching deps
		for (int i = 0; i < graph.getNumberOfEdges(); i++) {
			Edge edge = graph.getEdge(i);
			ContainerVertex pred = (ContainerVertex) edge.getParentVertex();
			ContainerVertex succ = (ContainerVertex) edge.getChildVertex();
			boolean found = false;
			int j = 0;
			while (!found && j < deps.length) {
				// just check if objects match
				if (deps[j].getPred().equals(pred.getContainer())
					&& deps[j].getSucc().equals(succ.getContainer())) {
					// matching dep found
					found = true;
				}
				j++;
			}
			if (!found) {
				// if we got here no matching dep was found ->modified!
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 *  Update vertices and positions
	 */
	public void updateValues() {

		// update vertices
		Vector tasks = container.getTasks();
		Hashtable oldTable = taskTable;
		taskTable = new Hashtable();
		for (int i = 0; i < tasks.size(); i++) {
			ActionContainer container = (ActionContainer) tasks.get(i);

			if (!oldTable.containsKey(container)) {
				ContainerVertex vertex = new ContainerVertex(container);
				// first try to apply container position
				Point position = vertex.getContainer().getIconPosition();

				if (position == null) {
					// give new vertex a random position on screen
					// if container did not contain a position
					// in case of adding a new task
					findPosition(vertex);
				} else {
					// test if container has been copied and pasted, so that
					// another vertex has same position
					boolean isClone = false;
					for (int j = 0; j < graph.getNumberOfVertices(); j++) {
						if (position.equals(graph.getVertex(j).getCenter())) {
							isClone = true;
						}
					}
					if (isClone) {
						findPosition(vertex);
					} else {
						// apply last container position (in case of loading a job)
						vertex.setCenter(position);
					}
				}

				// a new vertex will be added to the graph
				graph.addVertex(vertex);
				// put container into new table
				taskTable.put(container, vertex);
			}
			// container did exist at last apply time
			else {
				Vertex vertex = (Vertex) oldTable.get(container);
				// update name
				vertex.setName(container.getName());
				// put container into new table anyway
				taskTable.put(container, vertex);
				// remove old vertices that are still in the graph from table
				oldTable.remove(container);
			}
		}

		// remove vertices that disappeared from graph
		Enumeration deletedVertices = oldTable.elements();
		while (deletedVertices.hasMoreElements()) {
			Vertex deletedVertex = (Vertex) deletedVertices.nextElement();
			graph.removeVertex(deletedVertex);

			// update edges in graph
			for (int i = 0; i < graph.getNumberOfEdges(); i++) {
				if (graph.getEdge(i).contains(deletedVertex)) {
					graph.removeEdge(graph.getEdge(i));
				}
			}
		}

		// check if new edges have appeared
		DependencyContainer[] deps = container.getDependencies();
		for (int i = 0; i < deps.length; i++) {
			Vertex pred = (Vertex) taskTable.get(deps[i].getPred());
			Vertex succ = (Vertex) taskTable.get(deps[i].getSucc());
			Edge edge = new Edge(pred, succ);
			if (!((DAG) graph).contains(edge)) {
				try {
					graph.addEdge(edge);
				} catch (Exception e) {
					logger.log(Level.WARNING, "Cycle in dependencies.", e);
				}
			}
		}
		repaint();
	}
}
