package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import javax.swing.JPanel;

import com.pallas.unicore.client.graphics.Arrow;
import com.pallas.unicore.client.graphics.Drawable;
import com.pallas.unicore.client.graphics.Selectable;

/**
 *  Edge in a DAG. For cyclic graphs directions may be ignored.
 *
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: Edge.java,v 1.2 2003/06/30 06:54:08 rrate Exp $
 */
public class Edge implements Selectable, Drawable {
	private Arrow arrow =
		new Arrow(new Point2D.Double(0.0, 0.0), new Point2D.Double(0.0, 0.0));
	private Vertex childVertex, parentVertex;
	private Color color = Color.black, selectionColor = Color.red;
	private Line2D line = new Line2D.Double(0.0, 0.0, 0.0, 0.0);
	private String name;
	private boolean selected;

	/**
	 *  Constructor for the Edge object
	 *
	 *@param  name          name for edge
	 *@param  parentVertex  start vertex
	 *@param  childVertex   end vertex
	 */
	public Edge(String name, Vertex parentVertex, Vertex childVertex) {
		this.name = name;
		this.parentVertex = parentVertex;
		this.childVertex = childVertex;

		setSelected(false);
	}

	/**
	 *  Constructor for the Edge object
	 *
	 *@param  parentVertex  start vertex
	 *@param  childVertex   end vertex
	 */
	public Edge(Vertex parentVertex, Vertex childVertex) {
		this("", parentVertex, childVertex);
	}

	/**
	 *  Test if a vertex belongs to this edge
	 *
	 *@param  vertex  vertex to be tested
	 *@return         true if vertex belongs to edge
	 */
	public boolean contains(Vertex vertex) {
		if (vertex.equals(childVertex) || vertex.equals(parentVertex)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  g      Description of Parameter
	 *@param  panel  Description of Parameter
	 */
	public void draw(Graphics g, JPanel panel) {
		update();

		Graphics2D g2 = (Graphics2D) g;
		if (selected) {
			g2.setColor(selectionColor);
		} else {
			g2.setColor(color);
		}

		g2.draw(arrow);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public boolean equals(Object object) {
		Edge edge2 = (Edge) object;
		return childVertex.equals(edge2.childVertex)
			&& parentVertex.equals(edge2.parentVertex);
	}

	/**
	 *  Gets the Bounds attribute of the Edge object
	 *
	 *@return    The Bounds value
	 */
	public Rectangle getBounds() {
		return line.getBounds();
	}

	/**
	 *  Gets the Center attribute of the Edge object
	 *
	 *@return    The Center value
	 */
	public Point getCenter() {
		return new Point(
			(int) line.getBounds().getCenterX(),
			(int) line.getBounds().getCenterY());
	}

	/**
	 *  Gets the ChildVertex attribute of the Edge object
	 *
	 *@return    The ChildVertex value
	 */
	public Vertex getChildVertex() {
		return childVertex;
	}

	/**
	 *  Gets the Color attribute of the Edge object
	 *
	 *@return    The Color value
	 */
	public Color getColor() {
		return color;
	}

	/**
	 *  Gets the Line attribute of the Edge object
	 *
	 *@return    The Line value
	 */
	public Line2D getLine() {
		return line;
	}

	/**
	 *  Gets the ParentVertex attribute of the Edge object
	 *
	 *@return    The ParentVertex value
	 */
	public Vertex getParentVertex() {
		return parentVertex;
	}

	/**
	 *  Gets the SelectionColor attribute of the Edge object
	 *
	 *@return    The SelectionColor value
	 */
	public Color getSelectionColor() {
		return selectionColor;
	}

	/**
	 *  Gets the Selected attribute of the Edge object
	 *
	 *@return    The Selected value
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 *  Sets the Center attribute of the Edge object
	 *
	 *@param  center  The new Center value
	 */
	public void setCenter(Point center) {
		int tx = (int) (center.getX() - getBounds().getCenterX());
		int ty = (int) (center.getY() - getBounds().getCenterY());
		parentVertex.getCenter().translate(tx, ty);
		childVertex.getCenter().translate(tx, ty);
	}

	/**
	 *  Sets the Color attribute of the Edge object
	 *
	 *@param  color  The new Color value
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 *  Sets the Selected attribute of the Edge object
	 *
	 *@param  selected  The new Selected value
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 *  Sets the SelectionColor attribute of the Edge object
	 *
	 *@param  selectionColor  The new SelectionColor value
	 */
	public void setSelectionColor(Color selectionColor) {
		this.selectionColor = selectionColor;
	}

	/**
	 *  String representation for edge
	 *
	 *@return    String representation
	 */
	public String toString() {
		String result = "";
		if (!name.equals("")) {
			result += name + ": ";
		}
		return result + parentVertex + " -> " + childVertex;
	}

	/**
	 *  Description of the Method
	 */
	public void update() {
		line.setLine(parentVertex.getCenter(), childVertex.getCenter());

		if (parentVertex.getBounds().intersects(childVertex.getBounds())) {
			arrow.setLine(
				new Point2D.Double(0.0, 0.0),
				new Point2D.Double(0.0, 0.0));
			return;
		}

		Point2D start = parentVertex.getBounds().findIntersection(line);
		Point2D end = childVertex.getBounds().findIntersection(line);
		if (start == null || end == null) {
			start = new Point2D.Double(0.0, 0.0);
			end = new Point2D.Double(0.0, 0.0);
		}
		arrow.setLine(start, end);
	}

}
