package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Dimension;
import java.util.Vector;

/**
 *  A graph consisting of edges and vertices
 *
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: Graph.java,v 1.2 2003/06/30 06:54:08 rrate Exp $
 */
public class Graph {
	/**
	 *  colors
	 */
	protected Color color, selectionColor;
	/**
	 *  edges
	 */
	protected Vector edges = new Vector();
	/**
	 *  vertices
	 */
	protected Vector vertices = new Vector();

	/**
	 *  Constructor for the Graph object
	 */
	public Graph() {
		setColor(Color.black);
		setSelectionColor(Color.red);
	}

	/**
	 *  Adds a feature to the Edge attribute of the Graph object
	 *
	 *@param  edge           The feature to be added to the Edge attribute
	 *@exception  Exception  Description of the Exception
	 */
	public void addEdge(Edge edge) throws Exception {
		if (edges.contains(edge)) {
			throw new Exception(
				"Cannot add edge " + edge + ". Edge already exists.");
		}
		edges.add(edge);
		edge.getParentVertex().addEdge(edge);
		edge.getChildVertex().addEdge(edge);
		edge.setColor(color);
		edge.setSelectionColor(selectionColor);
	}

	/**
	 *  Adds a vertex to graph
	 *
	 *@param  vertex  new vertex in graph
	 */
	public void addVertex(Vertex vertex) {
		if (!vertices.contains(vertex)) {
			vertices.add(vertex);
			vertex.setColor(color);
			vertex.setSelectionColor(selectionColor);

		}
	}

	/**
	 *  Gets the Foreground attribute of the Graph object
	 *
	 *@return    The Foreground value
	 */
	public Color getColor() {
		return color;
	}

	/**
	 *  Gets get the ith edge from list of edges
	 *
	 *@param  i  edge index in list
	 *@return    The Edge value
	 */
	public Edge getEdge(int i) {
		return (Edge) edges.get(i);
	}

	/**
	 *  Gets the NumberOfEdges attribute of the Graph object
	 *
	 *@return    The NumberOfEdges value
	 */
	public int getNumberOfEdges() {
		return edges.size();
	}

	/**
	 *  Gets the NumberOfVertices attribute of the Graph object
	 *
	 *@return    The NumberOfVertices value
	 */
	public int getNumberOfVertices() {
		return vertices.size();
	}

	/**
	 *  Gets the SelectionColor attribute of the Graph object
	 *
	 *@return    The SelectionColor value
	 */
	public Color getSelectionColor() {
		return selectionColor;
	}

	/**
	 *  Gets the Size attribute of the Graph object
	 *
	 *@return    The Size value
	 */
	public Dimension getSize() {
		int width = 0;
		int height = 0;

		for (int i = 0; i < vertices.size(); i++) {
			Vertex vertex = (Vertex) vertices.elementAt(i);
			int xpos = (int) vertex.getBounds().getMaxX();
			int ypos = (int) vertex.getBounds().getMaxY();
			if (xpos > width) {
				width = xpos;
			}
			if (ypos > height) {
				height = ypos;
			}
		}

		return new Dimension(width, height);
	}

	/**
	 *  Gets the ith vertex fomr list of vertices
	 *
	 *@param  i  vertex index in list
	 *@return    The Vertex value
	 */
	public Vertex getVertex(int i) {
		return (Vertex) vertices.get(i);
	}

	/**
	 *  Description of the Method
	 */
	public void removeAllEdges() {
		while (edges.size() > 0) {
			removeEdge((Edge) edges.firstElement());
		}
	}

	/**
	 *  Remove an edge from the graph
	 *
	 *@param  edge  edge to be removed
	 */
	public void removeEdge(Edge edge) {
		edges.remove(edge);
		edge.getParentVertex().removeEdge(edge);
		edge.getChildVertex().removeEdge(edge);
	}

	/**
	 *  Remove a vertex from the graph
	 *
	 *@param  vertex  vertex to be removed
	 */
	public void removeVertex(Vertex vertex) {
		vertices.remove(vertex);
		for (int i = 0; i < edges.size(); i++) {
			Edge edge = (Edge) edges.elementAt(i);
			if (edge.contains(vertex)) {
				removeEdge(edge);
			}
		}
	}

	/**
	 *  Sets the Foreground attribute of the Graph object
	 *
	 *@param  color  The new Color value
	 */
	public void setColor(Color color) {
		this.color = color;
		for (int i = 0; i < vertices.size(); i++) {
			((Vertex) vertices.elementAt(i)).setColor(color);
		}
		for (int i = 0; i < edges.size(); i++) {
			((Edge) edges.elementAt(i)).setColor(color);
		}
	}

	/**
	 *  Sets the SelectionColor attribute of the Graph object
	 *
	 *@param  selectionColor  The new SelectionColor value
	 */
	public void setSelectionColor(Color selectionColor) {
		this.selectionColor = selectionColor;
		for (int i = 0; i < vertices.size(); i++) {
			((Vertex) vertices.elementAt(i)).setSelectionColor(selectionColor);
		}
		for (int i = 0; i < edges.size(); i++) {
			((Edge) edges.elementAt(i)).setSelectionColor(selectionColor);
		}
	}

	/**
	 *  Get a String description of graph
	 *
	 *@return    String description
	 */
	public String toString() {
		String result = "VERTICES:\n";
		for (int i = 0; i < getNumberOfVertices(); i++) {
			result += getVertex(i) + "\n";
		}

		result += "EDGES:\n";
		for (int i = 0; i < getNumberOfEdges(); i++) {
			result += getEdge(i) + "\n";
		}

		return result;
	}
}
