package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import com.pallas.unicore.client.graphics.ClipRectangle;
import com.pallas.unicore.client.graphics.Drawable;
import com.pallas.unicore.client.graphics.Selectable;

/**
 *  Representation of a vertex in a graph including an icon, a text a set of
 *  edges and a flag if it's selected.
 *
 *@author     Ralf Ratering
 *@created    15. August 2001
 *@version    $Id: Vertex.java,v 1.2 2003/06/30 06:54:05 rrate Exp $
 */
public class Vertex implements Selectable, Drawable {
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.graphs");

	private Point center = new Point(0, 0);

	private Color color, selectionColor;

	/**
	 *  Vector of edges connected to this vertex
	 */
	private Vector edges = new Vector();
	/**
	 *  Icon representing vertex
	 */
	private ImageIcon icon;
	/**
	 *  Vertex name
	 */
	private String name;

	/**
	 *  Flag if vertex is selected
	 */
	private boolean selected;

	private int xpos, ypos, width, height;

	/**
	 *  Constructor for the Vertex object
	 */
	public Vertex() {
		this("");
	}

	/**
	 *  Constructor for the Vertex object
	 *
	 *@param  name  vertex name
	 */
	public Vertex(String name) {
		this.name = name;

		setSelected(false);
		setColor(Color.black);
		setSelectionColor(Color.red);

		init();
	}

	/**
	 *  Adds a feature to the Edge attribute of the Vertex object
	 *
	 *@param  edge  The feature to be added to the Edge attribute
	 */
	public void addEdge(Edge edge) {
		if (!edges.contains(edge)) {
			edges.add(edge);
		}
	}

	/**
	 *  Draw icon at its position
	 *
	 *@param  panel  panel to draw into
	 *@param  g      Description of Parameter
	 */
	public void draw(Graphics g, JPanel panel) {
		int xOffset = 75;

		g.setColor(color);

		// icon
		icon = getImageIcon();
		int iconWidth = icon.getIconWidth();
		int iconHeight = icon.getIconHeight();

		// font
		g.setFont(new Font(null, Font.PLAIN, 10));
		int textHeight = g.getFontMetrics().getHeight();
		int textWidth = g.getFontMetrics().stringWidth(name);

		String displayName = name;
		int end = displayName.length() - 3;
		while (textWidth >= iconWidth + xOffset && end > 0) {
			displayName = name.substring(0, end) + "...";
			textWidth = g.getFontMetrics().stringWidth(displayName);
			end--;
		}

		width = textWidth > iconWidth ? textWidth : iconWidth;
		height = iconHeight + textHeight;

		xpos = (int) (center.getX() - width / 2.0);
		ypos = (int) (center.getY() - height / 2.0);

		g.drawImage(
			icon.getImage(),
			(int) (center.getX() - iconWidth / 2.0),
			ypos,
			panel);

		// draw identifier string centered under icon
		g.drawString(
			displayName,
			(int) (center.getX() - textWidth / 2),
			ypos + height);

		// draw selection box
		if (selected) {
			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(selectionColor);
			g2.fill(getBounds());
		}

	}

	/**
	 *  Gets the Bounds attribute of the Vertex object
	 *
	 *@return    The Bounds value
	 */
	public ClipRectangle getBounds() {
		int border = 4;

		return new ClipRectangle(
			(int) center.getX() - (width + border) / 2,
			(int) center.getY() - (height + border) / 2,
			width + border,
			height + border);
	}

	/**
	 *  Gets the Center attribute of the Vertex object
	 *
	 *@return    The Center value
	 */
	public Point getCenter() {
		return center;
	}

	/**
	 *  Gets the Color attribute of the Vertex object
	 *
	 *@return    The Color value
	 */
	public Color getColor() {
		return color;
	}

	/**
	 *  Gets the Edges attribute of the Vertex object
	 *
	 *@return    The Edges value
	 */
	public Vector getEdges() {
		return edges;
	}

	/**
	 *  Gets the Height attribute of the Vertex object
	 *
	 *@return    The Height value
	 */
	public int getHeight() {
		return height;
	}

	/**
	 *  Gets the ImageIcon attribute of the Vertex object
	 *
	 *@return    The ImageIcon value
	 */
	public ImageIcon getImageIcon() {
		return icon;
	}

	/**
	 *  Gets the Name attribute of the Vertex object
	 *
	 *@return    The Name value
	 */
	public String getName() {
		return name;
	}

	/**
	 *  Gets the SelectionColor attribute of the Vertex object
	 *
	 *@return    The SelectionColor value
	 */
	public Color getSelectionColor() {
		return selectionColor;
	}

	/**
	 *  Gets the Width attribute of the Vertex object
	 *
	 *@return    The Width value
	 */
	public int getWidth() {
		return width;
	}

	/**
	 *  Gets the XPos attribute of the Vertex object
	 *
	 *@return    The XPos value
	 */
	public int getXPos() {
		return xpos;
	}

	/**
	 *  Gets the YPos attribute of the Vertex object
	 *
	 *@return    The YPos value
	 */
	public int getYPos() {
		return ypos;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  neighbor  Description of Parameter
	 *@return           Description of the Returned Value
	 */
	public boolean hasEdgeTo(Vertex neighbor) {
		for (int i = 0; i < edges.size(); i++) {
			if (neighbor
				.equals(((Edge) edges.elementAt(i)).getChildVertex())) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 */
	private void init() {
		// icon
		icon = getImageIcon();

		int iconWidth = icon != null ? icon.getIconWidth() : 60;
		int iconHeight = icon != null ? icon.getIconHeight() : 60;

		// font
		int textHeight = 20;
		int textWidth = 60;

		width = textWidth > iconWidth ? textWidth : iconWidth;
		height = iconHeight + textHeight;

		xpos = (int) (center.getX() - width / 2.0);
		ypos = (int) (center.getY() - height / 2.0);
	}

	/**
	 *  Gets the Selected attribute of the Vertex object
	 *
	 *@return    The Selected value
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 *  Removes an edge from ths vertex
	 *
	 *@param  edge  edge to be removed
	 */
	public void removeEdge(Edge edge) {
		edges.remove(edge);
	}

	/**
	 *  Sets the Center attribute of the Vertex object
	 *
	 *@param  center  The new Center value
	 */
	public void setCenter(Point center) {
		this.center = center;
	}

	/**
	 *  Sets the Color attribute of the Vertex object
	 *
	 *@param  color  The new Color value
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 *  Sets the ImageIcon attribute of the Vertex object
	 *
	 *@param  icon  The new ImageIcon value
	 */
	public void setImageIcon(ImageIcon icon) {
		this.icon = icon;
	}

	/**
	 *  Sets the Name attribute of the Vertex object
	 *
	 *@param  name  The new Name value
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 *  Sets the Selected attribute of the Vertex object
	 *
	 *@param  selected  The new Selected value
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 *  Sets the SelectionColor attribute of the Vertex object
	 *
	 *@param  sc  The new SelectionColor value
	 */
	public void setSelectionColor(Color sc) {
		// make a transparent copy of color
		this.selectionColor =
			new Color(sc.getRed(), sc.getGreen(), sc.getBlue(), 100);
	}

	/**
	 *  Get the String description of this vertex
	 *
	 *@return    String description
	 */
	public String toString() {
		return name;
	}

}
