package com.pallas.unicore.client.menus;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

/**
 *  A menu that manages recently select files.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: RecentFileMenu.java,v 1.2 2003/06/30 06:54:18 rrate Exp $
 */
public class RecentFileMenu extends JMenu {

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.menus");
	// for the moment we have constantly a maximum of 4 recent files
	private final static int RECENT_FILES = 4;

	// filename where recent files will be stored
	private String filename;

	// listener for item selections
	private ActionListener itemListener;

	// collect files in a vector
	private Vector recentFiles = new Vector(RECENT_FILES);

	/**
	 *  Constructor builds GUI and loads recent filenames from file
	 *
	 *@param  title         menu title
	 *@param  filename      points to file where recent filenames are stored
	 *@param  itemListener  listens to menu item selections
	 */
	public RecentFileMenu(
		String title,
		String filename,
		ActionListener itemListener) {
		super(title);

		this.filename = filename;
		this.itemListener = itemListener;

		loadEntries();

		buildRecentFileMenu();

	}

	/**
	 *  Add a recent file to the menu
	 *
	 *@param  fileName  filename to be added
	 */
	public void addRecentFile(String fileName) {
		int offset;
		// Check wether this file is already in the list
		if ((offset = recentFiles.indexOf(fileName)) == -1) {
			// Not present - add at beginning
			recentFiles.insertElementAt(fileName, 0);
			if (recentFiles.size() > RECENT_FILES) {
				// remove last item
				recentFiles.removeElementAt(RECENT_FILES);
			}
		} else {
			// Already present: move to the front
			recentFiles.removeElementAt(offset);
			recentFiles.insertElementAt(fileName, 0);
		}

		// save recent files
		saveEntries();

		buildRecentFileMenu();
	}

	/**
	 *  Build the file menu entries
	 */
	private void buildRecentFileMenu() {
		if (recentFiles == null) {
			return;
		}
		int size = recentFiles.size();
		removeAll();
		for (int i = 0; i < size; i++) {
			String fileName = (String) recentFiles.elementAt(i);
			JMenuItem mi = new JMenuItem(fileName);
			mi.setForeground(getForeground());
			mi.setBackground(getBackground());
			add(mi);
			mi.addActionListener(itemListener);
		}
	}

	/**
	 *  Load current entries from file
	 */
	private void loadEntries() {
		File file = new File(filename);
		try {
			FileInputStream in = new FileInputStream(file);
			BufferedReader reader =
				new BufferedReader(new InputStreamReader(in));
			while (reader.ready()) {
				String recentFile = reader.readLine();
				if (recentFile != null && !recentFile.equals("")) {
					recentFiles.add(recentFile);
				}
			}
			reader.close();
		} catch (Exception e) {
			logger.log(
				Level.INFO,
				"Recent files not available:" + e.getMessage());
		}

	}

	/**
	 *  Write current entries to file
	 */
	private void saveEntries() {
		File file = new File(filename);
		try {
			FileOutputStream out = new FileOutputStream(file);
			BufferedWriter writer =
				new BufferedWriter(new OutputStreamWriter(out));
			for (int i = 0; i < recentFiles.size(); i++) {
				writer.write((String) recentFiles.get(i));
				writer.newLine();
			}
			writer.close();
		} catch (Exception e) {
			logger.info("Recent files do not exist: " + filename);
		}
	}

	/**
	 *  Sets the Background attribute of the RecentFileMenu object
	 *
	 *@param  c  The new Background value
	 */
	public void setBackground(Color c) {
		super.setBackground(c);
		buildRecentFileMenu();
	}

	/**
	 *  Sets the Foreground attribute of the RecentFileMenu object
	 *
	 *@param  c  The new Foreground value
	 */
	public void setForeground(Color c) {
		super.setForeground(c);
		buildRecentFileMenu();
	}

}
