package com.pallas.unicore.client.panels;

import java.awt.datatransfer.DataFlavor;
import java.util.Date;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;

import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileTransfer;

/**
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: FileTransferPanel.java,v 1.2 2003/06/30 06:54:39 rrate Exp $
 */
public class FileTransferPanel extends FilePanel {

	final static int COL_SOURCE = 0;
	final static int COL_TARGET = 1;

	/**
	 *  Vector containing possible destination job groups
	 */
	private Vector jobGroups;

	/**
	 *  Constructor
	 *
	 *@param  container  container to be modified
	 */
	public FileTransferPanel(TaskContainer container) {
		super(container);
	}

	/**
	 *  Add an empty row to table end
	 */
	protected void addEmptyRow() {
		Vector rowData = new Vector();
		rowData.add("");
		if (jobGroups != null && jobGroups.size() > 0) {
			rowData.add(jobGroups.elementAt(0));
		} else {
			rowData.add("");
		}

		tableModel.addRow(rowData);
		int selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 *  Apply gui values to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		// get file transfers from table
		int nrOfRows = table.getRowCount();
		FileTransfer[] fileOps = new FileTransfer[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			fileOps[i] = (FileTransfer) getFileOperationAt(i);
		}

		updateModifiedTime(fileOps);

		container.setFileTransfers(fileOps);
		container.checkContents();
	}

	/**
	 *  Gets the columnNames attribute of the FileTransferPanel object
	 *
	 *@return    The columnNames value
	 */
	protected String[] getColumnNames() {
		String[] columnNames =
			{ res.getString("USPACE_NAME"), res.getString("DESTINATION_JOB")};
		return columnNames;
	}

	/**
	 *  Gets the dataFlavor attribute of the FileTransferPanel object
	 *
	 *@return    The dataFlavor value
	 */
	protected DataFlavor getDataFlavor() {
		return FileTransfer.getFlavor();
	}

	/**
	 *  Get file transfer from a certain row
	 *
	 *@param  row  index of row
	 *@return      file transfer from row
	 */
	protected FileOperation getFileOperationAt(int row) {
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		Object destinationJobEntry = tableModel.getValueAt(row, COL_TARGET);
		JobContainer destinationJob = null;
		if (destinationJobEntry instanceof JobContainer) {
			destinationJob = (JobContainer) destinationJobEntry;
		}
		// renaming not yet implemented...
		return new FileTransfer(sourceName, destinationJob, sourceName);
	}

	/**
	 *  Reset current GUI entries to last container contents.
	 */
	public void resetValues() {

		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}

		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}

		clearTable();

		updateDestinationJobGroups();

		FileTransfer[] fileOps = container.getFileTransfers();
		for (int i = 0; i < fileOps.length; i++) {
			addFileOperation(fileOps[i]);
		}
	}

	/**
	 *  Fill table row with a certain file transfer
	 *
	 *@param  fileOp  file transfer to fill in
	 *@param  index   nr of table row
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileTransfer fileTransfer = (FileTransfer) fileOp;

		Vector rowData = new Vector();
		rowData.add(fileTransfer.getSourceName());
		rowData.add(fileTransfer.getDestinationJob());

		tableModel.insertRow(index, rowData);
	}

	/**
	 *  Add all possible destination sub job groups to combo box
	 */
	private void updateDestinationJobGroups() {
		JobContainer parentJob = container.getParentJobContainer();
		JobContainer topParentJob = parentJob;

		while (topParentJob.getParentContainer() != null) {
			topParentJob = topParentJob.getParentJobContainer();
		}
		jobGroups = topParentJob.getSubJobs();
		// add top level job itself
		jobGroups.add(topParentJob);

		// remove own job from vector
		for (int i = 0; i < jobGroups.size(); i++) {
			JobContainer jc = (JobContainer) jobGroups.elementAt(i);
			if (jc.getIdentifier().equals(parentJob.getIdentifier())) {
				jobGroups.remove(jc);
			}
		}

		JComboBox comboBox = new JComboBox(jobGroups);
		table.getColumnModel().getColumn(COL_TARGET).setCellEditor(
			new DefaultCellEditor(comboBox));
		table.getColumnModel().getColumn(COL_TARGET).setCellRenderer(
			new ComboBoxCellRenderer(jobGroups));
	}

	/**
	 *  Description of the Method
	 *
	 *@param  newOps  Description of the Parameter
	 */
	private void updateModifiedTime(FileTransfer[] newOps) {
		FileTransfer[] oldOps = container.getFileTransfers();

		// first a simple check...
		if (oldOps.length != newOps.length) {
			container.setModifiedTime(new Date());
			return;
		}

		// ... now try to find matching ops
		// loop over old ops
		for (int i = 0; i < oldOps.length; i++) {
			boolean found = false;
			// loop over new ops
			for (int j = 0; j < newOps.length; j++) {
				// try to find matching new import
				if (newOps[j].equals(oldOps[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one op was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 *  Update destination job groups. For consistence as explicit updateValues
	 *  method
	 */
	public void updateValues() {
		updateDestinationJobGroups();
	}

}
