package com.pallas.unicore.client.plugins.command;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  User interface to change command default settings
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: CommandDefaultsDialog.java,v 1.2 2003/06/30 06:55:14 rrate Exp $
 */
public class CommandDefaultsDialog extends GenericDialog {

	/**
	 *  Description of the Class
	 *
	 *@author     rrate
	 *@created    17. September 2001
	 */
	private class ButtonListener implements ActionListener {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == commandDirButton) {
				String value =
					chooseDirectory(
						res.getString("CHOOSE_DIR_TITLE"),
						commandDefaults.getCommandDirectory());
				if (value != null) {
					commandDirTextField.setText(value);
				}
			}
		}
	}

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.command.ResourceStrings");

	private CommandDefaults commandDefaults;

	private JButton commandDirButton;
	private JTextField commandDirTextField;

	/**
	 *  Constructor takes parent frame as input and builds non-modal dialog
	 *
	 *@param  parentFrame      reference to parent JFrame
	 *@param  commandDefaults  Description of Parameter
	 */
	public CommandDefaultsDialog(
		JFrame parentFrame,
		CommandDefaults commandDefaults) {
		super(parentFrame, res.getString("DEFAULTS_TITLE"), false, OK_CANCEL);

		this.commandDefaults = commandDefaults;

		buildComponents();
		fillTextFields();

		setStatusMessage(res.getString("ENTER_VALUES"));

		pack();
	}

	/**
	 *  Add tooltips to buttons from super class
	 */
	private void addTooltipsToButtons() {
		okButton.setToolTipText(res.getString("STORE_CURRENT"));
		applyButton.setToolTipText(res.getString("STORE_AND_CLOSE"));
		cancelButton.setToolTipText(res.getString("CLOSE"));
	}

	/**
	 *  Helper method to handle GridBagLayout
	 *
	 *@param  panel  The feature to be added to the ToPanel attribute
	 *@param  l      The feature to be added to the ToPanel attribute
	 *@param  r      The feature to be added to the ToPanel attribute
	 *@param  g      The feature to be added to the ToPanel attribute
	 *@param  c      The feature to be added to the ToPanel attribute
	 */
	private void addToPanel(
		JPanel panel,
		Component l,
		Component r,
		GridBagLayout g,
		GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);

		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}

	/**
	 *  Method will be called from parent class
	 *
	 *@return    true if applying was successful
	 */
	protected boolean applyValues() {
		// copy text field entries to user defaults
		commandDefaults.setCommandDirectory(
			commandDirTextField.getText().trim());

		// write defaults to file
		if (!commandDefaults.writeToFile()) {
			UserMessages.error(res.getString("COULD_NOT_WRITE"));
			setStatusMessage(res.getString("COULD_NOT_WRITE"));
		} else {
			setStatusMessage(res.getString("SUCCESS"));
		}

		// returning true closes dialog if ok was pressed
		return true;
	}

	/**
	 *  Build user panel
	 */
	private void buildComponents() {
		// fill text fields with previous defaults
		commandDirTextField = new JTextField(25);

		// file chooser buttons
		commandDirButton =
			new JButton(ResourceManager.getIcon(ResourceManager.OPEN));
		commandDirButton.addActionListener(new ButtonListener());

		JPanel commandDirPanel = new JPanel();
		commandDirPanel.add(commandDirTextField);
		commandDirPanel.add(commandDirButton);

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;

		JPanel subPanel = new JPanel(gridbag);
		addToPanel(
			subPanel,
			new JLabel(res.getString("DIR_LABEL")),
			commandDirPanel,
			gridbag,
			c);

		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(subPanel, BorderLayout.CENTER);
		mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));

		getContentPane().add(mainPanel, BorderLayout.CENTER);

		addTooltipsToButtons();
	}

	/**
	 *  When cancel was pressed reload user defaults from file
	 *
	 *@return    if canceling was successful
	 */
	protected boolean cancelValues() {
		commandDefaults.loadFromFile();
		fillTextFields();
		return true;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  title       Description of Parameter
	 *@param  initialdir  Description of Parameter
	 *@return             Description of the Returned Value
	 */
	private String chooseDirectory(String title, String initialdir) {
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(new File(initialdir), null);
		chooser.setTitle(title);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = chooser.showDialog();

		if (result == JFileChooser.APPROVE_OPTION) {
			try {
				return chooser.getSelectedFile().getCanonicalPath();
			} catch (IOException e) {
				return null;
			}
		}
		return null;
	}

	/**
	 *  Fill text fields with current class member values
	 */
	private void fillTextFields() {
		commandDirTextField.setText(commandDefaults.getCommandDirectory());
	}

	/**
	 *  Overwrite show method to let email text field always grab focus.
	 */
	public void show() {
		commandDirTextField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}

}
