package com.pallas.unicore.client.plugins.compile;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import org.unicore.ajo.*;
import org.unicore.resources.*;

import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.explorer.*;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  User interface to change command default settings
 *
 *@author     Thomas Kentemich
 *@created    22. April 2002
 *@version    $Id: CompileDefaultsDialog.java,v 1.1.1.1 2002/04/22 08:30:25
 *      thomas Exp $
 */
public class CompileDefaultsDialog extends GenericDialog {

    private JTextField
            commandDirTextField;

    private JButton
            commandDirButton;

    private CompileDefaults compileDefaults;


    /**
     *  Constructor takes parent frame as input and builds non-modal dialog
     *
     *@param  parentFrame      reference to parent JFrame
     *@param  compileDefaults  Description of the Parameter
     */
    public CompileDefaultsDialog(JFrame parentFrame, CompileDefaults compileDefaults) {
        super(parentFrame, "UNICOREpro: Compile Plugin Defaults", false, OK_CANCEL);

        this.compileDefaults = compileDefaults;

        buildComponents();
        fillTextFields();

        setStatusMessage("Enter default values for UNICOREpro compile plugin");

        pack();
    }


    /**
     *  Overwrite show method to let email text field always grab focus.
     */
    public void show() {
        commandDirTextField.requestFocus();
        getRootPane().setDefaultButton(okButton);
        super.show();
    }


    /**
     *  Method will be called from parent class
     *
     *@return    true if applying was successful
     */
    protected boolean applyValues() {
        // copy text field entries to user defaults
        compileDefaults.setCommandDirectory(commandDirTextField.getText().trim());

        // write defaults to file
        if(!compileDefaults.writeToFile()) {
            UserMessages.error("Could not write compile plugin  defaults to file");
            setStatusMessage("Could not write compile plugin defaults to file");
        }
        else {
            setStatusMessage("Successfully stored compile plugin defaults.");
        }

        // returning true closes dialog if ok was pressed
        return true;
    }


    /**
     *  When cancel was pressed reload user defaults from file
     *
     *@return    if canceling was successful
     */
    protected boolean cancelValues() {
        compileDefaults.loadFromFile();
        fillTextFields();
        return true;
    }


    /**
     *  Build user panel
     */
    private void buildComponents() {
        // fill text fields with previous defaults
        commandDirTextField = new JTextField(25);

        // file chooser buttons
        commandDirButton = new JButton(ResourceManager.getIcon(ResourceManager.OPEN));
        commandDirButton.addActionListener(new ButtonListener());

        JPanel commandDirPanel = new JPanel();
        commandDirPanel.add(commandDirTextField);
        commandDirPanel.add(commandDirButton);

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.WEST;

        JPanel subPanel = new JPanel(gridbag);
        addToPanel(subPanel, new JLabel("Command Directory: "), commandDirPanel, gridbag, c);

        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(subPanel, BorderLayout.CENTER);
        mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));

        getContentPane().add(mainPanel, BorderLayout.CENTER);

        addTooltipsToButtons();
    }


    /**
     *  Helper method to handle GridBagLayout
     *
     *@param  panel  The feature to be added to the ToPanel attribute
     *@param  l      The feature to be added to the ToPanel attribute
     *@param  r      The feature to be added to the ToPanel attribute
     *@param  g      The feature to be added to the ToPanel attribute
     *@param  c      The feature to be added to the ToPanel attribute
     */
    private void addToPanel(JPanel panel,
            Component l,
            Component r,
            GridBagLayout g,
            GridBagConstraints c) {
        c.gridwidth = GridBagConstraints.RELATIVE;
        g.setConstraints(l, c);
        panel.add(l);

        c.anchor = GridBagConstraints.WEST;
        c.gridwidth = GridBagConstraints.REMAINDER;
        g.setConstraints(r, c);
        panel.add(r);
    }


    /**
     *  Fill text fields with current class member values
     */
    private void fillTextFields() {
        commandDirTextField.setText(compileDefaults.getCommandDirectory());
    }


    /**
     *  Add tooltips to buttons from super class
     */
    private void addTooltipsToButtons() {
        okButton.setToolTipText("Store current settings as defaults to disk");
        applyButton.setToolTipText("Store current settings as defaults to " +
                "disk and close dialog");
        cancelButton.setToolTipText("Close dialog and dismiss changes");
    }


    /**
     *  Description of the Method
     *
     *@param  title       Description of the Parameter
     *@param  initialdir  Description of the Parameter
     *@return             Description of the Return Value
     */
    private String chooseDirectory(String title, String initialdir) {
        SelectorDialog chooser =
                ResourceManager.getSelectorDialog(new File(initialdir), null);
        chooser.setTitle(title);
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int result = chooser.showDialog();

        if(result == JFileChooser.APPROVE_OPTION) {
            try {
                return chooser.getSelectedFile().getCanonicalPath();
            }
            catch(IOException e) {
                return null;
            }
        }
        return null;
    }


    /**
     *  Description of the Class
     *
     *@author     rrate
     *@created    22. April 2002
     */
    private class ButtonListener implements ActionListener {
        /**
         *  Description of the Method
         *
         *@param  e  Description of the Parameter
         */
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();
            if(source == commandDirButton) {
                String value =
                        chooseDirectory("UNICOREpro: Select Command Directory",
                        compileDefaults.getCommandDirectory());
                if(value != null) {
                    commandDirTextField.setText(value);
                }
            }
        }
    }

}

