package com.pallas.unicore.client.plugins.povray;

import java.io.File;
import java.util.logging.Level;

import javax.swing.JPanel;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.Option;
import org.unicore.ajo.UserTask;
import org.unicore.resources.SoftwareResource;
import org.unicore.sets.OptionSet;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.client.panels.IPanelProvider;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.StringTools;

/**
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: PovrayContainer.java,v 1.7 2003/07/10 14:03:03 rrate Exp $
 */
public class PovrayContainer extends UserContainer implements IPanelProvider {
//	field definitions for storing the container in XML format
	 private static String[][] persistentXMLFields =
		 { { "INPUTSTRING", "inputString" }, 
		 	{ "INPUTFILE", "inputFile" }, 
		 	{ "WIDTH", "width" },
		 	{ "HEIGHT", "height"},
		 	{ "FILENAME", "filename"},
		 	{ "OUTPUTDIRECTORY", "outputDirectory"}
	 };
	final static long serialVersionUID = -3244691511027502609L;
	private double antiAliasing;

	private String filename = "PovrayFile";

	// associate an input file with each container
	private File inputFile;

	// povray input that will be executed
	private String inputString;

	private RenderPanel outcomePanel;

	private String outputDirectory = ".";

	private int width, height;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent group container
	 */
	public PovrayContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Build ActionGroup to transfer and execute the script
	 */
	protected void buildExecuteGroup() {

		// get input file contents as 2D byte array
		byte[][] contents = new byte[1][];
		contents[0] = StringTools.dos2Unix(inputString).getBytes();

		String inputFilename = filename + ".pov";

		String[] filenames = { inputFilename };

		// pass input filenam as environment variable
		OptionSet env = getEnv();
		if (env == null) {
			env = new OptionSet();
			setEnv(env);
		}
		env.add(new Option("POVRAY_INPUTFILE", inputFilename));

		// set command line params
		setCommandLine(
			getCommandLine()
				+ " -w"
				+ width
				+ " -h"
				+ height
				+ " +a"
				+ antiAliasing);

		// setup incarnate files task to transfer files to uspace
		IncarnateFiles incarnateFiles =
			new IncarnateFiles(
				getName()
					+ "_INCARNATEFILES_"
					+ ResourceManager.getNextObjectIdentifier(),
				new ResourceSet(),
				filenames,
				contents,
				true);

		ResourceSet taskResourceSet = getResourceSet().getResourceSetClone();
		taskResourceSet.add(getPreinstalledSoftware());

		UserTask userTask =
			new UserTask(
				getName(),
				null,
				taskResourceSet,
				getEnv(),
				getCommandLine(),
				null,
				getRedirectStdout(),
				getRedirectStderr(),
				isVerboseOn(),
				isVersionOn(),
				null,
				getMeasureTime(),
				getDebug(),
				getProfile());

		// put tasks into ActionGroup and add dependency
		executeGroup =
			new ActionGroup(
				getName()
					+ "_EXECUTION_"
					+ ResourceManager.getNextObjectIdentifier());
		executeGroup.add(incarnateFiles);
		executeGroup.add(userTask);
		try {
			executeGroup.addDependency(new Dependency(incarnateFiles, userTask));
		}
		catch(ActionGroup.InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add Dependency", e);
		}
	}

	/**
	 *  Add container specific errors to error set of super classes and select
	 *  appropriate icon for JPA display.
	 *
	 *@return    ErrorSet set containing all errors including those from super
	 *      classes
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		// allow empty inputs...
		if (inputString == null) {
			inputString = "";
		}

		if (getVsite() != null) {
			// set POV-Ray as preinstalled software resource if available
			SoftwareResource povrayResource =
				ResourceManager.getResourceSet(
					getVsite()).findSoftwareResourceByName(
					"povray");
			if (povrayResource != null) {
				setPreinstalledSoftware(povrayResource);
			} else {
				setPreinstalledSoftware(null);
				err.add(
					new UError(
						id,
						"POV-Ray is not available at " + getVsite().getName()));
			}
		}
		setErrors(err);
		return err;
	}

	/**
	 *  Description of the Method
	 */
	public void finalizePanel() {
	}

	/**
	 *  Gets the additionalExports attribute of the PovrayContainer object
	 *
	 *@return    The additionalExports value
	 */
	public FileExport[] getAdditionalExports() {
		// build export
		String outputFilename = filename + ".png";
		String localFilename =
			outputDirectory + File.separator + outputFilename;
		outcomePanel = new RenderPanel();
		outcomePanel.setFile(new File(localFilename));

		FileExport[] exports =
			{
				 new FileExport(
					this,
					FileStorage.NSPACE_STRING,
					outputFilename,
					localFilename,
					true,
					false)};
		return exports;
	}

	/**
	 *  Gets the antiAliasing attribute of the PovrayContainer object
	 *
	 *@return    The antiAliasing value
	 */
	public double getAntiAliasing() {
		return antiAliasing;
	}

	/**
	 *  Gets the height attribute of the PovrayContainer object
	 *
	 *@return    The height value
	 */
	public int getHeight() {
		return height;
	}

	/**
	 *  Get the File belonging to the script
	 *
	 *@return    script file object
	 */
	public File getInputFile() {
		return inputFile;
	}

	/**
	 *  Get the script contents as String
	 *
	 *@return    script contents String
	 */
	public String getInputString() {
		return inputString;
	}

	/**
	 *  Gets the nrOfPanels attribute of the FileWatcherContainer object
	 *
	 *@return    The nrOfPanels value
	 */
	public int getNrOfPanels() {
		return 1;
	}

	/**
	 *  Gets the outputDirectory attribute of the PovrayContainer object
	 *
	 *@return    The outputDirectory value
	 */
	public String getOutputDirectory() {
		return outputDirectory;
	}

	/**
	 *  Gets the Panel attribute of the GraphicsContainer object
	 *
	 *@param  i  Description of the Parameter
	 *@return    The Panel value
	 */
	public JPanel getPanel(int i) {
		return outcomePanel;
	}

	/**
	 *  Gets the panelTitle attribute of the FileWatcherContainer object
	 *
	 *@param  i  Description of the Parameter
	 *@return    The panelTitle value
	 */
	public String getPanelTitle(int i) {
		return "Image";
	}

	/**
	 *  Gets the width attribute of the PovrayContainer object
	 *
	 *@return    The width value
	 */
	public int getWidth() {
		return width;
	}

	/**
	 *  Sets the antiAliasing attribute of the PovrayContainer object
	 *
	 *@param  antiAliasing  The new antiAliasing value
	 */
	public void setAntiAliasing(double antiAliasing) {
		this.antiAliasing = antiAliasing;
	}

	/**
	 *  Sets the height attribute of the PovrayContainer object
	 *
	 *@param  height  The new height value
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 *  Set a File object for the sript
	 *
	 *@param  inputFile  The new inputFile value
	 */
	public final void setInputFile(File inputFile) {
		this.inputFile = inputFile;

		filename = inputFile.getName();
		int index = filename.indexOf(".");
		if (index > 0) {
			filename = filename.substring(0, index);
		}
	}

	/**
	 *  Set the script contents as String
	 *
	 *@param  inputString  The new inputString value
	 */
	public void setInputString(String inputString) {
		this.inputString = inputString;
	}

	/**
	 *  Sets the outputDirectory attribute of the PovrayContainer object
	 *
	 *@param  outputDirectory  The new outputDirectory value
	 */
	public void setOutputDirectory(String outputDirectory) {
		this.outputDirectory = outputDirectory;
	}

	/**
	 *  Sets the width attribute of the PovrayContainer object
	 *
	 *@param  width  The new width value
	 */
	public void setWidth(int width) {
		this.width = width;
	}
}
