package com.pallas.unicore.client.plugins.povray;

import java.awt.Font;
import java.io.File;

import com.pallas.unicore.client.util.Defaults;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Define, load and save plugin default values
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: PovrayDefaults.java,v 1.5 2003/06/30 06:55:25 rrate Exp $
 */
public class PovrayDefaults extends Defaults {

	/**
	 *  String constants defining property keys
	 */
	public final static String INPUTDIRECTORY = "plugin.povray.inputdirectory",
		OUTPUTDIRECTORY = "plugin.povray.outputdirectory",
		WIDTH = "plugin.povray.width",
		HEIGHT = "plugin.povray.height",
		ANTIALIASING = "plugin.povray.antialiasing",
		EDITORFONTNAME = "plugin.povray.editor.fontname",
		EDITORFONTSIZE = "plugin.povray.editor.fontsize",
		EDITORLINEWRAP = "plugin.povray.editor.linewrap";

	/**
	 *  Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(
			INPUTDIRECTORY,
			ResourceManager.getJarDirectory() + File.separator + "povray");
		put(OUTPUTDIRECTORY, userHome);
		put(WIDTH, "320");
		put(HEIGHT, "200");
		put(ANTIALIASING, "0.3");
		put(EDITORFONTNAME, "monospaced.plain");
		put(EDITORFONTSIZE, "12");
		put(EDITORLINEWRAP, "true");
	}

	/**
	 *  Gets the antiAliasing attribute of the PovrayDefaults object
	 *
	 *@return    The antiAliasing value
	 */
	public double getAntiAliasing() {
		if (get(ANTIALIASING) == null) {
			put(ANTIALIASING, "0.3");
		}
		return doubleValue(ANTIALIASING);
	}

	/**
	 *  Get the editor font
	 *
	 *@return    The editor font
	 */
	public Font getEditorFont() {
		String fontName = (String) get(EDITORFONTNAME);
		int fontSize = intValue(EDITORFONTSIZE);

		return new Font(fontName, Font.PLAIN, fontSize);
	}

	/**
	 *  Gets the height attribute of the PovrayDefaults object
	 *
	 *@return    The height value
	 */
	public int getHeight() {
		if (get(HEIGHT) == null) {
			put(HEIGHT, "200");
		}
		return intValue(HEIGHT);
	}

	/**
	 *  Get the script directory
	 *
	 *@return    script directory value
	 */
	public String getInputDirectory() {
		return getProperty(INPUTDIRECTORY);
	}

	/**
	 *  Get the line wrap flag for editor
	 *
	 *@return    boolean line wrap value
	 */
	public boolean getLineWrap() {
		return booleanValue(EDITORLINEWRAP);
	}

	/**
	 *  Gets the outputDirectory attribute of the PovrayDefaults object
	 *
	 *@return    The outputDirectory value
	 */
	public String getOutputDirectory() {
		return getProperty(OUTPUTDIRECTORY);
	}

	/**
	 *  Gets the width attribute of the PovrayDefaults object
	 *
	 *@return    The width value
	 */
	public int getWidth() {
		if (get(WIDTH) == null) {
			put(WIDTH, "320");
		}
		return intValue(WIDTH);
	}

	/**
	 *  Load script default values from file ${HOME}/.unicore/PovrayDefaults.txt
	 *
	 *@return    true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for  ${HOME}/.unicoreXX/PovrayDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "PovrayDefaults.txt");
	}

	/**
	 *  Sets the aA attribute of the PovrayDefaults object
	 *
	 *@param  aa  The new aA value
	 */
	public void setAntiAliasing(double aa) {
		put(ANTIALIASING, new Double(aa).toString());
	}

	/**
	 *  Set the editor font
	 *
	 *@param  font  The new editor font
	 */
	public void setEditorFont(Font font) {
		put(EDITORFONTNAME, font.getFontName());
		put(EDITORFONTSIZE, new Integer(font.getSize()).toString());
	}

	/**
	 *  Sets the height attribute of the PovrayDefaults object
	 *
	 *@param  height  The new height value
	 */
	public void setHeight(int height) {
		put(HEIGHT, new Integer(height).toString());
	}

	/**
	 *  Set the script directory
	 *
	 *@param  s  The new script directory
	 */
	public void setInputDirectory(String s) {
		put(INPUTDIRECTORY, s);
	}

	/**
	 *  Set the line wrap flag for editor
	 *
	 *@param  b  boolean line wrap flag
	 */
	public void setLineWrap(boolean b) {
		put(EDITORLINEWRAP, new Boolean(b).toString());
	}

	/**
	 *  Sets the outputDirectory attribute of the PovrayDefaults object
	 *
	 *@param  s  The new outputDirectory value
	 */
	public void setOutputDirectory(String s) {
		put(OUTPUTDIRECTORY, s);
	}

	/**
	 *  Sets the width attribute of the PovrayDefaults object
	 *
	 *@param  width  The new width value
	 */
	public void setWidth(int width) {
		put(WIDTH, new Integer(width).toString());
	}

	/**
	 *  Try to write script default values to file ${HOME}/.unicore/PovrayDefaults.txt
	 *
	 *@return    true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicoreXX/PovrayDefaults.txt
		return writeToFile(unicoreDir + File.separator + "PovrayDefaults.txt");
	}

}
