package com.pallas.unicore.client.plugins.povray;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import com.pallas.unicore.client.controls.DoubleTextField;
import com.pallas.unicore.client.controls.IntTextField;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.panels.FontSelectionPanel;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 *  User interface to change default settings
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: PovrayDefaultsDialog.java,v 1.4 2003/06/30 06:55:25 rrate Exp $
 */
public class PovrayDefaultsDialog extends GenericDialog {

	/**
	 *  Listen to button clicks
	 *
	 *@author     rrate
	 *@created    27. August 2001
	 */
	private class ChooseDirectoryAction extends AbstractAction {
		/**
		 *  Constructor for the OpenAction object
		 */
		public ChooseDirectoryAction() {
			super("", ResourceManager.getIcon(ResourceManager.OPEN));
		}

		/**
		 *  Depovrayion of the Method
		 *
		 *@param  e  Depovrayion of Parameter
		 */
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == inputDirButton) {
				String value =
					chooseDirectory(
						"POV-Ray Plugin: Select Input Directory",
						povrayDefaults.getInputDirectory());
				if (value != null) {
					inputDirField.setText(value);
				}
			} else if (source == outputDirButton) {
				String value =
					chooseDirectory(
						"POV-Ray Plugin: Select Output Directory",
						povrayDefaults.getInputDirectory());
				if (value != null) {
					outputDirField.setText(value);
				}
			}
		}
	}
	private DoubleTextField aaField;
	private FontSelectionPanel fontPanel;
	private JButton inputDirButton, outputDirButton;

	private JTextField inputDirField;

	private JCheckBox lineWrapCheckBox = new JCheckBox("Line wrap");
	private JTextField outputDirField;
	private PovrayDefaults povrayDefaults;
	private IntTextField widthField, heightField;

	/**
	 *  Constructor takes parent frame as input and builds modal dialog
	 *
	 *@param  parentFrame     reference to parent JFrame
	 *@param  povrayDefaults  Depovrayion of Parameter
	 */
	public PovrayDefaultsDialog(
		JFrame parentFrame,
		PovrayDefaults povrayDefaults) {
		super(parentFrame, "POV-Ray Plugin: Defaults", true, OK_CANCEL);

		this.povrayDefaults = povrayDefaults;

		buildComponents();
		resetValues();

		setStatusMessage("Enter POV-Ray default values");

		pack();
	}

	/**
	 *  Add tooltips to buttons from super class
	 */
	private void addTooltipsToButtons() {
		okButton.setToolTipText("Store values to file and close dialog.");
		cancelButton.setToolTipText("Discard changes and close dialog.");
	}

	/**
	 *  Method will be called from parent class
	 *
	 *@return    true if applying was successful
	 */
	protected boolean applyValues() {
		// copy text field entries to user defaults
		povrayDefaults.setInputDirectory(inputDirField.getText().trim());
		povrayDefaults.setOutputDirectory(outputDirField.getText().trim());
		povrayDefaults.setEditorFont(fontPanel.getSelectedFont());
		povrayDefaults.setLineWrap(lineWrapCheckBox.isSelected());
		povrayDefaults.setWidth(widthField.getValue());
		povrayDefaults.setHeight(heightField.getValue());
		povrayDefaults.setAntiAliasing(aaField.getValue());

		// write defaults to file
		if (!povrayDefaults.writeToFile()) {
			UserMessages.error("Could not write POV-Ray defaults to file");
		}

		// returning true closes dialog if ok was pressed
		return true;
	}

	/**
	 *  Build user panel
	 */
	private void buildComponents() {
		ChooseDirectoryAction action = new ChooseDirectoryAction();

		JPanel mainPanel = new JPanel(new GridBagLayout());
		mainPanel.add(
			new JLabel("Input Directory:"),
			LayoutTools.makegbc(0, 0, 1, 0, false));
		inputDirField = new JTextField();
		mainPanel.add(inputDirField, LayoutTools.makegbc(1, 0, 6, 0, true));
		inputDirButton = new JButton(new ChooseDirectoryAction());
		mainPanel.add(inputDirButton, LayoutTools.makegbc(7, 0, 1, 0, false));

		mainPanel.add(
			new JLabel("Output Directory:"),
			LayoutTools.makegbc(0, 1, 1, 0, false));
		outputDirField = new JTextField();
		mainPanel.add(outputDirField, LayoutTools.makegbc(1, 1, 6, 0, true));
		outputDirButton = new JButton(new ChooseDirectoryAction());
		mainPanel.add(outputDirButton, LayoutTools.makegbc(7, 1, 1, 0, false));

		widthField = new IntTextField(0, 9999);
		mainPanel.add(
			new JLabel("Width:"),
			LayoutTools.makegbc(1, 2, 1, 0, false));
		mainPanel.add(widthField, LayoutTools.makegbc(2, 2, 1, 0, true));

		heightField = new IntTextField(0, 9999);
		mainPanel.add(
			new JLabel("Height:"),
			LayoutTools.makegbc(3, 2, 1, 0, false));
		mainPanel.add(heightField, LayoutTools.makegbc(4, 2, 1, 0, true));

		aaField = new DoubleTextField(0.0, 1.0);
		mainPanel.add(
			new JLabel("Anti Aliasing:"),
			LayoutTools.makegbc(5, 2, 1, 0, false));
		mainPanel.add(aaField, LayoutTools.makegbc(6, 2, 1, 0, true));

		fontPanel = new FontSelectionPanel();
		mainPanel.add(
			new JLabel("Editor font: "),
			LayoutTools.makegbc(0, 3, 1, 0, false));
		mainPanel.add(fontPanel, LayoutTools.makegbc(1, 3, 6, 0, true));
		mainPanel.add(lineWrapCheckBox, LayoutTools.makegbc(7, 3, 1, 0, true));

		mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));

		getContentPane().add(mainPanel, BorderLayout.CENTER);

		addTooltipsToButtons();
	}

	/**
	 *  When cancel was pressed reload user defaults from file
	 *
	 *@return    true to close dialog
	 */
	protected boolean cancelValues() {
		povrayDefaults.loadFromFile();
		return true;
	}

	/**
	 *  Popup file chooser and return the selected directory
	 *
	 *@param  title       dialog title
	 *@param  initialdir  start directory
	 *@return             selected directory as String
	 */
	private String chooseDirectory(String title, String initialdir) {
		SelectorDialog chooser =
			ResourceManager.getSelectorDialog(new File(initialdir), null);
		chooser.setTitle(title);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			return chooser.getSelectedFile().getAbsolutePath();
		}
		return null;
	}

	/**
	 *  Fill controls with current values
	 */
	protected void resetValues() {
		inputDirField.setText(povrayDefaults.getInputDirectory());
		outputDirField.setText(povrayDefaults.getOutputDirectory());
		fontPanel.setSelectedFont(povrayDefaults.getEditorFont());
		lineWrapCheckBox.setSelected(povrayDefaults.getLineWrap());
		widthField.setValue(povrayDefaults.getWidth());
		heightField.setValue(povrayDefaults.getHeight());
		aaField.setValue(povrayDefaults.getAntiAliasing());
	}

	/**
	 *  Overwrite show method to let email text field always grab focus.
	 */
	public void show() {
		povrayDefaults.loadFromFile();
		resetValues();
		inputDirField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}

}
