package com.pallas.unicore.client.plugins.script;

import java.io.File;
import java.util.ResourceBundle;
import java.util.logging.Level;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.ExecuteScriptTask;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.ScriptType;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.StringTools;

/**
 *  Container that executes a script on the vsite and manages file imports and
 *  exports. Inherit from UserContainer to include file imports and exports, as
 *  well as stdin/stdout and stderr.
 *
 *@author     Ralf Ratering
 *@created    27. August 2001
 *@version    $Id: ScriptContainer.java,v 1.3 2003/06/30 06:55:38 rrate Exp $
 */
public class ScriptContainer extends UserContainer {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "SCRIPTTYPE", "scriptType" }, {
			"SCRIPTCONTENTS", "scriptContents" }, {
			"SCRIPTFILE", "scriptFile" }
	};

	// type of script to execute
	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.plugins.script.ResourceStrings");

	final static long serialVersionUID = 4167955261798674857L;

	// script contents that will be executed
	private String scriptContents;

	// associate a file with each container
	private File scriptFile;

	// script type
	private ScriptType scriptType;

	private boolean scriptTypeIsAvailable;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent group container
	 */
	public ScriptContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Build ActionGroup to transfer and execute the script
	 */
	protected void buildExecuteGroup() {

		// add newline to script contents
		String text = getScriptContents();
		if (text.length() > 0 && !text.endsWith("\n")) {
			text += "\n";
		}

		// get script contents as 2D byte array
		byte[][] contents = new byte[1][];
		contents[0] = StringTools.dos2Unix(text).getBytes();

		// get  unique uspace name for script (=script file name + id counter)
		String scriptFilename =
			"Script_" + getName() + ResourceManager.getNextObjectIdentifier();
		String[] files = { scriptFilename };

		// setup incarnate files task to transfer files to uspace
		IncarnateFiles incarnateFiles =
			new IncarnateFiles(
				getName()
					+ "_INCARNATEFILES_"
					+ ResourceManager.getNextObjectIdentifier(),
				new ResourceSet(),
				files,
				contents,
				true);

		// make portfolio for transferred files in uspace
		MakePortfolio makePortfolio =
			new MakePortfolio(
				getName()
					+ "_MAKEPORTFOLIO_"
					+ ResourceManager.getNextObjectIdentifier(),
				new ResourceSet(),
				files);
		// this one is necessary if script task is used in a do repeat loop, where
		// it is tested for successful/not_successful
		//makePortfolio.setPropagateSuccessfulIfNotSuccessful(true);

		// finally setup task that executes the script
		ExecuteScriptTask executeScriptTask =
			new ExecuteScriptTask(
				getName(),
				getResourceSet().getResourceSet(),
				getEnv(),
				getCommandLine(),
				null,
				getRedirectStdout(),
				getRedirectStderr(),
				isVerboseOn(),
				isVersionOn(),
				makePortfolio.getPortfolio(),
				getMeasureTime(),
				getDebug(),
				getProfile(),
				getScriptType());

		// try to delete previous portfolio in case of loops,
		// because NJS will complain about already existing
		// portfolio else
		DeletePortfolio deletePortfolio =
			new DeletePortfolio(
				getName()
					+ "_DELETEPORTFOLIO_"
					+ ResourceManager.getNextObjectIdentifier());
		deletePortfolio.setPortfolio(makePortfolio.getPortfolio().getId());
		// set ignore failure, because portfolio will not exist yet,
		// if task is not in a loop or in the first iteration of a loop
		deletePortfolio.setPropagateSuccessfulIfNotSuccessful(true);

		executeGroup =
			new ActionGroup(
				getName()
					+ "_EXECUTION_"
					+ ResourceManager.getNextObjectIdentifier());
		executeGroup.add(incarnateFiles);
		executeGroup.add(makePortfolio);
		executeGroup.add(executeScriptTask);
		executeGroup.add(deletePortfolio);

		try {
			// try to delete portfolio, before making a new one
			executeGroup.addDependency(
				new Dependency(deletePortfolio, makePortfolio));
			executeGroup.addDependency(
				new Dependency(incarnateFiles, makePortfolio));
			executeGroup.addDependency(
				new Dependency(makePortfolio, executeScriptTask));

			if (getRedirectStdout()) {
				executeScriptTask.setStdoutFileName(this.getStdout());
				// delete portfolio in case of loops
				DeletePortfolio delStdoutPortfolio =
					new DeletePortfolio(
						getName()
							+ "_DELETE_STDOUT_"
							+ ResourceManager.getNextObjectIdentifier());
				delStdoutPortfolio.setPortfolio(
					executeScriptTask.getStdout().getId());

				executeGroup.add(delStdoutPortfolio);
				executeGroup.addDependency(
					new Dependency(executeScriptTask, delStdoutPortfolio));
			}
			if (getRedirectStderr()) {
				executeScriptTask.setStderrFileName(this.getStderr());
				// delete portfolio in case of loops
				DeletePortfolio delStderrPortfolio =
					new DeletePortfolio(
						getName()
							+ "_DELETE_STDERR_"
							+ ResourceManager.getNextObjectIdentifier());
				delStderrPortfolio.setPortfolio(
					executeScriptTask.getStderr().getId());

				executeGroup.add(delStderrPortfolio);
				executeGroup.addDependency(
					new Dependency(executeScriptTask, delStderrPortfolio));
			}

			if (getRedirectStdin()) {
				String[] stdinfiles = { getStdin()};
				MakePortfolio stdinPortfolio =
					new MakePortfolio(
						getName()
							+ "_MP_STDIN_"
							+ ResourceManager.getNextObjectIdentifier(),
						new ResourceSet(getResourceSet().getUspace()),
						stdinfiles);
				executeScriptTask.setStdin(stdinPortfolio.getPortfolio());
				executeGroup.add(stdinPortfolio);
				executeGroup.addDependency(
					new Dependency(stdinPortfolio, executeScriptTask));
			}
		} catch (ActionGroup.InvalidDependencyException ex) {
			logger.log(Level.SEVERE, "Could not add dependency.", ex);
		}
	}

	/**
	 *  Add container specific errors to error set of super classes and select
	 *  appropriate icon for JPA display.
	 *
	 *@return    ErrorSet set containing all errors including those from super
	 *      classes
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();

		// scriptType
		if (scriptType == null) {
			err.add(new UError(getIdentifier(), res.getString("MISSING_TYPE")));
		} else {
			if (!scriptTypeIsAvailable) {
				err.add(
					new UError(
						getIdentifier(),
						scriptType.toString()
							+ res.getString("SCRIPT_TYPE_NOT_AVAILABLE")));
			}
		}

		if (scriptContents == null || scriptContents.trim().length() == 0) {
			err.add(
				new UError(getIdentifier(), res.getString("MISSING_CONTENTS")));
		}

		setErrors(err);
		return err;
	}

	/**
	 *  Get the script contents as String
	 *
	 *@return    script contents String
	 */
	public String getScriptContents() {
		return scriptContents;
	}

	/**
	 *  Get the File belonging to the script
	 *
	 *@return    script file object
	 */
	public final File getScriptFile() {
		return scriptFile;
	}

	/**
	 *  Get the script type
	 *
	 *@return    script type
	 */
	public ScriptType getScriptType() {
		return scriptType;
	}

	/**
	 *  Set the script contents as String
	 *
	 *@param  scriptContents  String containing script contents
	 */
	public void setScriptContents(String scriptContents) {
		this.scriptContents = scriptContents;
	}

	/**
	 *  Set a File object for the sript
	 *
	 *@param  scriptFile
	 */
	public final void setScriptFile(File scriptFile) {
		this.scriptFile = scriptFile;
	}

	/**
	 *  Set the script type
	 *
	 *@param  scriptType   type of script
	 *@param  isAvailable  The new scriptType value
	 */
	public void setScriptType(ScriptType scriptType, boolean isAvailable) {
		this.scriptType = scriptType;
		this.scriptTypeIsAvailable = isAvailable;
	}

}
