package com.pallas.unicore.client.remotefilechooser;

import java.io.File;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.Vsite;
import org.unicore.outcome.XFile;
import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.Home;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.requests.GetListing;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;

/**
 *  Layer between RemoteFileSystemView and NJS on remote system.
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: RemoteFileSystem.java,v 1.2 2003/06/30 06:55:54 rrate Exp $
 */
public class RemoteFileSystem implements Serializable, IObserver {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.remotefilechooser");

	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.remotefilechooser.ResourceStrings");

	/**
	 *  define file separator on remote file system for now we assume that we
	 *  only deal with remote UNIX systems
	 */
	public final static String SEPARATOR = "/";

	/**
	 *  Gets the ParentPath attribute of the RemoteFileSystem class
	 *
	 *@param  path  Description of Parameter
	 *@return       The ParentPath value
	 */
	public static String getParentPath(String path) {
		int index = path.lastIndexOf(SEPARATOR);
		return path.substring(0, index);
	}

	private RemoteFile[] currentListing;

	private boolean getListingIsRunning = false;

	// file system home
	private RemoteFile home;

	// file system roots
	private RemoteFile[] roots;

	// available storage resources on vsite
	private Vector storages;

	// vsite to connect to
	private Vsite vsite;

	/**
	 *  Constructor
	 *
	 *@param  vsite  to connect to
	 */
	public RemoteFileSystem(Vsite vsite) {
		this.vsite = vsite;
		updateStorageResources();
	}

	/**
	 *@return    file system's home
	 */
	public RemoteFile getHome() {
		return home;
	}

	/**
	 *  Submit a ListDirectory task to the NJS, wait for the result and return
	 *  it.
	 *
	 *@param  dir  directory to be listed
	 *@return      array with RemoteFiles from the directory
	 */
	public RemoteFile[] getListing(RemoteFile dir) {

		GetListing request = new GetListing(dir);
		getListingIsRunning = true;
		request.start();
		request.addObserver(this);

		// wait until thread returns and calls observableUpdate()
		while (getListingIsRunning) {
			try {
				logger.fine("Waiting for GetListing thread to return....");
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				logger.log(
					Level.WARNING,
					"GetListing Thread was interrupted.",
					e);
			}
		}
		return currentListing;
	}

	/**
	 *  Gets the path attribute of the RemoteFileSystem object
	 *
	 *@param  storage  Description of the Parameter
	 *@return          The path value
	 */
	public String getPath(Storage storage) {
		for (int i = 0; i < roots.length; i++) {
			if (roots[i].getStorage().equals(storage)) {
				return roots[i].getCompletePath();
			}
		}
		return null;
	}

	/**
	 *@return    array containing file system roots
	 */
	public RemoteFile[] getRoots() {
		return roots;
	}

	/**
	 *  Gets the vsite attribute of the RemoteFileSystem object
	 *
	 *@return    The vsite value
	 */
	public Vsite getVsite() {
		return vsite;
	}

	/**
	 *  Check if a file is a file system root
	 *
	 *@param  f  RemoteFile to be tested
	 *@return    true if f is a root
	 */
	public boolean isRoot(File f) {
		for (int i = 0; i < roots.length; i++) {
			if (f.equals(roots[i])) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  observable  Description of the Parameter
	 *@param  changeCode  Description of the Parameter
	 */
	public void observableUpdate(Object observable, Object changeCode) {
		GetListing request = (GetListing) observable;
		currentListing = request.getRemoteFileArray();
		getListingIsRunning = false;
	}

	/**
	 *  Gets the storageRemoteFile attribute of the RemoteFileSystem object
	 *
	 *@param  file  Description of the Parameter
	 */
	public void updatePath(RemoteFile file) {
		try {
			String parentSubspacePath = "";
			String subspacePath = file.getSubspacePath();
			int index = subspacePath.lastIndexOf(SEPARATOR);
			if (index > 0) {
				parentSubspacePath = subspacePath.substring(0, index);
			}
			RemoteFile parentDir =
				new RemoteFile(
					parentSubspacePath,
					file.getStorage(),
					file.getFileSystem());

			RemoteFile[] listing = getListing(parentDir);

			if (!subspacePath.equals("")) {
				for (int i = 0; i < listing.length; i++) {
					if (listing[i].getPath().endsWith(file.getName())) {
						file.setXFile(listing[i].getXFile());
						break;
					}
				}
			}
			// storage
			else {
				XFile xfile = listing[0].getXFile();
				String filePath = xfile.getName();
				index = filePath.lastIndexOf(SEPARATOR);
				String storagePath = filePath.substring(0, index - 1);
				file.getXFile().setName(storagePath);
			}

		} catch (Exception e) {
			logger.log(Level.WARNING, "Could not update path  for " + file, e);
		}

	}

	/**
	 *  Update current storage resources at vsite and build new file system
	 *  roots and home.
	 */
	private void updateStorageResources() {
		storages = new Vector();

		// build storage vector
		ResourceSet storageSet =
			ResourceManager.getResourceSet(vsite).getStorageResources();
		ResourceEnumeration storageEnum = storageSet.elements();
		while (storageEnum.hasMoreElements()) {
			Storage storage = (Storage) storageEnum.nextElement();
			// do not handle uspace storage
			if (!(storage instanceof USpace
				|| storage instanceof AlternativeUspace
				|| storage instanceof org.unicore.resources.Spool)) {
				storages.add(storage);
			}
		}
		if (storageSet == null) {
			roots = new RemoteFile[0];
			return;
		}

		// build roots array and find home
		roots = new RemoteFile[storages.size()];
		for (int i = 0; i < storages.size(); i++) {
			Storage s = (Storage) storages.elementAt(i);
			roots[i] = new RemoteFile(s, this);
			if (s instanceof Home) {
				home = roots[i];
			}
		}

		// no home available
		if (home == null) {
			home = roots[0];
		}
	}

}
