package com.pallas.unicore.client.themes;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;

/**
 *  A class that manages different look and feels and metal themes
 *
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: LookAndFeelManager.java,v 1.2 2003/06/30 06:56:12 rrate Exp $
 */
public class LookAndFeelManager {
	// look and feel
	private final static String mac =
		"com.sun.java.swing.plaf.mac.MacLookAndFeel";
	private final static String metal =
		"javax.swing.plaf.metal.MetalLookAndFeel";
	private final static String motif =
		"com.sun.java.swing.plaf.motif.MotifLookAndFeel";

	// table containing plaf name/class pairs
	private static Hashtable plafTable = getAvailableLookAndFeels(),
		themeTable = getAvailableThemes();
	private final static String windows =
		"com.sun.java.swing.plaf.windows.WindowsLookAndFeel";

	/**
	 *  Build a Hashtable that contains the Plaf name as key and the corresponding class name as entry.
	 *  This is needed to display the plafs with their short names in a GUI element.
	 *
	 *@return    Hashtable containing name/class pairs
	 */
	public static Hashtable getAvailableLookAndFeels() {

		Hashtable table = new Hashtable();

		if (isAvailableLookAndFeel(mac)) {
			table.put("Mac", mac);
		}

		if (isAvailableLookAndFeel(metal)) {
			table.put("Metal", metal);
		}

		if (isAvailableLookAndFeel(motif)) {
			table.put("Motif", motif);
		}

		if (isAvailableLookAndFeel(windows)) {
			table.put("Windows", windows);
		}

		return table;
	}

	/**
	 *  Gets the AvailableThemes attribute of the LookAndFeelManager class
	 *
	 *@return    The AvailableThemes value
	 */
	public static Hashtable getAvailableThemes() {

		Hashtable table = new Hashtable();

		DefaultMetalTheme theme = new DefaultMetalTheme();
		table.put(theme.getName(), theme);

		theme = new SystemTheme();
		table.put(theme.getName(), theme);

		theme = new AquaTheme();
		table.put(theme.getName(), theme);

		theme = new BigContrastTheme();
		table.put(theme.getName(), theme);

		theme = new CharcoalTheme();
		table.put(theme.getName(), theme);

		theme = new ContrastTheme();
		table.put(theme.getName(), theme);

		theme = new DemoTheme();
		table.put(theme.getName(), theme);

		theme = new EmeraldTheme();
		table.put(theme.getName(), theme);

		theme = new KhakiTheme();
		table.put(theme.getName(), theme);

		theme = new RubyTheme();
		table.put(theme.getName(), theme);

		theme = new UserTheme();
		table.put(theme.getName(), theme);

		return table;
	}

	/**
	 *  Gets the CrossPlatformPlaf attribute of the LookAndFeelManager class
	 *
	 *@return    The CrossPlatformPlaf value
	 */
	public static String getCrossPlatformPlaf() {
		return UIManager.getCrossPlatformLookAndFeelClassName();
	}

	/**
	 *  Gets the CrossPlatformPlafName attribute of the LookAndFeelManager class
	 *
	 *@return    The CrossPlatformPlafName value
	 */
	public static String getCrossPlatformPlafName() {
		return getPlafByFullName(getCrossPlatformPlaf());
	}

	/**
	 *  Gets the PlafByFullName attribute of the LookAndFeelManager class
	 *
	 *@param  name  Description of Parameter
	 *@return       The PlafByFullName value
	 */
	public static String getPlafByFullName(String name) {
		Enumeration keys = plafTable.keys();
		while (keys.hasMoreElements()) {
			String plafName = (String) keys.nextElement();
			if (plafTable.get(plafName).equals(name)) {
				return plafName;
			}
		}
		return null;
	}

	/**
	 *  Gets the PlafByName attribute of the LookAndFeelManager class
	 *
	 *@param  name  Description of Parameter
	 *@return       The PlafByName value
	 */
	public static String getPlafByName(String name) {
		return (String) plafTable.get(name);
	}

	/**
	 *  Gets the PlafNames attribute of the LookAndFeelManager class
	 *
	 *@return    The PlafNames value
	 */
	public static Object[] getPlafNames() {
		return plafTable.keySet().toArray();
	}

	/**
	 *  Gets the StandardTheme attribute of the LookAndFeelManager class
	 *
	 *@return    The StandardTheme value
	 */
	public static String getStandardTheme() {
		return new DefaultMetalTheme().getName();
	}

	/**
	 *  Gets the ThemeByName attribute of the LookAndFeelManager class
	 *
	 *@param  name  Description of Parameter
	 *@return       The ThemeByName value
	 */
	public static DefaultMetalTheme getThemeByName(String name) {
		return (DefaultMetalTheme) themeTable.get(name);
	}

	/**
	 *  Gets the ThemeNames attribute of the LookAndFeelManager class
	 *
	 *@return    The ThemeNames value
	 */
	public static Object[] getThemeNames() {
		return themeTable.keySet().toArray();
	}

	/**
	 *  Gets the AvailableLookAndFeel attribute of the LookAndFeelManager class
	 *
	 *@param  plaf  Description of Parameter
	 *@return       The AvailableLookAndFeel value
	 */
	public static boolean isAvailableLookAndFeel(String plaf) {
		try {
			Class lnfClass = Class.forName(plaf);
			LookAndFeel newLAF = (LookAndFeel) (lnfClass.newInstance());
			return newLAF.isSupportedLookAndFeel();
		} catch (Exception e) {
			// If ANYTHING weird happens, return false
			return false;
		}
	}

}
