package com.pallas.unicore.client.trees;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JOptionPane;

import org.unicore.AAIdentifier;
import org.unicore.ajo.Control;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.ActionGroup_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.UnicoreResponse;

import com.pallas.unicore.client.dialogs.OutputFilesDialog;
import com.pallas.unicore.client.panels.IPanelProvider;
import com.pallas.unicore.client.panels.OutcomePanel;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.requests.ControlActionService;
import com.pallas.unicore.requests.DeleteJob;
import com.pallas.unicore.requests.GetNewJobStatus;
import com.pallas.unicore.requests.GetOutcome;
import com.pallas.unicore.requests.GetRunningJobs;
import com.pallas.unicore.requests.GetSpooledFiles;
import com.pallas.unicore.requests.ObservableNodeThread;
import com.pallas.unicore.requests.ObservableRequestThread;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Helper class that starts UNICOREpro jobs as new threads and updates the jmc
 *  tree when the threads have finished
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    17. September 2001
 *@version    $Id: JMCTreeUpdater.java,v 1.3 2003/06/30 06:56:24 rrate Exp $
 */
public class JMCTreeUpdater implements IObserver {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.trees");

	// reference to jmc tree
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.trees.ResourceStrings");
	private Hashtable runningThreads = new Hashtable();
	private JMCTree tree;

	/**
	 *  Constructor
	 *
	 *@param  tree  jmcTree to be updated
	 */
	public JMCTreeUpdater(JMCTree tree) {
		this.tree = tree;
	}

	/**
	 *  Checks that a given FileExport actually was successfull
	 *
	 *@param  export  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	private boolean checkForSuccesfullSpooltask(FileExport export) {
		ActionContainer task = export.getContainer();
		Vector outcomeEntries = OutcomeManager.getOutcomes(task);
		for (int i = 0; i < outcomeEntries.size(); i++) {
			OutcomeEntry entry = (OutcomeEntry) outcomeEntries.elementAt(i);
			Outcome taskOutcome = entry.getOutcome();
			if (taskOutcome == null) {
				logger.warning(
					"Outcome of exporting container not yet set. " + task);
				return false;
			}
			if (!(taskOutcome instanceof ActionGroup_Outcome)) {
				logger.warning("Dubious outcome for task. " + task);
				return false;
			}
			PutPortfolio_Outcome ppfOutcome =
				(PutPortfolio_Outcome) findOutcome(export
					.getSpoolTask()
					.getId(),
					(ActionGroup_Outcome) taskOutcome);
			if (ppfOutcome == null) {
				logger.warning("Cannot find PutPortfolio_Outcome. " + task);
				return false;
			}

			// logger.info("Found PutPortfolio_Outcome. " + ppfOutcome);
			if (!ppfOutcome
				.getStatus()
				.isEquivalent(AbstractActionStatus.SUCCESSFUL)) {
				return false;
			}
		}
		return true;
	}

	/**
	 *  Submit service job that removes a specific job from a given vsite
	 *
	 *@param  jobNodes  Description of the Parameter
	 */
	public void deleteJob(Vector jobNodes) {
		for (int i = 0; i < jobNodes.size(); i++) {
			ContainerNode jobNode = (ContainerNode) jobNodes.elementAt(i);
			jobNode.setState(PanelNode.STATE_UPDATING);
			tree.updateNode(jobNode);

			// perform finalization on additional plugin outcome panels
			if (jobNode.getActionContainer() instanceof IPanelProvider) {
				IPanelProvider panelProvider =
					(IPanelProvider) jobNode.getActionContainer();
				panelProvider.finalizePanel();
			}
			DeleteJob dj = new DeleteJob(jobNode);
			dj.addObserver(this);
			dj.start();
			runningThreads.put(jobNode, dj);
		}
		jobNodes.clear();
	}

	/**
	 *  Fetch all spooled files belonging to this job. We currently do not
	 *  support spooled files that persist after the job is deleted
	 *
	 *@param  top        Description of the Parameter
	 *@param  topNode    Description of the Parameter
	 *@param  downloads  Description of the Parameter
	 *@return            Description of the Return Value
	 */
	private int fetchSpooledFiles(
		ContainerNode topNode,
		JobContainer top,
		Vector downloads) {
		Vector subJobs = top.getSubJobs();
		int totalDownloads = 0;

		// delete sub job spool files
		for (int j = 0; j < subJobs.size(); j++) {
			totalDownloads
				+= fetchSpooledFiles(
					topNode,
					(JobContainer) subJobs.elementAt(j),
					downloads);
		}
		if (top.getStatus() != null
			&& !top.getStatus().isEquivalent(AbstractActionStatus.DONE)) {
			return 0;
		}
		Vector newNspaceTransfers = top.getNspaceExportTasks();
		if (newNspaceTransfers.size() == 0) {
			return 0;
		}
		for (int i = 0; i < newNspaceTransfers.size(); i++) {
			TaskContainer ac = (TaskContainer) newNspaceTransfers.elementAt(i);
			Vector spooledFiles = ac.getSpooledFiles();
			if (spooledFiles != null) {
				for (int j = 0; j < spooledFiles.size(); j++) {
					FileExport export = (FileExport) spooledFiles.elementAt(j);
					if (!checkForSuccesfullSpooltask(export)) {
						continue;
					}

					// construct real absolute filename
					if (export.getDestinationName().equals("")
						&& !export.hasWildcards()) {
						export.setDestinationName(export.getSourceName());
					}
					File testFile = new File(export.getDestinationName());
					String absolutePath = testFile.getAbsolutePath();
					export.setDestinationName(absolutePath);
					GetSpooledFiles fetch =
						new GetSpooledFiles(topNode, top, export);
					downloads.add(fetch);
					totalDownloads++;
				}
			}
		}
		return totalDownloads;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  id            Description of the Parameter
	 *@param  groupOutcome  Description of the Parameter
	 *@return               Description of the Return Value
	 */
	private Outcome findOutcome(
		AAIdentifier id,
		ActionGroup_Outcome groupOutcome) {
		Outcome idOutcome = groupOutcome.getOutcome(id);
		if (idOutcome != null) {

			// outcome found, return...
			return idOutcome;
		}

		// find sub ActionGroups and recurse
		OutcomeEnumeration outEnum = groupOutcome.getOutcomes();
		while (outEnum.hasMoreElements()) {
			Outcome next = (Outcome) outEnum.nextElement();
			if (next instanceof ActionGroup_Outcome) {
				idOutcome = findOutcome(id, (ActionGroup_Outcome) next);
				if (idOutcome != null) {
					return idOutcome;
				}
			}
		}
		return null;
	}

	/**
	 *  Submit a control action job like abort, hold or resume for a specific
	 *  job
	 *
	 *@param  jobNode  job to be controlled
	 *@param  control  control action to be performed
	 */
	public void fireControlAction(ContainerNode jobNode, Control control) {
		ControlActionService cas = new ControlActionService(jobNode, control);
		cas.addObserver(this);
		cas.start();
		runningThreads.put(jobNode, cas);
	}

	/**
	 *  Submit service job that gets all jobs from a given vsite
	 *
	 *@param  vsiteNode  vsite to get jobs from
	 */
	public void getJobs(PanelNode vsiteNode) {
		GetRunningJobs grj = new GetRunningJobs(vsiteNode);
		grj.addObserver(this);
		grj.start();
		runningThreads.put(vsiteNode, grj);
		return;
	}

	/**
	 *  Submit service job that gets status of a specific job
	 *
	 *@param  jobNode  job to get status from
	 */
	public void getNewJobStatus(ContainerNode jobNode) {
		GetNewJobStatus gjs = new GetNewJobStatus(jobNode);
		gjs.addObserver(this);
		gjs.start();
		runningThreads.put(jobNode, gjs);
	}

	/**
	 *  Submit service job that gets the outcome of a specific job
	 *
	 *@param  jobNode  job to get outcome from
	 */
	public void getOutcomes(ContainerNode jobNode) {
		GetOutcome go = new GetOutcome(jobNode);
		go.addObserver(this);
		go.start();
		runningThreads.put(jobNode, go);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	public void interruptUpdate(PanelNode node) {
		ObservableRequestThread thread =
			(ObservableRequestThread) runningThreads.get(node);
		if (thread != null) {
			thread.setInterrupted(true);
		}
	}

	/**
	 *  Implementation of the IObserver interface - check for the returned
	 *  object - setup a innner class "Runnable" - let the event-dispatchng
	 *  thread perform the update see http://java.sun.com/products/jfc/tsc/articles/threads/threads1.html
	 *  This method will be called from the running threads that have submitted
	 *  jobs to the njs
	 *
	 *@param  theObserved  Description of Parameter
	 *@param  changeCode   Description of Parameter
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {

		// remove task from table
		if (theObserved instanceof ObservableRequestThread) {
			runningThreads.remove(theObserved);
		}

		// if job was deleted remove node from tree
		if (theObserved instanceof DeleteJob) {
			DeleteJob deleteJob = (DeleteJob) theObserved;
			ContainerNode jobNode = (ContainerNode) deleteJob.getNode();
			jobNode.setUpToDate();
			tree.updateNode(jobNode);
			UnicoreResponse resp = (UnicoreResponse) changeCode;
			if (resp.getReturnCode() != 0) {
				UserMessages.error(
					res.getString("CANNOT_DELETE"),
					resp.getComment());
				return;
			}
			if (jobNode != null) {
				tree.removeSubTree(jobNode);
			}
		}
		// if a control action has been fired
		else if (theObserved instanceof ControlActionService) {
			ControlActionService service = (ControlActionService) theObserved;
			ContainerNode node = (ContainerNode) service.getNode();
			UnicoreResponse resp = (UnicoreResponse) changeCode;
			if (resp.getReturnCode() != 0) {
				UserMessages.error(
					res.getString("CANNOT_CONTROL"),
					resp.getComment());
				return;
			}
			if (node != null) {
				node.setUpToDate();
				tree.updateNode(node);
				// automatically refresh job in tree
				ContainerNode jobNode = node;
				while (!((PanelNode) jobNode.getParent()).isVsite()) {
					jobNode = (ContainerNode) jobNode.getParent();
				}
				tree.refresh(jobNode);
			}
		}
		// if running jobs were fetched add them to tree
		else if (theObserved instanceof GetRunningJobs) {
			PanelNode vsn = ((GetRunningJobs) theObserved).getNode();
			Vector knownJobs = ((GetRunningJobs) theObserved).getKnownJobs();
			if (knownJobs != null) {
				tree.updateRunningJobs(knownJobs, vsn);
				Enumeration enum = vsn.children();
				while (enum.hasMoreElements()) {
					ContainerNode next = (ContainerNode) enum.nextElement();
					getNewJobStatus(next);
				}
			}
			vsn.setUpToDate();
			tree.updateNode(vsn);
		}
		// if a new job status was fetched update job node
		else if (theObserved instanceof GetNewJobStatus) {
			GetNewJobStatus getStatus = (GetNewJobStatus) theObserved;
			ContainerNode jobNode = (ContainerNode) getStatus.getNode();
			if (jobNode != null) {
				setUpToDate(jobNode);
				JobContainer njc = (JobContainer) jobNode.getActionContainer();

				// get correct job status from job outcome
				// this is done here, because we will distribute the
				// status over the job tasks. These tasks are only
				// known after the job has been decoded, so we cannot set
				// the status in requests.GetRunningJobs
				//njc.setJobOutcome(njc.getOutcome());
				tree.updateSubtree(jobNode);

				/*
				 *  if(ResourceManager.getUserDefaults().getAutofetchOutcome()) {
				 *  getOutcomes(jobNode);
				 *  }
				 */
			}
		}
		// if job outcome was fetched update node and menu
		else if (theObserved instanceof GetOutcome) {
			GetOutcome outcome = (GetOutcome) theObserved;
			ContainerNode jobNode = (ContainerNode) outcome.getNode();
			tree.updateNode(jobNode);
			updateOutcomePanels(jobNode);

			// if job node is still selected
			// enable showLogMenu by re-selecting it
			if (tree.getFocussedNode().equals(jobNode)) {
				tree.updateSelection();
			}
			Vector transferJobs = new Vector();
			int totalDownloads =
				fetchSpooledFiles(
					jobNode,
					(JobContainer) jobNode.getActionContainer(),
					transferJobs);
			if (!ResourceManager.getUserDefaults().getAutofetchOutcome()
				&& transferJobs.size() > 0) {
				OutputFilesDialog ofd =
					new OutputFilesDialog(
						ResourceManager.getCurrentInstance(),
						jobNode,
						transferJobs);
				ofd.show();
				transferJobs = ofd.getTransferJobs();
				if (transferJobs == null) {

					// update the status
					GetNewJobStatus gjs = new GetNewJobStatus(jobNode);
					gjs.addObserver(this);
					gjs.start();
					runningThreads.put(jobNode, gjs);
					return;
				}
				totalDownloads = transferJobs.size();
			}
			jobNode.setTotalDownloads(totalDownloads);
			if (transferJobs != null) {
				boolean copyFiles = false;
				jobNode.setCompletedDownloads(0);
				boolean overwriteAll = false;
				Vector allTargetFiles = new Vector();
				for (int i = 0; i < transferJobs.size(); i++) {
					GetSpooledFiles getSpooledFiles =
						(GetSpooledFiles) transferJobs.elementAt(i);
					FileExport export = getSpooledFiles.getFileExport();
					String basename = export.getDestinationBasename();
					File testFile = new File(export.getDestinationName());

					Vector targetFiles = new Vector();
					Vector sourceFiles = new Vector();

					Vector spoolOutcomes =
						OutcomeManager.getOutcomeEntries(
							getSpooledFiles.getFileExport().getSpoolTask());
					for (int k = 0; k < spoolOutcomes.size(); k++) {
						PutPortfolio_Outcome ppo =
							(PutPortfolio_Outcome)
								((OutcomeEntry) spoolOutcomes
								.elementAt(k))
								.getOutcome();
						String[] writtenFiles = ppo.getWrittenFiles();
						for (int l = 0; l < writtenFiles.length; l++) {
							String realName = export.getDestinationName();
							String writtenFile = writtenFiles[l];
							if (testFile.exists() && testFile.isDirectory()) {
								if (!writtenFile.equals(basename)) {
									realName += File.separator + writtenFile;
								}
								targetFiles.addElement(realName);
							} else {
								targetFiles.addElement(realName);
							}
							sourceFiles.addElement(writtenFile);
						}
					}

					Vector removeFiles = new Vector();
					if (!export.getOverwritePolicy() && !overwriteAll) {
						for (int j = 0; j < targetFiles.size(); j++) {
							String target = (String) targetFiles.elementAt(j);
							File targetFile = new File(target);
							allTargetFiles.addElement(target);
							String allTargets = allTargetFiles.toString();
							boolean flag =
								allTargets.indexOf(target)
									!= allTargets.lastIndexOf(target);
							if ((targetFile.exists() || flag)
								&& !targetFile.isDirectory()
								&& !overwriteAll) {
								String sourceFile =
									(String) sourceFiles.elementAt(j);
								String yesString = res.getString("YES_STRING");
								String noString = res.getString("NO_STRING");
								String allString = res.getString("ALL_STRING");
								int option =
									JOptionPane.showOptionDialog(
										null,
										res.getString("DOWNLOAD_1")
											+ sourceFile
											+ ":"
											+ "\n"
											+ targetFile
											+ "\n"
											+ res.getString("DOWNLOAD_2"),
										res.getString("FILE_EXISTS"),
										JOptionPane.DEFAULT_OPTION,
										JOptionPane.WARNING_MESSAGE,
										null,
										new String[] {
											yesString,
											allString,
											noString },
										noString);
								switch (option) {
									case JOptionPane.CLOSED_OPTION :
										// treat close as no
										removeFiles.addElement(target);
										allTargetFiles.remove(target);
										continue;
									case 0 :
										// yes
										continue;
									case 1 :
										//all
										overwriteAll = true;
										break;
									case 2 :
										// no
										removeFiles.addElement(target);
										allTargetFiles.remove(target);
										continue;
								}
							}
						}
					}
					targetFiles.removeAll(removeFiles);

					if (!targetFiles.isEmpty()) {
						export.setTargetFiles(targetFiles);
						export.setTargetFilesFlag(
							FileExport.CHOOSE_OVERWRITTEN_FILES);
						copyFiles = true;
						getSpooledFiles.addObserver(this);
						getSpooledFiles.start();
						runningThreads.put(jobNode, getSpooledFiles);
					}
				}
				if (!copyFiles) {
					jobNode.setUpToDate();
				}
			} else {
				jobNode.setUpToDate();
			}

			// assume we want to have the focus now,
			// because when the outcome arrives we want to see it.
			tree.requestFocus();

			// update the status
			GetNewJobStatus gjs = new GetNewJobStatus(jobNode);
			gjs.addObserver(this);
			gjs.start();
			runningThreads.put(jobNode, gjs);
		}
		// if job outcome was fetched update node and menu
		else if (theObserved instanceof GetSpooledFiles) {
			GetSpooledFiles getSpooledFiles = (GetSpooledFiles) theObserved;
			ContainerNode jobNode = (ContainerNode) getSpooledFiles.getNode();
			jobNode.setCompletedDownloads(jobNode.getCompletedDownloads() + 1);
			if (jobNode.getCompletedDownloads()
				>= jobNode.getTotalDownloads()) {
				jobNode.setUpToDate();
			}
			tree.updateNode(jobNode);
		}

		// update actions in tree
		tree.enableDefaultActions();

		//notify auto update
		if (theObserved instanceof ObservableNodeThread) {
			ObservableNodeThread thread = (ObservableNodeThread) theObserved;
			if (thread.getNode() instanceof ContainerNode) {
				ContainerNode node = (ContainerNode) thread.getNode();
				tree.fireChangeEvent(node);
			}
		}
	}

	/**
	 *  Sets the upToDate attribute of the JMCTreeUpdater object
	 *
	 *@param  groupNode  The new upToDate value
	 */
	private void setUpToDate(ContainerNode groupNode) {
		groupNode.setUpToDate();
		for (int i = 0; i < groupNode.getChildCount(); i++) {
			ContainerNode node = (ContainerNode) groupNode.getChildAt(i);
			if (node.isGroup()) {
				setUpToDate(node);
			} else {
				node.setUpToDate();
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  node  Description of the Parameter
	 */
	private void updateOutcomePanels(ContainerNode node) {
		OutcomePanel panel = (OutcomePanel) node.getPanel();
		panel.resetValues();
		for (int i = 0; i < node.getChildCount(); i++) {
			updateOutcomePanels((ContainerNode) node.getChildAt(i));
		}
	}
}
