package com.pallas.unicore.client.util;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.plaf.ColorUIResource;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Define, load and save user default values
 *
 *@author     Ralf Ratering
 *@created    14. August 2001
 *@version $Id: Defaults.java,v 1.6 2003/06/30 06:56:38 rrate Exp $
 */
public abstract class Defaults extends Properties {
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.util");
	static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.client.util.ResourceStrings");

	/**
	 *  Description of the Field
	 */
	protected String userHome, userName, unicoreDir;

	/**
	 *  Constructor looks for user defaults file. If no such file exists user
	 *  defaults are generated from scratch and a new defaults file will be
	 *  generated
	 */
	public Defaults() {

		// look for home directory
		userHome = ".";
		userName = "";
		try {
			userHome = System.getProperty("user.home");
			userName = System.getProperty("user.name");
		} catch (SecurityException se) {
			UserMessages.warning(res.getString("NO_HOME_DIR"));
			logger.log(Level.SEVERE, "", se);
		}

		unicoreDir = Client.getUnicoreDir();

		File dir = new File(unicoreDir);
		if (!dir.exists()) {
			UserMessages.info(res.getString("CREATE_DIRECTORY") + unicoreDir);
			dir.mkdirs();
		}

		// try to load defaults from file
		if (!loadFromFile()) {
			logger.info(res.getString("GENERATING_DEFAULTS"));

			// if loading failed, build new defaults...
			genSystemDefaults();

			// ...and write them to file
			if (!writeToFile()) {
				UserMessages.warning(res.getString("COULD_NOT_WRITE"));
			}
		}

	}

	/**
	 *  Helper function: Parse a String for a Boolean and return 0 if String
	 *  contains no Boolean
	 *
	 *@param  prop  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected boolean booleanValue(String prop) {
		try {
			Boolean value = Boolean.valueOf(getProperty(prop));
			return value.booleanValue();
		} catch (NumberFormatException e) {
			UserMessages.print(res.getString("WRONG_BOOLEAN") + prop);
			return false;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  color  Description of Parameter
	 *@return        Description of the Returned Value
	 */
	protected String colorString(Color color) {
		String result =
			color.getRed() + "," + color.getGreen() + "," + color.getBlue();
		return result;
	}

	/**
	 *  Parse a comma delimited list of 3 strings into a Color
	 *
	 *@param  prop  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected ColorUIResource colorValue(String prop) {
		int red = 0;
		int green = 0;
		int blue = 0;

		if (prop == null) {
			new ColorUIResource(red, green, blue);
		}

		try {
			StringTokenizer st = new StringTokenizer(prop, ",");

			red = Integer.parseInt(st.nextToken());
			green = Integer.parseInt(st.nextToken());
			blue = Integer.parseInt(st.nextToken());
		} catch (Exception e) {
			UserMessages.print("Couldn't parse color :" + prop);
		}
		return new ColorUIResource(red, green, blue);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  prop  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	protected int defaultIntValue(String prop) {
		if (prop.equals("unicore.sockettimeout")) {
			return 60000;
		}
		return 0;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  prop  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	protected long defaultLongValue(String prop) {
		return 0L;
	}

	/**
	 *  Helper function: Parse a String for a Double and return 0 if String
	 *  contains no Double
	 *
	 *@param  prop  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected double doubleValue(String prop) {
		try {
			return Double.parseDouble(getProperty(prop));
		} catch (NumberFormatException e) {
			UserMessages.print(res.getString("WRONG_DOUBLE") + prop);
			return 0.0;
		}
	}

	/**
	 *  Fill properties with default values
	 */
	protected abstract void genSystemDefaults();

	/**
	 *  Overwrite getProperty method to prevent return value null and return ""
	 *  instead. This is important to prevent null pointer execptions from
	 *  faulty UserDefaults files.
	 *
	 *@param  prop  property value
	 *@return       "" if property not available
	 */
	public String getProperty(String prop) {
		String value = super.getProperty(prop);

		if (value == null) {
			UserMessages.print(res.getString("COULD_NOT_READ") + prop);
			return "";
		} else {
			return value;
		}
	}

	/**
	 *  Get path to .unicore directory
	 *
	 *@return    path ending with ".unicore"
	 */
	public String getUnicoreDir() {
		return unicoreDir;
	}

	/**
	 *  Gets the userHome attribute of the Defaults object
	 *
	 *@return    The userHome value
	 */
	public String getUserHome() {
		return userHome;
	}

	/**
	 *  Gets the userName attribute of the Defaults object
	 *
	 *@return    The userName value
	 */
	public String getUserName() {
		return userName;
	}

	/**
	 *  Helper function: Parse a String for an Integer and return 0 if String
	 *  contains no integer
	 *
	 *@param  prop  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected int intValue(String prop) {
		try {
			return Integer.parseInt(getProperty(prop));
		} catch (NumberFormatException e) {
			UserMessages.print(res.getString("WRONG_INTEGER") + prop);
			return defaultIntValue(prop);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public abstract boolean loadFromFile();

	/**
	 *  Load user default values from file ${HOME}/.unicoreXX/UserDefaults.txt
	 *
	 *@param  filename  Description of the Parameter
	 *@return           true if loading was successful, else false
	 */
	protected boolean loadFromFile(String filename) {
		// look for  ${HOME}/.unicoreXX/UserDefaults.txt
		File defs = new File(filename);
		if (defs.exists()) {
			// try to load file
			try {
				load(new FileInputStream(defs));
			}
			// loading failed
			catch (Exception e) {
				// no error message here, but in calling method
				logger.log(Level.SEVERE, "", e);
				return false;
			}
		}

		// no ${HOME}/.unicoreXX/UserDefaults.txt found
		else {
			return false;
		}
		update();
		// loading was successful
		return true;
	}

	/**
	 *  Helper function: Parse a String for an Long and return 0 if String
	 *  contains no integer
	 *
	 *@param  prop  Description of Parameter
	 *@return       Description of the Returned Value
	 */
	protected long longValue(String prop) {
		try {
			return Long.parseLong(getProperty(prop));
		} catch (NumberFormatException e) {
			UserMessages.print(res.getString("WRONG_INTEGER") + prop);
			return defaultLongValue(prop);
		}
	}

	/**
	 *  Overwrite this method to update system with new default entries
	 */
	protected void update() {
		logger.fine("Update in Defaults not implemented.");
	}

	/**
	 *  Try to write user default values to file ${HOME}/.unicoreXX/UserDefaults.txt
	 *
	 *@return    true if saving was successful, else false
	 */
	public abstract boolean writeToFile();

	/**
	 *  Description of the Method
	 *
	 *@param  filename  Description of the Parameter
	 *@return           Description of the Return Value
	 */
	protected boolean writeToFile(String filename) {
		// always overwrite old ${HOME}/.unicoreXX/UserDefaults.txt
		File defs = new File(filename);
		// try to write
		try {
			store(
				new FileOutputStream(defs),
				"UNICOREpro Defaults File. DO NOT EDIT MANUALLY!!");
		}
		// writing failed
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
			return false;
		}

		update();

		// writing was successful
		return true;
	}
}
