package com.pallas.unicore.client.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;

import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Abstract base class for Client task plugins.
 *
 *@author     Ralf Ratering
 *@created    27. Mai 2002
 *@version $Id: TaskPlugable.java,v 1.2 2003/06/30 06:56:35 rrate Exp $
 */
public abstract class TaskPlugable extends UnicorePlugable {

	/**
	 *  Count container instances
	 */
	protected int counter = 0;
	/**
	 *  Image for icon representing the plugin task in the client GUI.
	 */
	protected Image iconImage;

	/**
	 *  Gets the container class from the plugin
	 *
	 *@return    The containerClass value
	 */
	public final Class getContainerClass() {
		counter--;
		if (counter < 0) {
			counter = 0;
		}
		return getContainerInstance(new JobContainer()).getClass();
	}

	/**
	 *  Build a new container for this plugin
	 *
	 *@param  parentContainer  JobContainer that plugin task belongs to or null
	 *      if the container was attached to the root of the tree.
	 *@return                  a new instance of the plugin specific task
	 *      container
	 */
	public abstract ActionContainer getContainerInstance(GroupContainer parentContainer);

	/**
	 *  Gets the Image representing the plugin task in the client GUI.
	 *
	 *@return    plugin icon image
	 */
	public Image getIconImage() {
		if (iconImage == null) {
			iconImage = loadImage(getIconPath());
		}
		return iconImage;
	}

	/**
	 *  Get path to image icon representaing the task in the GUI
	 *
	 *@return    path to icon image as String
	 */
	protected String getIconPath() {
		return null;
	}

	/**
	 *  Get name for plugin task task will be presented to the user e.g. in the
	 *  JPA menu.
	 *
	 *@return    String displaying task name to the user
	 */
	public abstract String getName();

	/**
	 *  Return a new JPA panel inatnace to set the parameter for the plugin
	 *  task. This panel will be added to the clients JPA area, when the task is
	 *  seleceted in the job preparation tree. Non-Task Plugins should return
	 *  null here, because they will not provide a panel for the job preparation
	 *  area.
	 *
	 *@param  container  Description of Parameter
	 *@return            a new instance of the plugin specific JPAPanel.
	 */
	public abstract JPAPanel getPanelInstance(ActionContainer container);

	/**
	 *  Check, if plugin will provide a certain container class
	 *
	 *@param  cls  container class to ask for
	 *@return      true if container class matches plugin container class
	 */
	public final boolean hasContainerClass(Class cls) {
		counter--;
		if (counter < 0) {
			counter = 0;
		}
		return getContainerInstance(new JobContainer()).getClass().equals(cls);
	}

	/**
	 *  Load an image from a given path
	 *
	 *@param  filename  path to Image as String
	 *@return           loaded Image
	 *@deprecated       use getIconPath() instead
	 */
	protected Image loadImage(String filename) {
		if (filename == null) {
			return ResourceManager.getImage(ResourceManager.UNKNOWNTASK);
		}
		ClassLoader loader = getClass().getClassLoader();
		URL url = loader.getResource(filename);
		return Toolkit.getDefaultToolkit().createImage(url);
	}

}
