package com.pallas.unicore.client.xml;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.w3c.dom.Element;

import com.pallas.unicore.utility.UserMessages;

/**
 *  XMLObjectClass keeps the fields of a class which get saved in XML format.
 *  These may be either all fields or the fields specified in variable
 *  persistentXMLfields of the class, except the fields which have modifiers of
 *  type NO_XML (i.e. static or transient fields).
 *
 *@author     kdoertel
 *@created    April 10, 2002
 *@version $Id: XMLObjectClass.java,v 1.3 2003/06/30 06:56:47 rrate Exp $
 */
public class XMLObjectClass {

	private final static Class[] createDOMElementArgs = { Element.class };

	// set debug mode and output prefix
	private static int DEBUG = XMLObjectIO.DEBUG;

	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.xml.XMLObjectClass");
	private static StringBuffer prefix = new StringBuffer("XMLObjectClass: ");
	private final static Class[] readExternalArgs = { ObjectInput.class };
	private final static Class[] readObjectArgs = { ObjectInputStream.class };
	private final static Class[] readReplaceArgs = {
	};

	// static and transient fields are not serialized in XML format
	private final static int TRANSIENT_STATIC =
		Modifier.STATIC | Modifier.TRANSIENT;
	private final static Class[] writeExternalArgs = { ObjectOutput.class };
	private final static Class[] writeObjectArgs = { ObjectOutputStream.class };
	private final static Class[] writeReplaceArgs = {
	};

	// global hashtable which keeps all XMLObjectClass objects
	private static Hashtable xmlClassTable = new Hashtable();

	/**
	 *  Retrieve XMLObjectClass from input class Looks for corresponding
	 *  XMLObjectClass in hashtable and creates new object if not yet available.
	 *
	 *@param  key  class for which the corresponding XMLObjectClass object is
	 *      searched for
	 *@return      The classXML value
	 */
	public static synchronized XMLObjectClass getClassXML(Class key) {

		// look
		XMLObjectClass xmlClass = (XMLObjectClass) xmlClassTable.get(key);

		if (xmlClass != null) {
			if (key.equals(xmlClass.clazz)) {
				if (DEBUG > 1) {
					System.out.println(
						prefix
							+ "key "
							+ key
							+ " value "
							+ xmlClass
							+ " already in hashtable");
				}
			} else {
				String error =
					"ERROR in XMLObjectClass: identical keys for distinct objects!!!"
						+ "\nPrev hashCode: "
						+ xmlClass.clazz.hashCode()
						+ " value "
						+ xmlClass
						+ " internal class "
						+ xmlClass.clazz
						+ "\nNew  hashCode: "
						+ key.hashCode()
						+ " value not yet generated, internal class "
						+ key;
				UserMessages.error(error);
				return null;
			}
		} else {
			xmlClass = new XMLObjectClass(key);
			xmlClassTable.put(key, xmlClass);
			if (DEBUG > 1) {
				System.out.println(
					prefix
						+ "key "
						+ key
						+ " value "
						+ xmlClass
						+ " NEW in hashtable");
			}
		}

		return xmlClass;
	}

	// class which fields are kept here
	private Class clazz;

	private Method createDOMElementMethod = null;
	private int jCurr = 0;
	private Method readExternalMethod = null;
	private Method readObjectMethod = null;
	private Method readReplaceMethod = null;

	private Vector tmpFields;
	private Method writeExternalMethod = null;
	private Method writeObjectMethod = null;
	private Method writeReplaceMethod = null;
	// array of the fields which are saved in XML format
	private XMLObjectField xmlFields[] = null;

	/**
	 *  Private constructor for XMLObjectClass New objects are created by call
	 *  to getClassXML
	 *
	 *@param  clazz  class which fields are analyzed and kept in the
	 *      XMLObjectClass class for later use
	 */
	private XMLObjectClass(Class clazz) {
		this.clazz = clazz;

		if (DEBUG > 0) {
			String c = clazz.getName();
			prefix =
				new StringBuffer(
					"XMLObjectClass " + c.substring(c.lastIndexOf('.')) + ": ");
		}

		// Objects may create DOM elements by themselves, look for existence of method createDOMElement

		createDOMElementMethod =
			XMLObjectIO.getDeclaredMethod(
				this.clazz,
				"createDOMElement",
				createDOMElementArgs);

		// Check existence of methods which implicitly require existence of createDOMElement method

		if (createDOMElementMethod == null) {
			writeObjectMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"writeObject",
					writeObjectArgs,
					Modifier.PRIVATE,
					Modifier.STATIC);
			if (writeObjectMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares writeObject() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares writeObject() method");
			}

			writeExternalMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"writeExternal",
					writeExternalArgs,
					Modifier.PUBLIC,
					Modifier.STATIC);
			if (writeExternalMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares writeExternal() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares writeExternal() method");
			}

			writeReplaceMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"writeReplace",
					writeReplaceArgs);
			if (writeReplaceMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares writeReplace() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares writeReplace() method");
			}

			readObjectMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"readObject",
					readObjectArgs,
					Modifier.PRIVATE,
					Modifier.STATIC);
			if (readObjectMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares readObject() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares readObject() method");
			}

			readExternalMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"readExternal",
					readExternalArgs,
					Modifier.PUBLIC,
					Modifier.STATIC);
			if (readExternalMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares readExternal() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares readExternal() method");
			}

			readReplaceMethod =
				XMLObjectIO.getDeclaredMethod(
					this.clazz,
					"readReplace",
					readReplaceArgs);
			if (readReplaceMethod != null && DEBUG > 0) {
				if (DEBUG >= 2)
					UserMessages.warning(
						"JOS: Class "
							+ clazz
							+ " declares readReplace() method");
				logger.warning(
					prefix
						+ "**************************** JOS: Class "
						+ clazz
						+ " declares readReplace() method");
			}
		}

		// Look for constructor which takes a DOM element

		tmpFields = new Vector(0);

		// scan over current class and its superclasses and
		// gather all fields in temporary vector tmpFields
		scanFieldsSuperClass(clazz.getSuperclass());
		scanFields(clazz);

		// copy temporary vector to array xmlFields for later use
		int size = tmpFields.size();
		xmlFields = new XMLObjectField[size];
		for (int i = 0; i < size; i++) {
			xmlFields[i] = (XMLObjectField) tmpFields.get(i);
		}

		tmpFields = null;

		if (DEBUG > 0) {
			prefix = new StringBuffer("XMLObjectClass: ");
		}
	}

	/**
	 *  Gets the createDOMElement attribute of the XMLObjectClass object
	 *
	 *@return    The createDOMElement value
	 */
	protected Method getCreateDOMElement() {
		return createDOMElementMethod;
	}

	// TODO: put fields into internal hashtable
	/**
	 *  Get the Field object which corresponds to its XML name
	 *
	 *@param  fieldNameXML  the XML name of the field
	 *@return               The field value
	 */
	public Field getField(String fieldNameXML) {
		Field field = null;

		// jCurr is used for performance reasons. It will work fine
		// if the fields are stored and loaded in identical order
		for (int j = jCurr; j < xmlFields.length; j++) {
			if (fieldNameXML.equals(xmlFields[j].getFieldNameXML())) {
				field = xmlFields[j].getField();
				jCurr = j + 1;
				break;
			}
		}
		if (field == null) {
			for (int j = 0; j < jCurr; j++) {
				if (fieldNameXML.equals(xmlFields[j].getFieldNameXML())) {
					field = xmlFields[j].getField();
					jCurr = j + 1;
					break;
				}
			}
		}

		if (field == null) {
			String error =
				"WARNING in XMLObjectClass: could not find field "
					+ fieldNameXML
					+ " of class "
					+ clazz;
			logger.warning(error);
			//	    UserMessages.error(error);
		}

		return field;
	}

	// return the internally kept field information
	/**
	 *  Gets the fieldsXML attribute of the XMLObjectClass object
	 *
	 *@return    The fieldsXML value
	 */
	public XMLObjectField[] getFieldsXML() {
		return xmlFields;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	protected boolean overwritesSerialization() {
		return (writeObjectMethod != null);
	}

	/**
	 *  Scan over fields of the current class
	 *
	 *@param  objClass  the current class
	 */
	private void scanFields(Class objClass) {

		// Return if top of class chain reached
		if (objClass == null || objClass == Object.class) {
			return;
		}

		// Return if special classes occur
		if (objClass == Vector.class || objClass == Hashtable.class) {
			return;
		}

		if (DEBUG > 1) {
			System.err.println(prefix + "CLASS: " + objClass.getName());
		}

		// Scan over all fields of the current class level

		String fieldNames[] = null;
		Field objFields[] = null;
		int j = 0;

		try {
			// search for field persistentXMLFields which defines the fields
			// which are serialized to XML
			Field persistent = objClass.getDeclaredField("persistentXMLFields");
			persistent.setAccessible(true);
			String persistentFieldsXML[][] = (String[][]) persistent.get(null);

			objFields = new Field[persistentFieldsXML.length];
			fieldNames = new String[persistentFieldsXML.length];
			for (int i = 0; i < persistentFieldsXML.length; i++) {
				try {
					String name = persistentFieldsXML[i][0];
					Field field =
						objClass.getDeclaredField(persistentFieldsXML[i][1]);
					// Warn if TRANSIENT or STATIC modifier
					if ((field.getModifiers() & TRANSIENT_STATIC) != 0) {
						if (DEBUG > 1) {
							System.out.println(
								prefix
									+ "WARNING: Using TRANSIENT or STATIC field["
									+ i
									+ "]: "
									+ name
									+ " % "
									+ field.getName());
						}
					}
					objFields[j] = field;
					fieldNames[j] = name;
					j++;
				} catch (NoSuchFieldException nsfe) {
					UserMessages.error(
						"ERROR: could not get declared field "
							+ persistentFieldsXML[i][0]
							+ ":"
							+ persistentFieldsXML[i][1]);
					logger.log(Level.SEVERE, "", nsfe);
				}
			}
		} catch (IllegalAccessException iae) {
			logger.log(Level.SEVERE, "", iae);
		} catch (NoSuchFieldException nof) {

			try {
				// search for field serialPersistentFields which defines the fields
				// which are serialized by the default Java Object Serialization
				Field persistent =
					objClass.getDeclaredField("serialPersistentFields");
				persistent.setAccessible(true);
				ObjectStreamField persistentFieldsXML[] =
					(ObjectStreamField[]) persistent.get(null);

				objFields = new Field[persistentFieldsXML.length];
				fieldNames = new String[persistentFieldsXML.length];
				for (int i = 0; i < persistentFieldsXML.length; i++) {
					try {
						String name = persistentFieldsXML[i].getName();
						Field field = objClass.getDeclaredField(name);
						// Warn if TRANSIENT or STATIC modifier
						if ((field.getModifiers() & TRANSIENT_STATIC) != 0) {
							if (DEBUG > 1) {
								System.out.println(
									prefix
										+ "WARNING: Using TRANSIENT or STATIC field["
										+ i
										+ "]: "
										+ name
										+ " % "
										+ field.getName());
							}
						}
						objFields[j] = field;
						fieldNames[j] = name;
						j++;
					} catch (NoSuchFieldException nsfe) {
						UserMessages.error(
							"ERROR: could not get declared field "
								+ persistentFieldsXML[i].getName());
						logger.log(Level.SEVERE, "", nsfe);
					}
				}
			} catch (IllegalAccessException iae) {
				logger.log(Level.SEVERE, "", iae);
			} catch (NoSuchFieldException noex) {
				// neither field persistentXMLFields nor field serialPersistentFields available:
				// take all class fields which are neither TRANSIENT nor STATIC
				objFields = objClass.getDeclaredFields();
				fieldNames = new String[objFields.length];
				for (int i = 0; i < objFields.length; i++) {
					Field field = objFields[i];
					// Ignore if TRANSIENT or STATIC modifier
					if ((field.getModifiers() & TRANSIENT_STATIC) != 0) {
						if (DEBUG > 1) {
							System.out.println(
								prefix
									+ "Ignoring TRANSIENT or STATIC field["
									+ i
									+ "]: "
									+ field.getName());
						}
						continue;
					}

					objFields[j] = field;
					fieldNames[j] = field.getName();
					j++;
				}
			}
		}

		// compress arrays to prepare subsequent call of Field.setAccessible
		// - if there were problems in getting all declared fields
		//   (typically due to wrong definition of persistentXMLFields or serialPersistentFields)
		// - if transient or static fields have to be ignored in the default case
		if (j < fieldNames.length) {
			Field tmpObjFields[] = new Field[j];
			String tmpFieldNames[] = new String[j];
			for (int i = 0; i < j; i++) {
				tmpObjFields[i] = objFields[i];
				tmpFieldNames[i] = fieldNames[i];
			}
			objFields = tmpObjFields;
			fieldNames = tmpFieldNames;
		}

		// declare fields to be accessible
		Field.setAccessible(objFields, true);

		// wrap fields in a vector of XMLObjectField
		for (int i = 0; i < objFields.length; i++) {
			XMLObjectField xmlField =
				new XMLObjectField(fieldNames[i], objFields[i]);
			tmpFields.add(xmlField);
			if (DEBUG > 1) {
				System.out.println(
					prefix
						+ "   XML field["
						+ i
						+ "]: "
						+ fieldNames[i]
						+ " % "
						+ objFields[i].getName());
			}
		}
	}

	/**
	 *  Scan recursively over fields of the superclasses i.e. the field
	 *  information is kept top-down in the XMLObjectClass objects.
	 *
	 *@param  objClass  the current (super)class type
	 */
	private void scanFieldsSuperClass(Class objClass) {
		if (objClass != Object.class) {
			scanFieldsSuperClass(objClass.getSuperclass());
			scanFields(objClass);
		}
	}
}
