package com.pallas.unicore.connection;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;
import java.net.ConnectException;
import java.net.Socket;
import java.security.KeyStoreException;
import java.security.SignatureException;
import java.security.cert.Certificate;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;

import org.unicore.AJOIdentifier;
import org.unicore.PortfolioIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Control;
import org.unicore.ajo.ControlAction;
import org.unicore.ajo.CopyPortfolioTask;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.ExportTask;
import org.unicore.ajo.GetPortfolio;
import org.unicore.ajo.ImportTask;
import org.unicore.ajo.ListDirectory;
import org.unicore.ajo.Portfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ListDirectory_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.XFileEnumeration;
import org.unicore.sets.XFileSet;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.ConsignJobReply;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;
import org.unicore.utility.ConsignForm;
import org.unicore.utility.PacketisedInputStream;
import org.unicore.utility.PacketisedOutputStream;

import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Class that manages direct connections between client and NJS. CAUTION: Do
 *  not forget to close the connection when it's not needed anymore!
 *
 *@author      Ralf Ratering
 *@created     10. Dezember 2001
 *@version:    $Id: DirectConnection.java,v 1.6 2003/06/30 06:56:55 rrate Exp $
 */
public class DirectConnection {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.connection");
	private ObjectInputStream inputStream;
	private Object lock = new Object();
	private ObjectOutputStream outputStream;
	private Socket socket;

	private UnicoreSSLSocketFactory socketFactory;

	private Vsite vsite;

	/**
	 *  Open a non-polling connection to a vsite
	 *
	 *@param  vsite                           Vsite to connect to
	 *@exception  IOException
	 *@exception  StreamCorruptedException
	 *@exception  ConnectException
	 *@exception  SSLPeerUnverifiedException
	 */
	public DirectConnection(Vsite vsite)
		throws
			IOException,
			StreamCorruptedException,
			ConnectException,
			SSLPeerUnverifiedException {
		logger.info("Open direct connection: " + vsite);

		this.vsite = vsite;
		socketFactory =
			new UnicoreSSLSocketFactory(ResourceManager.getUser(vsite));

		// open socket
		socket = socketFactory.createSocket(vsite.getAddress());
		socket.setSoTimeout(60000);

		// validate server certificate
		try {
			Certificate[] serverCert =
				((SSLSocket) socket).getSession().getPeerCertificates();
		} catch (SSLPeerUnverifiedException e) {
			logger.log(Level.INFO, "Server certificate validation failed.", e);
			return;
		}
		// build input&output streams
		outputStream = new ObjectOutputStream(socket.getOutputStream());
		inputStream = new ObjectInputStream(socket.getInputStream());
	}

	/**
	 *  Description of the Method
	 *
	 *@param  job                           Description of Parameter
	 *@param  polling                       Description of Parameter
	 *@return                               Description of the Return Value
	 *@exception  StreamCorruptedException  Description of Exception
	 *@exception  ClassNotFoundException    Description of Exception
	 *@exception  IOException               Description of Exception
	 *@exception  KeyStoreException         Description of the Exception
	 *@exception  SignatureException        Description of the Exception
	 *@exception  Exception                 Description of the Exception
	 */
	private ConsignJob buildConsignedJob(AbstractJob job, boolean polling)
		throws
			StreamCorruptedException,
			ClassNotFoundException,
			IOException,
			KeyStoreException,
			SignatureException,
			Exception {

		ConsignJob cj = JobConverter.convert(job, polling, true);
		return cj;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  job  Description of the Parameter
	 *@return      Description of the Return Value
	 */
	private boolean checkSuccess(AbstractJob job) {
		// now get reply
		try {
			Reply reply = (Reply) inputStream.readObject();
			UnicoreResponse response = reply.getLastEntry();
			if (response.getReturnCode() != 0) {
				return false;
			}

			if (reply instanceof RetrieveOutcomeReply) {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] serializedOutcome = roReply.getOutcome();
				if (serializedOutcome != null) {
					Outcome outcome =
						(Outcome) new ObjectInputStream(
							new ByteArrayInputStream(serializedOutcome))
							.readObject();
					return outcome.getStatus().equals(
						AbstractActionStatus.SUCCESSFUL);
				}
			}
			// if we have a ConsignJobReply, our job went into polling mode
			// kill it!
			else {
				killJob(job);
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Could not get outcome", e);
			e.printStackTrace();
		}

		return false;
	}

	/**
	 *  Close the connection.
	 *
	 *@exception  IOException  Description of Exception
	 */
	public void close() throws IOException {
		outputStream.close();
		inputStream.close();
		logger.info("Close direct connection: " + vsite);
	}

	/**
	 *  Get a portfolio containing multiple files from a job's USpace and store
	 *  it in a directory
	 *
	 *@param  jobId              ID of job with uspace
	 *@param  sourcePortfolioId  ID of portfolio inside USpace
	 *@param  targetDir          Directory where files are stored
	 *@param  isAscii            flag if file is ascii type
	 *@return                    The fileFromUspace value
	 */
	public boolean getFileFromUspace(
		PortfolioIdentifier sourcePortfolioId,
		AJOIdentifier jobId,
		String targetDir,
		boolean isAscii) {
		synchronized (lock) {
			GetPortfolio getPortfolio =
				new GetPortfolio(
					"GetPortfolio_" + ResourceManager.getNextObjectIdentifier(),
					null,
					sourcePortfolioId,
					jobId);

			// copy portfolio
			CopyPortfolioTask copyPortfolio =
				new CopyPortfolioTask(
					"CopyPortfolio_(" + vsite.getName() + ", " + jobId + ")");
			copyPortfolio.setTarget(getPortfolio.getTarget());

			// setup service job
			String servicePrefix = ResourceManager.getServicePrefix();
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_GetUspacePortfolio_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					ResourceManager.getUser(vsite));

			job.add(getPortfolio);
			job.add(copyPortfolio);
			try {
				job.addDependency(new Dependency(getPortfolio, copyPortfolio));
			} catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);
				write(consignedJob);
				return writeStreamedFilesToDir(job, targetDir, isAscii);
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Could not write streamed files", e);
			}
			return false;
		}
	}

	/**
	 *  Get a file listing from the remote file system
	 *
	 *@param  dir      Path to directory in storage sub space
	 *@param  storage  storage resource
	 *@return          A set of Xfiles
	 *@deprecated      Use com.pallas.requests.GetListing instead
	 */
	public XFileSet getListing(String dir, Storage storage) {
		synchronized (lock) {
			ListDirectory listDir =
				new ListDirectory(
					"_ListDir_("
						+ vsite.getName()
						+ ", "
						+ dir
						+ ")"
						+ ResourceManager.getNextObjectIdentifier());
			listDir.setResources(new ResourceSet(storage));
			listDir.setTarget(dir);

			String servicePrefix = ResourceManager.getServicePrefix();
			User user = ResourceManager.getUser(vsite);
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_FileAction_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);
			job.add(listDir);

			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);
				// submit job
				write(consignedJob);

				// get reply
				Reply reply = (Reply) inputStream.readObject();

				// reply has correct type -> nonPolling succeeded
				if (reply instanceof RetrieveOutcomeReply) {
					RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
					// deserialize the Outcome Object
					byte[] outcomeBytes = roReply.getOutcome();
					Outcome outcome =
						(Outcome) new ObjectInputStream(
							new ByteArrayInputStream(outcomeBytes))
							.readObject();
					AbstractJob_Outcome ajOut =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

					Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
					if (ajOutcome instanceof ListDirectory_Outcome) {
						ListDirectory_Outcome listDirOutcome =
							(ListDirectory_Outcome) ajOutcome;
						return listDirOutcome.getListing();
					}
				}
				// nonPolling failed->kill job
				else if (reply instanceof ConsignJobReply) {
					killJob(job);
					return null;
				}
				// unexpected reply->error
				else {
					logger.warning("Wrong outcome: " + reply);
					return null;
				}
			} catch (Exception e) {
				logger.log(
					Level.WARNING,
					"Could not get listing for directory "
						+ dir
						+ " at storage "
						+ storage,
					e);
			}
			return null;
		}
	}

	/**
	 *  Get a whole bunch of listings at once
	 *
	 *@param  dirs   Vector of RemoteFile directories to get listings from
	 *@return        Hashtable containing RemoteFile dirs as keys and listing
	 *      vectors as values
	 *@deprecated    Use com.pallas.requests.GetListings instead
	 */
	public Hashtable getListings(Vector dirs) {
		synchronized (lock) {
			String servicePrefix = ResourceManager.getServicePrefix();
			User user = ResourceManager.getUser(vsite);
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_FileAction_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);

			Vector listDirs = new Vector();
			for (int i = 0; i < dirs.size(); i++) {
				RemoteFile dir = (RemoteFile) dirs.elementAt(i);
				String dirname = dir.getSubspacePath();
				Storage storage = dir.getStorage();
				ListDirectory listDir =
					new ListDirectory(
						"_ListDir_("
							+ vsite.getName()
							+ ", "
							+ dirname
							+ ")"
							+ ResourceManager.getNextObjectIdentifier());
				listDir.setResources(new ResourceSet(storage));
				listDir.setTarget(dirname);
				listDirs.add(listDir);
				job.add(listDir);
				logger.fine(
					"Getting listing for "
						+ dirname
						+ " at storage "
						+ storage);
			}
			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);

				// submit job
				write(consignedJob);

				// get reply
				Reply reply = (Reply) inputStream.readObject();

				// reply has correct type -> non-Polling succeeded
				if (reply instanceof RetrieveOutcomeReply) {
					RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
					// deserialize the Outcome Object
					byte[] outcomeBytes = roReply.getOutcome();
					Outcome outcome =
						(Outcome) new ObjectInputStream(
							new ByteArrayInputStream(outcomeBytes))
							.readObject();
					AbstractJob_Outcome ajOut =
						ConsignForm.convertFrom((AbstractJob_Outcome) outcome);

					Hashtable table = new Hashtable();
					for (int i = 0; i < listDirs.size(); i++) {
						ListDirectory listDir =
							(ListDirectory) listDirs.elementAt(i);
						Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
						if (ajOutcome instanceof ListDirectory_Outcome) {
							ListDirectory_Outcome listDirOutcome =
								(ListDirectory_Outcome) ajOutcome;
							Vector listing = new Vector();
							RemoteFile parent = (RemoteFile) dirs.elementAt(i);
							if (listDirOutcome.getListing() == null) {
								logger.severe(
									"Could not get listing for "
										+ listDir
										+ " at site "
										+ vsite);
							} else {
								XFileEnumeration fileEnum =
									listDirOutcome.getListing().elements();
								while (fileEnum.hasMoreElements()) {
									XFile xfile =
										(XFile) fileEnum.nextElement();
									RemoteFile file =
										new RemoteFile(
											xfile,
											parent,
											parent.getFileSystem());
									listing.add(file);
								}
							}
							table.put(parent, listing);
						}
					}
					return table;
				}
				// nonPolling failed->kill job
				else if (reply instanceof ConsignJobReply) {
					logger.warning(
						"Get listing timed out and switched to polling mode - Killing job now: "
							+ reply);
					killJob(job);
					return null;
				}
				// unexcpected reply->error
				else {
					logger.warning("Wrong outcome: " + reply);
					return null;
				}
			} catch (Exception e) {
				logger.log(
					Level.WARNING,
					"Could not get listing for directories " + dirs + ".",
					e);
			}
			return null;
		}
	}

	/**
	 *  Get a RemoteFile as byte array out of a vsite's XSpace
	 *
	 *@param  file  RemoteFile object to be transferred
	 *@return       file as byte array
	 */
	public byte[] getRemoteFile(RemoteFile file) {
		String sourcePath = file.getSubspacePath();
		Storage storage = file.getStorage();
		return getRemoteFile(sourcePath, storage);
	}

	/**
	 *  Get a file specified by name and storage as byte array out of a vsite's
	 *  XSpace
	 *
	 *@param  sourcePath  path to file in storage subspace
	 *@param  storage     storage resource
	 *@return             file as byte array
	 */
	public byte[] getRemoteFile(String sourcePath, Storage storage) {
		synchronized (lock) {
			if (sourcePath.indexOf(" ") >= 0) {
				UserMessages.error(
					"Cannot transfer files with spaces in filename: "
						+ sourcePath);
				return null;
			}
			// import task
			ImportTask importTask =
				new ImportTask(
					"ImportTask_(" + vsite.getName() + ", " + sourcePath + ")");
			importTask.setResources(new ResourceSet(storage));
			importTask.addFile(sourcePath);

			// copy portfolio
			CopyPortfolioTask copyPortfolio =
				new CopyPortfolioTask(
					"CopyPortfolio_("
						+ vsite.getName()
						+ ", "
						+ sourcePath
						+ ")");
			copyPortfolio.setTarget(importTask.getPortfolio());

			// setup service job
			String servicePrefix = ResourceManager.getServicePrefix();
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_GetFile_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					ResourceManager.getUser(vsite));
			job.add(importTask);
			job.add(copyPortfolio);
			try {
				job.addDependency(new Dependency(importTask, copyPortfolio));
			} catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);
				write(consignedJob);
				String uspaceName =
					sourcePath.substring(sourcePath.lastIndexOf("/") + 1);
				String streamedName =
					copyPortfolio.getUPLDirectoryName()
						+ "/"
						+ copyPortfolio.getTarget().getUPLDirectoryName()
						+ "/"
						+ uspaceName;
				return readStreamedFile(job, streamedName);
			} catch (Exception e) {
				UserMessages.warning(
					"Transferring file from local workstation to remote system failed.",
					e.getMessage());
			}
			return null;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  target  Description of Parameter
	 */
	private void killJob(AbstractJob target) {

		String servicePrefix = ResourceManager.getServicePrefix();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job =
			new AbstractJob(
				servicePrefix
					+ "KillerJob"
					+ ResourceManager.getNextObjectIdentifier(),
				null,
				null,
				vsite,
				user);

		ControlAction killJob =
			new ControlAction(
				"Kill_Job" + ResourceManager.getNextObjectIdentifier(),
				target.getAJOId(),
				Control.CANCEL);
		job.add(killJob);

		// job will kill itself when it's finished, so
		// we do not have to send a retrieve outcome acknowledge
		ControlAction suicide =
			new ControlAction(
				"Suicide" + ResourceManager.getNextObjectIdentifier(),
				job.getAJOId(),
				Control.CANCEL);
		job.add(suicide);

		// send killing job in polling mode
		try {
			ConsignJob consignedJob = buildConsignedJob(job, true);
			write(consignedJob);
		} catch (Exception e) {
			logger.log(
				Level.SEVERE,
				"Exception occured while killing job " + target,
				e);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  filename                    Description of Parameter
	 *@param  job                         Description of the Parameter
	 *@return                             Description of the Returned Value
	 *@exception  IOException             Description of Exception
	 *@exception  ClassNotFoundException  Description of Exception
	 */
	private byte[] readStreamedFile(AbstractJob job, String filename)
		throws IOException, ClassNotFoundException {
		Reply reply = (Reply) inputStream.readObject();
		logger.fine("Reply from reading object: " + reply);

		UnicoreResponse response = reply.getLastEntry();
		if (response.getReturnCode() != 0) {
			// something went wrong
			throw new IOException(response.getComment());
		}

		if (reply instanceof RetrieveOutcomeReply) {
			RetrieveOutcomeReply ror = (RetrieveOutcomeReply) reply;
			if (ror.hasStreamed()) {
				PacketisedInputStream pis =
					new PacketisedInputStream(socket.getInputStream());
				ZipInputStream zis = new ZipInputStream(pis);
				ZipEntry entry = zis.getNextEntry();
				while (entry != null) {
					if (filename.equals(entry.getName())) {
						// this stuff can probably be optimized...
						ByteArrayOutputStream outputStream =
							new ByteArrayOutputStream();
						while (zis.available() > 0) {
							int read = zis.read();
							// do not write eof file byte
							if (zis.available() > 0) {
								outputStream.write(read);
							}
						}
						outputStream.flush();
						zis.closeEntry();
						zis.close();
						pis.close();
						return outputStream.toByteArray();
					}
					zis.closeEntry();
					entry = zis.getNextEntry();
				}
				zis.close();
				pis.close();
			}
		} else if (reply instanceof ConsignJobReply) {
			logger.fine("Switched to polling -> killing job...");
			killJob(job);
		} else {
			logger.warning("Wrong outcome: " + reply);
		}
		return null;
	}

	/**
	 *  Send a file into the USpace
	 *
	 *@param  destinationPortfolioId  portfolio in USpace where to put the file
	 *@param  jobId                   job id belonging to USpace
	 *@param  isAscii                 flag if it's ascii
	 *@param  destinationName         uspace filename
	 *@param  contents                file contents to transfer
	 *@return                         Description of the Return Value
	 */
	public boolean sendFileToUspace(
		PortfolioIdentifier destinationPortfolioId,
		AJOIdentifier jobId,
		String destinationName,
		byte[] contents,
		boolean isAscii) {
		synchronized (lock) {
			Portfolio streamedPortfolio =
				new Portfolio(
					"NspacePortfolio_"
						+ ResourceManager.getNextObjectIdentifier());

			DeclarePortfolio declarePortfolio =
				new DeclarePortfolio(
					"DeclarePortfolio_"
						+ ResourceManager.getNextObjectIdentifier());
			declarePortfolio.setPortfolio(streamedPortfolio);

			PutPortfolio putPortfolio =
				new PutPortfolio(
					"PutPortfolio_"
						+ ResourceManager.getNextObjectIdentifier());
			putPortfolio.setSource(declarePortfolio.getPortfolio());
			putPortfolio.setTarget(destinationPortfolioId);
			putPortfolio.setTargetUspace(jobId);
			putPortfolio.setTargetMustExist(false);

			// setup service job
			String servicePrefix = ResourceManager.getServicePrefix();
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_GetUspacePortfolio_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					ResourceManager.getUser(vsite));
			job.addStreamed(streamedPortfolio);
			job.add(declarePortfolio);
			job.add(putPortfolio);
			try {
				job.addDependency(
					new Dependency(declarePortfolio, putPortfolio));
			} catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);
				write(consignedJob);
				streamFile(
					destinationName,
					contents,
					streamedPortfolio.getUPLDirectoryName());
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Could not send file to uspace", e);
				return false;
			}
			return checkSuccess(job);
		}
	}

	/**
	 *  Send a RemoteFile object containing a byte array to a vsite's XSpace
	 *
	 *@param  file      RemoteFile object
	 *@param  contents  file contents as byte array
	 *@return           Description of the Return Value
	 */
	public boolean sendRemoteFile(RemoteFile file, byte[] contents) {
		String destinationPath = file.getSubspacePath();
		Storage storage = file.getStorage();
		return sendRemoteFile(destinationPath, storage, contents);
	}

	/**
	 *  Send a file containing a byte array to a vsite's XSpace
	 *
	 *@param  destinationPath  path to file in storage subspace
	 *@param  storage          storage rersource
	 *@param  contents         file contents as byte array
	 *@return                  Description of the Return Value
	 */
	public boolean sendRemoteFile(
		String destinationPath,
		Storage storage,
		byte[] contents) {
		synchronized (lock) {
			// setup service job
			String servicePrefix = ResourceManager.getServicePrefix();
			User user = ResourceManager.getUser(vsite);
			AbstractJob job =
				new AbstractJob(
					servicePrefix
						+ "_SendFile_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					vsite,
					user);

			Portfolio streamedPortfolio =
				new Portfolio(
					"NspacePortfolio_"
						+ ResourceManager.getNextObjectIdentifier());
			job.addStreamed(streamedPortfolio);

			USpace uspaceHome =
				ResourceManager.getResourceSet(vsite).getUspace();
			DeclarePortfolio declarePortfolio =
				new DeclarePortfolio(
					"DeclarePortfolio_"
						+ ResourceManager.getNextObjectIdentifier(),
					new ResourceSet(uspaceHome));
			declarePortfolio.setPortfolio(streamedPortfolio);
			job.add(declarePortfolio);

			// cut destination directory from subspace path
			int index = destinationPath.lastIndexOf("/");
			String destDir = "/";
			String destinationName = destinationPath;
			if (index > 0) {
				destDir = destinationPath.substring(0, index);
				destinationName = destinationPath.substring(index + 1);
			}
			ExportTask exportTask =
				new ExportTask(
					"ExportTask_" + ResourceManager.getNextObjectIdentifier(),
					null,
					new ResourceSet(storage),
					streamedPortfolio,
					true,
					destDir);
			job.add(exportTask);

			try {
				job.addDependency(new Dependency(declarePortfolio, exportTask));
			} catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
			try {
				ConsignJob consignedJob = buildConsignedJob(job, false);
				consignedJob.setStreamed(true);
				write(consignedJob);
				streamFile(
					destinationName,
					contents,
					streamedPortfolio.getUPLDirectoryName());
			} catch (Exception e) {
				logger.log(Level.SEVERE, "Could not send file to xspace.", e);
				return false;
			}

			return checkSuccess(job);
		}
	}

	/**
	 *  Stream a file to the USpace
	 *
	 *@param  contents         file contents as byte array
	 *@param  uplDirectory     uplDirectory for zip entry
	 *@param  uspaceName       filename in USpace
	 *@exception  IOException  thrown if file could not be streamed
	 */
	private void streamFile(
		String uspaceName,
		byte contents[],
		String uplDirectory)
		throws IOException {

		PacketisedOutputStream pos =
			new PacketisedOutputStream(socket.getOutputStream());
		org.unicore.utility.ZipOutputStream zos =
			new org.unicore.utility.ZipOutputStream(pos);

		String zipName = uplDirectory + "/" + uspaceName;
		zos.putNextEntry(new ZipEntry(zipName));

		// now send file
		System.err.println("WRITING: " + zipName);

		zos.write(contents);
		zos.flush();
		zos.closeEntry();
		pos.finish();
	}

	/**
	 *  Write an object through output stream
	 *
	 *@param  what             Object to be streamed
	 *@exception  IOException
	 */
	private synchronized void write(Object what) throws IOException {
		logger.fine("Direct writing object: " + what);
		outputStream.writeObject(what);
		outputStream.flush();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  dir                         Description of the Parameter
	 *@param  isAscii                     Description of the Parameter
	 *@param  job                         Description of the Parameter
	 *@return                             Description of the Return Value
	 *@exception  IOException             Description of Exception
	 *@exception  ClassNotFoundException  Description of Exception
	 */
	private boolean writeStreamedFilesToDir(
		AbstractJob job,
		String dir,
		boolean isAscii)
		throws IOException, ClassNotFoundException {
		Reply reply = (Reply) inputStream.readObject();

		UnicoreResponse response = reply.getLastEntry();
		if (response.getReturnCode() != 0) {
			// something went wrong
			throw new IOException(response.getComment());
		}

		if (reply instanceof RetrieveOutcomeReply) {
			RetrieveOutcomeReply ror = (RetrieveOutcomeReply) reply;
			if (ror.hasStreamed()) {
				PacketisedInputStream pis =
					new PacketisedInputStream(socket.getInputStream());
				ZipInputStream zis = new ZipInputStream(pis);
				ZipEntry entry = zis.getNextEntry();
				while (entry != null) {
					String entryName = entry.getName();
					String uspaceName =
						entryName.substring(entryName.lastIndexOf("/") + 1);
					String filename = dir + File.separator + uspaceName;
					FileOutputStream outputStream =
						new FileOutputStream(filename);
					while (zis.available() > 0) {
						int read = zis.read();

						// do not write eof file byte
						if (zis.available() > 0) {
							outputStream.write(read);
						}
					}
					outputStream.flush();
					zis.closeEntry();
					entry = zis.getNextEntry();
				}
				return true;
			}
		} else if (reply instanceof ConsignJobReply) {
			logger.fine("Switched to polling -> killing job...");
			killJob(job);
		} else {
			logger.warning("Wrong outcome: " + reply);
		}
		return false;
	}
}
