package com.pallas.unicore.connection;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StreamCorruptedException;
import java.net.ConnectException;
import java.net.Socket;
import java.net.URL;
import java.util.Vector;
import java.util.logging.Level;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.net.ssl.SSLPeerUnverifiedException;

import org.apache.oro.io.GlobFilenameFilter;
import org.unicore.User;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.outcome.PutPortfolio_Outcome;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.ConsignJobReply;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.ServerRequest;
import org.unicore.upl.UnicoreResponse;
import org.unicore.utility.ConsignForm;
import org.unicore.utility.PacketisedInputStream;
import org.unicore.utility.PacketisedOutputStream;

import com.pallas.ogsa.services.uplstatic.IUPL;
import com.pallas.ogsa.services.uplstatic.IUPLService;
import com.pallas.ogsa.services.uplstatic.IUPLServiceLocator;
import com.pallas.ogsa.services.uplstatic.UPLWrapper;
import com.pallas.ogsa.util.ObjectWrapperBean;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Class that manages connections between client and NJS
 *
 *@author     Thomas Kentemich
 *@created    12. September 2001
 *@version:    $Id: GSConnection.java,v 1.9 2003/07/07 12:46:46 thomas Exp $
 */
public class GSConnection extends Connection {
	private final static String UC_ITERATION_COUNTS_REGEX = "\\$UC_ITERATION_COUNTS";
	private final int NOT_READY = 0;
	private final int READY = 1;
	private final int WRITTEN = 2;
	private final int READ = 3;
	private final int CLOSED = 4;
	private int count = globalCount++;
	private long lifeTime = 3600000;
	private int state;
	private int errorcode;
	private UnicoreResponse response;
	private Reply reply;
	private String target;
	private Socket socket;
	private ObjectOutputStream outputStream;
	private ObjectInputStream inputStream;
	private User user;
	private UnicoreSSLSocketFactory socketFactory;


	private IUPLService service;
	private IUPL  uplPort;


	/**
	 *  Constructor for the Connection object
	 *
	 *@param  target                          Address of the target Gateway
	 *@param  user                            Description of the Parameter
	 *@exception  IOException                 Description of Exception
	 *@exception  StreamCorruptedException    Description of Exception
	 *@exception  ConnectException            Description of Exception
	 *@exception  SSLPeerUnverifiedException  Description of Exception
	 */
	public GSConnection(String target, User user)
	throws IOException, StreamCorruptedException, ConnectException,
		SSLPeerUnverifiedException, SecurityException {
		this.user = user;

		logger.info(" [" + count + "] " + "Open connection: " + target);

		System.setProperty("org.apache.axis.components.net.SecureSocketFactory","com.pallas.ogsa.client.SunJSSESocketFactory");
		System.setProperty("axis.socketSecureFactory", "com.pallas.ogsa.client.SunJSSESocketFactory");

		System.setProperty("clientauth","true");
		
		ResourceManager.getKeystoreManager().setupAxisKeymanger(user.getCertificate());


		// create an rpc service
		service = new IUPLServiceLocator();

		//get the deployed web service
		try {
			uplPort = service.getUPLService(new URL(target));
			} catch (Exception e) {
			throw new IOException("Cannot get UPL ervice");
			}


		state = READY;
		if(sessionDir == null) {
			UserDefaults userDefaults = ResourceManager.getUserDefaults();
			sessionDir = new String("Session" + ResourceManager.getNextObjectIdentifier());
			String scratchDir = userDefaults.getTmpDirectory();
			if(scratchDir == null || scratchDir.equals("")) {
				scratchDir = userDefaults.getUnicoreDir();
				}
			sessionDir = scratchDir + File.separator + sessionDir;
			}
		}


	/**
	 *  Constructor for the Connection object
	 *
	 *@param  target                          Address of the target Gateway
	 *@param  directOutput                    Outputstream for directly storing
	 *      streamed files
	 *@param  isAscii                         Description of the Parameter
	 *@param  user                            Description of the Parameter
	 *@exception  IOException                 Description of Exception
	 *@exception  StreamCorruptedException    Description of Exception
	 *@exception  ConnectException            Description of Exception
	 *@exception  SSLPeerUnverifiedException  Description of Exception
	 */
	public GSConnection(String target, User user, FileExport directOutput, boolean isAscii)
	throws IOException, StreamCorruptedException, ConnectException,
		SSLPeerUnverifiedException {

		// CAUTION: This will only work for single files in the stream !!!
		this(target, user);
		this.directOutput = directOutput;
		this.isAscii = isAscii;
		}


	/**
	 *  Sets the reply attribute of the Connection object
	 *
	 *@param  reply  The new reply value
	 */
	public void setReply(Reply reply) {
		this.reply = reply;
		}


	/**
	 *  Set the lifetime (in ms) of AJO's for this Connection
	 *
	 *@param  lifeTime  The new ajoLifetime value
	 */
	public void setAjoLifetime(long lifeTime) {
		this.lifeTime = lifeTime;
		}


	/**
	 *  Gets the SessionDir attribute of the Connection class
	 *
	 *@return    The SessionDir value
	 */
	public static String getSessionDir() {
		return sessionDir;
		}


	/**
	 *  Gets the user attribute of the Connection object
	 *
	 *@return    The user value
	 */
	public User getUser() {
		return user;
		}


	/**
	 *  Gets the OutputStream of the Connection
	 *
	 *@return                  The OutputStream value
	 *@exception  IOException  Description of Exception
	 */
	public OutputStream getOutputStream()
	throws IOException {
		return socket.getOutputStream();
		}


	/**
	 *  Gets the InputStream of the Connection
	 *
	 *@return                  The InputStream value
	 *@exception  IOException  Description of Exception
	 */
	public InputStream getInputStream()
	throws IOException {
		return socket.getInputStream();
		}


	/**
	 *  Gets the Reply attribute of the Connection object
	 *
	 *@return    The Reply value
	 */
	public Reply getReply() {
		state = READY;
		return this.reply;
		}


	/**
	 *  returns the lifetime of AJO's (in ms) for this connection
	 *  Can be used to limit the validity of "short" service requests
	 *  Shold not be used with user defined jobs
	 *
	 *@return    The ajoLifetime value
	 */

	public long getAjoLifetime() {
		return lifeTime;
		}


	/**
	 *  Minimal main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {}


	/**
	 *  Write a serialized AJO
	 *
	 *@param  what             Description of Parameter
	 *@exception  IOException  Description of Exception
	 */
	public void write(Object what)
	throws IOException {
		if(state == READY) {
			logger.fine(" [" + count + "] " + "Writing AJO: ");
			outputStream.writeObject(what);
			outputStream.flush();
			}
		state = WRITTEN;
		}


	/**
	 *  Description of the Method
	 *
	 *@return                             Description of the Returned Value
	 *@exception  IOException             Description of Exception
	 *@exception  ClassNotFoundException  Description of Exception
	 */
	public Object read()
	throws IOException, ClassNotFoundException {
		Object returnedObject = null;
		if(state == WRITTEN) {
			state = READY;
			logger.fine(" [" + count + "] " + "Reading NJS reply: ");
			returnedObject = inputStream.readObject();

			// does the gateway really close the socket ?
			return returnedObject;
			} else {
			state = READY;
			return returnedObject;
			}
		}


	/**
	 *  Description of the Method
	 *
	 *@param  what           Description of the Parameter
	 *@param  streamedFiles  Description of the Parameter
	 *@return                Description of the Return Value
	 *@exception  Exception  Description of the Exception
	 */
	public synchronized Reply sendMessage(Object what, Vector streamedFiles)
	throws Exception {

		logger.info(" [" + count + "] " + "Send message: " + target);
		ObjectWrapperBean result;
		ObjectWrapperBean wrapper;
		UPLWrapper uplWrapper = new UPLWrapper();
		if(what instanceof ServerRequest) {
			try {
				ConsignForm.AJO cfa = new ConsignForm.AJO((ConsignJob)what);
				AbstractJob job = cfa.getAJO();
				logger.fine("GS Endorser is: " + job.getEndorser().getCertificate().getSubjectDN());
				} catch(Exception e) {}

			//call the methods



			uplWrapper.setJob((ServerRequest)what);
			wrapper = new ObjectWrapperBean(uplWrapper);
			if(streamedFiles != null && streamedFiles.size()>0) {
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				writeStreamedFiles(streamedFiles, out);
				wrapper.setBytes(out.toByteArray());
				}


			result = uplPort.consign(wrapper);
			} else {
			throw new IllegalArgumentException("No UPL request provided");
			}

		uplWrapper = (UPLWrapper)result.deserialize();
		reply = uplWrapper.getReply();
		;
		if(reply instanceof RetrieveOutcomeReply) {

			// AJO was consigned in non-polling mode or this was a
			// RetrieveOutcome request
			RetrieveOutcomeReply ror = (RetrieveOutcomeReply)reply;
			if(ror.hasStreamed()) {
				readStreamedOutcome(new ByteArrayInputStream(uplWrapper.getStreamedBytes()));
				}
			}
		logger.info(" [" + count + "] " + "Send message reply: " + reply);
		//       close();
		return reply;
		}


	/**
	 *  Close the connection.
	 *
	 *@exception  IOException  Description of Exception
	 */
	public void close()
	throws IOException {
		if(state != READY) {
			return;
			}

		logger.fine(" [" + count + "] " + "Close connection: " + target);
		state = CLOSED;
		}


	/**
	 *  Description of the Method
	 *
	 *@exception  Exception  Description of the Exception
	 */
	public void flush()
	throws Exception {
		if(state == CLOSED) {
			return;
			}
		// outputStream.flush();

		//inputStream.flush();
		logger.fine(" [" + count + "] " + "Flush connection: " + target);
		state = READY;
		}


	/**
	 *  Description of the Method
	 *
	 *@exception  IOException               Description of Exception
	 *@exception  StreamCorruptedException  Description of Exception
	 *@exception  Exception                 Description of Exception
	 */
	public void reconnect()
	throws IOException, StreamCorruptedException, Exception {
		close();
		/*		this.socket = socketFactory.createSocket(target);
				this.outputStream = new ObjectOutputStream(socket.getOutputStream());
				this.inputStream = new ObjectInputStream(socket.getInputStream()); */
		state = READY;
		}


	/**
	 *  Description of the Method Read the NJS's reply after sending an AJO.
	 *  Retrieve all streamed files possibly following the reply.
	 *
	 *@return                             Description of the Returned Value
	 *@exception  IOException             Description of Exception
	 *@exception  ClassNotFoundException  Description of Exception
	 */
	public int readReply()
	throws IOException, ClassNotFoundException {
		if(state == WRITTEN) {
			try {
				reply = (Reply)read();
				} catch(EOFException eofx) {
				logger.warning("Problems reading reply (EOF): ignored");
				response = new UnicoreResponse(0, "Problems reading reply (EOF): ignored");
				reply = new ConsignJobReply();
				reply.addTraceEntry(response);
				}
			response = reply.getLastEntry();
			errorcode = response.getReturnCode();
			state = NOT_READY;
			if(errorcode != 0) {

				// something went wrong
				return errorcode;
				}

			// nothing special for ListVsites_Reply and RetrieveOutcomeAck_Reply
			if(reply instanceof ConsignJobReply) {

				// AJO was consigned in polling mode do nothing special
				}
			else if(reply instanceof RetrieveOutcomeReply) {

				// AJO was consigned in non-polling mode or this was a
				// RetrieveOutcome request
				RetrieveOutcomeReply ror = (RetrieveOutcomeReply)reply;
				if(ror.hasStreamed()) {

					// decode the streamed files
					try {
						readStreamedOutcome(socket.getInputStream());
						} catch(IOException e) {
						logger.log(Level.SEVERE, "Could not read streamed files", e);
						}
					}
				}
			state = READY;
			}
		return errorcode;
		}


	/**
	 *  Write files imported from nspace as zip stream following a ConsignJob
	 *  object.
	 *
	 *@param  importTasks                Vector with ImportContainers that
	 *      contain import tasks from nspace
	 *@exception  FileNotFoundException  Description of Exception
	 *@exception  IOException            Description of Exception
	 */
	public void writeStreamedFiles(Vector importTasks, OutputStream out)
	throws FileNotFoundException, IOException {
		if(importTasks.size() <= 0) {
			return;
			}
		PacketisedOutputStream pos = new PacketisedOutputStream(out);
		org.unicore.utility.ZipOutputStream zos = new org.unicore.utility.ZipOutputStream(pos);
		for(int i = 0; i < importTasks.size(); i++) {

			// get container from vector
			TaskContainer importContainer = (TaskContainer)importTasks.get(i);
			FileImport[] fileImports = importContainer.getFileImportsFromLocal();
			for(int j = 0; j < fileImports.length; j++) {

				// DeclarePortfolio defines portfolio containing files from container
				DeclarePortfolio portfolio = fileImports[j].getNspacePortfolio();
				String uplDirectory = portfolio.getPortfolio().getUPLDirectoryName();

				// check for wildcards
				if(fileImports[j].hasWildcards()) {
					logger.fine("Wildcard in Nspace import: " + fileImports[j]);
					streamWildcardFiles(fileImports[j], uplDirectory, zos);
					} else {
					streamFile(fileImports[j], uplDirectory, zos);
					}
				}
			}

		// close zip stream
		pos.finish();

		//   zos.close();
		}


	/**
	 *  Description of the Method
	 */
	public void finalize() {
		try {
			close();
			} catch(IOException e) {
			logger.fine("Connection already closed: " + e.getMessage());
			}
		}


	/**
	 *  Description of the Method Send files that have to be transfered from the
	 *  local file system to the Uspace by appending them as a ZIPStream after
	 *  the serialized AJO. Wildcard sensitive variant.
	 *
	 *@param  uplDirectory               Description of Parameter
	 *@param  zos                        Description of Parameter
	 *@param  wildImport                 Description of the Parameter
	 *@exception  FileNotFoundException  Description of Exception
	 *@exception  IOException            Description of Exception
	 */
	private void streamWildcardFiles(FileImport wildImport, String uplDirectory,
	                                 org.unicore.utility.ZipOutputStream zos)
	throws FileNotFoundException, IOException {
		String filename = wildImport.getSourceName();
		File file = new File(filename);
		String directoryPath = file.getParent();
		String pattern = file.getName();
		File directory = new File(directoryPath);
		GlobFilenameFilter fileFilter = new GlobFilenameFilter(pattern);
		File[] files = directory.listFiles((FilenameFilter)fileFilter);
		if(files != null) {
			for(int i = 0; i < files.length; i++) {
				FileImport fileImport = new FileImport(FileStorage.NSPACE_STRING,
				                                       files[i].getAbsolutePath(),
				                                       wildImport.getDestinationName());

				/*
				 *  To resolve all kinds of imports with renaming, a file/directory is first streamed
				 *  without renaming to a temporary hidden directory which has been set in ImportFactory
				 *  as the destination name of the FileImport at hand. A dependent FileImport (additionally
				 *  specified in ImportDirectory) will then copy the streamed file/directory to its actual
				 *  destination. The renaming will take place implicitly because the dependent FileImport
				 *  uses CopyFile.
				 *  Make a file destination from the hidden directory destination
				 */
				fileImport.setDestinationName(
				        fileImport.getDestinationName() + "/" + fileImport.getSourceBasename());
				streamFile(fileImport, uplDirectory, zos);
				}
			}
		}


	/**
	 *  Description of the Method Send files that have to be transfered from the
	 *  local file system to the Uspace by appending them as a ZIPStream after
	 *  the serialized AJO.
	 *
	 *@param  uplDirectory               Description of Parameter
	 *@param  zos                        Description of Parameter
	 *@param  fileImport                 Description of Parameter
	 *@exception  FileNotFoundException  Description of Exception
	 *@exception  IOException            Description of Exception
	 */
	private void streamFile(FileImport fileImport, String uplDirectory,
	                        org.unicore.utility.ZipOutputStream zos)
	throws FileNotFoundException, IOException {
		writeImportToZip(fileImport.getSourceName(), fileImport.getDestinationName(), uplDirectory,
		                 fileImport.getFileMode(), zos);
		}


	/**
	 *  Description of the Method
	 *
	 *@param  filename                   Description of the Parameter
	 *@param  uspaceName                 Description of the Parameter
	 *@param  uplDir                     Description of the Parameter
	 *@param  isAscii                    Description of the Parameter
	 *@param  zos                        Description of the Parameter
	 *@exception  FileNotFoundException  Description of the Exception
	 *@exception  IOException            Description of the Exception
	 */
	private void writeImportToZip(String filename, String uspaceName, String uplDir,
	                              boolean isAscii, org.unicore.utility.ZipOutputStream zos)
	throws FileNotFoundException, IOException {
		File importObj = new File(filename);
		if(importObj.exists() == true) {
			if(importObj.isDirectory() == true) {

				// Create an array of File objects, one for each file or directory in dirObj.
				File[] fileList = importObj.listFiles();

				// Loop through File array and display.
				for(int i = 0; i < fileList.length; i++) {
					if(fileList[i].isDirectory()) {
						writeImportToZip(fileList[i].getPath(),
						                 uspaceName + "/" + fileList[i].getName(), uplDir, isAscii,
						                 zos);
						} else if(fileList[i].isFile()) {

						// Call the zipFunc function
						writeFileToZip(fileList[i].getPath(),
						               uspaceName + "/" + fileList[i].getName(), uplDir, isAscii,
						               zos);
						}
					}

				// for loop
				}
			else {

				// Call the zipFunc function
				writeFileToZip(importObj.getPath(), uspaceName, uplDir, isAscii, zos);
				}
			}
		}


	/**
	 *  Description of the Method
	 *
	 *@param  filename                   Description of the Parameter
	 *@param  uspaceName                 Description of the Parameter
	 *@param  uplDirectory               Description of the Parameter
	 *@param  isAscii                    Description of the Parameter
	 *@param  zos                        Description of the Parameter
	 *@exception  FileNotFoundException  Description of the Exception
	 *@exception  IOException            Description of the Exception
	 */
	private void writeFileToZip(String filename, String uspaceName, String uplDirectory,
	                            boolean isAscii, org.unicore.utility.ZipOutputStream zos)
	throws FileNotFoundException, IOException {

		// use portfolio directory and uspacename for zip entry
		// expects UNIX filesystem !!!
		String zipName = uplDirectory + "/" + uspaceName;
		logger.info("Writing ZIP entry: " + zipName);
		zos.putNextEntry(new ZipEntry(zipName));
		FileInputStream fis = new FileInputStream(filename);
		if(!isAscii) {

			// now send file
			byte[] buffer = new byte[4096];
			int read;
			while((read = fis.read(buffer)) > 0) {
				zos.write(buffer, 0, read);
				logger.fine("next chunk: " + read);
				}
			} else {

			// now send file
			byte read;
			while((read = (byte)fis.read()) > 0) {

				// skip dos-sytle ^Ms (eol) and ^Zs (eof)
				if(read != 13 && read != 26) {
					zos.write((int)read);
					}
				}
			}
		zos.closeEntry();
		fis.close();
		}


	/**
	 *  Description of the Method Receive file(s) streamed together whit the
	 *  reply. As optimization a *single* file can be directly stored to the
	 *  provided FileOutputStream. All other files are temporarily stored in the
	 *  Sessiondir and must be retrieved form ther later. This holds especially
	 *  for the stdout/stderr files of the UserTasks.
	 *
	 *@exception  FileNotFoundException  Description of Exception
	 *@exception  IOException            Description of Exception
	 */
	private void readStreamedOutcome(InputStream is)
	throws FileNotFoundException, IOException {
		PacketisedInputStream pis = new PacketisedInputStream(is);
		ZipInputStream zis = new ZipInputStream(pis);
		int avail;
		int count = 0;
		logger.fine("Reading streamed outcome.");
		if((avail = zis.available()) != 0) {

			// unspool into Session dir
			if(directOutput == null) {
				FileOutputStream fos = null;
				File streamedDir = null;
				ZipEntry entry = zis.getNextEntry();
				byte[] buffer = new byte[4096];
				int read = 0;
				OutcomeManager.clearCache();
				while(entry != null) {
					String zipName = entry.getName();
					int lastSep = zipName.lastIndexOf("/");
					String filename = zipName.substring(lastSep + 1);
					String dir = sessionDir + File.separator + zipName.substring(0, lastSep);
					logger.fine("Receiving streamed file: " + zipName);

					// put streamed filename into outcome manager
					OutcomeManager.addStreamedFile(zipName);
					streamedDir = new File(dir);
					streamedDir.deleteOnExit();
					File streamedFile = new File(dir, filename);

					// delete streamed files when JVM stops
					streamedFile.deleteOnExit();
					streamedDir.mkdirs();
					fos = new FileOutputStream(streamedFile);
					while((read = zis.read(buffer)) > 0) {
						fos.write(buffer, 0, read);
						}
					fos.close();
					zis.closeEntry();

					// delete all spool files on exit
					File parent = streamedDir;
					while(parent.getParentFile() != null) {
						parent.deleteOnExit();
						parent = parent.getParentFile();
						}
					parent = parent.getParentFile();
					entry = zis.getNextEntry();
					}
				}
			// unspool directly into destination dir
			else {
				logger.fine("Direct output files into destination directory.");
				copyFile(zis);
				}
			long skipped = pis.skip(Long.MAX_VALUE);
			}
		}


	/**
	 *  Description of the Method
	 *
	 *@param  zis                        Description of the Parameter
	 *@exception  FileNotFoundException  Description of Exception
	 *@exception  IOException            Description of Exception
	 */
	private void copyFile(ZipInputStream zis)
	throws FileNotFoundException, IOException {
		Vector targetFiles = directOutput.getTargetFiles();

		String targetName = directOutput.getDestinationName();
		logger.fine(
		        "Direct copy to: " + targetName + " PF: " +
		        directOutput.getSpoolPortfolio().getUPLDirectoryName());
		ZipEntry entry = zis.getNextEntry();
		String zipName = null;
		String realName = null;
		String uplName = null;
		PutPortfolio_Outcome ppo = null;
		String iteration = "";
		Vector spoolOutcomes = OutcomeManager.getOutcomeEntries(directOutput.getSpoolTask());
		if(spoolOutcomes.size() <= 0) {

			// no Outcome
			logger.severe(
			        "Cannot find outcome of spool task for file: " +
			        directOutput.getSourceName());
			return;
			}
		while(entry != null) {

			// zipName = entry.getName();
			zipName = entry.getName().replace('/', File.separatorChar);
			logger.fine("Receiving streamed file (direct output)" + zipName);
			for(int i = 0; i < spoolOutcomes.size(); i++) {
				ppo = (PutPortfolio_Outcome)((OutcomeEntry)spoolOutcomes.elementAt(i)).getOutcome();

				//uplName = ppo.getPortfolio().getName();
				uplName = (new Portfolio("", ppo.getPortfolio().getValue())).getUPLDirectoryName();
				if(zipName.lastIndexOf(uplName) > 0) {
					iteration = ((OutcomeEntry)spoolOutcomes.elementAt(i)).getIteration();
					break;
					}
				}

			// cut of the Portfolio junk
			// realName = zipName.substring(zipName.indexOf(directOutput.getSpoolPortfolio().getUPLDirectoryName()));
			realName = zipName.substring(zipName.lastIndexOf(uplName));
			realName = realName.substring(realName.indexOf(File.separator) + 1);
			targetName = directOutput.getDestinationName().replaceAll(UC_ITERATION_COUNTS_REGEX,
			                iteration);
			File destination = new File(targetName);
			if(destination.exists()) {
				if(destination.isDirectory()) {

					// prepend the target directory name
					if(!targetName.endsWith(File.separator)) {
						targetName = targetName + File.separator;
						}
					realName = targetName + realName;
					File destDir = new File(realName.substring(0,
					                        realName.lastIndexOf(File.separator)));
					destDir.mkdirs();
					} else {

					// possibly rename it here
					realName = targetName;
					}
				} else {
				if(realName.indexOf(File.separator) > 0) {

					// directory
					// prepend the target directory name
					if(!targetName.endsWith(File.separator)) {
						targetName = targetName + File.separator;
						}

					// possibly rename
					logger.fine("non exixting  target:" + realName);
					String sourceDir = realName.substring(0, realName.indexOf(File.separator));
					realName = targetName +
					           realName.substring(realName.indexOf(File.separator) + 1);
					File destDir = new File(realName.substring(0,
					                        realName.lastIndexOf(File.separator)));
					destDir.mkdirs();
					if(this.directOutput.getTargetFilesFlag() == this.directOutput.CHOOSE_OVERWRITTEN_FILES) {
						if(targetFiles.contains(realName)) {
							copyFileFromZip(realName, zis);
							}
						} else {
						copyFileFromZip(realName, zis);
						}
					// shortcut loop here  otherwise we will run into the wrong branch for existing dir
					entry = zis.getNextEntry();
					while(entry != null) {
						zipName = entry.getName().replace('/', File.separatorChar);
						logger.fine("Receiving streamed file (direct output)" + zipName);

						// cut of the Portfolio junk
						realName = zipName.substring(zipName.indexOf(directOutput.getSpoolPortfolio().getUPLDirectoryName()));
						realName = realName.substring(realName.indexOf(File.separator) + 1);
						if(!realName.startsWith(sourceDir)) {

							// this is an error because we cannot handle multiple dirs in the stream here
							logger.severe("Multiple directories in zip stream are not allowed");
							throw new IOException("Multiple directorie in zip stream are not allowed");
							}
						realName = targetName +
						           realName.substring(realName.indexOf(File.separator) + 1);
						destDir = new File(realName.substring(0,
						                                      realName.lastIndexOf(File.separator)));
						destDir.mkdirs();
						if(this.directOutput.getTargetFilesFlag() == this.directOutput.CHOOSE_OVERWRITTEN_FILES) {
							if(targetFiles.contains(realName)) {
								copyFileFromZip(realName, zis);
								}
							} else {
							copyFileFromZip(realName, zis);
							}

						entry = zis.getNextEntry();
						}
					this.directOutput.resetTargetFilesFlag();
					return;
					} else {

					// possibly rename it here
					realName = targetName;
					}
				}
			if(this.directOutput.getTargetFilesFlag() == this.directOutput.CHOOSE_OVERWRITTEN_FILES) {
				if(targetFiles.contains(realName)) {
					copyFileFromZip(realName, zis);
					}
				} else {
				copyFileFromZip(realName, zis);
				}
			entry = zis.getNextEntry();
			}
		this.directOutput.resetTargetFilesFlag();
		}


	/**
	 *  Description of the Method
	 *
	 *@param  realName                   Description of the Parameter
	 *@param  zis                        Description of the Parameter
	 *@exception  FileNotFoundException  Description of the Exception
	 *@exception  IOException            Description of the Exception
	 */
	private void copyFileFromZip(String realName, ZipInputStream zis)
	throws FileNotFoundException, IOException {
		logger.fine("Real target:" + realName);
		byte[] buffer = new byte[4096];
		int read = 0;
		File destFile = new File(realName);
		if(!isAscii) {
			File parentFile = destFile.getParentFile();
			if(!parentFile.isDirectory()) {
				parentFile.mkdir();
				}

			FileOutputStream fos = new FileOutputStream(destFile);
			while((read = zis.read(buffer)) > 0) {
				fos.write(buffer, 0, read);
				}
			fos.close();
			} else {
			BufferedReader br = new BufferedReader(new InputStreamReader(zis));
			PrintWriter bw = new PrintWriter(new FileOutputStream(destFile));
			String line;
			while((line = br.readLine()) != null) {
				bw.println(line);
				}
			bw.close();
			}
		}



	}
