package com.pallas.unicore.container;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.ImageIcon;

import org.unicore.AAIdentifier;
import org.unicore.ajo.ActionGroup;
import org.unicore.outcome.AbstractActionStatus;

import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.Version;

/**
 *  Abstract base class managing an AJO ActionGroup, an icon and its position in
 *  the dependency graph and a status for each AJO task. The container classes
 *  are used for information exchange between a component constructing jobs
 *  (like the JPA), possibly GUI based and the classes responsible for
 *  submitting jobs.
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    24. August 2001
 *@version    $Id: ActionContainer.java,v 1.3 2003/07/02 07:18:42 rrate Exp $
 */
public abstract class ActionContainer
	extends Object
	implements Serializable, Cloneable {

	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.container");

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "VERSION", "versionInfo" }, {
			"PARENT", "parentContainer" }, {
			"NAME", "name" }, {
			"IDENTIFIER", "id" }, {
			"ICONPOSITION", "iconPosition" }, {
			"IGNOREFAILURE", "ignoreFailure" }
	};

	final static long serialVersionUID = 4421856744783358741L;
	/**
	 *  Description of the Field
	 */
	public final static int STATE_SUBMITTED = 2;
	/**
	 *  Description of the Field
	 */
	public final static int STATE_SUBMITTING = 1;
	/**
	 *  Description of the Field
	 */
	public final static int STATE_UNKNOWN = 0;
	/**
	 *  Description of the Field
	 */
	public final static int STATE_UP_TO_DATE = 4;
	/**
	 *  Description of the Field
	 */
	public final static int STATE_UPDATING = 3;

	/**
	 *  The ajo action group that is encapsulated by the container
	 */
	protected ActionGroup actionGroup = null;

	/**
	 *  Current status of container
	 */
	protected transient AbstractActionStatus actionStatus =
		AbstractActionStatus.NEVER_RUN;

	/**
	 *  internal state information e.g. for setting the icon symbol
	 */
	protected transient int containerState = STATE_UNKNOWN;

	//flagging that an error occurred
	private transient ErrorSet errors;

	/**
	 *  Icon that represents this type of container
	 */
	protected transient ImageIcon icon;

	/**
	 *  Position of icon in dependency graph
	 */
	protected Point iconPosition;

	/**
	 *  AJO identifier
	 */
	protected AAIdentifier id;

	/**
	 *  Description of the Field
	 */
	protected boolean ignoreFailure;

	// date storing last modify time
	private transient Date modifiedTime;

	private String name;

	/**
	 *  Reference to the parent container
	 */
	protected GroupContainer parentContainer = null;

	/**
	 *  Icon that displays the current container status in the gui
	 */
	protected transient ImageIcon statusIcon;

	/**
	 *  Status for this container
	 */
	protected transient Hashtable statusTable = new Hashtable();

	/**
	 *  Reference to the user defaults
	 */
	protected transient UserDefaults userDefaults =
		ResourceManager.getUserDefaults();

	// tags the container so that we can later retrieve the version info
	private String versionInfo = null;

	/**
	 *  Constructor for the ActionContainer object
	 */
	public ActionContainer() {
		this(null);
	}

	/**
	 *  Default Constructor
	 *
	 *@param  parentContainer  Description of the Parameter
	 */
	public ActionContainer(GroupContainer parentContainer) {
		this.parentContainer = parentContainer;
		this.id = new AAIdentifier("New ActionContainer");
		this.errors = new ErrorSet();
		updateContainerVersion();
		if (statusTable == null) {
			statusTable = new Hashtable();
		}
	}

	/**
	 *  Add an ActionStatus for one of the AbstractActions belonging to this
	 *  container e.g when executing GetNewJobStatus
	 *
	 *@param  actionID      Identifier of the AbstractAction
	 *@param  actionStatus  The new status value
	 */
	public final void addActionStatus(
		AAIdentifier actionID,
		AbstractActionStatus actionStatus) {
		if (actionStatus == null) {
			return;
		}
		if (statusTable == null) {
			statusTable = new Hashtable();
		}
		logger.info("ADDING ACTION STATUS TO "
		+ this + ": " + actionID + ", " + actionStatus);
		statusTable.put(actionID, actionStatus);
		updateIcon(getColor(getStatus()));
	}

	/**
	 *  Description of the Method
	 */
	protected void analyzeStatus() {
		if (statusTable == null) {
			statusTable = new Hashtable();
		}

		// move the status stuff away from here (later)
		if (statusTable.get(getIdentifier()) != null) {
			setStatus((AbstractActionStatus) statusTable.get(getIdentifier()));
			updateIcon(getColor(getStatus()));
		}
	}

	/**
	 *  Abstract method. Subclasses should buil the AJO ActionGroup here. The
	 *  method does not return a value, because the ActionGroup is already a
	 *  class member (actionGroup).
	 */
	public abstract void buildActionGroup();

	/**
	 *  Should be overriden by subclasses in order to add own error checking
	 *  methods.
	 *
	 *@return    ErrorSet containing ActionContainer specific errors
	 */
	public ErrorSet checkContents() {
		ErrorSet err = new ErrorSet();
		// check name
		if (id == null) {
			err.add(new UError(id, "Missing task identifier!"));
		}
		setErrors(err);

		return err;
	}

	/**
	 *  Cloneable interface. Copy all necessary informations and return cloned
	 *  ActionContainer
	 *
	 *@return    container clone
	 */
	public Object clone() {
		try {
			ActionContainer clone = (ActionContainer) super.clone();
			if (clone.getName() == null) {
				clone.setName("Unknown");
			}
			clone.setIdentifier(new AAIdentifier(getName()));
			clone.errors = new ErrorSet();
			//clone.checkContents());
			clone.statusIcon = new ImageIcon(statusIcon.getImage());
			clone.actionGroup = null;
			return clone;
		} catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning " + this +" failed.", e);
			return null;
		}
	}

	/**
	 *  Find top level JobContainer from an ActionContainer
	 *
	 *@param  ac  ActionContainer
	 *@return     top level JobContainer
	 */
	protected JobContainer findRootContainer(ActionContainer ac) {
		while (ac.getParentContainer() != null) {
			ac = ac.getParentContainer();
		}
		// last ActionContainer before null must be a JobContainer
		return (JobContainer) ac;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  job   Description of the Parameter
	 *@param  find  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	protected boolean findTask(JobContainer job, ActionContainer find) {
		// also check jobs....
		if (job == find) {
			return true;
		}
		Vector tasks = job.getAllTasks();
		for (int i = 0; i < tasks.size(); i++) {
			ActionContainer container = (ActionContainer) tasks.elementAt(i);
			if (container == find) {
				return true;
			} else if (container instanceof JobContainer) {
				if (findTask((JobContainer) container, find)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 *  Return the encoded AJO for this container.
	 *
	 *@return    The ActionGroup value
	 */
	public final ActionGroup getActionGroup() {
		return this.actionGroup;
	}

	/**
	 *  Gets the color attribute of the ActionContainer object
	 *
	 *@param  status  Description of the Parameter
	 *@return         The color value
	 */
	public final Color getColor(AbstractActionStatus status) {
		// no status == undefined
		if (status == null) {
			return Color.gray;
		}

		if (status.equals(AbstractActionStatus.PENDING)) {
			return Color.magenta;
		} else if (status.equals(AbstractActionStatus.QUEUED)) {
			return Color.blue;
		} else if (status.equals(AbstractActionStatus.RUNNING)) {
			return Color.orange;
		} else if (status.equals(AbstractActionStatus.CONSIGNED)) {
			return Color.blue.brighter();
		} else if (status.equals(AbstractActionStatus.EXECUTING)) {
			return Color.yellow;
		} else if (status.equals(AbstractActionStatus.HELD)) {
			return Color.white;
		} else if (status.equals(AbstractActionStatus.READY)) {
			return Color.cyan;
		} else if (status.equals(AbstractActionStatus.KILLED)) {
			return Color.black;
		} else if (status.equals(AbstractActionStatus.SUCCESSFUL)) {
			return Color.green;
		} else if (status.equals(AbstractActionStatus.NEVER_RUN)) {
			return Color.gray;
		} else if (status.isEquivalent(AbstractActionStatus.NOT_DONE)) {
			return Color.yellow;
		} else if (status.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
			return Color.red;
		}

		return Color.gray;
	}

	/**
	 *  Gets the ContainerVersion attribute of the ActionContainer object
	 *
	 *@return    The ContainerVersion value
	 */
	public final String getContainerVersion() {
		return versionInfo;
	}

	/**
	 *  Gets the Icon attribute of the ActionContainer object
	 *
	 *@return    The Icon value
	 */
	public ImageIcon getIcon() {
		if (icon == null) {
			icon = ResourceManager.getIcon(this);
		}
		return icon;
	}

	/**
	 *  Gets the IconPosition attribute of the ActionContainer object
	 *
	 *@return    The IconPosition value
	 */
	public final Point getIconPosition() {
		return iconPosition;
	}

	/**
	 *  Return the unique Identifier of this container. This is synchronized
	 *  with the Identifier of the AJO constructed from the container
	 *
	 *@return    The Identifier value
	 */
	public final AAIdentifier getIdentifier() {
		return id;
	}

	/**
	 *  Returns the ignore failure flag for this task
	 *
	 *@return    The ignoreFailure value
	 */
	public final boolean getIgnoreFailure() {
		return this.ignoreFailure;
	}

	/**
	 *  Get a time stamp when this container was modified for the last time.
	 *
	 *@return    System date of last modify time
	 */
	public final Date getModifiedTime() {
		return modifiedTime;
	}

	/**
	 *  Gets the Name attribute of the ActionContainer object
	 *
	 *@return    The Name value
	 */
	public final String getName() {
		if (name == null) {
			name = id.getName();
		}
		return name;
	}

	/**
	 *  Return the parent of this contianer
	 *
	 *@return    The ParentContainer value
	 */
	public final GroupContainer getParentContainer() {
		return parentContainer;
	}

	/**
	 *  Gets the parentJobContainer attribute of the ActionContainer object
	 *
	 *@return    The parentJobContainer value
	 */
	public final JobContainer getParentJobContainer() {
		GroupContainer parentJob = parentContainer;
		while (parentJob != null && !(parentJob instanceof JobContainer)) {
			parentJob = parentJob.getParentContainer();
		}
		return (JobContainer) parentJob;
	}

	/**
	 *  Return the action status of the container. The action status can be used
	 *  to obtain the color coding of the container for a GUI based job monitor.
	 *
	 *@return    The Status value
	 */
	public final AbstractActionStatus getStatus() {
		return actionStatus;
	}

	/**
	 *  Gets the statusIcon attribute of the ActionContainer object
	 *
	 *@return    The statusIcon value
	 */
	public final ImageIcon getStatusIcon() {
		if (statusIcon == null) {
			statusIcon = new ImageIcon(getIcon().getImage());
			updateIcon(Color.gray);
		}
		return statusIcon;
	}

	/**
	 *  Check if last checkContents call resulted in any errors
	 *
	 *@return    true if no errors occured
	 */
	public final boolean isComplete() {
		if (errors == null) {
			checkContents();
		}
		return !errors.containsError();
	}

	/**
	 *  Gets the succ attribute of the ActionContainer object
	 *
	 *@param  container  Description of the Parameter
	 *@return            The succ value
	 */
	public boolean isPredecessorOf(ActionContainer container) {
		GroupContainer groupContainer = getParentContainer();
		for (int i = 0; i < groupContainer.getDependencies().length; i++) {
			ActionContainer succ =
				groupContainer.getDependencies()[i].getSucc();
			ActionContainer pred =
				groupContainer.getDependencies()[i].getPred();
			if (pred.getIdentifier() == getIdentifier()) {
				if (succ.getIdentifier() == container.getIdentifier()) {
					return true;
				}
				//recurse
				if (succ.isPredecessorOf(container)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 */
	protected final void selectIcon() {
		if (isComplete()) {
			updateIcon(Color.green);
		} else {
			updateIcon(Color.red);
		}
	}

	/**
	 *  Sets the ActionGroup attribute of the ActionContainer object
	 *
	 *@param  actionGroup  The new ActionGroup value
	 */
	public final void setActionGroup(ActionGroup actionGroup) {
		this.actionGroup = actionGroup;
	}

	/**
	 *  Sets the vontainerVersion attribute of the ActionContainer object
	 *
	 *@param  versionInfo  The new vontainerVersion value
	 */
	public final void setContainerVersion(String versionInfo) {
		this.versionInfo = versionInfo;
	}

	/**
	 *  Set ErrorSet for this container
	 *
	 *@param  errors  The new Errors value
	 */
	public final void setErrors(ErrorSet errors) {
		if (errors == null) {
			throw new IllegalArgumentException("ErrorSet must not be null!");
		}
		this.errors = errors;

		if (!Client.getRunningMode()) {
			selectIcon();
		}
	}

	/**
	 *  Sets the Icon attribute of the ActionContainer object
	 *
	 *@param  icon  The new Icon value
	 */
	public final void setIcon(ImageIcon icon) {
		this.icon = icon;
	}

	/**
	 *  Sets the IconPosition attribute of the ActionContainer object
	 *
	 *@param  iconPosition  The new IconPosition value
	 */
	public final void setIconPosition(Point iconPosition) {
		this.iconPosition = iconPosition;
	}

	/**
	 *  Sets the Identifier attribute of the ActionContainer object
	 *
	 *@param  id  The new Identifier value
	 */
	public final void setIdentifier(AAIdentifier id) {
		this.id = id;
	}

	/**
	 *  Sets the ignore failure flag of this container Dependeing task will be
	 *  executed regardles wether this task failes or not Use with care !!
	 *
	 *@param  what  The new ignoreFailure value
	 */
	public final void setIgnoreFailure(boolean what) {
		this.ignoreFailure = what;
		this.selectIcon();
	}

	/**
	 *  Set time stamp when this container was last modified. Also set parent's
	 *  modified time if a parent is available.
	 *
	 *@param  modifiedTime  The new ModifiedTime value
	 */
	public final void setModifiedTime(Date modifiedTime) {
		this.modifiedTime = modifiedTime;
		// modify all parents
		if (getParentContainer() != null) {
			getParentContainer().setModifiedTime(modifiedTime);
		}
	}

	/**
	 *  Sets the Name attribute of the ActionContainer object
	 *
	 *@param  name  The new Name value
	 */
	public final void setName(String name) {
		this.name = name;
		// sync id name
		this.id.setName(name);
	}

	/**
	 *  Set the parent container. This can only be a JobContainer
	 *
	 *@param  groupContainer  The new parentContainer value
	 */
	public final void setParentContainer(GroupContainer groupContainer) {
		this.parentContainer = groupContainer;
	}

	/**
	 *  Set the action status. This is called by the update threads when
	 *  executing for example GetRunningJobs.
	 *
	 *@param  actionStatus  The new status value
	 */
	public final void setStatus(AbstractActionStatus actionStatus) {
		this.actionStatus = actionStatus;

		updateIcon(getColor(actionStatus));
	}

	/**
	 *  Get String representation for this object
	 *
	 *@return    id name
	 */
	public String toString() {
		return name;
	}

	/**
	 *  Sets the ContainerVersion attribute of the ActionContainer object
	 */
	protected void updateContainerVersion() {
		versionInfo =
			Version.dotVersion() + ".[" + Version.getBuildDate() + "]";
	}

	/**
	 *  Sets the IconColor attribute of the ActionContainer object
	 *
	 *@param  color  The new IconColor value
	 */
	protected void updateIcon(Color color) {

		ImageIcon originalIcon = getIcon();

		int width = originalIcon.getIconWidth();
		int height = originalIcon.getIconHeight();

		if (statusTable == null) {
			statusTable = new Hashtable();
		}

		if (userDefaults == null) {
			userDefaults = ResourceManager.getUserDefaults();
		}

		BufferedImage offscreenImage = null;

		if (userDefaults.getDetailedJobStatus()) {
			offscreenImage =
				new BufferedImage(
					width + 4 * statusTable.size(),
					height,
					BufferedImage.TYPE_INT_RGB);
		} else {
			offscreenImage =
				new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		}
		Graphics2D g2d = offscreenImage.createGraphics();

		g2d.drawImage(originalIcon.getImage(), 0, 0, width, height, null);

		if (userDefaults.getDetailedJobStatus()) {
			int count = 0;
			Enumeration enum = statusTable.elements();
			while (enum.hasMoreElements()) {
				g2d.setColor(
					getColor((AbstractActionStatus) enum.nextElement()));
				g2d.fillRect(width + count * 4, 0, 3, height);
				count++;
			}
		}

		g2d.setColor(color);
		g2d.fillRect(0, height - 4, width - 1, 3);
		g2d.setColor(Color.black);
		g2d.drawRect(0, height - 4, width, 4);

		if (getIgnoreFailure()) {
			g2d.setColor(Color.RED);
			int[] xPoints = { width - 8, width - 1, width - 1 };
			int[] yPoints = { 1, 1, 8 };
			g2d.fillPolygon(xPoints, yPoints, 3);
			g2d.setColor(Color.black);
			g2d.drawLine(xPoints[0], yPoints[0], xPoints[2], yPoints[2]);
		}
		if (statusIcon == null) {
			statusIcon = new ImageIcon();
		}
		statusIcon.setImage(offscreenImage);
	}

}
