package com.pallas.unicore.container;

import java.awt.Color;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.Identifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.If;
import org.unicore.ajo.IfSuccess;
import org.unicore.sets.PortfolioSet;

import com.pallas.unicore.client.graphs.ContainerVertex;
import com.pallas.unicore.client.graphs.DAG;
import com.pallas.unicore.client.graphs.Edge;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileTest;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.ReturnCodeTest;
import com.pallas.unicore.extensions.TimeTest;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.Version;

/**
 *  Abstract container class for a group of tasks and their dependencies.
 *
 *@author     Ralf Ratering
 *@created    August 16, 2001
 *@version    $Id: GroupContainer.java,v 1.2 2003/06/30 06:57:21 rrate Exp $
 */
public abstract class GroupContainer
	extends ActionContainer
	implements Serializable, Cloneable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields = { { "TASKS", "tasks" }, {
			"DEPENDENCIES", "deps" }
	};

	/**
	 *  Description of the Field
	 */
	protected static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");

	final static long serialVersionUID = -2044946196772880605L;

	/**
	 *  Description of the Field
	 */
	protected DependencyContainer[] deps = new DependencyContainer[0];

	/**
	 *  Tasks belonging to this Jobcontainer, as default an empty Vector
	 */
	private Vector tasks = new Vector();

	/**
	 *  Adds a feature to the Tasks attribute of the JobContainer object
	 *
	 *@param  groupContainer  The feature to be added to the Tasks attribute
	 *@param  allTasks        The feature to be added to the Tasks attribute
	 */
	private void addAllTasks(GroupContainer groupContainer, Vector allTasks) {
		Vector groupTasks = groupContainer.getTasks();
		for (int i = 0; i < groupTasks.size(); i++) {
			Object task = groupTasks.elementAt(i);
			allTasks.add(task);
			if (task instanceof GroupContainer
				&& !(task instanceof JobContainer)) {
				addAllTasks((GroupContainer) task, allTasks);
			}
		}
	}

	/**
	 *  Adds a feature to the Dependency attribute of the GroupContainer object
	 *
	 *@param  dep  The feature to be added to the Dependency attribute
	 */
	public void addDependency(DependencyContainer dep) {
		DependencyContainer[] newDeps =
			new DependencyContainer[deps.length + 1];
		for (int i = 0; i < deps.length; i++) {
			newDeps[i] = deps[i];
		}
		newDeps[deps.length] = dep;
		setDependencies(newDeps);
	}

	/**
	 *  Adds all task containers that belong to this job container, but not
	 *  those in sub jobs
	 *
	 *@param  groupContainer  The feature to be added to the ExecuteTasks
	 *      attribute
	 *@param  execTasks       The feature to be added to the ExecutionTasks
	 *      attribute
	 */
	private void addExecutionTasks(
		GroupContainer groupContainer,
		Vector execTasks) {
		Vector groupTasks = groupContainer.getTasks();
		for (int i = 0; i < groupTasks.size(); i++) {
			Object task = groupTasks.elementAt(i);

			if (task instanceof GroupContainer
				&& !(task instanceof JobContainer)) {
				addExecutionTasks((GroupContainer) task, execTasks);
			} else if (task instanceof TaskContainer) {
				execTasks.add(task);
			}
		}
	}

	/**
	 *  Adds all sub jobs that are in this job
	 *
	 *@param  groupContainer  The feature to be added to the SubJobs attribute
	 *@param  allTasks        The feature to be added to the SubJobs attribute
	 */
	private void addSubJobs(GroupContainer groupContainer, Vector allTasks) {
		Vector groupTasks = groupContainer.getTasks();
		for (int i = 0; i < groupTasks.size(); i++) {
			Object task = groupTasks.elementAt(i);
			// if we have a job container, add it
			if (task instanceof JobContainer) {
				allTasks.add(task);
			}
			if (task instanceof GroupContainer) {
				addSubJobs((GroupContainer) task, allTasks);
			}
		}
	}

	/**
	 *  Adds a feature to the Task attribute of the GroupContainer object
	 *
	 *@param  task  The feature to be added to the Task attribute
	 */
	public final void addTask(ActionContainer task) {
		addTaskAt(tasks.size(), task);
	}

	/**
	 *  Adds a feature to the TaskAt attribute of the GroupContainer object
	 *
	 *@param  task   The feature to be added to the TaskAt attribute
	 *@param  index  The feature to be added to the TaskAt attribute
	 */
	public final void addTaskAt(int index, ActionContainer task) {
		if (index > tasks.size()) {
			tasks.addElement(task);
		} else {
			tasks.add(index, task);
		}
	}

	/**
	 *  Description of the Method
	 */
	public void buildActionGroup() {
		// build the action group
		actionGroup =
			new ActionGroup(
				getName() + ResourceManager.getNextObjectIdentifier());
		buildActionGroup(actionGroup);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  actionGroup  Description of the Parameter
	 */
	public void buildActionGroup(ActionGroup actionGroup) {
		this.actionGroup = actionGroup;

		// synchronize identifiers to find corresponding outcomes for
		// container in jmc
		setIdentifier(actionGroup.getId());

		// get task Vector, sorted by dependencies
		Vector sortedTasks = getSortedTasks();

		for (int i = 0; i < sortedTasks.size(); i++) {
			ActionContainer container =
				(ActionContainer) sortedTasks.elementAt(i);
			container.buildActionGroup();
			ActionGroup taskActionGroup = container.getActionGroup();
			// set ignore failure flag
			taskActionGroup.setPropagateSuccessfulIfNotSuccessful(
				container.getIgnoreFailure());
			actionGroup.add(taskActionGroup);
		}

		// add dependencies
		for (int i = 0; i < deps.length; i++) {
			try {
				actionGroup.addDependency(deps[i].getDependency());
			} catch (ActionGroup.InvalidDependencyException e) {
				logger.log(Level.WARNING, "Could not add dependency.", e);
			}
		}
		// set ignore failure flag
		actionGroup.setPropagateSuccessfulIfNotSuccessful(getIgnoreFailure());

	}

	/**
	 *  Self check the correctness of the values relevant for this container
	 *
	 *@return    Description of the Returned Value
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();

		Enumeration e = tasks.elements();
		while (e.hasMoreElements()) {
			ActionContainer ac = (ActionContainer) e.nextElement();
			if (ac instanceof TaskContainer) {
				if (((TaskContainer) ac).getParentContainer() == null) {
					((TaskContainer) ac).setParentContainer(this);
				}
			}
			ErrorSet es = ac.checkContents();
			err.add(es);
		}

		if (err.containsError()) {
			updateIcon(Color.red);
		} else {
			updateIcon(Color.green);
		}

		setErrors(err);
		return err;
	}

	/**
	 *  Clonable Interface. Copy all tasks and dependencies and outcomes and
	 *  return cloned JobContainer
	 *
	 *@return    JobContainer clone
	 */
	public Object clone() {
		GroupContainer clone = (GroupContainer) super.clone();

		// clone tests for control container
		if (this instanceof ControlContainer) {
			ControlContainer controlContainer = (ControlContainer) this;
			ControlContainer controlClone = (ControlContainer) clone;
			controlClone.setReturnCodeTest(
				(ReturnCodeTest) controlContainer.getReturnCodeTest().clone());
			controlClone.setTimeTest(
				(TimeTest) controlContainer.getTimeTest().clone());
			controlClone.setFileTest(
				(FileTest) controlContainer.getFileTest().clone());
		}

		Vector taskClones = new Vector();
		Hashtable cloneTable = new Hashtable();
		Vector sortedTasks = getSortedTasks();
		for (int i = 0; i < sortedTasks.size(); i++) {
			ActionContainer container =
				(ActionContainer) sortedTasks.elementAt(i);
			ActionContainer containerClone =
				(ActionContainer) container.clone();
			containerClone.setParentContainer(clone);
			taskClones.add(containerClone);
			cloneTable.put(container, containerClone);
			// check for If Containers, because their
			// Test tasks must be substituted with their clones
			if (container instanceof IfContainer) {
				IfContainer ifContainer = (IfContainer) container;
				IfContainer ifClone = (IfContainer) containerClone;
				ReturnCodeTest test = ifContainer.getReturnCodeTest();
				ActionContainer testContainer = test.getTestContainer();
				if (testContainer != null) {
					ActionContainer testContainerClone =
						(ActionContainer) cloneTable.get(testContainer);
					ifClone.getReturnCodeTest().setTestContainer(
						testContainerClone);
				} else {
					ifClone.getReturnCodeTest().setTestContainer(null);
				}
				// for future use...
				ActionContainer addedFlag = test.getAddedFlag();
				if (addedFlag != null) {
					ActionContainer addedFlagClone =
						(ActionContainer) cloneTable.get(addedFlag);
					ifClone.getReturnCodeTest().setAddedFlag(addedFlagClone);
				} else {
					ifClone.getReturnCodeTest().setAddedFlag(null);
				}

			}
			// update resource tray in jobs
			else if (this instanceof JobContainer) {
				JobContainer job = (JobContainer) this;
				JobContainer jobClone = (JobContainer) clone;
				// set tray entry
				NamedResourceSet resourceSet =
					(NamedResourceSet) job.getResourceTray().get(
						container.getIdentifier());
				jobClone.getResourceTray().remove(container.getIdentifier());
				jobClone.getResourceTray().put(
					containerClone.getIdentifier(),
					resourceSet.clone());
			}

		}

		// replace test task in do repeat with clone
		if (this instanceof DoRepeatContainer) {
			DoRepeatContainer doRepeat = (DoRepeatContainer) this;
			DoRepeatContainer doRepeatClone = (DoRepeatContainer) clone;
			ReturnCodeTest test = doRepeat.getReturnCodeTest();
			ActionContainer testContainer = test.getTestContainer();
			if (testContainer != null) {
				ActionContainer testContainerClone =
					(ActionContainer) cloneTable.get(testContainer);
				doRepeatClone.getReturnCodeTest().setTestContainer(
					testContainerClone);
			} else {
				logger.warning(
					"Did not find test container clone: "
						+ doRepeat.getReturnCodeTest());
				doRepeatClone.getReturnCodeTest().setTestContainer(null);
			}
			// for future use...
			ActionContainer addedFlag = test.getAddedFlag();
			if (addedFlag != null) {
				ActionContainer addedFlagClone =
					(ActionContainer) cloneTable.get(addedFlag);
				doRepeatClone.getReturnCodeTest().setAddedFlag(addedFlagClone);
			} else {
				doRepeatClone.getReturnCodeTest().setAddedFlag(null);
			}
		}

		clone.setTasks(taskClones);

		// clone dependencies
		if (deps != null) {
			DependencyContainer[] depClones =
				new DependencyContainer[deps.length];
			for (int i = 0; i < deps.length; i++) {
				ActionContainer predClone =
					(ActionContainer) cloneTable.get(deps[i].getPred());
				ActionContainer succClone =
					(ActionContainer) cloneTable.get(deps[i].getSucc());
				depClones[i] = new DependencyContainer(predClone, succClone);
			}
			clone.setDependencies(depClones);
		}

		// replace added deps for if clones
		for (int i = 0; i < tasks.size(); i++) {
			if (tasks.elementAt(i) instanceof IfContainer) {
				IfContainer ifTask = (IfContainer) tasks.elementAt(i);
				IfContainer ifClone = (IfContainer) cloneTable.get(ifTask);
				DependencyContainer addedDep =
					ifTask.getReturnCodeTest().getAddedDependency();
				ifClone.getReturnCodeTest().setAddedDependency(null);
				if (addedDep != null) {
					ActionContainer predClone =
						(ActionContainer) cloneTable.get(addedDep.getPred());
					ActionContainer succClone =
						(ActionContainer) cloneTable.get(addedDep.getSucc());
					DependencyContainer[] depClones = ifClone.getDependencies();
					for (int j = 0; j < depClones.length; j++) {
						if (depClones[j].getPred().equals(predClone)) {
							ifClone.getReturnCodeTest().setAddedDependency(
								depClones[j]);
						}
					}
				}
			}

		}

		// When destination sub jobs in transfers have been cloned, substitute
		// sub jobs by their clones.
		Vector cloneTasks = clone.getTasks();
		for (int i = 0; i < cloneTasks.size(); i++) {
			if (cloneTasks.elementAt(i) instanceof FileTransferContainer) {
				FileTransferContainer container =
					(FileTransferContainer) cloneTasks.elementAt(i);
				com.pallas.unicore.extensions.FileTransfer transfers[] =
					container.getFileTransfers();
				for (int j = 0; j < transfers.length; j++) {
					JobContainer oldJob = transfers[j].getDestinationJob();
					JobContainer newJob = (JobContainer) cloneTable.get(oldJob);
					transfers[j].setDestinationJob(newJob);
				}
			}
		}

		return clone;
	}

	/**
	 *  Return the PortfolioSet containing the local Nspace imports
	 *
	 *@return    Description of the Returned Value
	 */
	protected PortfolioSet collectLocalNspacePortfolios() {
		PortfolioSet pfSet = new PortfolioSet();
		// loop over task containers
		Vector tasks = getExecutionTasks();
		for (int i = 0; i < tasks.size(); i++) {
			TaskContainer importContainer = (TaskContainer) tasks.elementAt(i);
			// fetch the Portfolios imported from nspace
			FileImport localImports[] =
				importContainer.getFileImportsFromLocal();
			for (int j = 0; j < localImports.length; j++) {
				pfSet.add(localImports[j].getNspacePortfolio().getPortfolio());
			}
		}
		return pfSet;
	}

	/**
	 *  Service routine for ActionGroup checking
	 *
	 *@param  job  Description of Parameter
	 *@return      Description of the Return Value
	 */
	protected String dumpActionGroup(ActionGroup job) {
		Identifier id = job.getId();
		Enumeration actions = job.getActions();
		Enumeration dependencies = job.getDependencies();

		String dump = "\n";

		if (job instanceof AbstractJob) {
			dump += "<AbstractJob: " + id.getName() + ",  ";
			dump += "Endorser: " + ((AbstractJob) job).getEndorser();
		} else {
			dump += "<ActionGroup: " + id.getName();
			if (job.propagateSuccessfulIfNotSuccessful()) {
				dump += "<IGNORED>";
			}
			dump += ", ";
		}
		while (actions.hasMoreElements()) {
			AbstractAction next = (AbstractAction) actions.nextElement();
			dump += next.getName();
			if (next.propagateSuccessfulIfNotSuccessful()) {
				dump += "<IGNORED>";
			}
			dump += ", ";

			if (next instanceof If) {
				If ifTask = (If) next;
				dump += dumpActionGroup((ActionGroup) ifTask.getTrueBranch());
				dump += dumpActionGroup((ActionGroup) ifTask.getFalseBranch());
			} else if (next instanceof IfSuccess) {
				IfSuccess ifTask = (IfSuccess) next;
				dump += dumpActionGroup((ActionGroup) ifTask.getTrueBranch());
				dump += dumpActionGroup((ActionGroup) ifTask.getFalseBranch());
			} else if (next instanceof ActionGroup) {
				dump += dumpActionGroup((ActionGroup) next);
			}
		}
		return dump + ">";
	}

	/**
	 *  Gets the allTasks attribute of the JobContainer object
	 *
	 *@return    The allTasks value
	 */
	public Vector getAllTasks() {
		Vector allTasks = new Vector();
		addAllTasks(this, allTasks);
		return allTasks;
	}

	/**
	 *  Get the dependencies for this group
	 *
	 *@return    The Dependencies value
	 */
	public final DependencyContainer[] getDependencies() {
		return deps;
	}

	/**
	 *  Gets the executeTasks attribute of the JobContainer object
	 *
	 *@return    The executeTasks value
	 */
	public Vector getExecutionTasks() {
		Vector executionTasks = new Vector();
		addExecutionTasks(this, executionTasks);
		return executionTasks;
	}

	/**
	 *  Gets the sortedTasks attribute of the GroupContainer object
	 *
	 *@return    The sortedTasks value
	 */
	public Vector getSortedTasks() {

		Vector unsorted = getTasks();

		if (this instanceof IfContainer) {
			return unsorted;
		}

		// build DAG
		DAG dag = new DAG();
		Hashtable table = new Hashtable();
		logger.fine("**** OLD ORDER ****");
		for (int i = 0; i < unsorted.size(); i++) {
			ActionContainer container = (ActionContainer) unsorted.elementAt(i);
			ContainerVertex vertex = new ContainerVertex(container);
			dag.addVertex(vertex);
			table.put(container, vertex);
			logger.fine(vertex.toString());
		}

		for (int i = 0; i < deps.length; i++) {
			ContainerVertex pred =
				(ContainerVertex) table.get(deps[i].getPred());
			ContainerVertex succ =
				(ContainerVertex) table.get(deps[i].getSucc());
			try {
				dag.addEdge(new Edge(pred, succ));
			} catch (Exception e) {
				logger.log(Level.WARNING, "Cycle in dependencies.", e);
			}
		}

		Vector sortedVertices = dag.getSortedVertices(false);
		Vector sorted = new Vector();
		logger.fine("**** NEW ORDER ****");
		for (int i = 0; i < sortedVertices.size(); i++) {
			ContainerVertex vertex =
				(ContainerVertex) sortedVertices.elementAt(i);
			sorted.add(vertex.getContainer());
			logger.fine(vertex.toString());
		}

		return sorted;
	}

	/**
	 *  Gets the subJobs attribute of the GroupContainer object
	 *
	 *@return    The subJobs value
	 */
	public Vector getSubJobs() {
		Vector subJobs = new Vector();
		addSubJobs(this, subJobs);
		return subJobs;
	}

	/**
	 *  Return all tasks belonging to this job group
	 *
	 *@return    The Tasks value
	 */
	public final Vector getTasks() {
		return tasks;
	}

	/**
	 *  Gets the vsite attribute of the GroupContainer object
	 *
	 *@return    The vsite value
	 */
	public Vsite getVsite() {
		return getParentJobContainer().getVsite();
	}

	/**
	 *  Query if any task are defined for the job group.
	 *
	 *@return    Description of the Returned Value
	 */
	public final boolean hasTasks() {
		return (tasks.size() > 0);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  dep  Description of the Parameter
	 */
	public void removeDependency(DependencyContainer dep) {
		if (deps.length == 0) {
			logger.info(
				"Trying to remove dependency "
					+ dep
					+ " from group without dependencies.");
			return;
		}
		DependencyContainer[] newDeps =
			new DependencyContainer[deps.length - 1];
		int offset = 0;
		for (int i = 0; i < deps.length - 1; i++) {
			if (deps[i].equals(dep)) {
				offset = 1;
			}
			newDeps[i] = deps[i + offset];
		}
		setDependencies(newDeps);
	}

	/**
	 *  Remove a task from this job group.
	 *
	 *@param  cont  Description of Parameter
	 */
	public final void removeTask(ActionContainer cont) {
		if (tasks == null) {
			return;
		}
		tasks.removeElement(cont);
	}

	/**
	 *  Define new dependencies for this group
	 *
	 *@param  deps  The new Dependencies value
	 */
	public final void setDependencies(DependencyContainer[] deps) {
		this.deps = deps;
	}

	/**
	 *  Set a new task list for this container.
	 *
	 *@param  tasks  The new Tasks value
	 */
	public final void setTasks(Vector tasks) {
		this.tasks = tasks;
	}

	/**
	 *  Description of the Method
	 */
	public void sortTasks() {
		setTasks(getSortedTasks());
		for (int i = 0; i < tasks.size(); i++) {
			if (tasks.elementAt(i) instanceof GroupContainer) {
				GroupContainer group = (GroupContainer) tasks.elementAt(i);
				group.sortTasks();
			}
		}
	}

	/**
	 *  Description of the Method
	 */
	protected void updateContainerVersion() {
		setContainerVersion(
			Version.dotVersion() + ".[" + Version.getBuildDate() + "]");
		if (tasks == null) {
			return;
		}
		for (int i = 0; i < tasks.size(); i++) {
			ActionContainer next = (ActionContainer) tasks.elementAt(i);
			if (next != null) {
				next.updateContainerVersion();
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  cont  Description of Parameter
	 */
	protected void updateName(ActionContainer cont) {
		if (cont.getName() == null) {
			cont.setName(cont.getIdentifier().getName());
		}
		if (cont instanceof GroupContainer) {
			GroupContainer group = (GroupContainer) cont;
			Vector tasks = group.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				updateName((ActionContainer) tasks.elementAt(i));
			}
		}
	}

}
