package com.pallas.unicore.container;

import java.io.Serializable;
import java.util.Date;
import java.util.logging.Level;

import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.AbstractService;
import org.unicore.ajo.ActionGroup;
import org.unicore.upl.ConsignJob;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.upl.UnicoreResponse;

import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;
import com.pallas.unicore.threadpool.GenericObservable;
import com.pallas.unicore.threadpool.IObservable;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.SerialLock;

/**
 *  Container used for service tasks in requests.
 *
 *@author     Thomas Kentemich
 *@created    17. September 2001
 *@version    $Id: ServiceContainer.java,v 1.1.1.1 2002/04/22 08:30:26 thomas
 *      Exp $
 */
public class ServiceContainer
	extends ActionContainer
	implements Serializable, IObservable, Runnable {

	final static long serialVersionUID = 1130011431582336137L;

	// internal state information e.g. for coloring symbols
	private int containerState = STATE_UNKNOWN;

	// serializable version of a lock
	private SerialLock lock = new SerialLock();
	private Reply reply;
	private AbstractService service = null;

	// task id (for future use)
	private transient int taskId;

	/**
	 *  Description of the Field
	 */
	protected GenericObservable theGenericObservable = new GenericObservable();
	private User user = null;
	private Vsite vsite = null;
	private AbstractJob wrapperJob = null;

	/**
	 *  Creates new ServiceContainer
	 *
	 *@param  service  Description of the Parameter
	 *@param  user     Description of the Parameter
	 *@param  vsite    Description of the Parameter
	 */

	//    public ServiceContainer() {
	//       this(null, null, null);
	//  }

	/**
	 *  Constructor for the ServiceContainer object
	 *
	 *@param  service  Description of Parameter
	 *@param  user     Description of Parameter
	 *@param  vsite    Description of Parameter
	 */
	public ServiceContainer(AbstractService service, User user, Vsite vsite) {
		super();
		this.service = service;
		this.user = user;
		this.vsite = vsite;
	}

	/**
	 *  Adds a feature to the Observer attribute of the UpdateObject object
	 *
	 *@param  anObserver  The feature to be added to the Observer attribute
	 */
	public void addObserver(IObserver anObserver) {
		this.theGenericObservable.addObserver(anObserver);
	}

	/**
	 *  abstract methods
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		actionGroup.add(service);
	}

	/**
	 *  Description of the Method
	 */
	public void deleteAllObservers() {
		this.theGenericObservable.deleteAllObservers();
	}

	/**
	 *  Description of the Method
	 *
	 *@param  anObserver  Description of Parameter
	 */
	public void deleteObserver(IObserver anObserver) {
		this.theGenericObservable.deleteObserver(anObserver);
	}

	/**
	 *  Gets the Service attribute of the ServiceContainer object
	 *
	 *@return    The Service value
	 */
	public AbstractService getService() {

		return service;
	}

	/**
	 *  Query the state of the container. This gets updated by the threads
	 *  actually submitting the job
	 *
	 *@return    The State value
	 */
	public final int getState() {

		synchronized (lock) {

			return containerState;
		}
	}

	/**
	 *  Gets the SubmissionResult attribute of the ServiceContainer object
	 *
	 *@return    The SubmissionResult value
	 */
	public Reply getSubmissionResult() {

		return reply;
	}

	/**
	 *  Gets the TaskId attribute of the ServiceContainer object
	 *
	 *@return    The TaskId value
	 */
	public int getTaskId() {

		return taskId;
	}

	/**
	 *  Gets the User attribute of the ServiceContainer object
	 *
	 *@return    The User value
	 */
	public User getUser() {

		return user;
	}

	/**
	 *  Gets the Vsite attribute of the ServiceContainer object
	 *
	 *@return    The Vsite value
	 */
	public Vsite getVsite() {

		return vsite;
	}

	/**
	 *  Gets the WrapperJob attribute of the ServiceContainer object
	 *
	 *@return    The WrapperJob value
	 */
	public AbstractJob getWrapperJob() {

		return wrapperJob;
	}

	//  implementation of the IObservable interface

	/**
	 *  Description of the Method
	 *
	 *@param  theObserved  Description of Parameter
	 *@param  changeCode   Description of Parameter
	 */
	public void notifyObservers(Object theObserved, Object changeCode) {
		theGenericObservable.notifyObservers(theObserved, changeCode);
	}

	/**
	 *  Description of the Method
	 */
	public void run() {

		try {

			Connection connection =
				ResourceManager.getConnectionManager().lookupConnection(
					vsite,
					ResourceManager.getUser(vsite));

			//now the wrapping AbstractJob
			AbstractJob job =
				new AbstractJob(
					ResourceManager.getServicePrefix()
						+ "ServiceJob"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					null,
					getVsite(),
					ResourceManager.getUser(vsite));
			job.add(this.getService());

			// Limit the lifetime of Service AJO's default 30 s
			job.setTerminationTime(
				new Date(
					System.currentTimeMillis() + connection.getAjoLifetime()));
			setWrapperJob(job);

			ConsignJob cj = JobConverter.convert(job, false, true);
			reply = null;

			int returnCode = 0;
			UnicoreResponse resp;
			reply = connection.sendMessage(cj, null);

			if (reply instanceof RetrieveOutcomeReply && (returnCode == 0)) {
			} else {
				resp = reply.getLastEntry();
				logger.log(
					Level.WARNING,
					"Error in ServiceContainer: " + resp.getReturnCode(),
					resp.getComment());
			}
		} catch (Exception e) {
			logger.warning(
				"Could not connect to host: "
					+ getVsite().getAddress()
					+ ".\nReason: "
					+ e.getMessage());
		}

		setState(ActionContainer.STATE_SUBMITTED);
		notifyObservers(this, service);
	}

	/**
	 *  Sets the Service attribute of the ServiceContainer object
	 *
	 *@param  service  The new Service value
	 */
	public void setService(AbstractService service) {
		this.service = service;
	}

	/**
	 *  Set the internal state of the container. This is used to control job
	 *  submission
	 *
	 *@param  state  The new State value
	 */
	public final void setState(int state) {

		synchronized (lock) {
			containerState = state;
		}
	}

	/**
	 *  Sets the TaskId attribute of the ServiceContainer object
	 *
	 *@param  taskId  The new TaskId value
	 */
	public void setTaskId(int taskId) {
		this.taskId = taskId;
	}

	/**
	 *  Sets the User attribute of the ServiceContainer object
	 *
	 *@param  user  The new User value
	 */
	public void setUser(User user) {
		this.user = user;
	}

	/**
	 *  Sets the Vsite attribute of the ServiceContainer object
	 *
	 *@param  vsite  The new Vsite value
	 */
	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}

	/**
	 *  Sets the WrapperJob attribute of the ServiceContainer object
	 *
	 *@param  job  The new WrapperJob value
	 */
	public void setWrapperJob(AbstractJob job) {
		this.wrapperJob = job;
	}
}
