package com.pallas.unicore.container;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;

import org.unicore.AAIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractTask;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.ExecuteTask;
import org.unicore.ajo.MakePortfolio;
import org.unicore.resources.Resource;
import org.unicore.resources.Spool;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.ExportFactory;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.FileTransfer;
import com.pallas.unicore.extensions.ImportFactory;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.resourcemanager.ResourceTray;

/**
 *  This is the superclass of all executable tasks. It knows about resources and
 *  priorities that should be used for execution.
 *
 *@author     ThomasKentemich
 *@author     Ralf Ratering
 *@created    21. August 2001 @version: $Id: TaskContainer.java,v 1.37
 *      2002/12/02 13:16:31 rrate Exp $
 */
public abstract class TaskContainer
	extends ActionContainer
	implements Serializable, Cloneable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "SPOOLEDFILES", "spooledFiles" }, {
			"SPOOLEDPORTFOLIO", "spoolPortfolio" }, {
			"IMPORTS", "fileImports" }, {
			"EXPORTS", "fileExports" }, {
			"TRANSFERS", "fileTransfers" }, {
			"FILEOPERATIONS", "fileSystemOperations" }, };

	private static ResourceBundle res =
		ResourceBundle.getBundle(
			"com.pallas.unicore.container.ResourceStrings");

	final static long serialVersionUID = -6015318006938114109L;

	/**
	 *  Description of the Field
	 */
	protected ActionGroup executeGroup;

	/**
	 *  Description of the Field
	 */
	protected ActionGroup exportGroup;

	/**
	 *  array with filenames to be exported, initially empty
	 */
	protected FileExport[] fileExports = new FileExport[0];

	/**
	 *  array with filenames to be imported, initially empty
	 */
	protected FileImport[] fileImports = new FileImport[0];

	/**
	 *  array with file operations in uspace, initially empty
	 */
	protected FileSystemOperation[] fileSystemOperations =
		new FileSystemOperation[0];

	/**
	 *  array with filenames to be transferred, initially empty
	 */
	protected FileTransfer fileTransfers[] = new FileTransfer[0];

	/**
	 *  Description of the Field
	 */
	protected ActionGroup importGroup;

	/**
	 *  vector containing filenames for file exported to nspace
	 */
	protected Vector spooledFiles;

	/**
	 *  Description of the Field
	 */
	// deprecated: just for compatibility to old jobs
	protected MakePortfolio spoolPortfolio;

	/**
	 *  declare a portfolio containing all files exported to nspace
	 */
	protected Spool spoolTask;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent JobContainer
	 */
	public TaskContainer(GroupContainer parentContainer) {
		super(parentContainer);
		//        parentContainer.addTask(this);
	}

	/**
	 *  Adds a feature to the FileExport attribute of the TaskContainer object
	 *
	 *@param  fileExport  The feature to be added to the FileExport attribute
	 */
	public void addFileExport(FileExport fileExport) {
		FileExport[] newExports = new FileExport[fileExports.length + 1];
		for (int i = 0; i < fileExports.length; i++) {
			newExports[i] = fileExports[i];
			// null out old imports for garbage collector
			fileExports[i] = null;
		}
		newExports[newExports.length - 1] = fileExport;
		fileExports = newExports;
	}

	/**
	 *  Adds a feature to the FileImport attribute of the TaskContainer object
	 *
	 *@param  fileImport  The feature to be added to the FileImport attribute
	 */
	public void addFileImport(FileImport fileImport) {
		FileImport[] newImports = new FileImport[fileImports.length + 1];
		for (int i = 0; i < fileImports.length; i++) {
			newImports[i] = fileImports[i];
			// null out old imports for garbage collector
			fileImports[i] = null;
		}
		newImports[newImports.length - 1] = fileImport;
		fileImports = newImports;
	}

	/**
	 *  Description of the Method
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());

		importGroup = new ActionGroup(getName() + "_IMPORT");
		exportGroup = new ActionGroup(getName() + "_EXPORT");

		// needed for dependency later
		AbstractTask transferTask = null;

		ImportFactory importFactory =
			new ImportFactory(this.getFileImports(), importGroup, this);
		importFactory.build();

		ExportFactory exportFactory =
			new ExportFactory(getFileExports(), exportGroup, this);
		exportFactory.build();

		buildExecuteGroup();

		// if we have an executeGroup and it has not been added somewhere else
		// (e.g. in UserContainer buildExecuteGroup), add it now.
		if (executeGroup != null && !actionGroup.isIn(executeGroup)) {
			actionGroup.add(executeGroup);
		}

		// only add import group if not empty
		if (importGroup.getActions().hasMoreElements()) {
			// check if importGroup has already been added somewhere else
			// (e.g. in UserContainer buildExecuteGroup)
			if (!actionGroup.isIn(importGroup)) {
				actionGroup.add(importGroup);
			}
			if (executeGroup != null) {
				try {
					actionGroup.addDependency(
						new Dependency(importGroup, executeGroup));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		}

		// only add import group if not empty
		if (exportGroup.getActions().hasMoreElements()) {
			actionGroup.add(exportGroup);
			if (executeGroup != null) {
				try {
					actionGroup.addDependency(
						new Dependency(executeGroup, exportGroup));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			} else if (importGroup.getActions().hasMoreElements()) {
				try {
					actionGroup.addDependency(
						new Dependency(importGroup, exportGroup));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}
			}
		}

		// copy identifier from action group to container to to have a unique id for the njs.
		syncIdentifiers();
	}

	/**
	 *  Build an ActionGroup that is put between import and export tasks in the
	 *  main ActionGroup
	 */
	protected void buildExecuteGroup() {
		// do nothing, executeGroup remains null
	}

	/**
	 *  Self check the correctness of the values relevant for this container and
	 *  add it to the ErrorSet of the super classes
	 *
	 *@return    complete ErrorSet
	 */
	public ErrorSet checkContents() {

		ErrorSet errorSet = super.checkContents();

		NamedResourceSet siteResources =
			ResourceManager.getResourceSet(getVsite());
		getResourceSet().addErrorsToSet(
			errorSet,
			id,
			getVsite(),
			siteResources);

		// check file exports
		checkExports(errorSet);

		// check file imports
		checkImports(errorSet);

		// check file operations
		checkFileOperations(errorSet);

		setErrors(errorSet);
		return errorSet;
	}

	/**
	 *@param  errorSet  Any errors will be added to this ErrorSet
	 */
	protected void checkExports(ErrorSet errorSet) {
		for (int i = 0; i < fileExports.length; i++) {
			if (!fileExports[i].isComplete()) {
				errorSet.add(
					new UError(
						id,
						res.getString("INCOMPLETE_FILE_EXPORT")
							+ fileExports[i]
							+ res.getString("AT_ROW")
							+ (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// check for export of a directory to an existing file
			if (fileExports[i].isLocal()
				&& fileExports[i].getSourceName().endsWith(
					fileExports[i].getSourceFileSeparator())) {
				String filename = fileExports[i].getDestinationName();
				File file = new File(filename);
				if (file.isFile()) {
					errorSet.add(
						new UError(
							id,
							res.getString("EXPORT_FROM_DIR_TO_FILE")
								+ fileExports[i].getSourceName()));
				}
			}
			// check, if file storage is available
			checkStorage(errorSet, fileExports[i]);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  errorSet  Description of the Parameter
	 */
	protected void checkFileOperations(ErrorSet errorSet) {
		for (int i = 0; i < fileSystemOperations.length; i++) {
			if (!fileSystemOperations[i].isComplete()) {
				errorSet.add(
					new UError(
						id,
						res.getString("INCOMPLETE_FILE_OPERATION")
							+ fileSystemOperations[i]
							+ res.getString("AT_ROW")
							+ (i + 1)));
				// skip other checks if incomplete...
				continue;
			}

			// check, if file storage is available
			checkStorage(errorSet, fileSystemOperations[i]);
		}
	}

	/**
	 *@param  errorSet  Any errors will be added to this ErrorSet
	 */
	protected void checkImports(ErrorSet errorSet) {
		Vector uspaceNames = new Vector();
		for (int i = 0; i < fileImports.length; i++) {
			if (!fileImports[i].isComplete()) {
				errorSet.add(
					new UError(
						id,
						res.getString("INCOMPLETE_FILE_IMPORT")
							+ fileImports[i]
							+ res.getString("AT_ROW")
							+ (i + 1)));
				// skip other checks if incomplete...
				continue;
			}
			// test, if local files for Nspace fileImports are available, but ignore wildcards
			if (fileImports[i].isLocal() && !fileImports[i].hasWildcards()) {
				String filename = fileImports[i].getSourceName();
				File file = new File(filename);
				if (!file.exists()) {
					errorSet.add(
						new UError(
							id,
							res.getString("LOCAL_FILE")
								+ filename
								+ res.getString("FOR_IMPORT")));
				}
			}
			// check for duplicate uspace names in imports
			String uspaceName = fileImports[i].getDestinationName();
			if (uspaceNames.contains(uspaceName)) {
				errorSet.add(
					new UError(
						id,
						res.getString("USPACE_NAME")
							+ uspaceName
							+ res.getString("FOR_IMPORT_OF")
							+ fileImports[i].getSourceName()
							+ res.getString("ALREADY_EXISTS")));
			} else {
				uspaceNames.add(fileImports[i].getDestinationName());
			}

			// check, if file storage is available
			checkStorage(errorSet, fileImports[i]);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  errorSet     Description of the Parameter
	 *@param  fileStorage  Description of the Parameter
	 */
	protected void checkStorage(ErrorSet errorSet, FileStorage fileStorage) {
		if (getVsite() == null) {
			return;
		}

		if (fileStorage.getStorageItem().equals(FileStorage.NSPACE_STRING)) {
			return;
		}

		if (fileStorage.getStorageItem().equals("")) {
			errorSet.add(
				new UError(
					id,
					res.getString("STORAGE")
						+ fileStorage.getStorageItem()
						+ res.getString("NOT_AVAILABLE")
						+ getVsite()
						+ "."));
		}
		NamedResourceSet siteResources =
			ResourceManager.getResourceSet(getVsite());
		if (!siteResources.contains((Resource) fileStorage.getStorageItem())) {
			errorSet.add(
				new UError(
					id,
					res.getString("STORAGE")
						+ fileStorage.getStorageItem()
						+ res.getString("NOT_AVAILABLE")
						+ getVsite()
						+ "."));
		}
	}

	/**
	 *  Clonable interface.
	 *
	 *@return    TaskContainer clone
	 */
	public Object clone() {
		TaskContainer clone = (TaskContainer) super.clone();

		clone.setFileExports((FileExport[]) cloneFileExports());
		clone.setFileImports((FileImport[]) cloneFileImports());
		clone.setFileSystemOperations(
			(FileSystemOperation[]) cloneFileSystemOperations());
		clone.setFileTransfers((FileTransfer[]) cloneFileTransfers());

		return clone;
	}

	/**
	 *@return    file export clones
	 */
	private FileExport[] cloneFileExports() {
		FileExport[] clones = new FileExport[fileExports.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileExport) fileExports[i].clone();
		}
		return clones;
	}

	/**
	 *@return    file import clones
	 */
	private FileImport[] cloneFileImports() {
		FileImport[] clones = new FileImport[fileImports.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileImport) fileImports[i].clone();
		}
		return clones;
	}

	/**
	 *@return    file system ops clones
	 */
	private FileSystemOperation[] cloneFileSystemOperations() {
		FileSystemOperation[] clones =
			new FileSystemOperation[fileSystemOperations.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileSystemOperation) fileSystemOperations[i].clone();
		}
		return clones;
	}

	/**
	 *@return    file transfer clones
	 */
	private FileTransfer[] cloneFileTransfers() {
		FileTransfer[] clones = new FileTransfer[fileTransfers.length];
		for (int i = 0; i < clones.length; i++) {
			clones[i] = (FileTransfer) fileTransfers[i].clone();
		}
		return clones;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  group  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public ExecuteTask findExecuteTask(ActionGroup group) {
		// try to find execute task in current group
		Enumeration actions = group.getActions();
		while (actions.hasMoreElements()) {
			AbstractAction action = (AbstractAction) actions.nextElement();
			if (action instanceof ExecuteTask) {
				return (ExecuteTask) action;
			}
		}
		// couldn't find execute task, search sub groups
		actions = group.getActions();
		while (actions.hasMoreElements()) {
			AbstractAction action = (AbstractAction) actions.nextElement();
			if (action instanceof ActionGroup) {
				return findExecuteTask((ActionGroup) action);
			}
		}
		// no success
		return null;
	}

	/**
	 *  method for returning exports that will not appear in the gui can be
	 *  overridden in sub-classes
	 *
	 *@return    The additionalExports value
	 */
	public FileExport[] getAdditionalExports() {
		return new FileExport[0];
	}

	/**
	 *  method for returning imports not related to data files e.g. executables
	 *  in the UserTask; can be overridden in sub-classes
	 *
	 *@return    The AdditionalImports value
	 */
	public FileImport[] getAdditionalImports() {
		return new FileImport[0];
	}

	/**
	 *  Gets the executeGroup attribute of the TaskContainer object
	 *
	 *@return    The executeGroup value
	 */
	public ActionGroup getExecuteGroup() {
		return executeGroup;
	}

	/**
	 *  Gets the AJO ExecuteTask that is responsible for return code of this
	 *  Container. This method will return the first ExcuteTask from the Action
	 *  graph. If different ExecuteTasks are required from sub classes, this
	 *  method should be overwritten.
	 *
	 *@return    The executeTask value
	 */
	public ExecuteTask getExecuteTask() {
		// if execute group has not been built yet, we do not have an execute task
		// cannot build exeute group here, because it has to be re-build
		// with different ids at submission time
		if (executeGroup == null) {
			return null;
		}
		return findExecuteTask(executeGroup);
	}

	/**
	 *  Gets the FileExports attribute of the TaskContainer object
	 *
	 *@return    The FileExports value
	 */
	public FileExport[] getFileExports() {
		return fileExports;
	}

	/**
	 *  Gets the FileImports attribute of the TaskContainer object
	 *
	 *@return    The FileImports value
	 */
	public FileImport[] getFileImports() {
		return fileImports;
	}

	/**
	 *  Gets the FileImports attribute of the TaskContainer object
	 *
	 *@return    The FileImports value
	 */
	public FileImport[] getFileImportsFromLocal() {

		// loop over file operations in container and identify imports from nspace
		FileImport[] additionalImports = getAdditionalImports();
		FileImport[] normalImports = getFileImports();

		int normalLength = 0;
		if (normalImports != null) {
			for (int i = 0; i < normalImports.length; i++) {
				if (normalImports[i].isLocal()) {
					normalLength++;
				}
			}
		}

		int additionalLength = 0;
		if (additionalImports != null) {
			additionalLength = additionalImports.length;
		}

		FileImport[] localImports =
			new FileImport[normalLength + additionalLength];

		if (normalLength > 0) {
			int j = 0;
			for (int i = 0; i < normalImports.length; i++) {
				if (normalImports[i].isLocal()) {
					localImports[j] = normalImports[i];
					j++;
				}
			}
		}

		if (additionalLength > 0) {
			System.arraycopy(
				additionalImports,
				0,
				localImports,
				normalLength,
				additionalLength);
		}

		return localImports;
	}

	/**
	 *  Gets the FileSystemOperations attribute of the TaskContainer object
	 *
	 *@return    The FileSystemOperations value
	 */
	public FileSystemOperation[] getFileSystemOperations() {
		return fileSystemOperations;
	}

	/**
	 *  Gets the FileTransfers attribute of the TaskContainer object
	 *
	 *@return    The FileTransfers value
	 */
	public FileTransfer[] getFileTransfers() {
		return fileTransfers;
	}

	/**
	 *  Gets the ResourceSet attribute of the TaskContainer object
	 *
	 *@return    The ResourceSet value
	 */
	public NamedResourceSet getResourceSet() {
		NamedResourceSet resourceSet =
			(NamedResourceSet) getParentJobContainer().getResourceTray().get(
				getIdentifier());
		return resourceSet;
	}

	/**
	 *  Gets the SpooledFiles attribute of the TaskContainer object
	 *
	 *@return    The SpooledFiles value
	 */
	public Vector getSpooledFiles() {
		return this.spooledFiles;
	}

	/**
	 *@return        The SpoolPortfolio value
	 *@deprecated    Gets the SpoolPortfolio attribute of the TaskContainer
	 *      object
	 */
	public MakePortfolio getSpoolPortfolio() {
		return spoolPortfolio;
	}

	/**
	 *@return        The SpoolTask value
	 *@deprecated    Gets the SpoolTask attribute of the TaskContainer object
	 */
	public Spool getSpoolTask() {
		return spoolTask;
	}

	/**
	 *  Get the Vsite we are running on from the parent container
	 *
	 *@return    The Vsite value
	 */
	public Vsite getVsite() {
		if (getParentJobContainer() == null) {
			return null;
		} else {
			return getParentJobContainer().getVsite();
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public boolean hasExecuteTask() {
		if (executeGroup == null) {
			buildExecuteGroup();
		}
		if (executeGroup == null) {
			return false;
		}
		return true;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public boolean hasNspaceExports() {
		for (int i = 0; i < fileExports.length; i++) {
			FileExport fileExport = (FileExport) fileExports[i];
			if (fileExport.isLocal()) {
				return true;
			}
		}
		// check additional hidden exports as well
		FileExport[] additionalExports = getAdditionalExports();
		for (int i = 0; i < additionalExports.length; i++) {
			FileExport fileExport = (FileExport) additionalExports[i];
			if (fileExport.isLocal()) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Check if container contains any import tasks from nspace. Method is
	 *  needed by JobContainer, where import files from nspace are collected and
	 *  streamed after the AJO.
	 *
	 *@return    true, if container has nspace imports
	 */
	public boolean hasNspaceImports() {
		if (getAdditionalImports().length > 0) {
			return true;
		}
		FileImport[] fileImports = getFileImports();
		for (int i = 0; i < fileImports.length; i++) {
			if (fileImports[i].isLocal()) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  stream                      Description of the Parameter
	 *@exception  IOException             Description of the Exception
	 *@exception  ClassNotFoundException  Description of the Exception
	 */
	private void readObject(ObjectInputStream stream)
		throws IOException, ClassNotFoundException {
		boolean DEBUG = false;

		// all containers from version 4 should be compatible
		if (getContainerVersion().startsWith("4")) {
			stream.defaultReadObject();
			return;
		}

		logger.info(
			"Reading TaskContainer "
				+ id
				+ " from stream (potentially old format)");
		ObjectInputStream.GetField getField = null;
		try {
			getField = stream.readFields();

			ObjectStreamField streamFields[] =
				getField.getObjectStreamClass().getFields();
			for (int i = 0; i < streamFields.length; i++) {
				String name = streamFields[i].getName();
				Object value = getField.get(name, null);

				Field field = null;
				try {
					field = TaskContainer.class.getDeclaredField(name);

					field.set(this, value);

					logger.info(
						"Setting field " + field + " to value " + value);
				} catch (NoSuchFieldException nsfe) {
					// Class clazz = (streamFields[i]).getType(); // returns useless java.lang.Object class (JVM error?)
					if (name.equals("resourceSet")
						&& (value instanceof ResourceSet)) {
						logger.warning(
							"Field "
								+ name
								+ " not found in TaskContainer.class (OLD JOB FORMAT)");
						AAIdentifier id = getIdentifier();
						String taskName = getName();
						if (taskName == null) {
							taskName = id.getName();
						}
						ResourceSet clone = ((ResourceSet) value).cloneSet();
						NamedResourceSet resources =
							new NamedResourceSet(
								"RS_"
									+ taskName
									+ "_"
									+ id.getValue()
									+ "[generated]",
								clone);
						JobContainer job = getParentJobContainer();
						//resources.setVsite(job.getVsite());
						job.getResourceTray().put(id, resources);
					} else {
						logger.log(
							Level.SEVERE,
							"Field "
								+ name
								+ " not found in TaskContainer.class (OLD JOB FORMAT)",
							nsfe);
					}
				} catch (Exception e) {
					logger.log(
						Level.SEVERE,
						"Field "
							+ field.getDeclaringClass()
							+ "."
							+ field.getName(),
						e);
				}
			}
			logger.info(
				"Finished reading TaskContainer " + id + " from stream");
		} catch (Exception e) {
			logger.log(
				Level.SEVERE,
				"Could not read TaskContainer " + id + " from stream",
				e);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  fileExport  Description of Parameter
	 */
	public void removeFileExport(FileExport fileExport) {
		Vector newExports = new Vector();
		for (int i = 0; i < fileExports.length; i++) {
			if (!(fileExports[i].equals(fileExport))) {
				newExports.add(fileExports[i]);
				fileExports[i] = null;
			}
		}
		fileExports = new FileExport[newExports.size()];
		for (int i = 0; i < fileExports.length; i++) {
			fileExports[i] = (FileExport) newExports.elementAt(i);
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  fileImport  Description of Parameter
	 */
	public void removeFileImport(FileImport fileImport) {
		Vector newImports = new Vector();
		for (int i = 0; i < fileImports.length; i++) {
			if (!(fileImports[i].equals(fileImport))) {
				newImports.add(fileImports[i]);
				fileImports[i] = null;
			}
		}
		fileImports = new FileImport[newImports.size()];
		for (int i = 0; i < fileImports.length; i++) {
			fileImports[i] = (FileImport) newImports.elementAt(i);
		}
	}

	/**
	 *  Sets the FileExports attribute of the TaskContainer object
	 *
	 *@param  fileExports  The new FileExports value
	 */
	public void setFileExports(FileExport[] fileExports) {
		this.fileExports = fileExports;
	}

	/**
	 *  Sets the FileImports attribute of the TaskContainer object
	 *
	 *@param  fileImports  The new FileImports value
	 */
	public void setFileImports(FileImport[] fileImports) {
		this.fileImports = fileImports;
	}

	/**
	 *  Sets the FileSystemOperations attribute of the TaskContainer object
	 *
	 *@param  fileSystemOperations  The new FileSystemOperations value
	 */
	public void setFileSystemOperations(FileSystemOperation[] fileSystemOperations) {
		this.fileSystemOperations = fileSystemOperations;
	}

	/**
	 *  Sets the FileTransfers attribute of the TaskContainer object
	 *
	 *@param  fileTransfers  The new FileTransfers value
	 */
	public void setFileTransfers(FileTransfer[] fileTransfers) {
		this.fileTransfers = fileTransfers;
	}

	/**
	 *  Sets the SpooledFiles attribute of the TaskContainer object
	 *
	 *@param  spooledFiles  The new SpooledFiles value
	 */
	public void setSpooledFiles(Vector spooledFiles) {
		this.spooledFiles = spooledFiles;
	}

	/**
	 *@param  spoolPortfolio  The new SpoolPortfolio value
	 *@deprecated             Sets the SpoolPortfolio attribute of the
	 *      TaskContainer object
	 */
	public void setSpoolPortfolio(MakePortfolio spoolPortfolio) {
		this.spoolPortfolio = spoolPortfolio;
	}

	/**
	 *@param  spoolTask  The new SpoolTask value
	 *@deprecated        Sets the SpoolTask attribute of the TaskContainer
	 *      object
	 */
	public void setSpoolTask(Spool spoolTask) {
		this.spoolTask = spoolTask;
	}

	/**
	 *  Copy identifier from action group to container to have a unique id for
	 *  the njs. Here also the resource tray of the job container is updated,
	 *  because it uses the id as hash key.
	 */
	protected void syncIdentifiers() {
		AAIdentifier newId = actionGroup.getId();
		AAIdentifier oldId = getIdentifier();

		// update resource tray
		ResourceTray tray = getParentJobContainer().getResourceTray();
		NamedResourceSet resource = (NamedResourceSet) tray.get(oldId);
		// resource tray will return site default resource instead of null, so check this!
		if (resource != null && !resource.isSiteDefault()) {
			tray.put(newId, resource);
			tray.remove(oldId);
		}

		// update id
		setIdentifier(newId);
	}
}
