package com.pallas.unicore.container;

import java.util.logging.Level;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.ChangePermissions;
import org.unicore.ajo.DeletePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.ExecuteTask;
import org.unicore.ajo.ImportTask;
import org.unicore.ajo.MakePortfolio;
import org.unicore.ajo.Portfolio;
import org.unicore.ajo.UserTask;
import org.unicore.resources.Resource;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Container needed to import and execute a command on a vsite.
 *
 *@author     Thomas Kentemich
 *@author     Ralf Ratering
 *@created    21. August 2001
 *@version    $Id: UserContainer.java,v 1.2 2003/06/30 06:57:17 rrate Exp $
 */
public abstract class UserContainer
	extends ExecuteContainer
	implements Cloneable {

	// field definitions for storing the container in XML format
	private static String[][] persistentXMLFields =
		{ { "COMMAND", "commandImport" }, {
			"SOFTWARERESOURCE", "softwareResource" }, {
			"EXECUTEDEBUG", "debug" }, {
			"EXECUTEPROFILING", "profile" }, {
			"EXECUTETIMING", "measureTime" }
	};

	final static long serialVersionUID = 777192701154458809L;

	/**
	 *  Description of the Field
	 */
	protected FileImport commandImport;

	private boolean debug, profile, measureTime;

	/**
	 *  this one should get type SoftwareResource some day
	 */
	protected Resource softwareResource;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  Description of Parameter
	 */
	public UserContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Description of the Method
	 */
	protected void buildExecuteGroup() {
		executeGroup = new ActionGroup(getName() + "_EXECUTION");

		ResourceSet taskResourceSet = getResourceSet().getResourceSetClone();

		ResourceSet uspaceResourceSet = new ResourceSet();
		NamedResourceSet siteResourceSet =
			ResourceManager.getResourceSet(getVsite());
		USpace uspace = siteResourceSet.getUspace();
		if (uspace != null) {
			uspaceResourceSet.add(uspace);
		}

		ExecuteTask userTask = null;

		FileImport commandImport = getCommand();
		if (commandImport != null) {

			Portfolio executablePortfolio = null;

			if (!(commandImport.isLocal())) {
				String[] commArray = { commandImport.getSourceName()};
				Storage storage = (Storage) (commandImport.getStorageItem());
				if (!(storage instanceof USpace)) {
					ResourceSet commandResource = new ResourceSet(storage);
					// set overwrite in case of loops
					boolean overwrite = true;
					// import files into uspace
					ImportTask importTask =
						new ImportTask(
							this.getName()
								+ "_IT_EXEC_"
								+ storage
								+ ResourceManager.getNextObjectIdentifier(),
							null,
							commandResource,
							commArray,
							overwrite);

					// put import task into actionGroup
					importGroup.add(importTask);

					// Executable imported from Xspace
					executablePortfolio = importTask.getPortfolio();
				}
			}

			// destination might include sub-directory in Uspace,
			// therefore the full destination name has to be taken (and NOT the basename)
			String lastEntry = commandImport.getDestinationName();

			// set execute permission for command, write permission is needed in case of loops
			ChangePermissions changePermission =
				new ChangePermissions(
					getName()
						+ "_CP_USER_"
						+ ResourceManager.getNextObjectIdentifier(),
					null,
					uspaceResourceSet,
					lastEntry,
					true,
					true,
					true);

			// make portfolio if there is none from the commandImport
			MakePortfolio makePortfolio = null;
			if (executablePortfolio == null) {
				String[] files = { lastEntry };
				makePortfolio =
					new MakePortfolio(
						getName()
							+ "_MP_USER_"
							+ ResourceManager.getNextObjectIdentifier(),
						null,
						uspaceResourceSet,
						files);

				executablePortfolio = makePortfolio.getPortfolio();
			}

			userTask =
				new UserTask(
					getName(),
					null,
					taskResourceSet,
					getEnv(),
					getCommandLine(),
					null,
					getRedirectStdout(),
					getRedirectStderr(),
					isVerboseOn(),
					isVersionOn(),
					executablePortfolio,
					getMeasureTime(),
					getDebug(),
					getProfile());

			// delete portfolio in case of loops
			DeletePortfolio deletePortfolio =
				new DeletePortfolio(
					getName()
						+ "_DELETE_MP_USER_"
						+ ResourceManager.getNextObjectIdentifier());
			deletePortfolio.setPortfolio(executablePortfolio.getId());
			// set ignore failure in case of no or first iteration
			deletePortfolio.setPropagateSuccessfulIfNotSuccessful(true);
			// add delete portfolio to main action group...
			actionGroup.add(deletePortfolio);
			// ... and execute it before any (executable) imports
			try {
				// we may not have neccessarily imports here
				if (!importGroup.getActions().hasMoreElements()) {
					// explicitly add importGroup here instead of in the TaskContainer,
					// to prevent a DependencyException
					actionGroup.add(importGroup);
					actionGroup.addDependency(
						new Dependency(deletePortfolio, importGroup));
				}
				// if we don't have imports, set dependency to executeGroup
				// because a portfolio may have been created from MakePortfolio
				else {
					actionGroup.add(executeGroup);
					actionGroup.addDependency(
						new Dependency(deletePortfolio, executeGroup));
				}
			} catch (ActionGroup.InvalidDependencyException e) {
				logger.log(Level.WARNING, "Could not add dependency.", e);
			}

			// put tasks into execute group
			executeGroup.add(changePermission);
			executeGroup.add(userTask);

			try {
				executeGroup.addDependency(
					new Dependency(changePermission, userTask));
				if (makePortfolio != null) {
					executeGroup.add(makePortfolio);
					executeGroup.addDependency(
						new Dependency(makePortfolio, userTask));
				}
			} catch (ActionGroup.InvalidDependencyException ex) {
				logger.log(Level.SEVERE, "Could not add dependency.", ex);
			}
		} else {
			taskResourceSet.add(getPreinstalledSoftware());
			userTask =
				new ExecuteTask(
					getName(),
					null,
					taskResourceSet,
					getEnv(),
					getCommandLine(),
					null,
					getRedirectStdout(),
					getRedirectStderr(),
					isVerboseOn(),
					isVersionOn());

			executeGroup.add(userTask);
		}
		try {
			if (getRedirectStdout()) {
				userTask.setStdoutFileName(this.getStdout());
				// delete portfolio in case of loops
				DeletePortfolio delStdoutPortfolio =
					new DeletePortfolio(
						getName()
							+ "_DELETE_STDOUT_"
							+ ResourceManager.getNextObjectIdentifier());
				delStdoutPortfolio.setPortfolio(userTask.getStdout().getId());

				executeGroup.add(delStdoutPortfolio);
				executeGroup.addDependency(
					new Dependency(userTask, delStdoutPortfolio));
			}

			if (getRedirectStderr()) {
				userTask.setStderrFileName(this.getStderr());
				// delete portfolio in case of loops
				DeletePortfolio delStderrPortfolio =
					new DeletePortfolio(
						getName()
							+ "_DELETE_STDERR_"
							+ ResourceManager.getNextObjectIdentifier());
				delStderrPortfolio.setPortfolio(userTask.getStderr().getId());

				executeGroup.add(delStderrPortfolio);
				executeGroup.addDependency(
					new Dependency(userTask, delStderrPortfolio));
			}

			if (getRedirectStdin()) {
				String[] stdinfiles = { this.getStdin()};
				MakePortfolio stdinPortfolio =
					new MakePortfolio(
						getName()
							+ "_MP_STDIN_"
							+ ResourceManager.getNextObjectIdentifier(),
						null,
						uspaceResourceSet,
						stdinfiles);

				userTask.setStdin(stdinPortfolio.getPortfolio());

				// delete portfolio in case of loops
				DeletePortfolio delStdinPortfolio =
					new DeletePortfolio(
						getName()
							+ "_DELETE_MP_STDIN_"
							+ ResourceManager.getNextObjectIdentifier());
				delStdinPortfolio.setPortfolio(
					stdinPortfolio.getPortfolio().getId());

				executeGroup.add(stdinPortfolio);
				executeGroup.add(delStdinPortfolio);

				executeGroup.addDependency(
					new Dependency(stdinPortfolio, userTask));
				executeGroup.addDependency(
					new Dependency(userTask, delStdinPortfolio));
			}
		} catch (ActionGroup.InvalidDependencyException ex) {
			logger.log(Level.SEVERE, "Could not add dependency.", ex);
		}

	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public Object clone() {
		try {
			UserContainer clone = (UserContainer) super.clone();
			if (commandImport != null) {
				clone.setCommand((FileImport) commandImport.clone());
			}
			if (softwareResource != null) {
				clone.setPreinstalledSoftware(
					(Resource) softwareResource.clone());
			}
			return clone;
		} catch (CloneNotSupportedException e) {
			logger.log(
				Level.WARNING,
				"Cloning conatiner " + this +" failed.",
				e);
			return null;
		}
	}

	/**
	 *  Gets the AdditionalImports attribute of the UserContainer object
	 *
	 *@return    The AdditionalImports value
	 */
	public FileImport[] getAdditionalImports() {
		//  method for returning imports not related to data files
		// e.g. executables in the UserTask; can be overridden in sub-classes
		if (commandImport != null) {
			if (commandImport.isLocal()) {
				FileImport[] add = { commandImport };
				return add;
			}
		}
		return new FileImport[0];
	}

	/**
	 *  Gets the Command attribute of the UserContainer object
	 *
	 *@return    The Command value
	 */
	public FileImport getCommand() {
		return commandImport;
	}

	/**
	 *  Gets the Debug attribute of the UserContainer object
	 *
	 *@return    The Debug value
	 */
	public boolean getDebug() {
		return debug;
	}

	/**
	 *  Gets the MeasureTime attribute of the UserContainer object
	 *
	 *@return    The MeasureTime value
	 */
	public boolean getMeasureTime() {
		return measureTime;
	}

	/**
	 *  Gets the PreinstalledSoftware attribute of the UserContainer object
	 *
	 *@return    The PreinstalledSoftware value
	 */
	public Resource getPreinstalledSoftware() {
		return softwareResource;
	}

	/**
	 *  Gets the Profile attribute of the UserContainer object
	 *
	 *@return    The Profile value
	 */
	public boolean getProfile() {
		return profile;
	}

	/**
	 *  Gets the RedirectStdin attribute of the UserContainer object
	 *
	 *@return    The RedirectStdin value
	 */
	public boolean getRedirectStdin() {
		return redirectStdin;
	}

	/**
	 *  Gets the Stdin attribute of the UserContainer object
	 *
	 *@return    The Stdin value
	 */
	public String getStdin() {
		return stdinPath;
	}

	/**
	 *  Sets the Command attribute of the UserContainer object
	 *
	 *@param  command  The new Command value
	 */
	public void setCommand(FileImport command) {
		this.commandImport = command;
	}

	/**
	 *  Sets the Debug attribute of the UserContainer object
	 *
	 *@param  what  The new Debug value
	 */
	public void setDebug(boolean what) {
		this.debug = what;
	}

	/**
	 *  Sets the MeasureTime attribute of the UserContainer object
	 *
	 *@param  what  The new MeasureTime value
	 */
	public void setMeasureTime(boolean what) {
		this.measureTime = what;
	}

	/**
	 *  Sets the PreinstalledSoftware attribute of the UserContainer object
	 *
	 *@param  softwareResource  The new PreinstalledSoftware value
	 */
	public void setPreinstalledSoftware(Resource softwareResource) {
		this.softwareResource = softwareResource;
	}

	/**
	 *  Sets the Profile attribute of the UserContainer object
	 *
	 *@param  what  The new Profile value
	 */
	public void setProfile(boolean what) {
		this.profile = what;
	}

	/**
	 *  Sets the RedirectStdin attribute of the UserContainer object
	 *
	 *@param  what  The new RedirectStdin value
	 */
	public void setRedirectStdin(boolean what) {
		this.redirectStdin = what;
	}

	/**
	 *  Sets the Stdin attribute of the UserContainer object
	 *
	 *@param  stdin  The new Stdin value
	 */
	public void setStdin(String stdin) {
		this.stdinPath = stdin;
	}

}
