package com.pallas.unicore.extensions;
import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.ajo.Portfolio;
import org.unicore.ajo.PutPortfolio;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.requests.GetSpooledFiles;

/**
 *  Class representing a file export to a specified location from uspace.
 *
 *@author     Ralf Ratering
 *@created    13. September 2001
 *@version    $Id: FileExport.java,v 1.3 2003/06/30 06:58:22 rrate Exp $
 */
public class FileExport extends FileStorage {

	/**
	 *  In case the overwrite flag of the FileExport is set, the user can choose the target files
	 *  which he wants to be overwritten.
	 */
	public final static int CHOOSE_OVERWRITTEN_FILES = 1;

	private static DataFlavor flavor;

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");
	public final static int NOT_CHOOSE_OVERWRITTEN_FILES = 2;

	final static long serialVersionUID = -8215834295484608115L;

	/**
	 *  Gets the flavor attribute of the FileExport class
	 *
	 *@return    The flavor value
	 */
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor =
					new DataFlavor(
						Class.forName(
							"com.pallas.unicore.extensions.FileExport"),
						"FileExport");
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error getting DataFlavore for com.pallas.unicore.extensions.FileExport",
					e);
			}
		}
		return flavor;
	}

	/**
	 *  Reference to the originating container
	 */
	private ActionContainer container;

	private transient String resolvedPath;

	/**
	 *  declare a portfolio containing all files exported to nspace
	 */
	private Portfolio spoolPortfolio;

	/**
	 *  the reference to the SpoolTask for Nspace exports
	 */
	private PutPortfolio spoolTask;

	private Vector targetFiles = null;

	private int targetFilesFlag = NOT_CHOOSE_OVERWRITTEN_FILES;
	private transient GetSpooledFiles unspoolTask;

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing destination location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 *@param  container        Description of the Parameter
	 */
	public FileExport(
		ActionContainer container,
		Object storageItem,
		String sourceName,
		String destinationName) {
		super(storageItem, sourceName, destinationName);
		this.container = container;

	}

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing destination location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 *@param  overwrite        overwrite policy
	 *@param  container        Description of the Parameter
	 */
	public FileExport(
		ActionContainer container,
		Object storageItem,
		String sourceName,
		String destinationName,
		boolean overwrite) {
		this(container, storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
	}

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing destination location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 *@param  overwrite        overwrite policy
	 *@param  isAscii          binary or ASCII format
	 *@param  container        Description of the Parameter
	 */
	public FileExport(
		ActionContainer container,
		Object storageItem,
		String sourceName,
		String destinationName,
		boolean overwrite,
		boolean isAscii) {
		this(container, storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
		this.isAscii = isAscii;
	}

	/**
	 *  Gets the ActionContainer originationg this Export
	 *
	 *@return    The ActionContainer
	 */
	public ActionContainer getContainer() {
		return this.container;
	}

	/**
	 *  Get separator of destination file system (Local)
	 *
	 *@return    separator string
	 */
	public String getDestinationFileSeparator() {
		if (this.isLocal()) {
			return File.separator;
		}
		// remote file systems are assumed to be UNIX
		else {
			return "/";
		}
	}

	/**
	 *  Get separator of source file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getSourceFileSeparator() {
		return "/";
	}

	/**
	 *  Gets the SpoolPortfolio attribute of the FileExport object
	 *
	 *@return    The SpoolPortfolio value
	 */
	public Portfolio getSpoolPortfolio() {
		return this.spoolPortfolio;
	}

	/**
	 *  Gets the SpoolTask attribute of the FileExport object
	 *
	 *@return    The SpoolTask value
	 */
	public PutPortfolio getSpoolTask() {
		return this.spoolTask;
	}

	/**
	 *  Gets the subClassFlavor attribute of the FileExport object
	 *
	 *@return    The subClassFlavor value
	 */
	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	/**
	 *  Gets the targetFiles attribute of the FileExport object
	 *
	 *@return    The targetFiles value
	 */
	public Vector getTargetFiles() {
		if (targetFiles == null) {
			return null;
		}
		return this.targetFiles;
	}

	/**
	 *  Gets the targetFilesFlag attribute of the FileExport object
	 *
	 *@return    The targetFilesFlag value
	 */
	public int getTargetFilesFlag() {
		return this.targetFilesFlag;
	}

	/**
	 *  Gets the UnSpool attribute of the FileExport object
	 *
	 *@return    The UnSpool value
	 */
	public GetSpooledFiles getUnSpool() {
		return unspoolTask;
	}

	/**
	 *  Description of the Method
	 */
	public void resetTargetFilesFlag() {
		this.targetFilesFlag = NOT_CHOOSE_OVERWRITTEN_FILES;
	}

	/**
	 *  Sets the ActionContainer originating this Export
	 *
	 *@param  container  The new container value
	 */
	public void setContainer(ActionContainer container) {
		this.container = container;
	}

	/**
	 *  Sets the SpoolPortfolio attribute of the FileExport object 7 *@param
	 *  spoolPortfolio The new SpoolPortfolio value
	 *
	 *@param  spoolPortfolio  The new spoolPortfolio value
	 */
	public void setSpoolPortfolio(Portfolio spoolPortfolio) {
		this.spoolPortfolio = spoolPortfolio;
	}

	/**
	 *  Sets the SpoolTask attribute of the FileExport object
	 *
	 *@param  spoolTask  The new SpoolTask value
	 */
	public void setSpoolTask(PutPortfolio spoolTask) {
		this.spoolTask = spoolTask;
	}

	/**
	 *  Sets the targetFiles attribute of the FileExport object
	 *
	 *@param  targetFiles  The new targetFiles value
	 */
	public void setTargetFiles(Vector targetFiles) {
		this.targetFiles = targetFiles;
	}

	/**
	 *  Sets the targetFilesFlag attribute of the FileExport object
	 *
	 *@param  flag  The new targetFilesFlag value
	 */
	public void setTargetFilesFlag(int flag) {
		this.targetFilesFlag = flag;
	}

	/**
	 *  Sets the UnSpool attribute of the FileExport object
	 *
	 *@param  gsf  The new UnSpool value
	 */
	public void setUnSpool(GetSpooledFiles gsf) {
		unspoolTask = gsf;
	}

	/**
	 *@return    String describing operation
	 */
	public String toString() {
		return sourceName + " -> " + storageItem + ": " + destinationName;
	}
}
