package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.ajo.DeclarePortfolio;

/**
 *  Class representing a file import from a specified location into uspace.
 *
 *@author     Ralf Ratering
 *@created    12. September 2001
 *@version    $Id: FileImport.java,v 1.3 2003/06/30 06:58:22 rrate Exp $
 */
public class FileImport extends FileStorage {

	private static DataFlavor flavor;

	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");

	final static long serialVersionUID = -1883378006618693063L;

	/**
	 *  Gets the flavor attribute of the FileImport class
	 *
	 *@return    The flavor value
	 */
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor =
					new DataFlavor(
						Class.forName(
							"com.pallas.unicore.extensions.FileImport"),
						"FileImport");
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error getting DataFlavore for com.pallas.unicore.extensions.FileImport",
					e);
			}

		}
		return flavor;
	}
	/**
	 *  declare a portfolio containing the file/directory imported from nspace
	 */
	private transient DeclarePortfolio nspacePortfolio;

	/**
	 *  remember actual destination if import is done from Nspace via hidden directory
	 */
	private String realDestination = null;

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing source location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 */
	public FileImport(
		Object storageItem,
		String sourceName,
		String destinationName) {
		super(storageItem, sourceName, destinationName);
		// mask any blanks in destination name
		this.destinationName = maskBlanks(destinationName);
	}

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing destination location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 *@param  overwrite        overwrite policy
	 */
	public FileImport(
		Object storageItem,
		String sourceName,
		String destinationName,
		boolean overwrite) {
		this(storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
	}

	/**
	 *  Constructor
	 *
	 *@param  storageItem      object describing destination location
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 *@param  overwrite        overwrite policy
	 *@param  isAscii          binary or ASCII format
	 */
	public FileImport(
		Object storageItem,
		String sourceName,
		String destinationName,
		boolean overwrite,
		boolean isAscii) {
		this(storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
		this.isAscii = isAscii;
	}

	/**
	 *  Get separator of destination file system (always UNIX)
	 *
	 *@return    separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 *@return    DeclarePortfolio defining a portfolio needed to import files
	 *      from nspace
	 */
	public DeclarePortfolio getNspacePortfolio() {
		return nspacePortfolio;
	}

	/**
	 *  Get separator of source file system (Local)
	 *
	 *@return    separator string
	 */
	public String getSourceFileSeparator() {
		if (this.isLocal()) {
			return File.separator;
		}
		// remote file systems are assumed to be UNIX
		else {
			return "/";
		}
	}

	/**
	 *  Gets the subClassFlavor attribute of the FileImport object
	 *
	 *@return    The subClassFlavor value
	 */
	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	/**
	 *  Mask blanks in Windows filenames with backslashes to be compatible with
	 *  the UNIX Uspace
	 *
	 *@param  filename  Description of Parameter
	 *@return           Description of the Returned Value
	 */
	private String maskBlanks(String filename) {
		/*
		 *  if (filename.indexOf(" ")>=0
		 *  && !(filename.startsWith("\"") && !filename.endsWith("\""))) {
		 *  return "\"" + filename + "\"";
		 *  }
		 *  else {
		 *  return filename;
		 *  }
		 */
		if (filename == null) {
			return null;
		}
		String result = "";
		StringTokenizer tokenizer = new StringTokenizer(filename, " ");
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			result = result + token;
			if (token.endsWith("\\")) {
				result += " ";
				continue;
			}

			int index = filename.indexOf(token) + token.length();
			String remainder = filename.substring(index);
			while (!token.endsWith("\\") && remainder.startsWith(" ")) {
				result += "\\ ";
				index++;
				remainder = filename.substring(index);
			}

		}

		if (result.endsWith("\\ ")) {
			result = result.substring(0, result.length() - 2);
		}

		return result;
	}

	/**
	 *  Sets the DestinationName attribute of the FileImport object
	 *
	 *@param  destinationName  The new DestinationName value
	 */
	public void setDestinationName(String destinationName) {
		maskBlanks(destinationName);
		super.setDestinationName(destinationName);
	}

	/**
	 *  Sets the NspacePortfolio attribute of the FileImport object
	 *
	 *@param  nspacePortfolio  The new NspacePortfolio value
	 */
	public void setNspacePortfolio(DeclarePortfolio nspacePortfolio) {
		this.nspacePortfolio = nspacePortfolio;
	}

	/**
	 *  Sets the DestinationName attribute of the FileImport object
	 *
	 *@param  destinationName  The new DestinationName value
	 */
	protected void setRealDestination(String realDestination) {
		this.realDestination = realDestination;
	}

	/**
	 *@return    String describing operation
	 */
	public String toString() {
		if (realDestination != null) {
			return storageItem + ": " + sourceName + " -> " + realDestination;
		}
		return storageItem + ": " + sourceName + " -> " + destinationName;
	}

}
