package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.ajo.AbstractAction;

/**
 *  Abstract class representing a file operation that takes a source file as
 *  input and may produce a destination file as output
 *
 *@author     Ralf Ratering
 *@created    29. August 2001
 *@version    $Id: FileOperation.java,v 1.2 2003/06/30 06:58:22 rrate Exp $
 */
public abstract class FileOperation
	implements Serializable, Cloneable, Transferable {
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");

	final static long serialVersionUID = -6307530181526319154L;

	/**
	 *  Keep track of ajo tasks that are represented by this object
	 */
	protected AbstractAction abstractAction;

	/**
	 *  each operation has a source and eventually a destination
	 */
	protected String sourceName, destinationName;

	/**
	 *  byte size of source file
	 */
	protected long totalBytes, transferredBytes;

	/**
	 *  flag if file operation is currently executed
	 */
	protected boolean transferring = false;

	/**
	 *  Constructor for operations with source and destination
	 *
	 *@param  sourceName       name of source file
	 *@param  destinationName  name of destination file
	 */
	public FileOperation(String sourceName, String destinationName) {
		this.sourceName = sourceName;
		this.destinationName = destinationName;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public Object clone() {
		try {
			FileOperation clone = (FileOperation) super.clone();
			return clone;
		} catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning " + this +" failed.", e);
			return null;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  object  Description of Parameter
	 *@return         Description of the Returned Value
	 */
	public boolean equals(Object object) {
		if (object == null) {
			return false;
		}
		FileOperation op = (FileOperation) object;

		// this should catch the null-case
		if (sourceName == op.getSourceName()
			&& destinationName == op.getDestinationName()) {
			return true;
		}

		return sourceName.equals(op.getSourceName())
			&& destinationName.equals(op.getDestinationName());
	}

	/**
	 *  Gets the actions attribute of the FileOperation object
	 *
	 *@return    The actions value
	 */
	public AbstractAction getAbstractAction() {
		return abstractAction;
	}

	/**
	 *  Gets the base name in the destination name
	 *
	 *@return    The base name value of the destination name
	 */
	public String getDestinationBasename() {
		String basename;

		if (destinationName.endsWith(getDestinationFileSeparator())) {
			basename =
				destinationName.substring(0, destinationName.length() - 1);
		} else {
			basename = destinationName;
		}

		basename =
			basename.substring(
				basename.lastIndexOf(getDestinationFileSeparator()) + 1);
		return basename;
	}

	/**
	 *  Get separator of destination file system
	 *
	 *@return    separator string
	 */
	public abstract String getDestinationFileSeparator();

	/**
	 *@return    name of destination file
	 */
	public String getDestinationName() {
		return destinationName;
	}

	/**
	 *  Gets the base name in the source name
	 *
	 *@return    The base name value of the source name
	 */
	public String getSourceBasename() {
		String basename;

		if (sourceName.endsWith(getSourceFileSeparator())) {
			basename = sourceName.substring(0, sourceName.length() - 1);
		} else {
			basename = sourceName;
		}

		basename =
			basename.substring(
				basename.lastIndexOf(getSourceFileSeparator()) + 1);
		return basename;
	}

	/**
	 *  Get separator of source file system
	 *
	 *@return    separator string
	 */
	public abstract String getSourceFileSeparator();

	/**
	 *@return    name of source file
	 */
	public String getSourceName() {
		return sourceName;
	}

	/**
	 *  Gets the flavor attribute of the FileOperation object
	 *
	 *@return    The flavor value
	 */
	public abstract DataFlavor getSubClassFlavor();

	/**
	 *  Gets the TotalBytes attribute of the FileOperation object
	 *
	 *@return    The TotalBytes value
	 */
	public long getTotalBytes() {
		return this.totalBytes;
	}

	/**
	 *  Transferable interface:
	 *
	 *@param  flavor                          Description of Parameter
	 *@return                                 The TransferData value
	 *@exception  UnsupportedFlavorException  Description of Exception
	 */
	public synchronized Object getTransferData(DataFlavor flavor)
		throws UnsupportedFlavorException {

		if (getSubClassFlavor().equals(flavor)) {
			return this;
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 *  Transferable interface:
	 *
	 *@return    The TransferDataFlavors value
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		DataFlavor[] flavors = { getSubClassFlavor()};
		return flavors;
	}

	/**
	 *  Gets the TransferredBytes attribute of the FileOperation object
	 *
	 *@return    The TransferredBytes value
	 */
	public long getTransferredBytes() {
		return this.transferredBytes;
	}

	/**
	 *  Alwayas override hashCode, when overriding equals.
	 *
	 *@return    Description of the Return Value
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + sourceName.hashCode();
		result = 37 * result + destinationName.hashCode();
		return result;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public boolean hasWildcards() {
		return (
			(sourceName.indexOf("*") >= 0)
				|| (sourceName.indexOf("?") >= 0)
				|| (sourceName.indexOf("[") >= 0));
	}

	/**
	 *  Check if source and destination names have been set
	 *
	 *@return    true if operation is complete
	 */
	public boolean isComplete() {
		// removed [ && !destinationName.equals("")) ];
		// destinationName.equals("") specifies the Storage, i.e. Uspace, Home which defines a path already
		return (
			sourceName != null
				&& !sourceName.equals("")
				&& destinationName != null);
	}

	/**
	 *  Transferable interface:
	 *
	 *@param  flavor  Description of Parameter
	 *@return         The DataFlavorSupported value
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return (getSubClassFlavor().equals(flavor));
	}

	/**
	 *  Gets the Transferring attribute of the FileOperation object
	 *
	 *@return    The Transferring value
	 */
	public boolean isTransferring() {
		return transferring;
	}

	/**
	 *  Sets the id attribute of the FileOperation object
	 *
	 *@param  abstractAction  The new abstractAction value
	 */
	public void setAbstractAction(AbstractAction abstractAction) {
		this.abstractAction = abstractAction;
	}

	/**
	 *@param  destinationName  The new DestinationName value
	 */
	public void setDestinationName(String destinationName) {
		this.destinationName = destinationName;
	}

	/**
	 *@param  sourceName  The new SourceName value
	 */
	public void setSourceName(String sourceName) {
		this.sourceName = sourceName;
	}

	/**
	 *  Sets the TotalBytes attribute of the FileOperation object
	 *
	 *@param  totalBytes  The new TotalBytes value
	 */
	public void setTotalBytes(long totalBytes) {
		this.totalBytes = totalBytes;
	}

	/**
	 *  Sets the TransferredBytes attribute of the FileOperation object
	 *
	 *@param  transferredBytes  The new TransferredBytes value
	 */
	public void setTransferredBytes(long transferredBytes) {
		this.transferredBytes = transferredBytes;
	}

	/**
	 *  Sets the Transferring attribute of the FileOperation object
	 *
	 *@param  transferring  The new Transferring value
	 */
	public void setTransferring(boolean transferring) {
		this.transferring = transferring;
	}

	/**
	 *  Abstract method describing this operation as String e.g. for gui
	 *  elements
	 *
	 *@return    description as String
	 */
	public abstract String toString();
}
