package com.pallas.unicore.extensions;

import java.util.Vector;
import java.util.logging.Level;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.CopyFile;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.Portfolio;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 *  Class builds AJO ActionGroup from an input Array of FileImports
 *
 *@author     Ralf Ratering
 *@created    10. September 2001
 *@version    $Id: ImportFactory.java,v 1.2 2003/06/30 06:58:17 rrate Exp $
 */
public class ImportFactory extends FileOperationFactory {

	/**
	 *  Constructor
	 *
	 *@param  fileOpsArray  input FileImports
	 *@param  actionGroup   input ActionGroup that will be filled with import
	 *      tasks
	 *@param  container     container for ActionGroup
	 */
	public ImportFactory(
		FileImport[] fileOpsArray,
		ActionGroup actionGroup,
		TaskContainer container) {
		super(fileOpsArray, actionGroup, container);
	}

	/**
	 *  Go, build all tasks and put them into the ActionGroup
	 */
	public void build() {

		// Build DeclarePortfolios for import of executables from Nspace without any renaming
		// (without renaiming there is no need to import via an additional hidden directory)
		FileImport localNotRenamedImport[] = container.getAdditionalImports();
		for (int i = 0; i < localNotRenamedImport.length; i++) {
			buildDeclarePortfolio(localNotRenamedImport[i]);
		}

		// import from XSpace and additional hidden imports required for NSpace imports

		// collect different storage resources
		Vector storageLocations = new Vector();
		for (int i = 0; i < fileOps.size(); i++) {
			FileImport fileImport = (FileImport) fileOps.get(i);

			Object storageItem = fileImport.getStorageItem();
			if (!storageLocations.contains(storageItem)) {
				storageLocations.add(storageItem);
			}
		}

		// loop over different storage locations and build a task for each one
		for (int i = 0; i < storageLocations.size(); i++) {
			buildImportTask(storageLocations.get(i));
		}
	}

	/**
	 *  Go, build all tasks and put them into the ActionGroup
	 *
	 *@param  fileImport  Description of the Parameter
	 */
	private void buildDeclarePortfolio(FileImport fileImport) {
		if (fileImport.isLocal()) {
			// import from NSpace requires DeclarePortfolio
			Portfolio streamedPortfolio =
				new Portfolio(
					"NspacePortfolio_"
						+ ResourceManager.getNextObjectIdentifier());
			DeclarePortfolio nspacePortfolio =
				new DeclarePortfolio("DeclarePortfolio" + fileImport);
			nspacePortfolio.setPortfolio(streamedPortfolio);

			// remember portfolio
			fileImport.setNspacePortfolio(nspacePortfolio);
		}
	}

	/**
	 *  Build an import task for one specific storage location in Xspace
	 *
	 *@param  storage  one specific storage location
	 */
	private void buildImportTask(Object storage) {

		Vector copyImports = new Vector();

		for (int i = 0; i < fileOps.size(); i++) {
			FileImport fileImport = (FileImport) fileOps.get(i);

			if (fileImport.getStorageItem().equals(storage)) {

				// beginning with NJS 4.0.0 beta10 all imports to Xspace are handled by file copies
				copyImports.add(fileImport);
			}
		}

		constructCopyTask(copyImports, storage);
		logger.fine("Found " + copyImports.size() + " imports of directories");
	}

	/**
	 *  Description of the Method
	 *
	 *@param  imports  Description of Parameter
	 *@param  storage  Description of Parameter
	 */
	private void constructCopyTask(Vector imports, Object storage) {

		// nothing to do ?
		if (imports.size() == 0) {
			return;
		}

		// build resource sets for different tasks
		USpace uspaceHome =
			ResourceManager.getResourceSet(container.getVsite()).getUspace();
		if (uspaceHome == null) {
			return;
		}

		// loop over imports and build one copy per import;
		// reason: provides the best outcome details to the user
		// (previous version gathered imports to the same destination dir in a portfolio
		// and a corresponding import task)
		for (int i = 0; i < imports.size(); i++) {
			FileImport fileImport = (FileImport) imports.get(i);

			// create enclosing action group for details panel
			ActionGroup importGroup =
				new ActionGroup(createActionName("IMPORTGROUP_" + fileImport));
			logger.fine(
				"New ActionGroup: "
					+ importGroup.getId().getName()
					+ " ID: "
					+ importGroup.getId());

			// >>> AJO4.0
			CopyFile copyFile = new CopyFile();

			importGroup.add(copyFile);

			// resources
			ResourceSet storageSet = new ResourceSet(uspaceHome);

			// copy from Xspace to Uspace
			boolean copy_from_uspace = false;

			String sourceName;

			String destName;
			boolean overwrite;

			if (fileImport.isLocal()) {
				/*
				 *  To resolve all kinds of imports with renaming, a file/directory from Local is first streamed
				 *  without renaming to a temporary hidden directory (set in the following
				 *  as the destination name of the FileImport at hand). A dependent hidden CopyFile (additionally
				 *  specified here) will then copy the streamed file/directory to its actual
				 *  destination. The renaming will take place implicitly due to the use of CopyFile.
				 */
				// Create the temporary hidden directory
				String internalDir =
					".imports_"
						+ fileImport.getStorageItem().toString().replace(
							' ',
							'_')
						+ "_"
						+ ResourceManager.getNextObjectIdentifier();

				// The source of the additional copy is the streamed file/directory in the hidden directory
				sourceName = internalDir + "/" + fileImport.getSourceBasename();

				// The destination of the additional copy is the final destination specified by the user
				destName = fileImport.getDestinationName();

				// The overwrite mode of the additional copy is the original overwrite mode specified by the user
				overwrite = fileImport.getOverwritePolicy();

				// The streamed file/directory is revealed in the internalDir which is in the Uspace.
				// Therefore the source storage for the additional import is also the Uspace
				storageSet.add(uspaceHome);

				// Now prepare the streaming of the current FileImport to the hidden directory (cf. Connection class)

				// Specify the temporary hidden directory as the destination of the current import
				if (fileImport.hasWildcards()) {
					fileImport.setDestinationName(internalDir);
				} else {
					fileImport.setDestinationName(sourceName);
				}
				// Remember the real destination e.g. for details table
				fileImport.setRealDestination(destName);

				// There is nothing which can be overwritten in the temporary hidden directory
				fileImport.setOverwritePolicy(true);

				// import from NSpace requires DeclarePortfolio
				buildDeclarePortfolio(fileImport);
				DeclarePortfolio nspacePortfolio =
					fileImport.getNspacePortfolio();

				importGroup.add(nspacePortfolio);
				// An Uspace to Uspace import is only possible as an additional hidden import
				// when importing (streaming) from Nspace; add the corresponding dependency
				try {
					importGroup.addDependency(
						new Dependency(nspacePortfolio, copyFile));
				} catch (ActionGroup.InvalidDependencyException ex) {
					logger.log(Level.SEVERE, "Could not add dependency.", ex);
				}

			} else {

				// create a renamed copy of the import file in the internal directory

				// source (including wildcards) and destination are taken from import
				sourceName = fileImport.getSourceName();
				destName = fileImport.getDestinationName();

				// set overwrite flag
				overwrite = fileImport.getOverwritePolicy();

				// resources
				storageSet.add((Storage) storage);
			}

			copyFile.setName(
				createActionName("IMPORT_BY_COPYFILE_" + sourceName) + i);
			copyFile.setResources(storageSet);
			copyFile.setFrom(sourceName);
			copyFile.setTo(destName);
			copyFile.setOverwrite(overwrite);
			copyFile.setCopyFromUspace(copy_from_uspace);

			logger.fine(
				"New CopyFile: "
					+ copyFile.getId().getName()
					+ " ID: "
					+ copyFile.getId());

			actionGroup.add(importGroup);

			// remember import actions
			fileImport.setAbstractAction(importGroup);
		}

	}
}
