package com.pallas.unicore.extensions;

import java.io.Serializable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.Identifier;
import org.unicore.Vsite;
import org.unicore.resources.CapacityResource;
import org.unicore.resources.Context;
import org.unicore.resources.Home;
import org.unicore.resources.Memory;
import org.unicore.resources.Node;
import org.unicore.resources.PerformanceResource;
import org.unicore.resources.Priority;
import org.unicore.resources.PriorityValue;
import org.unicore.resources.Processor;
import org.unicore.resources.Resource;
import org.unicore.resources.RunTime;
import org.unicore.resources.SoftwareResource;
import org.unicore.resources.Storage;
import org.unicore.resources.StorageServer;
import org.unicore.resources.TextInfoResource;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.client.tables.NamedObject;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;

/**
 *  Class encapsulating a ResourceSet to manage loading, saving and editing
 *
 *@author     Ralf Ratering
 *@created    4. Oktober 2001
 *@version    $Id: NamedResourceSet.java,v 1.3 2003/06/30 06:58:17 rrate Exp $
 */
public class NamedResourceSet implements Serializable, Cloneable, NamedObject {

	/**
	 *  Description of the Field
	 */
	public final static Context DECISION_CONTEXT =
		new Context("", "MakeReturnCodeDecision", "");
	/**
	 *  Description of the Field
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.extensions");

	final static long serialVersionUID = 4464281534699904799L;
	/**
	 *  Description of the Field
	 */
	public final static String SITE_DEFAULTS = "Site Defaults";

	/**
	 *  Static helper method to generate an empty default set from scratch
	 *
	 *@return    a NamedResourceSet containing Memory, Node Processor and
	 *      RunTime resources
	 */
	public static NamedResourceSet getEmptyResourceSet() {
		TextInfoResource textInfo =
			new TextInfoResource(
				"Default Resource",
				"",
				"No resources have been loaded from virtual site. Using default resources");
		Memory memory = new Memory("Memory", Float.MAX_VALUE, 0.0, 0.0);
		Node nodes = new Node("Nodes", Float.MAX_VALUE, 0.0, 0.0);
		Processor processor =
			new Processor("Processors per Node", Float.MAX_VALUE, 0.0, 0.0);
		RunTime runTime =
			new RunTime("CPU Time per Job", Float.MAX_VALUE, 0.0, 0.0);

		ResourceSet resourceSet = new ResourceSet();
		resourceSet.add(textInfo);
		resourceSet.add(memory);
		resourceSet.add(nodes);
		resourceSet.add(processor);
		resourceSet.add(runTime);
		NamedResourceSet namedSet =
			new NamedResourceSet("Empty Set", resourceSet);
		return namedSet;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  args  Description of the Parameter
	 */
	public static void main(String args[]) {
		NamedResourceSet resourceSet = new NamedResourceSet("Honk");
		resourceSet.add(new SoftwareResource("Bla1", "CPMD1", "1.0"));
		resourceSet.add(new SoftwareResource("Bla2", "CPMD2", "2.0"));
		resourceSet.add(new SoftwareResource("Bla3", "CPMD3", "3.0"));
		resourceSet.add(new SoftwareResource("Bla4", "CPMD4", "4.0"));

		System.err.println(resourceSet.toStringDetailed());

		System.err.println("REMOVING...");
		resourceSet.removeAllElements();

		System.err.println(resourceSet.toStringDetailed());

	}

	private String name, filename;
	private ResourceSet resourceSet;
	private Vsite vsite;

	/**
	 *  Constructor for the NamedResourceSet object
	 */
	public NamedResourceSet() {
		this(SITE_DEFAULTS);
	}

	/**
	 *  Constructor for the NamedResourceSet object
	 *
	 *@param  name  Description of Parameter
	 */
	public NamedResourceSet(String name) {
		this(name, new ResourceSet());
	}

	/**
	 *  Constructor for the NamedResourceSet object
	 *
	 *@param  name         Description of Parameter
	 *@param  resourceSet  Description of Parameter
	 */
	public NamedResourceSet(String name, ResourceSet resourceSet) {
		this.name = name;
		setResourceSet(resourceSet);
	}

	/**
	 *  Adds a resource to the encapsulated ResourceSet. If there are resources
	 *  of the same type already in the resource set they will be removed.
	 *
	 *@param  resource  resource to be added
	 */
	public void add(Resource resource) {
		// remove all resources ogfsame type from set
		Resource alreadyIn = findResource(resource);
		while (alreadyIn != null) {
			resourceSet.remove(alreadyIn);
			alreadyIn = findResource(resource);
		}

		resourceSet.add(resource);
	}

	/**
	 *  Adds a feature to the ApplicationResources attribute of the
	 *  NamedResourceSet object
	 *
	 *@param  result  The feature to be added to the ApplicationResources
	 *      attribute
	 *@return         Description of the Return Value
	 */
	public Vector addApplicationResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Application", result);
	}

	/**
	 *@param  result  The feature to be added to the CapabilityResources
	 *      attribute
	 *@return         The CapabilityResources value
	 */
	public Vector addCapabilityResources(Vector result) {
		return addResourcesByName(
			"org.unicore.resources.CapabilityResource",
			result);
	}

	/**
	 *@param  result  The feature to be added to the CapacityResources attribute
	 *@return         The CapacityResources value
	 */
	public Vector addCapacityResources(Vector result) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof CapacityResource
				&& !(resource instanceof PerformanceResource)) {
				result.add(resource);
			}
		}
		return result;
	}

	/**
	 *  Adds a feature to the ContextResources attribute of the NamedResourceSet
	 *  object
	 *
	 *@param  result  The feature to be added to the ContextResources attribute
	 *@return         Description of the Return Value
	 */
	public Vector addContextResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Context", result);
	}

	/**
	 *  Adds a feature to the ErrorsToSet attribute of the NamedResourceSet
	 *  object
	 *
	 *@param  errorSet       The feature to be added to the ErrorsToSet
	 *      attribute
	 *@param  taskId         The feature to be added to the ErrorsToSet
	 *      attribute
	 *@param  vsite          The feature to be added to the ErrorsToSet
	 *      attribute
	 *@param  siteResources  The feature to be added to the ErrorsToSet
	 *      attribute
	 */
	public void addErrorsToSet(
		ErrorSet errorSet,
		Identifier taskId,
		Vsite vsite,
		NamedResourceSet siteResources) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			Resource siteResource = siteResources.findResource(resource);
			if (siteResource == null) {
				errorSet.add(
					new UError(
						taskId,
						"Requested resource <"
							+ resource
							+ ">"
							+ " not available on "
							+ vsite));
			} else if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				CapacityResource siteCapacity = (CapacityResource) siteResource;

				double request = capacity.getRequest();
				double min = siteCapacity.getMinRequest();
				double max = siteCapacity.getMaxRequest();

				if (request < min || request > max) {
					errorSet.add(
						new UError(
							taskId,
							"Requested "
								+ capacity
								+ ": <"
								+ (int) request
								+ " "
								+ capacity.getUnits()
								+ " not available on "
								+ vsite));
				}
			}
		}
	}

	/**
	 *@param  result  The feature to be added to the StorageResources attribute
	 *@return         The StorageResources value
	 *@deprecated     use getHome, because since AJO4.0 there is only one home
	 *      resource
	 */
	public Vector addHomeResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Home", result);
	}

	/**
	 *@param  result  The feature to be added to the InformationResources
	 *      attribute
	 *@return         The InformationResources value
	 */
	public Vector addInformationResources(Vector result) {
		return addResourcesByName(
			"org.unicore.resources.InformationResource",
			result);
	}

	/**
	 *@param  result  The feature to be added to the PerformanceResources
	 *      attribute
	 *@return         The PerformanceResources value
	 */
	public Vector addPerformanceResources(Vector result) {
		return addResourcesByName(
			"org.unicore.resources.PerformanceResource",
			result);
	}

	/**
	 *@param  result  The feature to be added to the PriorityResources attribute
	 *@return         Description of the Returned Value
	 */
	public Vector addPriorityResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Priority", result);
	}

	/**
	 *  Gets the ResourcesByName attribute of the NamedResourceSet object
	 *
	 *@param  className  Description of Parameter
	 *@param  result     The feature to be added to the ResourcesByName
	 *      attribute
	 *@return            The ResourcesByName value
	 */
	private Vector addResourcesByName(String className, Vector result) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			try {
				if (Class.forName(className).isInstance(resource)) {
					result.add(resource);
				}
			} catch (ClassNotFoundException e) {
				logger.log(
					Level.SEVERE,
					"Error adding Resource ro NamedResourceSet " + className,
					e);
			}
		}
		return result;
	}

	/**
	 *@param  result  The feature to be added to the SoftwareResources attribute
	 *@return         Description of the Returned Value
	 */
	public Vector addSoftwareResources(Vector result) {
		return addResourcesByName(
			"org.unicore.resources.SoftwareResource",
			result);
	}

	/**
	 *@param  result  The feature to be added to the StorageResources attribute
	 *@return         The StorageResources value
	 */
	public Vector addStorageResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Storage", result);
	}

	/**
	 *  Adds a feature to the StorageServerResources attribute of the
	 *  NamedResourceSet object
	 *
	 *@param  result  The feature to be added to the StorageServerResources
	 *      attribute
	 *@return         Description of the Return Value
	 */
	public Vector addStorageServerResources(Vector result) {
		return addResourcesByName(
			"org.unicore.resources.StorageServer",
			result);
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public Object clone() {
		try {
			NamedResourceSet clone = (NamedResourceSet) super.clone();
			clone.setResourceSet(resourceSet.cloneSet());
			return clone;
		} catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning " + this +" failed.", e);
			return null;
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@param  resource  Description of the Parameter
	 *@return           Description of the Return Value
	 */
	public boolean contains(Resource resource) {
		return (!(findResource(resource) == null));
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public boolean containsStorageResources() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			if (resourceEnum.nextElement() instanceof Storage) {
				return true;
			}
		}
		return false;
	}

	/**
	 *  Try to find a resource of a specific type in the resource set. For
	 *  storage resources also the subspace values have to match. For priorities
	 *  the priority values are compared.
	 *
	 *@param  toBeFound  Resource to be found
	 *@return            The matching resource from the resource set
	 */
	public Resource findResource(Resource toBeFound) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			// cannot use object equals here, because CapacityResources
			// are only equal when min/max/etc. are equal
			if (resource.getClass().equals(toBeFound.getClass())) {
				// for storages compare subspaces
				if (resource instanceof Storage) {
					Storage storage = (Storage) resource;
					Storage toBeFoundStorage = (Storage) toBeFound;
					// compare names of storage servers
					if (storage instanceof StorageServer) {
						StorageServer server = (StorageServer) storage;
						StorageServer toBeFoundServer =
							(StorageServer) toBeFoundStorage;
						if (server
							.getName()
							.equals(toBeFoundServer.getName())) {
							return storage;
						}
					} else {
						return storage;
					}
				}
				// for priorities compare values
				else if (resource instanceof Priority) {
					Priority priority = (Priority) resource;
					Priority toBeFoundPriority = (Priority) toBeFound;
					if (toBeFoundPriority
						.getValue()
						.equals(priority.getValue())) {
						return priority;
					}
				}
				// for software resources use equals to compare name and version
				else if (resource instanceof SoftwareResource) {
					if (resource.equals(toBeFound)) {
						return resource;
					}
				} else {
					return resource;
				}
			}
		}
		// if we got here no resource was found
		return null;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  softwareName  Description of the Parameter
	 *@return               Description of the Return Value
	 */
	public SoftwareResource findSoftwareResourceByName(String softwareName) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof SoftwareResource) {
				SoftwareResource software = (SoftwareResource) resource;
				if (software.getName().equals(softwareName)) {
					return software;
				}
			}
		}
		return null;
	}

	/**
	 *  Gets the filename attribute of the NamedResourceSet object
	 *
	 *@return    The filename value
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 *  Gets the home attribute of the NamedResourceSet object
	 *
	 *@return    The home value
	 */
	public Home getHome() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Home) {
				return (Home) resource;
			}
		}
		return null;
	}

	/**
	 *  Gets the Name attribute of the NamedResourceSet object
	 *
	 *@return    The Name value
	 */
	public String getName() {
		return name;
	}

	/**
	 *  Gets the priorityValue attribute of the NamedResourceSet object
	 *
	 *@return    The priorityValue value
	 */
	public PriorityValue getPriorityValue() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Priority) {
				Priority priority = (Priority) resource;
				return priority.getValue();
			}
		}
		return null;
	}

	/**
	 *  Get the encapsulated resource set. CAUTION: The resource set may be
	 *  referenced by multiple tasks, so if you modify the returned set, you may
	 *  affect the resources of outher tasks. Please, check this carefully and
	 *  use getResourceSetClone instead!!
	 *
	 *@return    the encasulated resource set
	 */
	public ResourceSet getResourceSet() {
		return resourceSet;
	}

	/**
	 *  Get a clone of the encapsulated resource set. Use this method instead of
	 *  the plain getResourceSet(), if you want to modify the set without
	 *  affecting other tasks that may have a reference to it.
	 *
	 *@return    A clone of the encapsulated resource set
	 */
	public ResourceSet getResourceSetClone() {
		return resourceSet.cloneSet();
	}

	/**
	 *  Gets the storageResources attribute of the NamedResourceSet object
	 *
	 *@return    The storageResources value
	 */
	public ResourceSet getStorageResources() {
		ResourceSet storages = new ResourceSet();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Storage) {
				storages.add(resource);
			}
		}
		return storages;
	}

	/**
	 *  Gets the UspaceHome resource from the resource set
	 *
	 *@return    The UspaceHome value
	 */
	public USpace getUspace() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof USpace) {
				return (USpace) resource;
			}
		}
		return null;
	}

	/**
	 *  Gets the uspaceHome attribute of the NamedResourceSet object
	 *
	 *@return        The uspaceHome value
	 *@deprecated    use getUspace() instead
	 */
	public USpace getUspaceHome() {
		return getUspace();
	}

	/**
	 *  Gets the vsite attribute of the NamedResourceSet object
	 *
	 *@return    The vsite value
	 */
	public Vsite getVsite() {
		return vsite;
	}

	/**
	 *  Gets the Correct attribute of the NamedResourceSet object
	 *
	 *@return    The Correct value
	 */
	public boolean isCorrect() {
		return true;
	}

	/**
	 *  Check if the reousrce set is empty
	 *
	 *@return    The empty value
	 */
	public boolean isEmpty() {
		return !resourceSet.elements().hasMoreElements();
	}

	/**
	 *  If resource set is empty and name equals SITE_DEFAULTS we have a site
	 *  default resource set.
	 *
	 *@return    The SiteDefault value
	 */
	public boolean isSiteDefault() {
		return name.equals(SITE_DEFAULTS);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  siteResources  Description of the Parameter
	 *@return                Description of the Return Value
	 */
	public boolean matchesSiteResources(NamedResourceSet siteResources) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			Resource siteResource = siteResources.findResource(resource);
			if (siteResource == null) {
				return false;
			}
			if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				CapacityResource siteCapacity = (CapacityResource) siteResource;

				double request = capacity.getRequest();
				double min = siteCapacity.getMinRequest();
				double max = siteCapacity.getMaxRequest();

				if (request < min || request > max) {
					return false;
				}
			}
		}

		// if we got here resources match
		return true;
	}

	/**
	 *  Description of the Method
	 */
	public void removeAllElements() {
		// @TODO: Remove this when clear() is implemented in ResourceSet
		Vector tmpVec = new Vector();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			tmpVec.add(resourceEnum.nextElement());
		}
		for (int i = 0; i < tmpVec.size(); i++) {
			resourceSet.remove((Resource) tmpVec.elementAt(i));
		}
	}

	/**
	 *  Sets the filename attribute of the NamedResourceSet object
	 *
	 *@param  filename  The new filename value
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 *  Sets the Name attribute of the NamedResourceSet object
	 *
	 *@param  name  The new Name value
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 *  Sets the ResourceSet attribute of the NamedResourceSet object
	 *
	 *@param  resourceSet  The new ResourceSet value
	 */
	public void setResourceSet(ResourceSet resourceSet) {
		this.resourceSet = resourceSet;
		// always add decision context to make ExecuteTasks produce Decision file
		if (!resourceSet.isIn(DECISION_CONTEXT)) {
			resourceSet.add(DECISION_CONTEXT);
		}
	}

	/**
	 *  Sets the vsite attribute of the NamedResourceSet object
	 *
	 *@param  vsite  The new vsite value
	 */
	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public String toString() {
		return name;
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public String toStringDetailed() {
		String result = "NamedResourceSet " + name + "\n";
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			result += resource + ":" + resource.getDescription();
			if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				result += ", Request: " + capacity.getRequest();
				result += ", Min: " + capacity.getMinRequest();
				result += ", Max: " + capacity.getMaxRequest();
				result += ", Default: " + capacity.getDefaultRequest();
			}
			result += "\n";
		}
		return result;
	}
}
