package com.pallas.unicore.resourcemanager;

import java.io.ByteArrayInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractTask;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.If;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.AbstractTask_Outcome;
import org.unicore.outcome.ActionGroup_Outcome;
import org.unicore.outcome.If_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.RepeatGroup_Outcome;
import org.unicore.outcome.StatusHistory;
import org.unicore.sets.ActionGroup_OutcomeList;
import org.unicore.sets.OutcomeEnumeration;

import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileImport;
import com.pallas.unicore.extensions.FileSystemOperation;
import com.pallas.unicore.extensions.FileTransfer;

/**
 * Store all outcomes with their ActionGroups and ActoinContainers 
 * 
 *@author     Ralf Ratering
 *@version    $Id: OutcomeManager.java,v 1.5 2003/07/10 17:27:10 rrate Exp $
 */
public class OutcomeManager {
	private static Hashtable detailsTable = new Hashtable();
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.resourcemanager");
	private static Hashtable outcomeTable = new Hashtable();
	private static Vector streamedFiles = new Vector();
	private final static String UC_ITERATION_COUNTS_REGEX =
		"\\$UC_ITERATION_COUNTS";

	/**
	 *  Add a streamed file
	 *
	 *@param  filename  streamed file
	 */
	public static void addStreamedFile(String filename) {
		if (!streamedFiles.contains(filename)) {
			streamedFiles.add(filename);
		}
	}
	
	/**
	 * Get streamed files
	 * @return Vector containing streamed files
	 */
	public static Vector getStreamedFiles() {
		return streamedFiles;
	}

	private static void buildDetailsEntry(
		AbstractAction action,
		Vector entries,
		TaskContainer container,
		String description) {

		Vector outcomeEntries = (Vector) outcomeTable.get(action.getId());
		if (outcomeEntries == null) {
			return;
		}
		for (int i = 0; i < outcomeEntries.size(); i++) {
			OutcomeEntry outcomeEntry =
				(OutcomeEntry) outcomeEntries.elementAt(i);
			Outcome outcome = outcomeEntry.getOutcome();
			StatusHistory history = outcome.getStatusHistory();
			String iteration = outcomeEntry.getIteration();

			Iterator iterator = history.iterator();
			while (iterator.hasNext()) {
				StatusHistory.Entry statusEntry =
					(StatusHistory.Entry) iterator.next();
				String reason = statusEntry.getReason();
				if (statusEntry
					.getStatus()
					.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
					// find real reason in nested outcome
					if (outcome instanceof ActionGroup_Outcome) {
						reason = findReason((ActionGroup_Outcome) outcome);
					} else {
						reason = outcome.getReason();
					}
				}

				// resolve $UCITERATION_COUNTS
				String resolvedDescription =
					description.replaceAll(
						UC_ITERATION_COUNTS_REGEX,
						iteration);

				DetailsEntry detailsEntry =
					new DetailsEntry(
						statusEntry.getTime(),
						container,
						resolvedDescription,
						statusEntry.getStatus(),
						reason,
						iteration,
						outcome.getId());
				entries.add(detailsEntry);
			}
		}
	}

	private static void buildDetailsTable(GroupContainer group) {
		Vector tasks = group.getTasks();
		for (int i = 0; i < tasks.size(); i++) {
			ActionContainer task = (ActionContainer) tasks.elementAt(i);
			if (task instanceof TaskContainer) {
				buildTaskDetails((TaskContainer) task);
			} else if (task instanceof GroupContainer) {
				buildDetailsTable((GroupContainer) task);
			}
		}
	}

	private static void buildTableEntry(
		Outcome outcome,
		AbstractAction action,
		String path,
		String iteration) {
		Vector entries = (Vector) outcomeTable.get(action.getId());
		if (entries == null) {
			entries = new Vector();
		}

		OutcomeEntry entry = new OutcomeEntry(action);
		entry.setIteration(iteration);
		entry.setOutcome(outcome);

		// only search stdout/stderr files for abstract tasks
		if (action instanceof AbstractTask) {
			logger.info("Searching streamed files for " + action);
			String actionPath = path + "/" + action.getUPLDirectoryName();
			entry.setStderrFile(findStreamedFile(actionPath + "/stderr"));
			entry.setStdoutFile(findStreamedFile(actionPath + "/stdout"));
		}
		entries.add(entry);

		outcomeTable.put(action.getId(), entries);
	}


	private static void buildTaskDetails(TaskContainer container) {
		Vector entries = new Vector();

		ActionGroup executeGroup = container.getExecuteGroup();
		if (executeGroup != null) {
			buildDetailsEntry(executeGroup, entries, container, "Execution");
		}

		FileImport[] fileImports = container.getFileImports();
		for (int i = 0; i < fileImports.length; i++) {
			buildDetailsEntry(
				fileImports[i].getAbstractAction(),
				entries,
				container,
				"Import: <" + fileImports[i] + ">");
		}
		FileExport[] fileExports = container.getFileExports();
		for (int i = 0; i < fileExports.length; i++) {
			buildDetailsEntry(
				fileExports[i].getAbstractAction(),
				entries,
				container,
				"Export: <" + fileExports[i] + ">");
		}
		FileTransfer[] fileTransfers = container.getFileTransfers();
		for (int i = 0; i < fileTransfers.length; i++) {
			buildDetailsEntry(
				fileTransfers[i].getAbstractAction(),
				entries,
				container,
				"Transfer: <" + fileTransfers[i] + ">");
		}
		FileSystemOperation[] fileOps = container.getFileSystemOperations();
		for (int i = 0; i < fileOps.length; i++) {
			buildDetailsEntry(
				fileOps[i].getAbstractAction(),
				entries,
				container,
				"File System Operation: <" + fileOps[i] + ">");
		}

		detailsTable.put(container.getIdentifier(), entries);
	}

	/**
	 *  Clear all stored information
	 */
	public static void clearCache() {
		logger.finest("Clearing outcome manager cache!");
		streamedFiles.clear();
	}

	/**
	 *  Build outcomes
	 *
	 *@param  original                      input outcome
	 *@return                               
	 *@exception  ClassNotFoundException    
	 *@exception  InvalidClassException     
	 *@exception  StreamCorruptedException  
	 *@exception  OptionalDataException     
	 *@exception  IOException               
	 */
	public static AbstractJob_Outcome convertFrom(AbstractJob_Outcome original)
		throws
			ClassNotFoundException,
			InvalidClassException,
			StreamCorruptedException,
			OptionalDataException,
			IOException {
		byte[] annon_outcomes = original.getAnonOutcomes();

		if (annon_outcomes == null) {
			return original;
		}

		original.setAnonOutcomes((byte[]) null);
		// here coz the call clears the unsered ones

		ObjectInputStream ois =
			new ObjectInputStream(new ByteArrayInputStream(annon_outcomes));

		// The container had the non-anon cleared so set them from the deserialised
		try {
			while (true) {
				Outcome o = (Outcome) ois.readObject();
				original.addOutcome(o);
			}
		} catch (EOFException eex) {
			// OK, assume all read
		}

		return original;
	}


	private static void findDetails(ActionContainer container, Vector result) {
		if (container instanceof GroupContainer) {
			GroupContainer group = (GroupContainer) container;
			Vector tasks = group.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				findDetails((ActionContainer) tasks.elementAt(i), result);
			}
		} else if (container instanceof TaskContainer) {
			Vector details =
				(Vector) detailsTable.get(container.getIdentifier());
			if (details != null) {
				result.addAll(details);
			}
		}
	}

	private static void findOutcomeEntries(
		AbstractAction action,
		Vector entries) {
		if (action instanceof If) {
			If group = (If) action;
			findOutcomeEntries(group.getTrueBranch(), entries);
			findOutcomeEntries(group.getFalseBranch(), entries);
		} else if (action instanceof ActionGroup) {
			ActionGroup group = (ActionGroup) action;
			Enumeration actions = group.getActions();
			while (actions.hasMoreElements()) {
				findOutcomeEntries(
					(AbstractAction) actions.nextElement(),
					entries);
			}
		}

		Vector taskEntries = (Vector) outcomeTable.get(action.getId());
		if (taskEntries == null) {
			taskEntries = new Vector();
		}
		entries.addAll(taskEntries);

	}

	private static String findReason(ActionGroup_Outcome outcome) {
		OutcomeEnumeration outcomes = outcome.getOutcomes();
		while (outcomes.hasMoreElements()) {
			Outcome next = (Outcome) outcomes.nextElement();
			if (next
				.getStatus()
				.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
				if (next instanceof AbstractTask_Outcome) {
					return next.getReason();
				} else if (next instanceof ActionGroup_Outcome) {
					return findReason((ActionGroup_Outcome) next);
				}
			}
		}

		return "";
	}

	private static File findStreamedFile(String path) {
		// remove leading "/" which might not occur in zip stream
		if (path.startsWith("/")) {
			path = path.substring(1);
		}
		for (int i = 0; i < streamedFiles.size(); i++) {
			String streamedName = (String) streamedFiles.elementAt(i);
			if ((path.indexOf(streamedName) >= 0)
				|| (streamedName.indexOf(path) >= 0)) {
				logger.fine(
					"COMPARING "
						+ path
						+ " TO "
						+ streamedName
						+ " SUCCESSFUL!!!!");
				String filename =
					Connection.getSessionDir() + "/" + streamedName;
				return new File(filename);
			}
			logger.finest(
				"COMPARING " + path + " TO " + streamedName + " FAILED...");
		}
		return null;
	}

	/**
	 *  Gets the details belonging to an ActionContainer
	 *
	 *@param  container  
	 *@return            Vector containing details, one for each iteration 
	 */
	public static Vector getDetails(ActionContainer container) {
		Vector result = new Vector();
		findDetails(container, result);
		return result;
	}

	/**
	 *  Gets the Outcome entries belonging to an ActionGroup
	 *
	 *@param  group ActionGroup  
	 *@return            Vector containing outcome entries, one for each iteration 
	 */
	public static Vector getOutcomeEntries(AbstractAction group) {
		Vector entries = new Vector();
		findOutcomeEntries(group, entries);
		// sort entries by last time stamp
		return entries;
	}

	/**
	 *  Gets the Outcomes belonging to an ActionContainer
	 *
	 *@param  container  
	 *@return            Vector containing Outcomes, one for each iteration 
	 */
	public static Vector getOutcomes(ActionContainer container) {
		Vector outcomeEntries =
			(Vector) outcomeTable.get(container.getActionGroup().getId());
		if (outcomeEntries == null) {
			outcomeEntries = new Vector();
		}
		return outcomeEntries;
	}

	/**
	 *  Set an outcomefor a JobContainer
	 *
	 *@param  jobOutcome  
	 *@param  jobContainer 
	 */
	public static void setOutcome(
		AbstractJob_Outcome jobOutcome,
		JobContainer jobContainer) {
		outcomeTable.clear();
		detailsTable.clear();
		spreadOutcome(jobOutcome, jobContainer.getActionGroup(), "", false, "");
		logger.finest("OUTCOME TABLE:\n" + outcomeTable);
		buildDetailsTable(jobContainer);
		logger.info("Streamed files cache:\n" + streamedFiles.toString());
	}

	/**
	 *  For plain action groups, loop over children
	 *
	 */
	private static void spreadActionGroupOutcome(
		ActionGroup_Outcome outcome,
		ActionGroup group,
		String path,
		boolean addPath,
		String iteration) {
		Enumeration actions = group.getActions();
		String newPath = path;
		// skip directories for jobs
		if (addPath) {
			newPath += "/" + group.getUPLDirectoryName();
		}
		while (actions.hasMoreElements()) {
			AbstractAction task = (AbstractAction) actions.nextElement();

			spreadOutcome(
				outcome.getOutcome(task.getId()),
				task,
				newPath,
				true,
				iteration);
		}
	}

	/**
	 *  Only for AbstractTasks and ActionGroups we fill the Hashtable
	 */
	private static void spreadOutcome(
		Outcome outcome,
		AbstractAction action,
		String path,
		boolean addPath,
		String iteration) {

		if (outcome instanceof AbstractJob_Outcome) {
			try {
				convertFrom((AbstractJob_Outcome) outcome);
			} catch (Exception e) {
				logger.log(
					Level.SEVERE,
					"Could not decode anonymous outcome.",
					e);
			}
		}
		if (outcome instanceof If_Outcome) {
			If_Outcome ifOutcome = (If_Outcome) outcome;
			If ifGroup = (If) action;
			if (addPath) {
				path += "/" + ifGroup.getUPLDirectoryName();
			}
			spreadOutcome(
				ifOutcome.getTrueBranchOutcome(),
				ifGroup.getTrueBranch(),
				path,
				addPath,
				iteration);
			spreadOutcome(
				ifOutcome.getFalseBranchOutcome(),
				ifGroup.getFalseBranch(),
				path,
				addPath,
				iteration);
		}
		if (outcome instanceof RepeatGroup_Outcome) {
			spreadRepeatGroupOutcome(
				(RepeatGroup_Outcome) outcome,
				(ActionGroup) action,
				path,
				addPath,
				iteration);
		} else if (outcome instanceof ActionGroup_Outcome) {
			buildTableEntry(outcome, action, path, iteration);
			spreadActionGroupOutcome(
				(ActionGroup_Outcome) outcome,
				(ActionGroup) action,
				path,
				addPath,
				iteration);
		} else {
			buildTableEntry(outcome, action, path, iteration);
		}
	}

	/**
	 *  Iterate over repeat/doN group outcomes here
	 *
	 */
	private static void spreadRepeatGroupOutcome(
		RepeatGroup_Outcome outcome,
		ActionGroup group,
		String path,
		boolean addPath,
		String iteration) {
		ActionGroup_OutcomeList list = outcome.getOutcomeList();
		String newIteration = iteration;
		if (!newIteration.equals("")) {
			newIteration += "_";
		}
		for (int i = 0; i < list.size(); i++) {
			String newPath = path + "/" + group.getUPLDirectoryName() + "_" + i;
			// strange: do not add subdirs for iterated action groups
			spreadOutcome(
				list.getAt(i),
				group,
				newPath,
				false,
				newIteration + (i + 1));
		}
	}

	/**
	 * Private constructor for static OutcomeManager
	 */
	private OutcomeManager() {
	}

}
