package com.pallas.unicore.resourcemanager;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Logger;
import org.unicore.Vsite;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.utility.VectorSorter;

/**
 *  Cached information about usites, vsites and resources
 *
 *@author     Ralf Ratering
 *@created    4. April 2002
 *@version    $Id: ResourceCache.java,v 1.3 2003/06/30 06:58:38 rrate Exp $
 */

public class ResourceCache implements Serializable {
	private static Logger logger =
		Logger.getLogger("com.pallas.unicore.resourcemanager");
	final static long serialVersionUID = -4561301205740529479L;

	private final static String[] specialVsiteTypes = { "AFT", "SAFE" };
	private Hashtable resourceTable = new Hashtable();
	private Hashtable specialVsiteTable = new Hashtable();
	private Hashtable unavailableUsiteTable = new Hashtable();
	private Hashtable unavailableVsiteTable = new Hashtable();
	private Vector usites = new Vector();
	private Hashtable vsiteTable = new Hashtable();

	/**
	 *  Constructor for the ResourceCache object
	 */
	public ResourceCache() {
	}

	/**
	 *  Adds a feature to the ResourceSet attribute of the ResourceCache object
	 *
	 *@param  vsite        The feature to be added to the ResourceSet attribute
	 *@param  resourceSet  The feature to be added to the ResourceSet attribute
	 */
	public void addResourceSet(Vsite vsite, NamedResourceSet resourceSet) {
		resourceTable.put(vsite, resourceSet);
	}

	/**
	  *  Adds a vsite to the specialVsite list of the ResourceCache
	  *  object
	  *
	  *@param  vsite    The feature to be added to the specialVsite attribute
	  *@param  message  The feature to be added to the specialVsite attribute
	  */
	public void addSpecialVsite(Vsite vsite, String message) {
		logger.info(message);
		if (specialVsiteTable == null) {
			specialVsiteTable = new Hashtable();
		}
		specialVsiteTable.put(vsite, message);
	}

	/**
	 *  Adds a feature to the UnavaiableUsite attribute of the ResourceCache
	 *  object
	 *
	 *@param  usite    The feature to be added to the UnavaiableUsite attribute
	 *@param  message  The feature to be added to the UnavaiableUsite attribute
	 */
	public void addUnavailableUsite(Usite usite, String message) {
		logger.info(message);
		unavailableUsiteTable.put(usite, message);
	}

	/**
	 *  Adds a feature to the UnavailableVsite attribute of the ResourceCache
	 *  object
	 *
	 *@param  vsite    The feature to be added to the UnavailableVsite attribute
	 *@param  message  The feature to be added to the UnavailableVsite attribute
	 */
	public void addUnavailableVsite(Vsite vsite, String message) {
		logger.info(message);
		unavailableVsiteTable.put(vsite, message);
	}

	/**
	 *  Adds a feature to the Vsites attribute of the ResourceCache object
	 *
	 *@param  usite   The feature to be added to the Vsites attribute
	 *@param  vsites  The feature to be added to the Vsites attribute
	 */
	public void addVsites(Usite usite, Vector vsites) {
		// sort vsites
		VectorSorter.sort(vsites);
		vsiteTable.put(usite, vsites);
	}

	/**
	 *  Description of the Method
	 */
	public void clearSpecialVsites() {
		specialVsiteTable.clear();
	}

	/**
	 *  Description of the Method
	 */
	public void clearUnavailableUsites() {
		unavailableUsiteTable.clear();
	}

	/**
	 *  Description of the Method
	 */
	public void clearUnavailableVsites() {
		unavailableVsiteTable.clear();
	}

	/**
	 *  This method is needed as long as we do not have a Vsite.equals() and
	 *  Vsite.hashCode() method. It returns the Vsite from the cache matching
	 *  the characteristics of the provided vsite, which may be a different
	 *  object.
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The matchingVsite value
	 */
	public Vsite getMatchingVsite(Vsite vsite) {
		Enumeration vsites = resourceTable.keys();
		while (vsites.hasMoreElements()) {
			Vsite nextKnown = (Vsite) vsites.nextElement();
			if (vsite.equals(nextKnown)) {
				return nextKnown;
			}
		}
		return null;
	}

	/**
	 *  Get resources from a specific VSite.
	 *
	 *@param  vsite  Vsite to get resources from
	 *@return        ResourceSet containing all resources from the VSite
	 */

	public NamedResourceSet getResourceSet(Vsite vsite) {
		// if no resources have been loaded return default empty set
		if (vsite == null || resourceTable.get(vsite) == null) {
			return NamedResourceSet.getEmptyResourceSet();
		}
		return (NamedResourceSet) resourceTable.get(vsite);
	}

	/**
	 *  Gets the Usite for a Vsite
	 *
	 *@param  vsite  The Vsite for which the Usite is asked for
	 *@return        The Usite value
	 */
	public Usite getUsite(Vsite vsite) {
		Usite usite = null;

		for (int i = 0; i < usites.size(); i++) {
			usite = (Usite) usites.elementAt(i);
			Vector vsites = getVsites(usite);
			for (int j = 0; j < vsites.size(); j++) {
				if (vsite.equals((Vsite) vsites.elementAt(j))) {
					return usite;
				}
			}
		}

		// this should only happen if no Usites exist
		return usite;
	}

	/**
	 *  Gets the usiteErrorMessage attribute of the ResourceCache object
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The usiteErrorMessage value
	 */
	public String getUsiteErrorMessage(Usite usite) {
		Object message = unavailableUsiteTable.get(usite);
		if (message == null) {
			return null;
		}
		return (String) message;
	}

	/**
	 *  Gets the usites attribute of the ResourceCache object
	 *
	 *@return    The usites value
	 */
	public Vector getUsites() {
		return usites;
	}

	/**
	 *  Gets the vsiteErrorMessage attribute of the ResourceCache object
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The vsiteErrorMessage value
	 */
	public String getVsiteErrorMessage(Vsite vsite) {
		Object message = unavailableVsiteTable.get(vsite);
		if (message == null) {
			return null;
		}
		return (String) message;
	}

	/**
	 *  Gets the vsites attribute of the ResourceCache object
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The vsites value
	 */
	public Vector getVsites(Usite usite) {
		// if no vsites have been loaded return empty Vector
		if (vsiteTable.get(usite) == null) {
			return new Vector();
		}
		return (Vector) (vsiteTable.get(usite));
	}

	/**
	 *  Gets the aJOCompatible attribute of the ResourceCache object
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The aJOCompatible value
	 */
	private boolean isAJOCompatible(Vsite vsite) {
		String version =
			Package.getPackage("org.unicore").getSpecificationVersion();
		String vsiteVersion = vsite.getAJOSpecificationVersion();
		return (version != null) && (version.compareTo(vsiteVersion) <= 0);
	}

	/**
	*  Checks if this Vsite can accept AJO's
	*  Special Vsites, like AFT and SAFE will appear on the gateway, but cannot handle AJO's
	**/

	private boolean isOnSpecialList(Vsite vsite) {
		for (int j = 0; j < specialVsiteTypes.length; j++) {
			if (vsite
				.getType()
				.getDescription()
				.equals(specialVsiteTypes[j])) {
				logger.info("Vsite " + vsite.getName() + " excluded");
				return true;
			}
		}
		return false;
	}

	/**
	*  Gets the special attribute of the ResourceCache object
	*
	*@param  vsite  Description of the Parameter
	*@return        The special value
	*/
	public boolean isSpecialVsite(Vsite vsite) {
		if (specialVsiteTable == null) {
			specialVsiteTable = new Hashtable();
		}
		return specialVsiteTable.get(vsite) != null;
	}

	/**
	 *  Gets the usiteAvailable attribute of the ResourceCache object
	 *
	 *@param  usite  Description of the Parameter
	 *@return        The usiteAvailable value
	 */
	public boolean isUsiteAvailable(Usite usite) {
		return unavailableUsiteTable.get(usite) == null;
	}

	/**
	 *  Gets the vsiteAvailable attribute of the ResourceCache object
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        The vsiteAvailable value
	 */
	public boolean isVsiteAvailable(Vsite vsite) {
		return unavailableVsiteTable.get(vsite) == null;
	}

	/**
	 *  Remove a Vector of outdated vsite entries from resource table
	 *
	 *@param  oldVsites  Description of the Parameter
	 */
	private void removeResourceEntries(Vector oldVsites) {
		Enumeration vsiteEnum = resourceTable.keys();
		while (vsiteEnum.hasMoreElements()) {
			Vsite vsite = (Vsite) vsiteEnum.nextElement();
			if (oldVsites.contains(vsite)) {
				resourceTable.remove(vsite);
			}
		}
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		String dump = "*** RESOURCE CACHE ***";
		dump += "\nUSITES: " + usites;
		dump += "\nUNAVAILABLE USITES: " + unavailableUsiteTable;
		dump += "\nVSITES: " + vsiteTable;
		dump += "\nUNAVAILABLE VSITES: " + unavailableVsiteTable;
		dump += "\nRESOURCES: " + resourceTable;
		return dump;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  newUsites  Description of the Parameter
	 */
	void updateUsites(Vector newUsites) {
		for (int i = 0; i < newUsites.size(); i++) {
			Usite newUsite = (Usite) newUsites.elementAt(i);
			boolean found = false;
			for (int j = 0; !found && j < usites.size(); j++) {
				Usite oldUsite = (Usite) usites.elementAt(j);
				if (newUsite.equals(oldUsite)) {
					//newUsites.setElementAt(oldUsite, newUsites.indexOf(newUsite));
					usites.remove(oldUsite);
					// setting flag will exit loop
					found = true;
				}
			}
		}
		// remove remaining usites from vsite table and
		// corresponding vsites from resource table
		for (int i = 0; i < usites.size(); i++) {
			Usite oldUsite = (Usite) usites.elementAt(i);
			Vector oldVsites = (Vector) vsiteTable.get(oldUsite);
			// if vsites have been available remove them from
			// vsite and resource table
			if (oldVsites != null) {
				removeResourceEntries(oldVsites);
				vsiteTable.remove(oldUsite);
			}
		}

		// finally we have the new usites
		usites = newUsites;
		VectorSorter.sort(usites);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  newVsites  Description of the Parameter
	 *@param  usite      Description of the Parameter
	 *@return            Description of the Return Value
	 */
	Vector updateVsites(Usite usite, Vsite[] newVsites) {
		Vector result = new Vector();
		// get old vsites from table if available
		Vector oldVsites = (Vector) vsiteTable.get(usite);
		for (int i = 0; i < newVsites.length; i++) {
			boolean found = false;
			// if we had vsites for our usite before,
			// use old ones if they still exist
			if (oldVsites != null) {
				for (int j = 0; !found && j < oldVsites.size(); j++) {
					Vsite oldVsite = (Vsite) oldVsites.elementAt(j);
					// check also if a vsite changed its type
					if (newVsites[i].equals(oldVsite)
						&& newVsites[i].getType().equals(oldVsite.getType())) {
						//result.add(oldVsite);
						// removing existing vsites will
						// outdated ones in the vector
						oldVsites.remove(oldVsite);
						found = true;
					}
				}
			}
			//            if(!found) {
			result.add(newVsites[i]);
			//            }
		}

		// result now contains the current vsites and oldVsites
		// the outdated ones

		// loop over new vsites and check AJO compatibility
		for (int i = 0; i < result.size(); i++) {
			Vsite vsite = (Vsite) result.elementAt(i);
			if (!isAJOCompatible(vsite)) {
				addUnavailableVsite(
					vsite,
					"Incompatible software version: "
						+ vsite.getAJOSpecificationVersion());
			}
			if (isOnSpecialList(vsite)) {
				addSpecialVsite(
					vsite,
					"Vsite type excluded: " + vsite.getType().getDescription());
			}

		}

		// remove old vsites from resource table, if available
		if (oldVsites != null) {
			removeResourceEntries(oldVsites);
		}
		return result;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  usite  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public boolean usiteExists(Usite usite) {
		return usites.contains(usite);
	}

	/**
	 *  Description of the Method
	 *
	 *@param  vsite  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public boolean vsiteExists(Vsite vsite) {
		return resourceTable.containsKey(vsite);
	}

}
