package com.pallas.unicore.security;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Enumeration;

import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.ConditionalAction;

/**
 * Converts AJOsto a form suitable for consigning.
 * <p>
 * AJOs and Outcomes are transported in the UPL through two levels of serialisation.
 * The UPL Objects,e.g. {@link org.unicore.upl.ConsignJob}, are transported
 * using the usual serialisation. However, when deserialised UPL objects contain
 * a byte array which is the serialisation of the AJO or Outcome. This double serialisation
 * means that intermediate servers, such as Gateways, which deserialise the UPL
 * objects do not have to deserialise the AJO or Outcome object and so do not
 * need to have the full AJO classes available to them.
 * <p>
 * In addition AJOs are are transmitted with a signature which mean that
 * NJSs can check that the AJOs have not been tampered with in transit.
 * <p>
 * The static methods in this class convert to and from the consign form
 * serialising the AJOs and Outcomes or deserialising them.
 *
 * @see org.unicore.ajo.AbstractJob#setConsignForm
 * @see org.unicore.ajo.AbstractJob#setSignature
 * @see org.unicore.outcome.AbstractJob_Outcome#setAnonOutcomes
 * @see org.unicore.upl.ConsignJob
 * @see org.unicore.upl.RetrieveOutcomeReply
 *
 * @author S. van den Berghe (fecit), T. Kentemich (Pallas)
 *
 *
 *
 **/
public final class JobSigner {

	public static AbstractJob convertTo(AbstractJob ajo)
		throws
			InvalidClassException,
			NotSerializableException,
			IOException,
			SignatureException,
			ClassNotFoundException {

		AbstractJob ajo_copy = null;

		{ // set scope for memory management

			// get a copy of the AJO
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bos);
			oos.writeObject(ajo);
			oos.close();

			//  try {
			ObjectInputStream ois =
				new ObjectInputStream(
					new ByteArrayInputStream(bos.toByteArray()));

			bos = null; // memory management

			ajo_copy = (AbstractJob) ois.readObject();
			// }

			// catch(Exception ex) {
			// We have just created the stream so none of the exceptions
			// will be thrown so just swallow them
			// What about IO?
			// RRCHANGE:
			// Do not swallow exceptions here, because also ClassNotFoundExceptions
			// may occur. Anyway, if something goes wrong here, pass the exception
			// to the calling method.
			// }

		} // end of memory management scope

		toConsignForm(ajo_copy);

		return ajo_copy;

	}

	private static void scanActionGroup(ActionGroup me)
		throws SignatureException {
		// Convert all children
		Enumeration actions = me.getActions();
		while (actions.hasMoreElements()) {
			AbstractAction child = (AbstractAction) actions.nextElement();
			if (child instanceof AbstractJob) {
				toConsignForm((AbstractJob) child);
			} else if (child instanceof ConditionalAction) {

				ConditionalAction child_ca = (ConditionalAction) child;

				if (child_ca.getTrueBranch() instanceof AbstractJob) {
					toConsignForm((AbstractJob) child_ca.getTrueBranch());
				} else if (child_ca.getTrueBranch() instanceof ActionGroup) {
					scanActionGroup((ActionGroup) child_ca.getTrueBranch());
				}

				if (child_ca.getFalseBranch() instanceof AbstractJob) {
					toConsignForm((AbstractJob) child_ca.getFalseBranch());
				} else if (child_ca.getFalseBranch() instanceof ActionGroup) {
					scanActionGroup((ActionGroup) child_ca.getFalseBranch());
				}

			} else if (child instanceof ActionGroup) {
				scanActionGroup((ActionGroup) child);
			}
		}

	}

	private static void toConsignForm(AbstractJob me)
		throws SignatureException {

		// Make sure all descendant AJOs are done first
		scanActionGroup(me);

		// Convert self
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bos);

			// Write the AJO specification version to the output bytes
			oos.writeObject(
				Package.getPackage("org.unicore").getSpecificationVersion());

			// Write a time stamp to the output bytes
			oos.writeObject(new java.util.Date());

			// Do not want to transport old rubbish
			me.setConsignForm(null);
			me.setSignature(null);

			// This redundantly writes Id, Vsite, endorser to the byte stream
			// can live with this? (Must as the NJS can then safely
			// deserialise?)
			oos.writeObject(me);
			oos.close();

			// Memory management: can clear the AJOs editable DAG as
			// we have a copy in the bos
			me.removeAllActions(); // Clear the editable DAG

			me.setConsignForm(bos.toByteArray());
		} catch (Exception ex) {
			// Should not get any as we serialised/deserialised for a copy
		}

		Signature signature =
			AjoSigner.getSigner(me.getEndorser()).getInstance();
		if (signature == null) {
			throw new SignatureException("ConsignForm.convertTo() SignatureFactory did not return an instance.");
		}

		signature.update(me.getConsignForm());
		me.setSignature(signature.sign());

		// and clear the fields
		// No - needed for sub-AJOs me.setId((null);
		me.setExtraInformation(null);
		me.setSSO(null);
		// No - needed for sub-AJOs me.setVsite(null);
		// No - needed for sub-AJOs me.setEndorser(null);

		// Note that this does not clear any fields created by sub-classes
		// of AJO (e.g. Idiomatics) so that they are sent twice. This
		// is safe (see below)  but redundant

	}
}
