package org.unicore.ajo;

import org.unicore.Vsite;
import org.unicore.User;
import org.unicore.AJOIdentifier;

import org.unicore.sets.PortfolioSet;

/**
 * An AbstractJob contains contains a description of work to be performed
 * at a Vsite. 
 * <p>
 * An AbstractJob is the thing that is sent between client and server
 * to request the work, the AbstractActions contained in the AbstractJob 
 * define the work to be done.
 * <p>
 * @see org.unicore.outcome.AbstractJob_Outcome
 * @see org.unicore.utility.ConsignForm
 * @see org.unicore.utility.ResetIdentifiers
 * @see org.unicore.upl.ConsignJob
 *
 * @author M. Drescher (Pallas)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version AbstractJob.java 1.10 09/09/02 
 * 
 **/
public class AbstractJob extends ActionGroup {

    static final long serialVersionUID = -8858470115080050696L;

    public AbstractJob() {
        this("");
    }

    /**
     * Create a new AbstractJob.
     * <p>
     * @param name The name of the AbstractJob
     * @param extra_information Client supplied extra information about the AbstractJob
     * @param sso The SSO
     * @param vsite Where to execute the AJO
     * @param endorser The endorser of the AJO
     *
     **/
    public AbstractJob( String name,
                        byte[] extra_information,
                        byte[] sso, 
                        Vsite vsite,
                        User endorser
                      ) {
        super(name,extra_information);

        // Change Identifier type
        setId(new AJOIdentifier(name));

        this.sso = sso;
        this.vsite = vsite;
        this.endorser = endorser;
    }

    /**
     * Create a new AbstractJob.
     * <p>
     * @param name The name of the AbstractJob
     * @param vsite Where to execute the AJO
     *
     **/
    public AbstractJob( String name,
                        Vsite vsite
                      ) {
        this(name, (byte[]) null, (byte[]) null, vsite, null);
    }

    /**
     * Create a new AbstractJob.
     *
     **/
    public AbstractJob(String name) {
        this(name, (byte[]) null, (byte[]) null, (Vsite) null, (User) null);
    }

    /**
     * Return the AJO's Identifier.
     * <p>
     * This methods returns the correct class
     * (not AAIdentifier from getId(), which we cannot override)
     *
     **/
    public AJOIdentifier getAJOId() {return (AJOIdentifier)getId();}

    private byte[] sso;
    /**
     * Get the SSO.
     * <p>
     * The SSO (Site Security Object) is an extra means
     * of authentication of the user planted into the AJO for security
     * conscious sites. The format and details of this information
     * are the property of the target site and of no concern to others.
     *
     **/
    public byte[] getSSO() {return sso;}

    /**
     * Set the SSO.
     *
     **/
    public void setSSO(byte[] sso) {this.sso = sso;}

    private Vsite vsite;
    /**
     * Return the Vsite where the AJO is to be run.
     *
     **/
    public Vsite getVsite() {return vsite;}

    /**
     * Set the Vsite where the AJO is to be run.
     *<p>
     * This must be set so that the NJS can check that the Endorser
     * wanted to execute at the Vsite
     *
     **/
    public void setVsite(Vsite vsite) {this.vsite = vsite;}

    private User endorser;
    /**
     * Return the endorser of the AJO.
     * <p>
     * This is needed for sub-AJOs whwere the user might want to make
     * Xlogin or project selections for the sub-AJO's Vsite. This is optional,
     * if not found the NJS will use the endorser of the parent AJO when
     * consigning the sub-AJO.
     *
     **/
    public User getEndorser() {return endorser;}

    /**
     * Set the endorser of the AJO.
     *
     **/
    public void setEndorser(User endorser) {this.endorser = endorser;}

    private byte[] consign_form;
    /**
     * Return the data in the AJO in a form suitable to be placed
     * in a ConsignJob request (if set).
     * <p>
     * AJOs are passed between Unicore
     * clients and servers in an anonymous form which means
     * that they do not have to be deserialised at Gateways
     * or at other NJSs. This anonymous form is the serialisation
     * of the AJO into a byte array.
     * <p>
     * Before consigning an AJO must be placed in this form,
     * {@link org.unicore.utility.ConsignForm} will do this.
     * <p>
     * It is also advisable to ensure uniqueness of the AbstractAction
     * Ids by resetting them, see {@link org.unicore.utility.ResetIdentifiers} for how.
     *
     * @since AJO 3
     *
     **/
    public byte[] getConsignForm() {return consign_form;}

    /**
     * Set the data in the AJO in a form suitable to be placed
     * in a ConsignJob request.
     * <p>
     * Any existing signature is cleared.
     *
     **/
    public void setConsignForm(byte[] consign_form) {
        this.consign_form = consign_form;
        signature = null;  //can't be valid any more
    }

    private byte[] signature;
    /**
     * Get the signature of the consign form of the AJO (if set).
     * <p>
     * NJSs will check
     * the integrity of the consigned AJOs using this signature, which
     * should be created using the private key of the endorser of
     * this AJO (the person under whose Xlogin the AJO will be
     * executed).
     * <p>
     * The methods in {@link org.unicore.utility.ConsignForm} will set
     * this when the AJO is converted to its consign form.
     *
     * @since AJO 3
     *
     **/
    public byte[] getSignature() {return signature;}

    /**
     * Set the signature of the consign form of the AJO.
     * 
     **/
    public void setSignature(byte[] signature) { this.signature = signature;}

    private PortfolioSet streamed;
    /**
     * Portfolios consigned with this sub-AJO. 
     * <p>
     * The UPL provides a mechanism for transferring large files
     * along with an AJO without having to serialise the data in the files.
     * This is called streaming the files. Any files that need to be transferred from
     * the client to the Vsite can be streamed,  and if they are streamed, then
     * they need to be mentioned here (grouped into a {@link Portfolio}).
     * Each streamed Portfolio should also be mentioned inside
     * the AbstractJob as the product of a {@link DeclarePortfolio}, using
     * the {@link DeclarePortfolio#setPortfolio} methods to link 
     * the PortfolioIdentifiers.
     * <p>
     * If this AbstractJob is a sub-AJO, then
     * the current incarnations of the Portfolios in the parent AJO's Uspace 
     * will be streamed with the AJO consign.
     *
     * @since AJO 3
     *
     **/
    public PortfolioSet getStreamed() {return streamed;}

    /**
     * Set the Portfolios to be streamed with sub-AJOs. 
     * <p>
     * This will overwrite any streamed Portfolios alreay set.
     *
     * @since AJO 3
     *
     **/
    public void setStreamed(PortfolioSet streamed) {this.streamed =  streamed;}

    /**
     * Add a Portfolio to the streamed set.
     * <p>
     * Creates the set if required.
     *
     * @since AJO 3
     *
     **/
    public void addStreamed(Portfolio stream) {
        if(streamed == null) streamed = new PortfolioSet();
        if(!streamed.isIn(stream)) streamed.add(stream);
    }
        
    /**
     * Remove a Portfolio from the streamed set.
     *
     * @since AJO 3
     *
     **/
    public void removeStreamed(Portfolio stream) {
        if(streamed != null) streamed.remove(stream);
    }

    private org.unicore.resources.USpace uspace;
    /**
     * @return  The location of the Porttfolios to be streamed
     *
     **/
    public org.unicore.resources.USpace getStreamedLocation() {return uspace;}

    /**
     * @param uspace The location of the Porttfolios to be streamed
     *
     **/
    public void setStreamedLocation(org.unicore.resources.USpace uspace) {this.uspace = uspace;}

}
//
//  Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, Uspace for streamed Portfolios
//           6 Apr 2001 SvdB Added endorser field (for sub-AJOs)
//          21 Sep 2000 svdB added getAJOId
//          15 May 2000 SvdB Added remove streamed
//           5 May 2000 SvdB V3.0 changes
//           9 Feb 2000 SvdB V2.9 added consign form and set methods
//           7 Dec 1998 SvdB Second release to Unicore
//          12 Jun 1998 SvdB getAccount was mis-spelled
//          12 Jun 1998 SvdB Initial release to Unicore
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
