package org.unicore.ajo;

import org.unicore.sets.PortfolioSet;
import org.unicore.sets.PortfolioList;
import org.unicore.sets.OptionSet;
import org.unicore.sets.ResourceSet;

/**
 * CompileTask compiles source files.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version CompileTask.java 1.5 09/09/02 
 * 
 **/
public abstract class CompileTask extends ExecuteTask {

    static final long serialVersionUID = 7508358776176402351L;

    /**
     * Create a new CompileTask.
     * <p>
     * @param name The name of the CompileTask
     * @param extra_information Client supplied extra information about the CompileTask
     * @param resources Resources needed to execute the CompileTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param sources The source files
     * @param list_level Level of source listing to produce
     * @param opt_level Optimisation level to apply during compilation
     * @param includes Files included in compilation
     * @param compile_for_debug Compile for debug?
     * @param compile_for_profile Compile for profile?
     *
     **/
    protected CompileTask( String name,
                           byte[] extra_information,
                           ResourceSet resource_limits,
                           OptionSet environment_variables,
                           String command_line,
                           Portfolio stdin,
                           boolean redirect_stdout,
                           boolean redirect_stderr,
                           boolean verbose,
                           boolean version,
                           PortfolioSet sources,
                           ListingLevel list_level,
                           OptimisationLevel opt_level,
                           PortfolioList includes,
                           boolean compile_for_debug,
                           boolean compile_for_profile
                      ) {

        super(name,extra_information,resource_limits,
              environment_variables,
              command_line,stdin,redirect_stdout,redirect_stderr,verbose,version);

        this.sources = sources;
        this.list_level = list_level;
        this.opt_level = opt_level;
        this.includes = includes;
        this.compile_for_debug = compile_for_debug;
        this.compile_for_profile = compile_for_profile;

        objects = new Portfolio("objects");
    }

    private PortfolioSet sources;
    /**
     * Get the souce files to compile. 
     * <p>
     * There may be multiple sources of source files.
     * <p>
     * There must be at least one file in the incarnation 
     * of this set.
     *
     **/
    public PortfolioSet getSources() {return sources;}

    /**
     * Set the souce files to compile. 
     * <p>
     * This will overwrite any source code Portfolios alreay set.
     *
     * @since AJO 3
     *
     **/
    public void setSources(PortfolioSet sources) {this.sources =  sources;}

    /**
     * Add a Portfolio containing source code.
     * <p>
     * Creates the set if required.
     *
     * @since AJO 3
     *
     **/
    public void addSource(Portfolio source_files) {
        if(sources == null) sources = new PortfolioSet();
        if(!sources.isIn(source_files)) sources.add(source_files);
    }
        
    private ListingLevel list_level;
    /**
     * Get the level of code listing that the compilation should provide (if any).
     *
     **/
    public ListingLevel getListingLevel() {return list_level;}

    /**
     * Change the listing level.
     *
     * @since AJO 3
     **/
    public void setListingLevel(ListingLevel list_level) {this.list_level = list_level;}

    private Portfolio listings;
    /**
     * Get the files produced by the listing.
     *
     **/
    public Portfolio getListings() {
        if(list_level == ListingLevel.NONE) {
            return (Portfolio) null;
        }
        else {
            if(listings == null) listings = new Portfolio("listings");
            return listings;
        }
    }

    private Portfolio objects;
    /**
     * Get the object files produced by the  compilation.
     *
     **/
    public Portfolio getObjects() {return objects;}

    private OptimisationLevel opt_level;
    /**
     * Get the level of optimisation that the compiler should use.
     *
     **/
    public OptimisationLevel getOptimisationLevel() {return opt_level;}

    /**
     * Change the optimisation level.
     *
     * @since AJO 3
     *
     **/
    public void setOptimisationLevel(OptimisationLevel opt_level) {
        this.opt_level = opt_level;
    }

    private PortfolioList includes;
    /**
     * Get the files to add to the search path for includes. 
     * <p>
     * The NJS
     * may add further files if the resource set requests any
     * software resources that require them.
     *
     **/
    public PortfolioList getIncludes() {return includes;}

    /**
     * Change the files to add to the include path. 
     * <p>
     * This overwrites any previous values.
     *
     * @since AJO 3
     *
     **/
    public void setIncludes(PortfolioList includes) {this.includes = includes;}

    /**
     * Add a Portfolio to the end of the includes list.
     * <p>
     * Creates the list if required.
     *
     * @since AJO 3
     *
     **/
    public void addInclude(Portfolio include) {
        if(includes == null) includes = new PortfolioList();
        if(!includes.isIn(include)) includes.add(include);
    }
        
    private boolean compile_for_debug;
    /**
     * Should the compile produce debug code?
     * <p>
     * If this is true, then the source code should
     * be compiled with whatever flags are necessary to
     * produce debugging information at execute time
     * (note that this is still batch mode, interactive
     * debugging is not supported).
     *
     * @since AJO 3 (previously was a Context)
     *
     **/
    public boolean compileForDebug() {return compile_for_debug;}

    /**
     * Set debug code policy.
     *
     **/
    public void setCompileForDebug(boolean compile_for_debug) {
        this.compile_for_debug = compile_for_debug;
    }

    private boolean compile_for_profile;
    /**
     * Should the compile produce code for profiling the executable.
     * <p>
     * If this is true, then the source code should
     * be compiled to support profiling of the executable.
     *
     * @since AJO 3 (previously was a Context)
     *
     **/
    public boolean compileForProfile() {return compile_for_profile;}

    /**
     * Set the profile code policy.
     *
     **/
    public void setCompileForProfile(boolean compile_for_profile) {
        this.compile_for_profile = compile_for_profile;
    }

}
//
// Change history:
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//          04 Nov 1998 SvdB Added Includes field 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
