package org.unicore.ajo;

import org.unicore.AAIdentifier;

/**
 * Apply a control to an AbstractAction. 
 * <p>
 * The control action is applied to the target only 
 * and not to any children of the target. However, control actions may 
 * have effects that propagate to other AbstractActions in the same
 * job. For example, killing an AbstractAction will
 * cause all dependent AbstractActions to fail with {@link org.unicore.outcome.AbstractActionStatus#NEVER_RUN}.
 *<p>
 * Applying an action to an AJO has a direct effect only on the
 * AJO, the controls are not propagated to any sub-AJOs that
 * may have been consigned to other NJSs.
 *<p>
 * @see org.unicore.outcome.ControlAction_Outcome
 * @see Control
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version ControlAction.java 1.6 09/09/02 
 *
 **/
public final class ControlAction extends ControlService {

    static final long serialVersionUID = -6390449605356616516L;

    /**
     * Create a new ControlService.
     * <p>
     * @param name The name of the ControlService
     * @param extra_information Client supplied extra information about the ControlService
     * @param target_action The target AbstractAction for the control
     * @param control The type of control action to apply
     *
     **/
    public ControlAction(String name,
                         byte[] extra_information,
                         AAIdentifier target_action,
                         Control control) {
                            
        super(name,extra_information);

        this.target_action = target_action;
        this.control = control;
    }

    /**
     * Create a new ControlService.
     * <p>
     * @param name The name of the ControlService
     * @param target_action The target AbstractAction for the control
     * @param control The type of control action to apply
     *
     **/
    public ControlAction(String name,
                         AAIdentifier target_action,
                         Control control) {
        this(name, (byte[]) null, target_action, control);
    }

    /**
     * Create a new ControlService.
     * <p>
     * @param name The name of the ControlService
     *
     **/
    public ControlAction(String name) {
        this(name, (byte[]) null, (AAIdentifier) null, (Control) null);
    }

    public ControlAction() {
        this("");
    }

    private AAIdentifier target_action;
    /**
     * Get the AbstractAction to which the control should be applied.
     *
     **/
    public AAIdentifier getActionId() {return target_action;}

    /**
     * Set the AbstractAction to which the control should be applied.
     *
     * @since AJO 3
     *
     **/
    public void setActionId(AAIdentifier target_action) {this.target_action = target_action;}

    private Control control;
    /**
     * Get the control to apply to the AbstractAction.
     *
     **/
    public Control getControl() {return control;}

    /**
     * Set the control to apply to the AbstractAction.
     *
     * @since AJO 3
     *
     **/
    public void setControl(Control control) {this.control = control;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          21 Dec 2000 SvdB fixed setActionId (this) from Thomas Janke
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Constructor to public 
//           7 Dec 1998 SvdB Second release to Unicore 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
