package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Declare a Portfolio to hold files that are created in the Uspace
 * without executing a task.
 * <p>
 * Usually files appear in an AJO's Uspace after the execution
 * of a task (e.g. the executable file after a link). However, sometimes
 * Portfolios can be created in a Uspace by agents outside the AJO. Examples
 * are the execution of an external task,
 * such as {@link PutPortfolio}, or the UPL transfer of streamed
 * files.  These Portfolios are created "hidden" so that the receiving AJO
 * can control when they become visible and so overwrite 
 * existing files or replace the contents of existing Portfolios.
 * <p>
 * DeclarePortfolio succeeds if the target Portfolio was already revealed
 * in the Uspace or if the target Portfolio was hidden and the execution of 
 * DeclarePortfolio caused it to be revealed.
 * <p>
 * DeclarePortfolio supplies a Portfolio handle to 
 * files created by outside agents. This means that that other tasks in the AJO 
 * can reference and so use the files. The 
 * Portfolio is also passed to the outside agents so that they
 * can reference the files too.
 *<p>
 * The Portfolio is revealed in either the current Uspace directory (path in instance
 * of {@link org.unicore.resources.USpace}) or in another Uspace referred to by an instance
 * of {@link org.unicore.resources.AlternativeUspace} in the DeclarePortfolio's resources.
 *
 * @see Portfolio
 * @see org.unicore.outcome.DeclarePortfolio_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 1 as CreatePortfolioTask)
 *
 * @version DeclarePortfolio.java 1.7 09/09/02
 * 
 **/
public final class DeclarePortfolio extends FileTask {

    static final long serialVersionUID = 2212291408459464678L;

    /**
     * Create a new DeclarePortfolio with this name that
     * requests the given resources and saves the 
     * supplied extra information.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     *
     **/
    public DeclarePortfolio( String name,
                             byte[] extra_information,
                             ResourceSet resources
                           ) {

        super(name,extra_information,resources);
        this.portfolio = new Portfolio(name);
    }

    /**
     * Create a new DeclarePortfolio with this name that
     * requests the given resources.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     * @param name The name of the task.
     * @param resources The resources requested by this task.
     *
     **/
    public DeclarePortfolio( String name,
                             ResourceSet resources
                              ) {

        this(name,(byte[]) null,resources);
    }

    /**
     * Create a new DeclarePortfolio with this name and
     * empty values otherwise.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     **/
    public DeclarePortfolio(String name) {
        this(name, (byte[]) null, (ResourceSet) null);
    }

    public DeclarePortfolio() {
        this("");
    }

    private Portfolio portfolio;
    /**
     * Return the Portfolio created by this task.
     *
     **/
    public Portfolio getPortfolio() {
        return portfolio;
    }

    /**
     * Set the Portfolio created by this task.
     * <p>
     * This method is provided so that Portfolios
     * streamed from one Uspace to another can be properly matched.
     * The streamed Portfolio is created in the parent Uspace and
     * its identity is used for {@link AbstractJob#addStreamed(Portfolio)}
     * this method allows the same identity to be set in the child Uspace
     * and so linking the source files to the destination files.
     *
     **/
    public void setPortfolio(Portfolio portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          24 Nov 2000 SvdB No longer setPortfolio, improved comments
//          30 Jun 2000 SvdB Minimal constructor creates a Portfolio
//          10 May 2000 SvdB Created from MakePortfolio
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
