package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Test a file.
 * <p>
 * The attributes that can be tested are the same as those returned in 
 * directory listings ({@link org.unicore.outcome.XFile})
 * <p>
 * The value of each attribute is tested against the template in the FileCheck.
 * If any attribute fails to match the template value, then the {@link org.unicore.outcome.Decision}
 * is {@link org.unicore.outcome.FileCheck_Outcome}.NOT_SATISFIED.
 * If all the set attributes match, then the {@link org.unicore.outcome.Decision}
 * is {@link org.unicore.outcome.FileCheck_Outcome}.SATISFIED.
 * <p>
 * The location of the file is given by an instance of {@link org.unicore.resources.Storage}
 * in the set of resources to check (in the same way as for {@link FileAction}
 *
 * @see org.unicore.outcome.FileCheck_Outcome
 * @see org.unicore.outcome.XFile
 * @see If
 * @see RepeatGroup
 * 
 * @author D. Snelling (Fujitsu Laboratories of Europe)
 * @author S. van den Berghe (Fujitsu Laboratories of Europe)
 *
 * @since AJO 3
 *
 * @version FileCheck.java 1.12 10/30/02
 *
 **/
public final class FileCheck extends FileAction implements Decidable {

    static final long serialVersionUID = 400;

    /**
     * Create a new FileCheck.
     * 
     * @param name The name of the CheckResources
     * @param extra_information Client supplied extra information about the CheckResources
     * @param resources Resources needed to execute the FileCheck
     * @param file The file to check
     *
     **/
    public FileCheck(String name, byte[] extra_information, ResourceSet resources, String file) {
        super(name,extra_information,resources);

        this.file = file;
    }

    /**
     * Create a new FileCheck.
     * 
     * @param name The name of the CheckResources
     * @param file The file to check
     *
     **/
    public FileCheck(String name, String file) {
        this(name, (byte[]) null, (ResourceSet)null, file);
    }

    /**
     * Create a new FileCheck.
     * <p>
     * @param name The name of the CheckResources
     *
     **/
    public FileCheck(String name) {
        this(name,(byte[]) null, (ResourceSet)null, (String) null);
    }

    public FileCheck() {
        this("");
    }

    private String file;

    /**
     * Return the name of the file to check.
     *
     **/
    public String getFileToCheck() {return file;}

    /**
     * Set the name of the file to check.
     *
     **/
    public void setFileToCheck(String file) {this.file = file;}

    private Boolean file_exists;
    /**
     * Set the test on file existance.
     * <p>
     * file_exists = true, test result is true if the file exists at run time <p>
     * file_exists = false, test result is true if the file does not exist at run time <p>
     * file_exists = null, test result is not included in the final result
     *
     * @param file_exists The test condition on file existance
     *
     **/
    public void setFileExists(Boolean file_exists) {this.file_exists = file_exists;}

    /**
     * @return The value of the file existance test
     *
     **/
    public Boolean getFileExists() {return file_exists;}

    private Boolean can_read_file;
    /**
     * Set the test on file readability.
     * <p>
     * can_read_file = true, test result is true if the file can be read by the user (as incarnated) <p>
     * can_read_file = false, test result is true if the file cannot be read by the user (as incarnated) <p>
     * can_read_file = null, test result is not included in the final result
     *
     * @param file_readable The test condition on ability to read the file
     *
     **/
    public void setCanReadFile(Boolean can_read_file) {this.can_read_file = can_read_file;}

    /**
     * @return The value of the test condition on ability to read the file
     *
     **/
    public Boolean getCanReadFile() {return can_read_file;}

    private Boolean can_write_file;
    /**
     * Set the test on file writable.
     * <p>
     * can_write_file = true, test result is true if the file can be written by the user (as incarnated) <p>
     * can_write_file = false, test result is true if the file cannot be written by the user (as incarnated) <p>
     * can_write_file = null, test result is not included in the final result
     *
     * @param can_write_file The test condition on ability to write to the file
     *
     **/
    public void setCanWriteFile(Boolean can_write_file) {this.can_write_file = can_write_file;}

    /**
     * @return The value of the test condition on ability to write to the file
     *
     **/
    public Boolean getCanWriteFile() {return can_write_file;}

    private Boolean can_execute_file;
    /**
     * Set the test on file execution.
     * <p>
     * can_execute_file = true, test result is true if the file can be executed by the user (as incarnated) <p>
     * can_execute_file = false, test result is true if the file cannot be executed by the user (as incarnated) <p>
     * can_execute_file = null, test result is not included in the final result
     *
     * @param can_write_file The test condition on ability to execute the file
     *
     **/
    public void setCanExecuteFile(Boolean can_execute_file) {this.can_execute_file = can_execute_file;}

    /**
     * @return The value of the test condition on ability to execute the file
     *
     **/
    public Boolean getCanExecuteFile() {return can_execute_file;}

    private Boolean file_owned_by_user;
    /**
     * Set the test on file ownership.
     * <p>
     * file_owned_by_user = true, test result is true if the file is owned by the user (as incarnated) <p>
     * file_owned_by_user = false, test result is true if the file is not owned by the user (as incarnated) <p>
     * file_owned_by_user = null, test result is not included in the final result
     *
     * @param file_owned_by_user The test condition on ownership of the the file
     *
     **/
    public void setFileOwnedByUser(Boolean file_owned_by_user) {this.file_owned_by_user = file_owned_by_user;}

    /**
     * @return The value of the test condition on ownership of the the file
     *
     **/
    public Boolean getFileOwnedByUser() {return file_owned_by_user;}

    private Boolean is_directory;
    /**
     * Set the test on file being a directory.
     * <p>
     * is_directory = true, test result is true if the file is a directory (as incarnated) <p>
     * is_directory = false, test result is true if the file is not a directory (as incarnated) <p>
     * is_directory = null, test result is not included in the final result
     *
     * @param is_directory The test condition on the file being a directory
     *
     **/
    public void setIsDirectory(Boolean is_directory) {this.is_directory = is_directory;}

    /**
     * @return The value of the test condition on file being a directory.
     *
     **/
    public Boolean getIsDirectory() {return is_directory;}

    private Expression file_size;
    /**
     * Set the file size test expression. The value of this expression must be an java.lang.Long.
     * The expression can be null, in which case there is no test on file size. 
     *
     * @param file_size The test expression (null if this condition should not be part of the final result)
     * @throws ClassCastException If the expression value is not an java.lang.Long
     *
     **/
    public void setFileSize(Expression file_size) {
        if(file_size == null || file_size.getValue() instanceof java.lang.Long) {
            this.file_size = file_size;
        }
        else {
            throw new ClassCastException("The value of the expression for the file size in FileCheck must be a Long, got: "+file_size.getValue().getClass() );
        }
    }

    /**
     * @return The current file size expression
     *
     **/
    public Expression getFileSize() {return file_size;}

    private Expression file_modification_date;
    /**
     * Set the file date test expression. The value of this expression must be a java.util.Date
     * The expression can be null, in which case there is no test on the file modification date.
     *
     * @param file_modification_date The test expression (null if this condition should not be part of the final result)
     * @throws ClassCastException If the expression value is not a java.util.Date
     *
     **/
    public void setFileModificationDate(Expression file_modification_date) {
        if(file_modification_date == null || file_modification_date.getValue() instanceof java.util.Date) {
            this.file_modification_date = file_modification_date;
        }
        else {
            throw new ClassCastException("The value of the expression for the file date in FileCheck must be a java.util.Date, got: "+file_modification_date.getValue().getClass() );
        }
    }

    /**
     * @return The current file modification date expression
     *
     **/
    public Expression getFileModificationDate() {return file_modification_date;}

}
//
// Change history:
//          29 Oct 2002 SvdB added isDirectory()
//           8 Aug 2002 SvdB Decidable
//          14 May 2002 SvdB Moved from subclass of org.unicore.ajo.QueryService
//          15 Apr 2002 SvdB Modified to allow many tests
//          11 Dec 2001 DFS  Moved from a subclass of org.unicore.ajo.CheckResources
//          15 Aug 2000 SvdB Changed name of parent
//          15 May 2000 SvdB Created for V3.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
