package org.unicore.ajo;

/**
 * IfSuccess provides support for conditional execution of actions depending
 * on the success, or otherwise, of a preceeding action.
 * <p>
 * The IfSuccess action tests the final status of an AbstractAction and
 * if that action was successful the success branch is taken otherwise the fail
 * branch is taken. 
 * Both branches are AbstractActions,
 * and in practice they will be ActionGroups or AbstractJobs.
 * <p>
 * The IfSuccess will fail if the test action has not completed
 * execution.
 * <p>
 *
 * @see org.unicore.outcome.IfSuccess_Outcome
 *
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version IfSuccess.java 1.5 09/09/02
 *
 **/
public final class IfSuccess extends ConditionalAction {

    static final long serialVersionUID = -6205999028012095665L;

    /**
     * Create a new IfSuccess action.
     * <p>
     * @param name The name of the IfSuccess action
     * @param extra_information Client supplied extra information about the IfSuccess action
     * @param test_action The action tested for success. Must be a predecessor (direct or 
     *                    indirect) of the IfSuccess action and within the same JobGroup.
     * @param success_branch The action executed if the test action was successful.
     * @param fail_branch The action executed if the test action was not successful.
     *
     **/
    public IfSuccess (String name, 
                      byte[] extra_information, 
                      AbstractAction test_action,
                      AbstractAction success_branch,
                      AbstractAction fail_branch) {
        super(name,extra_information, success_branch, fail_branch);

        this.test_action    = test_action;
    }

    /**
     * Create a new IfSuccess action.
     * <p>
     * @param name The name of the IfSuccess action
     * @param test_action The action tested for success. Must be a predecessor (direct or 
     *                    indirect) of the IfSuccess action and within the same JobGroup.
     * @param success_branch The action executed if the test action was successful.
     * @param fail_branch The action executed if the test action was not successful.
     *
     **/
    public IfSuccess (String name, 
                      AbstractAction test_action,
                      AbstractAction success_branch,
                      AbstractAction fail_branch) {
        this(name, (byte[]) null, test_action, success_branch, fail_branch);
    }

    /**
     * Create a new IfSuccess action.
     * <p>
     * @param name The name of the IfSuccess action
     *
     **/
    public IfSuccess (String name) {
        this(name,(byte[]) null, (AbstractAction) null, (AbstractAction) null, (AbstractAction) null);
    }

    public IfSuccess () {
        this("");
    }

    private AbstractAction test_action;

    /**
     * Return the action to be tested for success of failure.
     *
     **/
    public AbstractAction getTestAction() {return test_action;}

    /**
     * Set the test action. The test action can be null, in which case the true branch is taken.
     *
     **/
    public void setTestAction (AbstractAction test_action) {this.test_action = test_action;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          31 Jan 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
