package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * IncarnateFiles creates files in the Uspace from data contained in the instance.
 * <p>
 * This is provided as an alternative way of transferring
 * files to Uspaces. This task places
 * the bytes of the files into the AJO and so is practical
 * only for small files. Large files should be streamed.
 * <p>
 * The files are incarnated in the the current Uspace directory (path
 * of the {@link org.unicore.resources.USpace}).
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version IncarnateFiles.java 1.6 09/18/02 
 * 
 **/
public final class IncarnateFiles extends FileTask {

    static final long serialVersionUID = -502036403802020091L;

    /**
     * Create a new IncarnateFiles.
     * <p>
     * @param name The name of the IncarnateFiles
     * @param extra_information Client supplied extra information about the IncarnateFiles
     * @param resources Resources needed to execute the IncarnateFiles
     * @param names The names of the files to create
     * @param contents The contents of the files to craete
     * @param overwrite Whether the new file should overwrite an existing file
     *
     **/
    public IncarnateFiles( String name,
                          byte[] extra_information,
                          ResourceSet resource_limits,
                          String[] names,
                          byte[][] contents,
                          boolean overwrite
                              ) {

        super(name,extra_information,resource_limits);

        this.names = names;
        this.contents = contents;
        this.overwrite = overwrite;
    }

    /**
     * Create a new IncarnateFiles.
     * <p>
     * @param name The name of the IncarnateFiles
     * @param resources Resources needed to execute the IncarnateFiles
     * @param names The names of the files to create
     * @param contents The contents of the files to craete
     * @param overwrite Whether the new file should overwrite an existing file
     *
     **/
    public IncarnateFiles( String name,
                          ResourceSet resource_limits,
                          String[] names,
                          byte[][] contents,
                          boolean overwrite
                              ) {
        this(name,(byte[]) null,resource_limits, names, contents, true);
    }

    /**
     * Create a new IncarnateFiles.
     * <p>
     * @param name The name of the IncarnateFiles
     *
     **/
    public IncarnateFiles(String name) {
        this(name,(byte[]) null, (ResourceSet) null, (String[]) null, (byte[][]) null, true);
    }

    public IncarnateFiles() {
        this("");
    }


    private String[] names;
    /**
     * Get the names of the files to be created.
     * <p>
     * This array must match the file contents array. No checking
     * done.
     **/
    public String[] getNames() {return names;}

    /**
     * Set the names of the files to be created.
     *
     **/
    public void setNames(String[] names) {this.names = names;}

    private byte[][] contents;
    /**
     * Return the contents for all the files.
     * <p>
     * This array must match the file names array. No checking
     * done.
     **/
    public byte[][] getContents() {return contents;}

    /**
     * Set new contents for all files.
     *
     **/
    public void setContents(byte[][] contents) {this.contents = contents;}

    /**
     * Add a file to the ones to be created.
     *
     * @param  name The name of the file to create
     * @param contents The file's contents when created
     *
     **/
    public void addFile(String name, byte[] file_contents) {
        // Assume that array set up properly
        if(names == null) {
            names = new String[1];
            names[0] = name;
        }
        else {
            String[] temp = new String[names.length+1];
            System.arraycopy(names,0,temp,0,names.length);
            temp[names.length] = name;
            names = temp;
        }

        if(contents == null) {
            contents = new byte[1][];
            contents[0] = file_contents;
        }
        else {
            byte[][] temp = new byte[contents.length+1][];
            System.arraycopy(contents,0,temp,0,contents.length);
            temp[contents.length] = file_contents;
            contents = temp;
        }
    }

    boolean overwrite;
    /**
     * Overwrite existing destination files?
     * <p>
     * If no overwriting is allowed (this method returns false), then the task will
     * fail if the copy would overwrite an existing file. 
     * <p>
     * Overwriting existing files is always
     * subject to permissions on the existing files and the task
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, etc
//          30 Apr 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
