package org.unicore.ajo;

import org.unicore.sets.PortfolioList;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.OptionSet;

/**
 * Link.
 * 
 * @see org.unicore.outcome.LinkTask_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version LinkTask.java 1.6 09/09/02 
 * 
 **/

public final class LinkTask extends ExecuteTask {

    static final long serialVersionUID = 5270898051818994506L;

    /**
     * Create a new LinkTask.
     * <p>
     * @param name The name of the LinkTask
     * @param extra_information Client supplied extra information about the LinkTask
     * @param resources Resources needed to execute the LinkTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param objects The object files to link
     * @param libraries The libraries to include in the link
     * @param map_level Level of mapping toproduce
     * @param link_for_debug Link for debug?
     * @param link_for_profile Link for profile?
     *
     **/
    public LinkTask( String name,
                     byte[] extra_information,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     PortfolioList objects,
                     PortfolioList libraries,
                     LinkMapLevel map_level,
                     boolean link_for_debug,
                     boolean link_for_profile
                   ) {

        super(name,extra_information,resource_limits,
              environment_variables,
              command_line,stdin, redirect_stdout, redirect_stderr, verbose,version);

        this.objects = objects;
        this.libraries = libraries;
        this.map_level = map_level;
        this.link_for_debug = link_for_debug;
        this.link_for_profile = link_for_profile;
    }

    /**
     * Create a new LinkTask.
     * <p>
     * @param name The name of the LinkTask
     * @param resources Resources needed to execute the LinkTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     * @param objects The object files to link
     * @param libraries The libraries to include in the link
     * @param map_level Level of mapping toproduce
     * @param link_for_debug Link for debug?
     * @param link_for_profile Link for profile?
     *
     **/
    public LinkTask( String name,
                     ResourceSet resource_limits,
                     OptionSet environment_variables,
                     String command_line,
                     Portfolio stdin,
                     boolean redirect_stdout,
                     boolean redirect_stderr,
                     boolean verbose,
                     boolean version,
                     PortfolioList objects,
                     PortfolioList libraries,
                     LinkMapLevel map_level,
                     boolean link_for_debug,
                     boolean link_for_profile
                   ) {

        this(name,(byte[]) null,resource_limits,
              environment_variables,
              command_line,stdin, redirect_stdout, redirect_stderr, verbose, version,
              objects, libraries, map_level, link_for_debug, link_for_profile);
    }

    /**
     * Create a new LinkTask.
     * <p>
     * @param name The name of the LinkTask
     *
     **/
    public LinkTask(String name) {
        this(name,(byte[]) null, (ResourceSet) null,
              (OptionSet) null,
              (String) null, (Portfolio) null, false, false, false, false,
              (PortfolioList) null, (PortfolioList) null, LinkMapLevel.NONE, false, false);
    }

    public LinkTask() {
        this("");
    }

    private PortfolioList objects;
    /**
     * Return the object files to be linked.
     *
     **/
    public PortfolioList getObjects() {return objects;}

    /**
     * Replace object files to be linked.
     *
     **/
    public void setObjects(PortfolioList objects) {this.objects = objects;}

    /**
     * Add an Object Portfolio to the end of the current list.
     *
     **/
    public void addObject(Portfolio object) {
        if(objects == null) objects = new PortfolioList();
        if(!objects.isIn(object)) objects.add(object);
    }

    private PortfolioList libraries;
    /**
     * Libraries to scan during the link. 
     * <p>
     * System and package libraries
     * will be added seamlessly. However, the user may have
     * his own, transferred in from elsewhere. 
     *
     **/
     public PortfolioList getLibraries() {return libraries;}

    /**
     * Replace the set of libraries.
     *
     **/
    public void setLibraries(PortfolioList libraries) {this.libraries = libraries;}

    /**
     * Add a Library to the end of the current list.
     *
     **/
    public void addLibrary(Portfolio library) {
        if(libraries == null) libraries = new PortfolioList();
        if(!libraries.isIn(library)) libraries.add(library);
    }

    private Portfolio executable;
    /**
     * The executable resulting from the link.
     * <p>
     * This will incarnate to a single file.
     *
     **/
    public Portfolio getExecutable() {
        if(executable == null) executable = new Portfolio("executable");
        return executable;
    }

    private LinkMapLevel map_level;
    /**
     * The linker may produce a map at various levels.
     *
     **/
    public LinkMapLevel getMapLevel() {return map_level;}

    /**
     * Set a new map level.
     *
     **/
    public void setMapLevel(LinkMapLevel map_level) {
        this.map_level =  map_level;
    }

    private Portfolio map;
    /**
     * Any map is placed in a file to be saved by the user.
     * <p>
     * If the system sends this to stdout, then the incarnation
     * process redirects this to this file.
     **/
    public Portfolio getMap() {
        if(map_level.equals(LinkMapLevel.NONE)) {
            return map;
        }
        else {
            if(map == null) map = new Portfolio("map");
            return map;
        }
    }


    private boolean link_for_debug;
    /**
     * If this is true, then the object code should
     * be linked to produce debugging information at execute time.
     * <p>
     * Note that this is still batch mode, interactive
     * debugging is not supported.
     *
     **/
    public boolean linkForDebug() {return link_for_debug;}

    /**
     * Set debug policy.
     *
     **/
    public void setLinkForDebug(boolean link_for_debug) {this.link_for_debug = link_for_debug;}

    private boolean link_for_profile;
    /**
     * If this is true, then the object code should
     * be linked to support profiling of the executable.
     *
     **/
    public boolean linkForProfile() {return link_for_profile;}

    /**
     * Set the profile policy.
     *
     **/
    public void setLinkForProfile(boolean link_for_profile) {this.link_for_profile = link_for_profile;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor
//          29 Jun 2000 SvdB added setMapLevel 
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
