package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Create a Portfolio from an explicit list of file names.
 * <p>
 * The execution of instances of {@link UserTask} or its
 * subclasses can create files in the AJO's Uspace. These
 * files need to be placed into Portfolios so that they can
 * be used by other tasks in the AJO. MakePortfolio does
 * this by creating a Portfolio from a list of files names.
 *<p>
 * Another use of this task is to perform a regrouping of
 * files so that processing can be applied to different
 * arrangements of files that appear in other Portfolios.
 * <p>
 * The Portfolio is created in the Uspace named by the
 * instance of {@link org.unicore.resources.USpace} or
 * {@link org.unicore.resources.AlternativeUspace}
 *
 * @see Portfolio
 * @see org.unicore.outcome.MakePortfolio_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 1 as CreatePortfolioTask)
 *
 * @version MakePortfolio.java 1.6 05/13/02 
 * 
 **/
public class MakePortfolio extends FileTask {

    static final long serialVersionUID = 2506689521180731219L;

    /**
     * Create a new MakePortfolio.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     * @param name The name of the task.
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     * @param list List of files that will make up the Portfolio at execute time.
     *
     **/
    public MakePortfolio( String name,
                          byte[] extra_information,
                          ResourceSet resources,
                          String[] list
                              ) {
        super(name,extra_information,resources);
        files = list;
    }

    /**
     * Create a new MakePortfolio.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     * @param name The name of the task.
     * @param resources The resources requested by this task.
     * @param list List of files that will make up the Portfolio at execute time.
     *
     **/
    public MakePortfolio( String name,
                          ResourceSet resources,
                          String[] files
                              ) {

        this(name,(byte[]) null,resources,files);
    }

    /**
     * Create a new MakePortfolio with this name and
     * no files in the Portfolio.
     * <p>
     * The name is also given to the created Portfolio
     * (this can be changed using {@link Portfolio#setName}).
     *
     **/
    public MakePortfolio(String name) {
        this(name, (byte[]) null, (ResourceSet) null, (String[]) null);
    }

    public MakePortfolio() {
        this("");
    }

    private String[] files;
    /**
     * Return the list of files that will make up
     * the Portfolio when this task is executed.
     *
     **/
    public String[] getFiles() {return files;}

    /**
     * Set a new list of files for the Portfolio that this
     * task creates.
     * <p>
     * Each file name must follow the rules of {@link #addFile}.
     * 
     **/
    public void setFiles(String[] list) {files = list;}

    /**
     * Add a file to the list of files making up this
     * Portfolio.
     * <p>
     * The file must exist in the Uspace of the AJO when
     * this task is executed. The current working directory
     * is set to the Uspace when the task is executed.
     * <p>
     * The file name can be globbed (*, ? and []) and
     * any files that match the globbing when the task is 
     * executed are included in the Portfolio.
     * 
     **/
    public void addFile(String name) {
        if(files == null) {
            files = new String[1];
            files[0] = name;
        }
        else {
            String[] temp = new String[files.length+1];
            System.arraycopy(files,0,temp,0,files.length);
            temp[files.length] = name;
            files = temp;
        }
    }

    private Portfolio portfolio;
    /**
     * Return the Portfolio created by this task.
     *
     **/
    public Portfolio getPortfolio() {
        if(portfolio == null) portfolio = new Portfolio(getName());
        return portfolio;
    }


}
//
// Change history:
//          11 Apr 2002 SvdB Removed deprecated constructor
//          18 Mar 2002 SvdB V4 no arg constructor
//          18 Jan 2001 SvdB No longer final (NJS hack)
//          24 Nov 2000 SvdB Improved comments
//          10 May 2000 SvdB Split make and declare functions 
//          30 Apr 2000 SvdB Created from CreatePortfolioTask 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
