package org.unicore.idiomatic;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.resources.*;

/**
 * Parent to TransferFile and Stream.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version StreamFile.java 1.1 04/17/03 <br>
 * 
 **/
public abstract class StreamFile extends ActionGroup implements Idiomatic {

    public static class AJO extends AbstractJob implements Idiomatic {

        public AJO(String name, byte[] sso, Vsite target_vsite, User endorser, Portfolio streamed) {

            super(name+"-StreamFileAJO", null, sso, target_vsite, endorser);

            addStreamed(streamed);

            // Export the file
            export_it = new CopyFile(name+"-StreamFileAJO-ExportTask");
            export_it.setCopyFromUspace(true);

            // Get transferrred file visible in Uspace
            declare_portfolio = new DeclarePortfolio(name+"-StreamFileAJO-DeclarePortfolio");
            declare_portfolio.setPortfolio(streamed);

            add(declare_portfolio);
            add(export_it);
            add(new Dependency(declare_portfolio, export_it));

        }

        private CopyFile export_it;
        /**
         * Return the AbstractAction that represents the copy of the file to its
         * target (the file transfer initially places the file in a temporary
         * directory).
         *
         * @return The exporting AbstractAction (always present).
         *
         **/
        public CopyFile getExport() {return export_it;}

        private DeclarePortfolio declare_portfolio;
        /**
         * Return the AbstractAction that maked teh transferred file visible to
         * the export.
         *
         * @return The DeclarePortfolio (always present).
         *
         **/
        public DeclarePortfolio getDeclarePortfolio() {return declare_portfolio;}

        public void setSourceFile(String source_file) {
            export_it.setFrom(source_file);
        }

        public String getSourceFile() {
            return export_it.getFrom();
        }

        private PathedStorage target_storage;
        /**
         * @return The target Storage
         *
         **/
        public PathedStorage getTargetStorage() {return target_storage;}

        /**
         * @param target_storage The new value for the target storage
         *
         **/
        public void setTargetStorage(PathedStorage target_storage) {
            export_it.removeResource(this.target_storage);
            export_it.addResource(target_storage);
            this.target_storage = target_storage;
        }

        /**
         * @return The target file
         *
         **/
        public String getTargetFile() {return export_it.getTo();}

        /**
         * @param target_file The new target file
         *
         **/
        public void setTargetFile(String target_file) {
            export_it.setTo(target_file);
        }


        public boolean willOverwrite() {return export_it.willOverwrite();}

        public void setOverwrite(boolean overwrite) {export_it.setOverwrite(overwrite);}

    }

    /**
     * Create a new StreamFile
     *
     * @param name A name for the TransferFile
     * @param extra_information Storage for data from the creator of this TransferFile
     * @param sso security object
     * @param endorser The endorser of the AJO (see {@link org.unicore.ajo.AbstractJob#setEndorser})
     * @param source_file The file to be copied
     * @param source_storage The {@link org.unicore.resources.Storage} where the file can be found
     * @param target_file The destination file name
     * @param target_storage The {@link org.unicore.resources.Storage} where the file is placed
     * @param target_vsite The destination Vsite
     * @param overwrite Should the target overwrite an existing to file
     *
     **/
    protected StreamFile( String name,
                     byte[] extra_information,
                     byte[] sso, 
                     User endorser,
                     String source_file,
                     PathedStorage source_storage,
                     String target_file,
                     PathedStorage target_storage,
                     Vsite target_vsite,
                     boolean overwrite
                   ) {
        super(name,extra_information);

        makepf_it = new MakePortfolio(name+"-MakePortfolio");

        // Transfer to the destination
        remote_job = new AJO(name,sso,target_vsite,endorser, makepf_it.getPortfolio());
            remote_job.setTargetFile(target_file);
            remote_job.setTargetStorage(target_storage);
            remote_job.setOverwrite(overwrite);

        add(makepf_it);
        add(remote_job);
        add(new Dependency(makepf_it, remote_job));

        setSourceFile(source_file);
        _setSourceStorage(source_storage);

    }

    private CopyFile import_it;
    /**
     * Return the AbstractAction that will copy the source file from Xspace
     * to Uspace (if required).
     *
     * @return The importing AbstractAction (null if not required i.e. source file is in Uspace)
     *
     **/
    protected CopyFile getImport() {return import_it;}

    private MakePortfolio makepf_it;
    /**
     * Return the AbstractAction use to wrap the source file (in Uspace) in
     * A Portfolio so that it can be transferred to  teh destination Vsite.
     *
     * @return The MakePortfolio (always present).
     *
     **/
    public MakePortfolio getMakePortfolio() {return makepf_it;}

    private AJO remote_job;
    /**
     * Return the AbtsractJob that effects the transfer of the file between Uspaces.
     *
     * @return The AJO (always prtesent).
     *
     **/
    public AJO getRemoteJob() {return remote_job;}

    private DeleteFile delete_it;
    /**
     * Return the AbstractAction that deletes the copy of the source in Uspace.
     *
     * @return The DeleteFile (null if not required i.e. source file was in Uspace)
     *
     **/
    protected DeleteFile getDelete() {return delete_it;}

    public Vsite getTargetVsite() {return remote_job.getVsite();}

    public void setTargetVsite(Vsite target_vsite) {
        remote_job.setVsite(target_vsite);
    }

    public User getUser() {return remote_job.getEndorser();}

    public void setUser(User user) {remote_job.setEndorser(user);}

    /**
     * @return The source file
     *
     **/
    public String getSourceFile() {return import_it.getFrom();}

    /**
     * @param source_file The new source file
     *
     **/
    public void setSourceFile(String source_file) {
        if(import_it == null) {
            makepf_it.setFiles(new String[] {source_file});
            remote_job.setSourceFile(source_file);
        }
        else {
            import_it.setFrom(source_file);
        }
    }

    private PathedStorage source_storage;
    /**
     * @return The source Storage
     *
     **/
    protected PathedStorage _getSourceStorage() {return source_storage;}

    /**
     * @param source_storage The new value for the source storage
     *
     **/
    protected void _setSourceStorage(PathedStorage source_storage) {
        if(source_storage instanceof USpace) {
            if(import_it != null) {
                // This is now from within the Uspace (previously from outside). We can
                // clean up a bit to avoid an unnecessary copy.
                makepf_it.setFiles(new String[] {import_it.getFrom()});
                remote_job.setSourceFile(import_it.getFrom());
                remove(import_it);
                import_it = null;
                remove(delete_it);
                delete_it = null;
            }
            makepf_it.removeResource(this.source_storage);
            makepf_it.addResource(source_storage);
        }
        else {
            if(import_it==null) {
                // This is now from outside the Uspace but we were previously set up to
                // transfer from the Uspace. We must add tasks to import the file to the
                // Uspace, adjust the file making up the transferred Portfolio and then clean up
                // after the transfer
                import_it = new CopyFile(getName()+"-Import");
                import_it.setCopyFromUspace(false);

                String temp_file = ".transferfile.tempfile"+Integer.toHexString(import_it.getId().getValue());;

                import_it.setTo(temp_file);
                import_it.setOverwrite(false);  // just in case

                add(import_it);
                add(new Dependency(import_it, makepf_it));

                makepf_it.setFiles(new String[] {temp_file});

                remote_job.setSourceFile(temp_file);

                delete_it = new DeleteFile(getName()+"-DeleteTempFile",(byte[])null,null,temp_file);
                add(delete_it);
                add(new Dependency(remote_job,delete_it));
            }

            import_it.removeResource(this.source_storage);
            import_it.addResource(source_storage);
        }

        this.source_storage = source_storage;
    }

    /**
     * @return The target Storage
     *
     **/
    protected PathedStorage _getTargetStorage() {return remote_job.getTargetStorage();}

    /**
     * @param target_storage The new value for the target storage
     *
     **/
    protected void _setTargetStorage(PathedStorage target_storage) {
        remote_job.setTargetStorage(target_storage);
    }

    /**
     * @return The target file
     *
     **/
    public String getTargetFile() {return remote_job.getTargetFile();}

    /**
     * @param target_file The new target file
     *
     **/
    public void setTargetFile(String target_file) {
        remote_job.setTargetFile(target_file);
    }


    public boolean willOverwrite() {return remote_job.willOverwrite();}

    public void setOverwrite(boolean overwrite) {remote_job.setOverwrite(overwrite);}

    static final long serialVersionUID = 400;
}
//
//  Change history:
//          04 Apr 2003 SvdB Avoid source file copy if in Uspace, made parent class
//          23 Apr 2002 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2002 - 2003
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
