package org.unicore.idiomatic;

import org.unicore.PortfolioIdentifier;
import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.User;

import org.unicore.sets.ResourceSet;

import org.unicore.ajo.Portfolio;
import org.unicore.ajo.DeclarePortfolio;
import org.unicore.ajo.PutPortfolio;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.AbstractJob;

import org.unicore.resources.*;


/**
 * Transfer a Portfolio from one Uspace to another (on a different Vsite).
 * <p>
 * The implementation transfers the source Portfolio to
 * the remote Vsite as streamed bytes following the consign of the AJO.
 * The executed AJO contains a {@link org.unicore.ajo.DeclarePortfolio} to announce
 * the Portfolio to the Uspace followed by a {@link org.unicore.ajo.PutPortfolio}
 * to write the Portfolio to the target Uspace.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version Transfer.java 1.12 08/13/02 <br>
 * 
 **/
public class Transfer extends AbstractJob implements Idiomatic {

    /**
     * Create a new Transfer initialised with the supplied information.
     *
     * @param name A name for the Transfer
     * @param extra_information Storage for data from the creator of this Transfer.
     * @param sso security object
     * @param endorser The endorser of the AJO (see {@link org.unicore.ajo.AbstractJob#setEndorser})
     * @param source_portfolio The Portfolio to be copied
     * @param target_portfolio The destination Portfolio
     * @param target_uspace The Uspace containing the target Portfolio
     * @param target_vsite The destination Vsite
     * @param resources The resources required by the Transfer
     * @param target_must_exist If this is true, then the target Portfolio must exist
     *                          when the transfer takes place (otherwise the Transfer fails).
     *                          If this is false, then the Portfolio is created if it does
     *                          not exist.
     *
     * @deprecated AJO 4.0 No target_must_exist
     *
     **/
    public Transfer( String name,
                     byte[] extra_information,
                     byte[] sso, 
                     User endorser,
                     Portfolio source_portfolio,
                     PortfolioIdentifier target_portfolio,
                     AJOIdentifier target_uspace,
                     Vsite target_vsite,
                     ResourceSet resources,
                     boolean target_must_exist
                   ) {
        this(name,extra_information,sso,endorser,source_portfolio,(USpace)null,target_portfolio,target_uspace,target_vsite,true);
        setTargetMustExist(target_must_exist);
    }

    /**
     * Create a new Transfer initialised with the supplied information.
     *
     * @param name A name for the Transfer
     * @param extra_information Storage for data from the creator of this Transfer.
     * @param sso security object
     * @param endorser The endorser of the AJO (see {@link org.unicore.ajo.AbstractJob#setEndorser})
     * @param source_portfolio The Portfolio to be copied
     * @param uspace Location of Portfolio in current Uspace
     * @param target_portfolio The destination Portfolio
     * @param target_uspace The Uspace containing the target Portfolio
     * @param target_vsite The destination Vsite
     * @param overwrite Should the transferred files overwrite existing files
     *
     **/
    public Transfer( String name,
                     byte[] extra_information,
                     byte[] sso, 
                     User endorser,
                     Portfolio source_portfolio,
                     USpace uspace,
                     PortfolioIdentifier target_portfolio,
                     AJOIdentifier target_uspace,
                     Vsite target_vsite,
                     boolean overwrite
                   ) {
        super(name,extra_information,sso,target_vsite,endorser);

        // Also add the Portfolio to be streamed with the consign
        addStreamed(source_portfolio);
        setStreamedLocation(uspace);

        // -------- Create the constituent tasks

        // Declare the streamed Portfolio in the staging Uspace
        declare_portfolio = new DeclarePortfolio( name+"-DeclarePortfolio", 
                                                  (byte []) null, 
                                                  (ResourceSet) null); 

        declare_portfolio.setPortfolio(source_portfolio);

        // Now the task to transfer the files from the staging Uspace
        // into the target Uspace.
        put_portfolio = new PutPortfolio(name+"-PutPortfolio");
        setOverwrite(overwrite);
        setSourcePortfolio(source_portfolio);
        setTargetPortfolio(target_portfolio);
        setTargetUspace(target_uspace);

        // -------- add to AJO
        add(declare_portfolio);
        add(put_portfolio);
        add(new Dependency(declare_portfolio,put_portfolio));

        // Keep a copy of the supplied resources, these apply to the 
        // resources needed for the transfer between Vsites _not_
        // to resources used by constituent tasks.
        this.resources = resources;
    }

    /**
     * Created an unitialised Transfer.
     *
     **/
    public Transfer(String name) {
        this(name, (byte[]) null, (byte[]) null, (User)null, (Portfolio) null, (USpace)null,
                  (PortfolioIdentifier) null, (AJOIdentifier) null, (Vsite) null, true);
    }

    public Transfer() {
        this("");
    }

    private DeclarePortfolio declare_portfolio;
    public DeclarePortfolio getDeclarePortfolio() {return declare_portfolio;}

    private PutPortfolio put_portfolio;
    public PutPortfolio getPutPortfolio() {return put_portfolio;}

    /**
     * Return the Portfolio to be transferred.
     *
     **/
    public Portfolio getSourcePortfolio() {return declare_portfolio.getPortfolio();}

    /**
     * Set the Portfolio to be transferred.
     *
     **/
    public void setSourcePortfolio(Portfolio source_portfolio) {
        // Remove old one from the streamed set
        removeStreamed(declare_portfolio.getPortfolio());

        addStreamed(source_portfolio);
        declare_portfolio.setPortfolio(source_portfolio);
        put_portfolio.setSource(source_portfolio);
    }

    /**
     * Return the target Portfolio
     *
     **/
    public PortfolioIdentifier getTargetPortfolio() {return put_portfolio.getTarget();}

    /**
     * Set the target Portfolio
     *
     **/
    public void setTargetPortfolio(PortfolioIdentifier target_portfolio) {
        put_portfolio.setTarget(target_portfolio);
    }

    private AJOIdentifier target_uspace;
    private AlternativeUspace aus;
    /**
     * Return the Uspace containing the target Portfolio
     *
     **/
    public AJOIdentifier getTargetUspace() {return target_uspace;}

    /**
     * Set the Uspace containing the target Portfolio
     *
     **/
    public void setTargetUspace(AJOIdentifier target_uspace) {
        AlternativeUspace new_aus = new AlternativeUspace("Transfer created",0.0,0.0,0.0,"",target_uspace); // to root!
        put_portfolio.removeResource(aus);
        put_portfolio.addResource(new_aus);
        aus = new_aus;
        this.target_uspace = target_uspace;
    }

    /**
     * Return the Vsite containing the Uspace containing the target Portfolio
     *
     **/
    public Vsite getTargetVsite() {return getVsite();}

    /**
     * Set the Vsite containing the Uspace containing the target Portfolio
     *
     **/
    public void setTargetVsite(Vsite target_vsite) {
        setVsite(target_vsite);
    }

    /**
     * @since AJO 4.0
     *
     **/
    public boolean willOverwrite() {return put_portfolio.willOverwrite();}

    /**
     * @since AJO 4.0
     *
     **/
    public void setOverwrite(boolean overwrite) {put_portfolio.setOverwrite(overwrite);}

    /**
     * Action to take if the target Portfolio does not exist.
     * <p>
     * If this is true, then the Transfer succeeds only if the 
     * target Portfolio exists. Otherwise, the Transfer creates
     * a new Portfolio and, if necessary, the Uspace to hold it.
     *
     * @deprecated AJO 4.0 Since the equivalent field in {@link org.unicore.ajo.PutPortfolio} is deprecated
     * 
     **/
    public boolean targetMustExist() {return put_portfolio.targetMustExist();}

    /**
     * Set the action to take if the target Portfolio does not exist.
     *
     * @deprecated AJO 4.0 Since the equivalent field in {@link org.unicore.ajo.PutPortfolio} is deprecated
     * 
     **/
    public void setTargetMustExist(boolean target_must_exist) {
        put_portfolio.setTargetMustExist(target_must_exist);
    }
    
    private ResourceSet resources;
    /**
     * The resources needed by the transfer. 
     *
     **/
    public ResourceSet getResources() {return resources;}

    /**
     * Change the resources needed. 
     * <p>
     * This overwrites any previous values.
     *
     **/
    public void setResources(ResourceSet resources) {
        this.resources = resources;
    }

    /**
     * Add a resource to the requested set.
     *
     **/
    public void addResource(Resource resource) {
        if(resources == null) resources = new ResourceSet();
        if(!resources.isIn(resource)) resources.add(resource);
    }

    static final long serialVersionUID = 6683014485927016621L;
}
//
//  Change history:
//          15 Mar 2002 SvdB V4, no arg constructor, etc
//          15 May 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
