package org.unicore.outcome;

import java.util.Hashtable;

import org.unicore.ajo.ActionGroup;
import org.unicore.AAIdentifier;

import org.unicore.sets.OutcomeEnumeration;

/**
 * The results of executing an {@link org.unicore.ajo.ActionGroup}. 
 * <p>
 * An ActionGroup is a container for a number of other
 * AbstractAction (its children) and so its Outcome is a
 * collection of the Outcomes of its children.
 * <p>
 * The logging and status information in the ActionGroup_Outcome
 * relate to the execution of the ActionGroup alone (information
 * about the execution of each of the children is contained in the
 * Outcome of the child).
 * <p>
 * The status will be EXECUTING while any child AbstractAction
 * is still executing. It will be SUCCESSFUL if all its children
 * have finished executing and have a SUCCESSFUL status. If any
 * of the ActionGroup's children is NOT_SUCCESSFUL, then the
 * ActionGroup finishes with a status of NOT_SUCCESSFUL.
 *
 * @see org.unicore.ajo.ActionGroup
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2 (as AbstractJob_Outcome)
 *
 * @version ActionGroup_Outcome.java 1.7 09/09/02 
 * 
 **/
public class ActionGroup_Outcome extends Outcome {

    static final long serialVersionUID = -7099362075416022029L;

    protected Hashtable child_outcomes;

    public ActionGroup_Outcome() {
        this((AAIdentifier)null, (AbstractActionStatus)null);
    }

    /**
     * Create a new ActionGroup_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus the Outcomes of the child AbstractActions.
     *
     * @param source The ActionGroup for which this Outcome will hold the results.
     * @param initial The status of the ActionGroup at the time that the Outcome is created.
     *
     **/
    public ActionGroup_Outcome( ActionGroup          source,
                                AbstractActionStatus initial_status
                              ) {
        super(source, initial_status);
        child_outcomes = new Hashtable(11);
    }

    /**
     * Create a new ActionGroup_Outcome.
     *
     * @param source The Identifier of the ActionGroup for which this Outcome will hold the results.
     * @param initial The status of the ActionGroup at the time that the Outcome is created.
     *
     **/
    public ActionGroup_Outcome( AAIdentifier          source_id,
                                AbstractActionStatus initial_status
                              ) {
        super(source_id, initial_status);
        child_outcomes = new Hashtable(11);
    }

    /**
     * Add an Outcome to the collection of child Outcomes.
     * <p>
     * <b>Pre:</b> There is no Outcome for the AbstractAction
     *               in this AJOOoutcome
     * <br>
     * <b>Post:</b> The Outcome is in this AJOOutcome
     * <p>
     **/
    public void addOutcome(Outcome outcome) {
        child_outcomes.put(outcome.getId(),outcome);
    }

    /**
     * Returns an Outcome for an AbstractAction.
     * <p>
     * <b>Pre:</b> true
     * <br>
     * <b>Post:</b> Returns the Outcome or null if there is no
     *              Outcome for this AbstractAction
     *
     **/
    public Outcome getOutcome(AAIdentifier action_id) {
        return (Outcome) child_outcomes.get(action_id);
    }

    /**
     * Returns an enumeration over the Outcomes held by this.
     * <p>
     * <b>Pre:</b> true
     * <br>
     * <b>Post:</b> Returns the enumeration
     * <p>
     **/
    public OutcomeEnumeration getOutcomes() {
        return new OutcomeEnumeration(child_outcomes);
    }

    
}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor
//          31 Jan 2001 SvdB Added Identifier constructor
//          20 Dec 2000 SvdB Removed clone method
//           5 May 2000 SvdB Created from AbstractJob_Outcome for V3.0 
//          11 Mar 1999 SvdB added getOutcomes method 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
