package org.unicore.outcome;

import org.unicore.Unicore;

import org.unicore.ajo.AbstractAction;
import org.unicore.AAIdentifier;

/**
 * An Outcome contains the results 
 * of executing an {@link org.unicore.ajo.AbstractAction}.
 * <p>
 * All Outcomes contain information from the NJS
 * about their handling and execution. Many Outcomes
 * also contain results produced by their execution
 * on the target systems.
 * <p>
 * All Outcomes contain the status of their AbstractAction. If the
 * AbstractAction has completed execution, then the status is the final status
 * of the AbstractAction. If the AbstractAction has not finished
 * execution (for example because the Outcome was fetched by a 
 * {@link org.unicore.ajo.CopyActionOutcome}), then the status is the 
 * latest status available to the NJS.
 * <p>
 * A history of status changes is maintained (with time of change and the given reason)
 * <p>
 * All Outcomes have a reason field that describes why the
 * status was set. This is useful when the AbstractAction's execution
 * failed when the reason should explain why the AbstractAction failed.
 * <p>
 * All Outcomes contain a {@link Log} which contains a more detailed
 * history of the processing and execution of the AbstractAction.
 * The log should detailed explanations of failures or problems.
 * <p>
 * The format of the extra output from the execution of the AbstractAction
 * is different for each type and is described in their documentation.
 *
 * @see org.unicore.Identifier
 * @see org.unicore.ajo.AbstractAction
 * @see org.unicore.upl.RetrieveOutcomeReply
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version Outcome.java 1.10 09/09/02 
 * 
 **/
public class Outcome extends Unicore {

    static final long serialVersionUID = 5260478148461056178L;
 
    public Outcome() {
        this((AbstractAction)null,null);
    }

    /**
     * Create a new Outcome.
     * <p>
     * This Outcome will hold log information only with no results
     * from the AbstractAction's execution. Instance of Outcome are
     * returned from a UPL {@link org.unicore.upl.RetrieveOutcome} request
     * when the target AJO has not finished execution.
     *
     * @param source The AbstractAction for which this Outcome will hold the results.
     * @param initial The status of the AbstractAction at the time that the Outcome is created
     *
     **/
    public Outcome(AbstractAction source, AbstractActionStatus initial) {
        if(source != null) id = source.getId();

        log = new Log();
        status_history = new StatusHistory();

        setStatus(initial,"Initial status");
    }

    /**
     * Create a new Outcome.
     * <p>
     * @param source The Identifier of tthe AbstractAction for which this Outcome will hold the results.
     * @param initial The status of the AbstractAction at the time that the Outcome is created
     *
     **/
    public Outcome(AAIdentifier source_id, AbstractActionStatus initial) {
        id = source_id;

        status_history = new StatusHistory();
        log = new Log();

        setStatus(initial,"Initial status");
    }

    protected AAIdentifier id;

    /**
     * Return the identifier of the Outcome.
     * <p>
     * This Id is used to identify a unique instance of an AbstractAction within
     * Unicore. It is also used to tie this Outcome to the source
     * AbstractAction as they have the same Identifier (but they are not equal
     * as the class is different).
     *
     **/
    public AAIdentifier getId() {return id;}

    /**
     * Set the Identifier of the Outcome.
     * <p>
     * Note that as the Identifier is the link between
     * the Outcome and its source AbstractAction and Outcome's
     * Identifier should be changed only if the source's
     * Identifier has been changed.
     *
     **/
    public void setId(AAIdentifier id) {this.id = id;}

    /**
     * Test for equality. 
     * <p>
     * An Outcome is identified by its Identifier - which is
     * the same as the Identifier of the AbstractAction that is
     * the source of the Outcome. As the Id is used to identify the Outcome, we
     * need to provide an implementation of equals that uses
     * this value.
     *
     **/
    public boolean equals(Object in) {
        return( in.getClass() ==  this.getClass() &&
                id.equals(((Outcome)in).getId()) 
              );
    }

    /**
     * Return a hash code based on the the Identifier value.
     * <p>
     * This is needed as with have changed the implementation of 
     * equals.
     *
     **/
    public int hashCode() {return id.hashCode();}

    /**
     * Return the last set status of the source AbstractAction.
     *
     **/
    public AbstractActionStatus getStatus() {return status_history.getCurrentStatus();}

    /**
     * Change the status.
     *
     **/
    public void setStatus(AbstractActionStatus new_status) {
        setStatus(new_status, (String) "");
    }

    /**
     * Change the status with the given reason
     *
     * @since AJO 4.0
     *
     **/
    public void setStatus(AbstractActionStatus new_status, String new_reason) {
        status_history.add(new_status, new_reason);
    }
    

    /**
     * Get the reason for the last status change.
     *
     **/
    public String getReason() {return status_history.getLastReason();}

    /**
     * Set the reason for the last status change.
     *
     * @deprecated At AJO 4.0 Correct call is through {@link #setStatus(AbstractActionStatus,String)}
     *
     **/
    public void setReason(String reason) {};

    protected Log log;
    /**
     * Get the reference to the log of the execution of the 
     * source AbstractAction.
     *
     **/
    public Log getLog() {return log;}
    
    /**
     * Set a new log.
     *
     * @deprecated As of AJO 4.0 use {@link Log#merge} to stop information loss
     *
     **/
    public void setLog(Log log) {this.log = log;}

    /**
     * Add an entry to the log of the execution of the
     * source AbstractAction.
     *
     **/
    public void addLogEntry(String entry) {log.add(entry);}

    private StatusHistory status_history;
    /**
     * Return the history of status changes
     *
     * @since AJO 4.0
     *
     **/
    public StatusHistory getStatusHistory() {return status_history;}

    public void setStatusHistory(StatusHistory status_history) {this.status_history = status_history;}

    private java.util.Date current_termination_time;

    public void setCurrentTerminationTime(java.util.Date new_termination_time) {
        current_termination_time = new_termination_time;
    }

    /**
     * Return the current termination time of the AbstactAction.
     *
     * @return The current termination time, or null if none set
     *
     **/
    public java.util.Date getCurrentTerminationTime() {
        return current_termination_time;
    }

}
//
// Change history:
//          04 Sep 2002 SvdB Corrected equals (not to AbstractAction!!)
//          28 Jul 2002 SvdB Added current termination time
//          11 Apr 2002 SvdB Added status history recording (corrected reason etc)
//          18 Mar 2002 SvdB V4 no arg constructor
//          31 Jan 2001 SvdB Added constructor taking Identifier
//          11 May 2000 SvdB Added reason field 
//           5 May 2000 SvdB V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
