package org.unicore.outcome;

import java.util.Hashtable;

import org.unicore.AAIdentifier;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.RepeatGroup;
import org.unicore.sets.ActionGroup_OutcomeList; 

/**
 * The results of executing an {@link org.unicore.ajo.RepeatGroup}. 
 * <p>
 * A RepeatGroup is a container for a number of outcomes of a repeatedly
 * executed ActionGroup, and so its Outcome is a list of the Outcomes of
 * repeated executions of the ActionGroup. This list is orderd according
 * to the order of execution. Each element of the list is
 * ActionGroup_Outcome.
 * <p>
 * The logging and status information in the RepeatGroup_Outcome
 * relate to the execution of the RepeatGroup alone (information
 * about the execution of each of the interations is contained in
 * each element of the ActionGroup_OutcomeList - each of which contains
 * the Outcoomes of the actions in the RepeatGroup's DAG). In partictular,
 * here is only one instance of the outcome of the RepeatGroup itself.
 * <p>
 * The status will be EXECUTING while any child AbstractAction of any (i.e. the
 * current) iteration is still executing. It will be SUCCESSFUL if all its children
 * have finished executing, have a SUCCESSFUL status, and the expression
 * has evaluated to true. If any of the RepeatGroup's children is NOT_SUCCESSFUL,
 * then the RepeatGroup finishes with a status of NOT_SUCCESSFUL at the
 * end of that iteration, regardless of the expression evaluation.
 * <p>
 * Note that the RepeatGroup is a subtype of ActionGroup and therefore all
 * ActionGroup methods apply and act on the last iteration of the RepeatGroup.
 *
 * @see org.unicore.ajo.RepeatGroup
 *
 * @author David F. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version RepeatGroup_Outcome.java 1.5 07/18/02
 * 
 **/
public class RepeatGroup_Outcome extends ActionGroup_Outcome {

    static final long serialVersionUID = 0;

    private ActionGroup_OutcomeList iteration_outcomes;
  
    public RepeatGroup_Outcome() {
        this((AAIdentifier)null,null);
    }

    /**
     * Create a new RepeatGroup_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus the ActionGroup_Outcomes of the repeated iterations of the 
     * action group contained in the RepeatGroup.
     *
     * @param source The RepeatGroup for which this Outcome will hold the results.
     * @param initial The status of the RepeatGroup at the time that the Outcome is created.
     *
     **/
    public RepeatGroup_Outcome(RepeatGroup source, AbstractActionStatus initial) {
        this(source.getId(),initial);
    }

    public RepeatGroup_Outcome (AAIdentifier id, AbstractActionStatus initial) {
        super (id, initial); 
        iteration_outcomes = new ActionGroup_OutcomeList();
        startNewIteration();
    }
    
    /**
     * Returns a list of all ActionGroup iterations.
     * <p>
     * <b>Pre:</b> true
     * <br>
     * <b>Post:</b> Returns the list
     * <p>
     **/
    public ActionGroup_OutcomeList getOutcomeList() {
        return iteration_outcomes;
    }

    private ActionGroup_Outcome this_iterations_outcome;
    /**
     * This method must be called at the start of every
     * new iteration (after the first).
     *
     **/
    public void startNewIteration() {
        this_iterations_outcome = new ActionGroup_Outcome (this.getId(),this.getStatus());
        child_outcomes = this_iterations_outcome.child_outcomes;
        iteration_outcomes.add(this_iterations_outcome);
    }

    /**
     * Add an entry to the log of the RepeatGroup_Outcome <em>and</em> the
     * log of the ActionGroup_Outcome for the current iteration;
     *
     **/
    public void addLogEntry(String entry) {
        super.addLogEntry(entry);
        this_iterations_outcome.addLogEntry(entry);
    }

    public void setStatus(AbstractActionStatus new_status) {
        this.setStatus(new_status,(String)"");
    }

    public void setStatus(AbstractActionStatus new_status, String new_reason) {
        super.setStatus(new_status,new_reason);
        if(this_iterations_outcome != null) this_iterations_outcome.setStatus(new_status,new_reason);
    }

    private int completed_iterations;
    /**
     * Return teh number of completed iterations
     *
     **/
    public int getCompletedIterations() {return completed_iterations;}

    public void setCompletedIterations(int i) {completed_iterations=i;}

}
//
// Change history:
//       12 Jul 2002 SvdB Reworked holding iterations
//       18 Mar 2002 SvdB V4 no arg constructor
//       31 Jan 2001  DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
