package org.unicore.resources;

/**
 * This Resource is for performance related resources, flops, data transaction, data transfer.
 * <p>
 * In addition to being defined in terms of their units (bytes, flops), 
 * performance resources can be described in terms of time at a given level of
 * performance. Therefore, performance resources are described interms of 
 * three values: the resquest units, the performance of the resource, and the
 * time equivalent to the requested units at the given performance.
 * Setting the time or units values for max request and request will automaitcally
 * alter the other. Also, changing the performance will change the time field.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version PerformanceResource.java 1.11 09/09/02 
 *
 **/
public abstract class PerformanceResource extends CapacityResource {

    static final long serialVersionUID = -5299074927623319370L;

    protected PerformanceResource() {
        this("",0.0,0.0,0.0,0.0);
    }

    /**
     * Create a new PerformanceResource.
     *
     * @param description A description ofthe PerformanceResource
     * @param maxrequest Maximum amount of resource allowed
     * @param defaultrequest Amount of resource granted if no specific request received
     * @param minrequest Minimum amount of resource allowed
     * @param performance The performance of the resource
     *
     **/
    protected PerformanceResource (String description, 
                                double maxrequest,
                                double defaultrequest,
                                double minrequest,
                                double performance) {
        super (description, maxrequest, defaultrequest, minrequest);
        this.setPerformance (performance);
    }

    private double performance;
    /**
     * Get the performance of this resource with respect to its units.
     * <p>
     * The performance is interpreted as this.units per second.
     *
     **/
    public  double getPerformance () {
        return performance;
    }

    /**
     * Set a new value for the performance.
     * <p>
     * <em>Precondition:</em> performance is not zero.
     *
     **/
    public  void   setPerformance (double performance) {
        this.performance  = performance;
        this.time_request = this.getRequest() / this.performance;
        this.max_time     = this.getMaxRequest() / this.performance;
        this.min_time     = this.getMinRequest() / this.performance;
        this.default_time     = this.getDefaultRequest() / this.performance;
    }

    private double time_request;
    /**
     * Get the time of this resource.
     * 
     **/
    public  double getTime () {
        return time_request;
    }

    /**
     * Set the time of this resource.
     * <p>
     * This is relative to the
     * current performance of this resource. The value of the request
     * is therefore changed accordingly.
     *
     **/
    public  void   setTime (double time_request) {
        this.time_request = time_request;
        super.setRequest (this.performance * this.time_request);
    }

    private double max_time;
    /**
     * Get the  max time allowed for this resource.
     *
     **/
    public  double getMaxTime () {
        return max_time;
    }

    /**
     * Set the  max time allowed for this resource.
     * <p>
     * The max time request of this resource is relative to the
     * current performance of this resource. The value of the max request
     * is therefore changed accordingly.
     *
     **/
    public  void   setMaxTime (double max_time) {
        this.max_time = max_time;
        super.setMaxRequest (this.performance * this.max_time);
    }

    private double min_time;
    /**
     * Get the  min time allowed for this resource.
     *
     **/
    public  double getMinTime () {
        return min_time;
    }

    /**
     * Set the min time allowed for this resource.
     * <p>
     * The min time request of this resource is relative to the
     * current performance of this resource. The value of the min request
     * is therefore changed accordingly.
     *
     **/
    public  void   setMinTime (double min_time) {
        this.min_time = min_time;
        super.setMinRequest (this.performance * this.min_time);
    }

    private double default_time;
    /**
     * Get the  default time allowed for this resource.
     *
     **/
    public  double getDefaultTime () {
        return default_time;
    }

    /**
     * Set the default time allowed for this resource.
     * <p>
     * The default time request of this resource is relative to the
     * current performance of this resource. The value of the default request
     * is therefore changed accordingly.
     *
     **/
    public  void   setDefaultTime (double default_time) {
        this.default_time = default_time;
        super.setDefaultRequest (this.performance * this.default_time);
    }

    /**
     * The setMaxRequest method needs to be overridden to support the
     * automatic alternation of max_request and max_time.
     *
     **/
    public void setMaxRequest (double maxrequest) {
        super.setMaxRequest (maxrequest);
        this.max_time = this.getMaxRequest() / this.performance;
    }

    /**
     * The setMinRequest needs to be overridden to support the
     * automatic alternation of min_request and min_time.
     *
     **/
    public void setMinRequest (double minrequest) {
        super.setMinRequest (minrequest);
        this.min_time = this.getMinRequest() / this.performance;
    }

    /**
     * The setDefaultRequest needs to be overridden to support the
     * automatic alternation of default_request and default_time.
     *
     **/
    public void setDefaultRequest (double defaultrequest) {
        super.setDefaultRequest(defaultrequest);
        this.default_time = this.getDefaultRequest() / this.performance;
    }

    /**
     * The setRequest method needs to be overridden to support the
     * automatic alternation of time_request and request.
     *
     **/
    public void setRequest (double request) {
        super.setRequest (request);
        this.time_request = this.getRequest() / this.performance;
    }

}
//
// Change history:
//          15 Mar 2002 SvdB V4, no arg constructor
//          05 Oct 2000 SvdB added min request changes 
//          14 Aug 2000 DFS Added minimum request.
//          21 Jan 2000 DFS  First release with V3.0 resource model.
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
