package org.unicore.resources;

import org.unicore.Unicore;

/**
 * Describes a resource on a Vsite.
 * <p>
 * These classes are used for both resource description and resource requests.
 * Resource descriptions are sent from Vsites to clients (in reply 
 * to a {@link org.unicore.ajo.GetResourceDescription}) to tell the client
 * what resources are available. Resource requests are sent from client to 
 * Vsite with instances of {@link org.unicore.ajo.ExecuteTask} to say
 * that the incarnation of the task needs the listed resources. The same
 * classes are used as resource description and resource request although
 * some fields are significant in one mode and not significant in the other.
 * <p>
 * Subclasses are cloneable so that they can be copied by clients
 * for return as requests. However, all clones are shallow (only
 * copies references) and so changing values of fields in the clones must be
 * done using the set methods and <em>not</em> directly
 * on any returned field instances.
 * <p>
 * As these instances are passed from Java VM to Java VM the equals()
 * method has been overidden. Instances of Resources are equal if they refer to
 * the same actual resources in the resource description.
 * Changing just the requested quantity will not make a resource different
 * according to the equals() method.
 *
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version Resource.java 1.11 05/13/02 
 *
 **/
public abstract class Resource extends Unicore implements Cloneable {

    static final long serialVersionUID = -3575429329054146821L;

    protected Resource() {
        this("");
    }

    /**
     * Create a new Resource with the given description.
     *
     **/
    protected Resource(String description) {
        this.description = description;
    }

    private String description;
    /**
     * Return the Resource's description.
     * <p>
     * The string describing the resource. This is informational only
     * and has no semantic meaning. It may be used by a client to describe
     * the resource to the user.
     *
     **/
    public  String getDescription ()                   { return description; }

    /**
     * Change the Resource's description.
     *
     **/
    public  void   setDescription (String description) { this.description = description; }

    /**
     * Make a shallow clone available for all Resources.
     *
     **/
    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }

}
// 
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor
//          22 Feb 2001 SvdB Properly cloneable, stripped toString
//          13 Jul 2000 SvdB Made cloneable, properly 22-02-01
//          21 Jan 2000 DFS First release with V3.0 resource model. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
