package org.unicore.sets;

import java.util.Vector;

import org.unicore.*;
import org.unicore.ajo.*;
import org.unicore.outcome.*;
import org.unicore.resources.*;
import org.unicore.upl.*;

/**
 * <tt>PortfolioList</tt> provides an ordered list of <tt>Portfolio</tt> elements. 
 *
 * @author M. Drescher
 *
 * @since AJO 1
 *
 * @version listTemplate 1.11 09/09/02 
 *
 */
public class PortfolioList extends Unicore {

    private Vector list;

    public PortfolioList() {
        list = new Vector();
    }
        
    public PortfolioList(Portfolio first) {
        list = new Vector();
        add(first);
    }
        
    /**
     * Test if a <tt>Portfolio</tt> is in the list.
     * <p>
     * <b>Pre:</b>  True.  
     * <b>Post:</b> Return true iff the <tt>Portfolio</tt> is in the list
     * <p>
     *
     * @return Return true iff the <tt>Portfolio</tt> is in the list
     *
     **/
    public boolean isIn(Portfolio thing) { return list.contains(thing); }

    /**
     * Return the number of elements in the PortfolioList.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> Return the number of elements
     *
     * @return the number of elements in the PortfolioList
     *
     **/
    public int size() { return list.size();}

    /**
     * Append a <tt>Portfolio</tt> to this list.
     * <p>
     * <b>Pre:</b>  true
     * <b>Post:</b> If the <tt>Portfolio</tt> was non-null, then the <tt>Portfolio</tt> is at the end of the list.
     * <p>
     *
     * @param thing A <tt>Portfolio</tt>.
     */
    public void add(Portfolio thing) { if(thing != null) list.addElement(thing); }

    /**
     * Sets a <tt>Portfolio</tt> at the specified position <tt>pos</tt>.
     * The <tt>Portfolio</tt> occupying this position is discarded. 
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> If the <tt>Portfolio</tt> was non-null, then the <tt>Portfolio</tt> is in the list at pos and
     *              the <tt>Portfolio</tt> that was at pos in not in the list
     *
     * @param thing A <tt>Portfolio</tt>
     * @param pos An index in the list
     */
    public void setAt(Portfolio thing, int pos) {
        if(thing != null) {
            try{
                 list.setElementAt(thing,pos);
            }
            catch(ArrayIndexOutOfBoundsException ex) {
                System.err.println("This must not happen in org.unicore.sets.PortfolioList (pre-condition)"+ex);
            }
        }
    }
            
    /**
     * Insert a <tt>Portfolio</tt> at the specified position.
     *
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> If the <tt>Portfolio</tt> was non-null, then the <tt>Portfolio</tt> is in the list at pos 
     *
     * @param thing A <t>Portfolio</tt>
     * @param pos An index in the list
     */
    public void insertAt(Portfolio thing, int pos) {
        if(thing != null) {
            try{
                 list.insertElementAt(thing,pos);
            }
            catch(ArrayIndexOutOfBoundsException ex) {
                System.err.println("This must not happen in org.unicore.sets.PortfolioList (pre-condition)"+ex);
            }
        }
    }

    /**
     * Removes the <tt>Portfolio</tt> from the list.
     *
     * <p>
     * <b>Pre:</b>  True. 
     * <b>Post:</b> The <tt>Portfolio</tt> is not in the list
     * <p>
     *
     * @param thing A <tt>Portfolio</tt>
     */
    public void remove(Portfolio thing) { list.removeElement(thing);}

    /**
     * Removes the Portfolio at the specified position.
     *
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> the <tt>Portfolio</tt> that was at pos in not in the list
     * <p>
     *
     * @param pos An index in the list
     */
    public void removeAt(int pos) {
        try{
             list.removeElementAt(pos);
        }
        catch(ArrayIndexOutOfBoundsException ex) {
            System.err.println("This cannot happen (pre-condition)"+ex);
        }
    }

    /**
     * Removes all objects in the list.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> The list is empty
     *
     **/
    public void removeAll() {list.removeAllElements();}

    /**
     * Returns the position of the <tt>Portfolio</tt> in the list. 
     *
     * <p>
     * <b>Pre:</b> The <tt>Portfolio</tt> is in the list 
     * <b>Post:</b> Returns the position of the <tt>Portfolio</tt>
     * <p>
     *
     * @param thing The <tt>Portfolio</tt> to locate
     * @return The position of the <tt>Portfolio</tt>
     *
     **/
    public int pos(Portfolio thing) { return list.indexOf(thing);}

    /**
     * Returns the <tt>Portfolio</tt> at the specified position.
     *
     * <p>
     * <b>Pre:</b> 0 <= pos < size() 
     * <b>Post:</b> the <tt>Portfolio</tt> at pos
     * <p>
     *
     * @param pos An index in the list
     * @return A <tt>Portfolio</tt>
     *
     **/
    public Portfolio getAt(int pos) {
        try{
             return (Portfolio) list.elementAt(pos);
        }
        catch(ArrayIndexOutOfBoundsException ex) {
            System.err.println("This must not happen in org.unicore.sets.PortfolioList (pre-condition)"+ex);
            return null;
        }
    }

    /**
     * Returns a <tt>PortfolioEnumeration</tt>.
     *
     * <p>
     * <b>Pre:</b> true 
     * <b>Post:</b> the enumeration (in no particular order)
     * <p>
     *
     *
     * @return A <tt>PortfolioEnumetaion</tt>
     */
    public PortfolioEnumeration elements() {return new PortfolioEnumeration(list);}

    /**
     * Return a shallow clone of the list.
     *
     * The clone is shallow to match the style
     * of the java.util collections.
     *
     **/
    public PortfolioList cloneList() {
        // Do not use clone as we do not return an Object
        // so compiler will complain (Object sits poorly with rest)
        PortfolioList result = new PortfolioList(); 
        result.list = (Vector) list.clone();
        return result;
    }

    static final long serialVersionUID = 2L;
}
//
// Change history
//          31 Jan 2001 SvdB Rejects null elements
//           4 Oct 2001 SvdB Unfinalised
//           5 May 2000 SvdB V3.0 
//          11 Mar 1999 SvdB Includes serialVersionUID 
//          22 Dec 1998 SvdB SvdB, Added constructor with initial element 
//           7 Dec 1998 SvdB Second release to Unicore 
//           6 Nov 1998 SvdB SvdB Added clone method 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
