package org.unicore.utility;

import java.io.OutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * An implementation of a writer of a packetised
 * data stream as specified in the UPL.
 *
 * Each write causes a packet to be sent.
 *
 * Each packet is preceeded by a 4 byte header
 * giving the size of the following packet. The final packet
 * in a group is indicated by a 0 or negative size (absolute
 * value of this is the length of the final packet). Multiple
 * groups of packets can be written.
 *
 * The size of packets can be controlled by wrapping
 * this in a BufferedOutputStream.
 * 
 * @see org.unicore.utility.PacketisedInputStream
 *
 * @author Sven van den Berghe (fecit) 
 *
 * @since AJO 3
 *
 * @version PacketisedOutputStream.java 1.4 05/13/02
 *
 **/
public class PacketisedOutputStream extends OutputStream {

    private DataOutputStream stream;

    /**
     * Create a PacketisedOutputStream based on the output stream.
     *
     **/
    public PacketisedOutputStream(OutputStream base_stream) {

        // Build a convenient way to write in the header bytes
        // on top of this stream. Most calls are onto the
        // write() methods which should be efficient.

        stream = new DataOutputStream(base_stream); 

    }

    /**
     * Write a single byte packet to the underlying output stream. (This is preceeded
     * by a 4 byte header with the value 1)
     *
     **/
    public void write(int b) throws IOException {
        // DEBUG System.out.println("PacketisedOutputStream writing 1 byte");
        stream.writeInt(1);
        stream.write(b);
    }

    /**
     * Write a b.length bytes packet to the underlying output stream. (This is preceeded
     * by a 4 byte header with the value b.length)
     *
     **/
    public void write(byte[] b) throws IOException {
        // DEBUG System.out.println("PacketisedOutputStream writing "+b.length+" bytes");
        stream.writeInt(b.length);
        stream.write(b);
    }

    /**
     * Write a len bytes packet to the underlying output stream. (This is preceeded
     * by a 4 byte header with the value len)
     *
     **/
    public void write(byte[] b, int off, int len) throws IOException {
        // DEBUG System.out.println("PacketisedOutputStream writing "+len+" bytes offset "+off);
        stream.writeInt(len);
        stream.write(b,off,len);
    }


    /**
     * Write a single byte packet to the underlying output stream marking it
     * as the final packet in this group. (This is preceeded
     * by a 4 byte header with the value -1)
     *
     **/
    public void writeFinal(int b) throws IOException {
        stream.writeInt(-1);
        stream.write(b);
    }

    /**
     * Write a b.length bytes packet to the underlying output stream marking it
     * as the final packet in this group. (This is preceeded
     * by a 4 byte header with the value -b.length)
     *
     **/
    public void writeFinal(byte[] b) throws IOException {
        stream.writeInt(-b.length);
        stream.write(b);
    }

    /**
     * Write a len bytes packet to the underlying output stream marking it
     * as the final packet in this group. (This is preceeded
     * by a 4 byte header with the value -len)
     *
     **/
    public void writeFinal(byte[] b, int off, int len) throws IOException {
        stream.writeInt(-len);
        stream.write(b,off,len);
    }


    /**
     * Flush this and any underlyiny output streams.
     * Passes this call directly to the underlying stream.
     *
     **/
    public void flush() throws IOException {
        stream.flush();
    }

    /**
     * Close this group of packets. This method writes
     * a final packet 0 length and flushes it and call the close method
     * on the underlying stream.
     *
     **/
    public void close() throws IOException {
        stream.writeInt(0);
        stream.flush();
        stream.close();
    }

    /**
     * Finish writing a group of packets without closing
     * the underlying stream.
     *
     **/
    public void finish() throws IOException {
        stream.writeInt(0);
        stream.flush();
    }

}
//
// Change history:
//          26 Sep 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
